/*
 * Copyright 1995, 1996, 1997 by Jun Yei.	All Rights Reserved
 *
 * Permission to retain, use, modify, copy, and distribute ChinesePower
 * in source or binary and its documentation (hereafter, the Software)
 * for noncommercial purpose is hereby granted to you without a fee,
 * provided that this entire copyright and permission notice appear in
 * all such copies, that no charge be associated with such copies,
 * that distribution of derivative works (including value-added
 * distributions such as with additional input dictionaries or fonts)
 * include clarification that such added or derived parts are not from
 * the original Software, and that the names of the author(s) not be
 * used to endorse or promote such works. The author(s) of the software
 * makes no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * THE AUTHOR(S) OF THE SOFTWARE DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 * IN NO EVENT SHALL THE AUTHOR(S) OF THE SOFTWARE BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Jun Yei, Ottawa, ON, Canada
 */

/*
 *	$Id: HZInput.c,v 3.3 1995/01/17 10:56:02 ygz Exp $
 */

/***********************************************************
Copyright 1994,1995 by Yongguang Zhang.  All Rights Reserved

Permission to retain, use, modify, copy, and distribute CXTERM 5.0
in source or binary and its documentation (hereafter, the Software)
for non-commercial purpose is hereby granted to you without a fee,
provided that this entire copyright and permission notice appear in
all such copies, that no charge be associated with such copies,
that distribution of derivative works (including value-added
distributions such as with additional input dictionaries or fonts)
include clarification that such added or derived parts are not from
the original Software, and that the names of the author(s) not be
used to endorse or promote such works.

THE AUTHOR(S) DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
THE AUTHOR(S) BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

******************************************************************/

/* HZInput.c
 *
 * This file defines the interface between xterm and cxterm input module.
 *
 * The following external function are called by xterm.
 *
 *	function		called by
 * -------------------		----------
 * CreateCXtermInput()		charproc.c 	(to create the input module)
 * DestroyCXtermInput()		charproc.c 	(to destroy the input module)
 * RealizeCXtermInput()		charproc.c 	(when xterm widget realizes)
 * RefreshCXtermInput()		charproc.c util.c
 *				(when xterm refreshes the screen, e.g. expose)
 * ResetCXtermInput()		charproc.c	(when xterm is full-reset)
 * ResizeCXtermInput()		charproc.c screen.c scrollbar.c
 *				(when xterm resizes the screen)
 * HandleSwitchHZMode()		charproc.c	(switch-HZ-mode Xt action)
 * HandleSetHZParam()		charproc.c	(set-HZ-parameter Xt action)
 * HandleClickHZinArea()	charproc.c	(click-HZ-area Xt action)
 * HandlePopupPanel()		charproc.c menu.c
 *				(popup-panel Xt action/cxtermconfig menu item)
 * SetHZinDir()			misc.c		(esc seq to set HZINPUTDIR)
 * SetHZinMethod()		misc.c		(esc seq to switch HZ mode)
 * SetHZinParam()		misc.c		(esc seq to set parameters)
 * HZParseInput()		input.c		(keyseq => HZ)
 *
 * The following external function are called by cxterm.
 *	function		called by
 * -------------------		----------
 * XtermWritePty()		HZinArea.c 	(to write data to pty)
 *
 */

#include "HZinput.h"		/* X headers included here. */

/* could be moved to ptyx.h */
extern void CreateCXtermInput();
extern void DestroyCXtermInput();
extern void RealizeCXtermInput();
extern void ResizeCXtermInput();
extern void RefreshCXtermInput();
extern void ResetCXtermInput();
extern void HandleSwitchHZMode();
extern void HandleSetHZParam();
extern void HandleClickHZinArea();
extern void HandlePopupPanel();
extern void SetHZinDir();
extern void SetHZinMethod();
extern void SetHZAssoc();
extern void SetHZinParam();
extern  int HZParseInput();

/*CXtermInputModule cxtermInput; 
*/
/* The only global variable in the input module: _THE_ CXterm Input Module */

void CreateCXtermInput(cxtermInput, xw)
XtermWidget xw;
CXtermInputModule *cxtermInput;
{
  TScreen *screen = &xw->screen;

    cxtermInput->realized = FALSE;
    screen->hzIwin.rows = ROWSINPUTAREA;	/* num of rows in Input Area */
    screen->hzIwin.cols = Width(screen)/FontWidth(screen) - 1;
    xw->misc.assoc_files = strdup(DEFAULT_ASSO_FILE);
    xw->misc.f_hn = NULL;
    xw->misc.f_hb = NULL;
    xw->misc.hz_mode = NULL;
    HZimInit (cxtermInput, xw, screen);
    HZiaInit (&cxtermInput->hzia, &xw->screen);
}

/*ARGSUSED*/
void DestroyCXtermInput(cxtermInput)
CXtermInputModule *cxtermInput;
{
    HZimCleanUp (cxtermInput);
    HZiaCleanUp (&cxtermInput->hzia);
}

void RealizeCXtermInput(cxtermInput, xw)
CXtermInputModule *cxtermInput;
XtermWidget xw;
{
    cxtermInput->realized = TRUE;

    if (xw->misc.hz_mode == NULL)
    {
    /* association list must be loaded before the first input method */ 
    	if (xw->misc.assoc_files)
		SetHZAssoc (cxtermInput, xw->misc.it_dirs, xw->misc.assoc_files);

	HZswitchModeByNum (cxtermInput, 0);	/* initially, ASCII mode */
    /* don't draw it now, leave it to the Expose event */
	}
}

void ResizeCXtermInput(cxtermInput, screen)
CXtermInputModule *cxtermInput;
TScreen *screen;
{
    screen->hzIwin.rule_x = 0;
    screen->hzIwin.rule_y = 0;
    screen->hzIwin.rule_length = Width(screen);
    screen->hzIwin.x = 0;
    screen->hzIwin.y = 1;
    screen->hzIwin.width = Width(screen);
    screen->hzIwin.height = Height(screen);
    screen->hzIwin.rows = ROWSINPUTAREA; 
    screen->hzIwin.cols = Width(screen)/FontWidth(screen) - 1;

    HZiaResize (&cxtermInput->hzia, screen, cxtermInput);
    /* don't redraw, leave it to the Expose event */
}

void RefreshCXtermInput(cxtermInput, screen)
CXtermInputModule *cxtermInput;
TScreen *screen;
{
    HZiaRedraw (&cxtermInput->hzia, screen);
}

void ResetCXtermInput(cxtermInput, xw)
CXtermInputModule *cxtermInput;
XtermWidget xw;
{
  TScreen *screen = &xw->screen;

    HZimCleanUp (cxtermInput);
    HZimInit (cxtermInput, xw, screen);
    if (xw->misc.assoc_files)
	SetHZAssoc (cxtermInput, xw->misc.it_dirs, xw->misc.assoc_files);
    HZswitchModeByNum (cxtermInput, 0);	/* back to ASCII mode */
    RefreshCXtermInput(cxtermInput, screen);
}

void SetHZinDir (xw, dir)
    XtermWidget xw;
    String dir;
{
  char *cp = malloc((unsigned)strlen(dir) + 1);

    if (cp == NULL)
	return;
    strcpy(cp, dir);
    if(xw->misc.it_dirs)
	free(xw->misc.it_dirs);
    xw->misc.it_dirs = cp;
}

void SetHZinMethod (cxin, xw, name, doredraw)
CXtermInputModule *cxin;
XtermWidget xw;
String name;
Bool doredraw;
{
  int i;

    if (!name || !(*name))
	return;		/* nothing to switch */

    for (i = 0; i < cxin->numHZim; i++) {
	if (! strcmp(name, cxin->imtbl[i].name)) {
	    /* already loaded */
	    break;
	}
    }
    if (i == cxin->numHZim) {	/* not loaded */
	if (cxin->numHZim == MAX_HZIM) {	/* too many input methods */
	    HZprintfMsg(cxin, "Too many input methods (max = %d)", MAX_HZIM);
	    return;
	} else {
	    if (HZLoadInputMethod (cxin, name, xw->misc.it_dirs,
			cxin->xterm->misc.hz_encoding, &(cxin->imtbl[i]) ))
	    {
		/* non-zero return: loading unsuccessfully */
		return;
	    }
	    cxin->numHZim++ ;
	}
    }
    xw->misc.hz_mode = name;
    HZswitchModeByNum (cxin, i);

    /* enable it if temporarily disable. */
    if (cxin->temp_disable) {
	cxin->temp_disable = False;
	HZiaSetSensitive (&cxin->hzia, True);
    }
    if (doredraw)
	RefreshCXtermInput (cxin, &xw->screen);
}

void SetHZAssoc (cxin, inputdir, name)
CXtermInputModule *cxin;
char *inputdir;
String name;
{
    cxin->hzal = LoadAssocList (cxin, name, inputdir, cxin->xterm->misc.hz_encoding);
    if (cxin->hzal)
    {
	cxin->global.do_auto_segment = True;
	cxin->global.do_ps_assoc = True;
    }
}

void SetHZinParam (cxtermInput, str)
CXtermInputModule *cxtermInput;
char *str;
{
    HZimSetParam (cxtermInput, str);
}

int HZParseInput (cxtermInput, screen, nbytes, strbuf)
CXtermInputModule *cxtermInput;
TScreen *screen;
int nbytes;
char strbuf[];
{
    if (cxtermInput->mode == 0)	/* ASCII */
	return (nbytes);
    if (cxtermInput->temp_disable)	/* temporarily disable  */
	return (nbytes);

    /* optimize the simply cases */
    if (nbytes == 1) {
	
	switch(strbuf[0]) {
	  case '<':
	   strbuf[0] = GlobalDefaults.comma[0];
	   strbuf[1] = GlobalDefaults.comma[1];
	   return(2);

	  case '>':
	   strbuf[0] = GlobalDefaults.period[0];
	   strbuf[1] = GlobalDefaults.period[1];
	   return(2);
	
	  default:

	   return ((* cxtermInput->chzif) (cxtermInput, strbuf[0], strbuf));
   	} 
    } else if (nbytes == 0) {
	return (0);
    } else {
	char istr[100];	/* match the STRBUFSIZE in input.c */
	int i, nbytesOut = 0;

	strncpy (istr, strbuf, nbytes);
	for (i = 0; i < nbytes; i++) {

	    nbytesOut += (* cxtermInput->chzif) (cxtermInput,
				istr[i], &strbuf[nbytesOut]);
	}
	return (nbytesOut);
    }
}

#include <stdio.h>

/*
 * HZprintfMsg(...)	printf an error message
 */
/*VARARGS2*/
void HZprintfMsg(cxin, str, s1, s2, s3, s4, s5, s6)
CXtermInputModule *cxin;
char *str;
char *s1, *s2, *s3, *s4, *s5, *s6;
{
    if (cxin->realized) {
	char tmpstr[128];

	(void) sprintf(tmpstr, str, s1, s2, s3, s4, s5, s6);
	HZiaShowMesg (&(cxin->hzia), cxin->screen, tmpstr);
    } else {
	(void) fprintf(stderr, str, s1, s2, s3, s4, s5, s6);
	(void) fprintf(stderr, "\n");
    }
}

Bell(cxtermInput)
CXtermInputModule *cxtermInput;
{
	XBell(cxtermInput->xterm->screen.display, 0);
}

