/*
 * Copyright 1995, 1996, 1997 by Jun Yei.       All Rights Reserved
 *
 * Permission to retain, use, modify, copy, and distribute ChinesePower
 * in source or binary and its documentation (hereafter, the Software)
 * for noncommercial purpose is hereby granted to you without a fee,
 * provided that this entire copyright and permission notice appear in
 * all such copies, that no charge be associated with such copies,
 * that distribution of derivative works (including value-added
 * distributions such as with additional input dictionaries or fonts)
 * include clarification that such added or derived parts are not from
 * the original Software, and that the names of the author(s) not be
 * used to endorse or promote such works. The author(s) of the software
 * makes no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * THE AUTHOR(S) OF THE SOFTWARE DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 * IN NO EVENT SHALL THE AUTHOR(S) OF THE SOFTWARE BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Jun Yei, Ottawa, ON, Canada
 */

#include "chpower.h"

void CloseCharSetCB();
void ApplyCharSetCB();

CharSetClass charset;

void DestroyCharSet()
{
	int i, total;

	total = TotalNumOfFont() + 2;

	for(i = 0; i < total; i++)
	{
		if(charset.fontlist[i]) XmStringFree(charset.fontlist[i]);
		else break;
	}

	free(charset.fontlist);
}

void CreateCharSet(parent)
Widget parent;
{
	Arg args[MAX_ARGS];
	register int n;
	int i, total;	
	char line[10];
	Widget label;

	charset.PopUpWg = XtCreatePopupShell("Char Window", transientShellWidgetClass, parent, NULL, 0);
	charset.charsetmapped = False;

	n = 0;
	XtSetArg(args[n], XmNheight, 170); n++;
	XtSetArg(args[n], XmNwidth, 250); n++;
	XtSetArg(args[n], XmNhorizontalSpacing, 0); n++;
	XtSetArg(args[n], XmNverticalSpacing, 0); n++;
	XtSetArg(args[n], XmNrubberPositioning, True); n++;
	charset.Root = XmCreateForm(charset.PopUpWg, "CharSet", args, n);
	XtManageChild(charset.Root);
	XtAddCallback(charset.Root, XmNdestroyCallback, MainDestroyCB, NULL);

	total = TotalNumOfFont();
	
	charset.fontlist = (XmString *)malloc((total+2) * sizeof(XmString *));

	charset.fontlist[0] = XmStringCreateLtoR("AS IS", XmSTRING_DEFAULT_CHARSET);

	for(i = 1; i <= total; i++)
		charset.fontlist[i] = XmStringCreateLtoR(GetFontId(i), XmSTRING_DEFAULT_CHARSET);

	charset.fontlist[i] = NULL;

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Font Type:", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNx, 15); n++;
	XtSetArg(args[n], XmNy, 10); n++;
	label = XmCreateLabelGadget(charset.Root, "Ftype", args, n);
	XtManageChild(label);

	n = 0;
	XtSetArg(args[n], XmNitems, charset.fontlist); n++;
	XtSetArg(args[n], XmNitemCount, total+1); n++; 
	XtSetArg(args[n], XmNx, 15); n++;
	XtSetArg(args[n], XmNy, 30); n++;
	XtSetArg(args[n], XmNheight, 120); n++;
	XtSetArg(args[n], XmNwidth, 100); n++;
	charset.HzFontLstWg = XmCreateScrolledList(charset.Root, "HZfontlist", args, n);
	XtManageChild(charset.HzFontLstWg);

	n = 0;  
	XtSetArg(args[n], XmNlabelString, XmStringCreate(" Width:", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNx, 135); n++;
	XtSetArg(args[n], XmNy, 45); n++;
	label = XmCreateLabelGadget(charset.Root, "Width", args, n);
	XtManageChild(label);

	n = 0;
	XtSetArg(args[n], XmNwidth, 50); n++;	
	XtSetArg(args[n], XmNscrollHorizontal, False); n++;
	XtSetArg(args[n], XmNscrollVertical, False); n++;
	XtSetArg(args[n], XmNresizeHeight, False); n++;
	XtSetArg(args[n], XmNresizeWidth, False); n++;
	XtSetArg(args[n], XmNmarginHeight, 0); n++;
	XtSetArg(args[n], XmNmarginWidth, 0); n++;
	XtSetArg(args[n], XmNmaxLength, 3); n++;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(args[n], XmNleftWidget, label); n++;
	XtSetArg(args[n], XmNy, 45); n++;
	charset.WidthWg = XmCreateText(charset.Root, "width", args, n);
	XtManageChild(charset.WidthWg);

	n = 0;         
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Height:", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNx, 135); n++;
	XtSetArg(args[n], XmNy, 80); n++;
	label = XmCreateLabelGadget(charset.Root, "Height", args, n);
	XtManageChild(label);

	n = 0;
	XtSetArg(args[n], XmNwidth, 50); n++;
	XtSetArg(args[n], XmNscrollHorizontal, False); n++;
	XtSetArg(args[n], XmNscrollVertical, False); n++;
	XtSetArg(args[n], XmNresizeHeight, False); n++;
	XtSetArg(args[n], XmNresizeWidth, False); n++;
	XtSetArg(args[n], XmNmarginHeight, 0); n++;
	XtSetArg(args[n], XmNmarginWidth, 0); n++;
	XtSetArg(args[n], XmNmaxLength, 3); n++;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(args[n], XmNleftWidget, label); n++;
	XtSetArg(args[n], XmNy, 80); n++;
	charset.HeightWg = XmCreateText(charset.Root, "height", args, n); 
	XtManageChild(charset.HeightWg);

	n = 0;
	XtSetArg(args[n], XmNwidth, 50); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Apply", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNx, 130); n++;
	XtSetArg(args[n], XmNy, 120); n++;
	charset.ApplyBtn = XmCreatePushButtonGadget(charset.Root, "applyBtn", args, n);
	XtManageChild(charset.ApplyBtn);

	n = 0;
	XtSetArg(args[n], XmNwidth, 50); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Close", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNx, 182); n++;
	XtSetArg(args[n], XmNy, 120); n++;
	charset.CloseBtn = XmCreatePushButtonGadget(charset.Root, "closeBtn", args, n);
	XtManageChild(charset.CloseBtn);
	XtAddCallback(charset.CloseBtn, XmNactivateCallback, CloseCharSetCB, NULL);

	XtRealizeWidget(charset.PopUpWg);
}

int ChangeCharSet(viewport, hzdata1, hzdata2, opid1, opid2)
VIewportdata *viewport;
HZdata *hzdata1, *hzdata2;
int opid1, opid2;
{
	char line[20];
	unsigned short result[40];
	Arg args[MAX_ARGS];
	register int n;
	int i, width, height, fontchange = False, sizechange = False, NumOfInput;	
	int changefont = True, changesize = True;	
	char *line1 = NULL, *line2 = NULL;
	char *name = NULL;
	XmString *newfont = NULL;

	n = 0;
	XtSetArg(args[n], XmNvalue, &line1); n++;
	XtGetValues(charset.WidthWg, args, n);

	n = 0;
	XtSetArg(args[n], XmNvalue, &line2); n++;
	XtGetValues(charset.HeightWg, args, n); 

	if(strlen(line1) == 0 || strlen(line2) == 0) changesize = False;
	else
	{
		sscanf(line1, "%d", &width);
		sscanf(line2, "%d", &height);
	
		if(height > MAX_FONT_SIZE || width > MAX_FONT_SIZE)
		{
			if(line1) XtFree(line1);
			if(line2) XtFree(line2);
			ErrorPopup("Font Size is Too Big!");
			return(0);
		}

		if(height < MIN_FONT_SIZE || width < MIN_FONT_SIZE)     
		{
			if(line1) XtFree(line1);
			if(line2) XtFree(line2);
			ErrorPopup("Font Size is Too Small!");
			return(0);
		}

		width *= 100;
		height *= 100;
	}

	result[0] = 0x00;

	n = 0;
	XtSetArg(args[n], XmNselectedItems, &newfont); n++;
	XtGetValues(charset.HzFontLstWg, args, n);
	XmStringGetLtoR(*newfont, XmSTRING_DEFAULT_CHARSET, &name);

	if(strncmp(name, "AS IS", 5) == 0) changefont = False;

	if(changefont && strcmp(name, hzdata1->FontId))
	{
		fontchange = True;
		DBstrcat(result, "@[");
		DBstrcat(result, name);
		DBstrcat(result, "]");
	}
	
	if(changesize && (width !=  hzdata1->point_width || height != hzdata1->point_height))
	{
		sizechange = True;
		DBstrcat(result, "@[");
		DBstrcat(result, line1);
		if(width != height) 
		{
			DBstrcat(result, ",");
			DBstrcat(result, line2);
		}
		DBstrcat(result, "]");
	}
	
	NumOfInput = DBlen(result);
	if(NumOfInput > 0) 
	{
		Input(viewport, opid1, result, NumOfInput);
		viewport->set.StartPtr = viewport->set.TmpEPtr;	
		
		if(opid2 != CHARSETINS2) 
		{
			viewport->cursor.EndCurPtr += NumOfInput;	
			if(opid2 == CHARSETINS1)
				viewport->cursor.CurPtr = viewport->cursor.EndCurPtr;
			else
				viewport->cursor.MoveCurPtr = viewport->cursor.EndCurPtr; 
		}
	}
	else 
	{
		if(opid1 == CHARSETINS3) viewport->set.StartPtr = viewport->cursor.MoveCurPtr;
		else viewport->set.StartPtr = viewport->cursor.CurPtr;
	}
	
	result[0] = 0x00;
	
	if(changefont && ((fontchange == True && opid2 == CHARSETINS2) || (strcmp(name, hzdata2->FontId)!=0 && opid2 != CHARSETINS2)))
	{
		DBstrcat(result, "@[");
		DBstrcat(result, hzdata2->FontId);
		DBstrcat(result, "]");
	}
	
	if(changesize && ((sizechange == True && opid2 == CHARSETINS2) || ((width != hzdata2->point_width || height !=
	hzdata2->point_height) && opid2 != CHARSETINS2)))
	{
		if(hzdata2->point_width != hzdata2->point_height)
			sprintf(line, "@[%d,%d]", hzdata2->point_width/100, hzdata2->point_height/100);
		else
			sprintf(line, "@[%d]", hzdata2->point_width/100);
		DBstrcat(result, line);
	}

	NumOfInput = DBlen(result);
	if(NumOfInput > 0) 
	{
		Input(viewport, opid2, result, NumOfInput);
		viewport->set.EndPtr = viewport->set.TmpSPtr;  
	}
	else
	{
		if(opid2 == CHARSETINS3) viewport->set.TmpEPtr = viewport->set.EndPtr = viewport->cursor.MoveCurPtr;
		else viewport->set.TmpEPtr = viewport->set.EndPtr = viewport->cursor.CurPtr;
	}

	if(changefont && strcmp(name, hzdata1->FontId))
	{
		hzdata1->hbf = GetFontHandle(name, &hzdata1->FontId,&hzdata1->FontType);

	}

	if(changesize && (width !=  hzdata1->point_width || height != hzdata1->point_height))
	{
		hzdata1->point_width = width;
		hzdata1->point_height = height;
	}

	if(line1) XtFree(line1);
	if(line2) XtFree(line2);
	if(name) XtFree(name);

	if(!changesize && !changefont) return(0);
	if(changesize && !changefont) return(1);
	if(!changesize && changefont) return(2);
	return(3);
}

void GetCharSet(viewport)
VIewportdata *viewport;
{
	int i, j, id;
	
	if(viewport->cursor.StartCurPtr == viewport->cursor.EndCurPtr)
	{
		if(!ChangeCharSet(viewport, &viewport->cursor.hzdata, &viewport->cursor.hzdata, CHARSETINS1, CHARSETINS2)) return;
		viewport->cursor.CurPtr -= rmanysamecomponent(viewport->cursor.CurPtr, LEFTSEARCH);
		viewport->cursor.MoveCurPtr = viewport->cursor.StartCurPtr = viewport->cursor.EndCurPtr = viewport->cursor.newcurptr = viewport->cursor.CurPtr;
	}
	else
	{
		viewport->cursor.RedrawCurPtr = viewport->cursor.StartCurPtr;
		
		if(viewport->cursor.StartCurPtr == viewport->cursor.CurPtr)
		{
			id = ChangeCharSet(viewport, &viewport->cursor.hzdata, &viewport->cursor.Mohzdata, CHARSETINS1, CHARSETINS3);
			if(!id) return;
			j = rmanysamecomponent(viewport->set.EndPtr, WHOLESEARCHCL);	
			viewport->set.EndPtr -= j;
			switch(id)
			{
				case 1:
				j += rmtypedcomponent(viewport->set.StartPtr, viewport->set.EndPtr, SizeMask);
				break;

				case 2:
				j += rmtypedcomponent(viewport->set.StartPtr, viewport->set.EndPtr, FontMask);
				break;

				default:
				j += rmtypedcomponent(viewport->set.StartPtr, viewport->set.EndPtr, SizeMask | FontMask);
			}	
			i = rmanysamecomponent(viewport->set.StartPtr, WHOLESEARCHCL);   
			viewport->cursor.CurPtr = viewport->set.StartPtr - i;
			viewport->cursor.MoveCurPtr = viewport->set.TmpEPtr - i - j;
			viewport->cursor.StartCurPtr = viewport->cursor.CurPtr;
			viewport->cursor.EndCurPtr = viewport->cursor.MoveCurPtr;
		}
		else
		{
			id = ChangeCharSet(viewport, &viewport->cursor.Mohzdata, &viewport->cursor.hzdata, CHARSETINS3, CHARSETINS1);
			if(!id) return;
			i = rmanysamecomponent(viewport->set.EndPtr, WHOLESEARCHCL);  
			viewport->set.EndPtr -= i;
			switch(id)
			{
				case 1:
				i += rmtypedcomponent(viewport->set.StartPtr, viewport->set.EndPtr, SizeMask);
				break;

				case 2:
				i += rmtypedcomponent(viewport->set.StartPtr, viewport->set.EndPtr, FontMask);
				break;

				default:
				i += rmtypedcomponent(viewport->set.StartPtr, viewport->set.EndPtr, SizeMask | FontMask);
			}
			j = rmanysamecomponent(viewport->set.StartPtr, WHOLESEARCH);
			viewport->cursor.CurPtr = viewport->set.TmpEPtr - i - j;
			viewport->cursor.newcurptr -= i + j;
			viewport->cursor.MoveCurPtr = viewport->set.StartPtr - j;
			viewport->cursor.StartCurPtr = viewport->cursor.MoveCurPtr;
			viewport->cursor.EndCurPtr = viewport->cursor.CurPtr;
		}
		MyDrawPage(viewport, INVIRDRAW);	
		CheckCurrentPage(viewport);
		MyDrawPage(viewport, CLEARDRAW);	
	}
}

void RenewCharSet(viewport)
VIewportdata *viewport;
{
	Arg args[MAX_ARGS];
	register int n;
	int i, j, width, height;	
	char line[15];
	unsigned short *cp, *searchprt, chrmem;

	if(!charset.charsetmapped) return;
	
	i = 1;

	if(viewport->cursor.StartCurPtr != viewport->cursor.EndCurPtr)
	{
		if(strcmp(viewport->cursor.hzdata.FontId, viewport->cursor.Mohzdata.FontId) == 0)
		{
			for(cp = viewport->cursor.StartCurPtr; cp < viewport->cursor.EndCurPtr - 4; cp++)
			{
				if(*cp == '@' && *(cp+1) == '[' && *(cp+2) >= 'a' && *(cp+2) <= 'z')
				{
					cp += 2;
				
					strDBcpy(line, cp, 14);

					if(strncmp(viewport->cursor.hzdata.FontId, line, strlen(viewport->cursor.hzdata.FontId)) != 0) 
					{
						i = 0; break;
					}	
				}
			}
		
		}
		else i = 0;	
	}

	if(i) i = SearchFontList(viewport->cursor.hzdata.FontId);

	n = 0;
	XtSetArg(args[n], XmNselectedItems, &charset.fontlist[i]); n++;
	XtSetArg(args[n], XmNselectedItemCount, 1); n++; 
	XtSetValues(charset.HzFontLstWg, args, n);

	i = 1;
	j = 1;

	if(viewport->cursor.StartCurPtr != viewport->cursor.EndCurPtr)
	{
		if(viewport->cursor.hzdata.point_width != viewport->cursor.Mohzdata.point_width) i = 0;
		if(viewport->cursor.hzdata.point_height != viewport->cursor.Mohzdata.point_height) j = 0;
			
		if(i || j)
		{
			for(cp = viewport->cursor.StartCurPtr; cp < viewport->cursor.EndCurPtr - 4; cp++)
			{
				if(*cp == '@' && *(cp+1) == '[' && *(cp+2) >= '1' && *(cp+2) <= '9')
				{
					for(searchprt = cp, n = 0; *searchprt != '\0' && *searchprt != ']' && n < MAXIDNUM; searchprt++, n++);
					
					if(n >= MAXIDNUM)
					{
						cp += 2;
						continue;
					}

					chrmem = *searchprt;
					*searchprt = '\0';

					strDBcpy(line, cp, 14);

					if(sscanf(line, "%d,%d", &width, &height) == 2)
					{
						width *= 100;
						height *= 100;
					}
					else
					{
						width *= 100;
						height = width;
					}
					
					*searchprt = chrmem;
					cp = searchprt + 1;
					
					if(i && viewport->cursor.hzdata.point_width != width) i = 0;
					if(j && viewport->cursor.hzdata.point_height != height) j = 0;
			
					if(i == 0 && j == 0) break;
				}
			}
		}
	}

	
	if(i) sprintf(line, "%d\0", viewport->cursor.hzdata.point_width/100);
	else line[0] = '\0';

	n = 0;
	XtSetArg(args[n], XmNvalue, line); n++;
	XtSetArg(args[n], XmNmarginHeight, 0); n++;
	XtSetArg(args[n], XmNmarginWidth, 0); n++;
	XtSetValues(charset.WidthWg, args, n);

	if(j) sprintf(line, "%d\0", viewport->cursor.hzdata.point_height/100);
	else line[0] = '\0';   

	n = 0;
	XtSetArg(args[n], XmNvalue, line); n++;
	XtSetArg(args[n], XmNmarginHeight, 0); n++;
	XtSetArg(args[n], XmNmarginWidth, 0); n++;
	XtSetValues(charset.HeightWg, args, n); 

	XtRemoveAllCallbacks(charset.ApplyBtn, XmNactivateCallback);
	XtAddCallback(charset.ApplyBtn, XmNactivateCallback, ApplyCharSetCB, (caddr_t *)viewport);
}

void CharSetCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	if(!charset.charsetmapped)
	{
		charset.charsetmapped = True;
		active_viewport = (VIewportdata *)client_data;
		RenewCharSet((VIewportdata *)client_data);
		PopupPosition(active_viewport->resource.PopUpShell, charset.PopUpWg);	
		XtPopup(charset.PopUpWg, XtGrabNone); 
	}
	else XRaiseWindow(XtDisplay(charset.PopUpWg), XtWindow(charset.PopUpWg));
}

void ApplyCharSetCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	GetCharSet((VIewportdata *)client_data);
}

void CloseCharSetCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	
	charset.charsetmapped = False;
	XtPopdown(charset.PopUpWg);
}

void DeleteCharSet(viewport, hzdata1, hzdata2, col1, col2, alin1, alin2, colr1, colr2, under1, under2)
VIewportdata *viewport;
HZdata *hzdata1, *hzdata2;
int col1, col2, alin1, alin2, colr1, colr2, under1, under2;
{
	char line[20];
	int NumOfInput;
	unsigned short InputPtr[80];

	InputPtr[0] = 0x00;

	if(strcmp(hzdata1->FontId, hzdata2->FontId)!=0)
	{
		DBstrcat(InputPtr, "@[");
		DBstrcat(InputPtr, hzdata2->FontId);
		DBstrcat(InputPtr, "]");
	}
	
	if(hzdata1->point_width != hzdata2->point_width || hzdata1->point_height != hzdata2->point_height)
	{
		if(hzdata2->point_width != hzdata2->point_height)
			sprintf(line, "@[%d,%d]", hzdata2->point_width/100, hzdata2->point_height/100);
		else
			sprintf(line, "@[%d]", hzdata2->point_width/100);
		DBstrcat(InputPtr, line);
	}

	if(col1 != col2)
	{
		sprintf(line, "@[C%d]", col2);
		DBstrcat(InputPtr, line);
	}

	if(alin1 != alin2)
	{
		sprintf(line, "@[A%d]", alin2);
		DBstrcat(InputPtr, line);
	}

	if(colr1 != colr2)
	{
		sprintf(line, "@[Z%d]", colr2);
		DBstrcat(InputPtr, line);
	}

	if(under1 != under2)
	{
		sprintf(line, "@[U%d]", under2);
		DBstrcat(InputPtr, line);
	}

	NumOfInput = DBlen(InputPtr);
	
	if(NumOfInput > 0) 
	{
		insert(viewport->cursor.CurPtr, InputPtr, NumOfInput);
		rmanysamecomponent(viewport, viewport->cursor.CurPtr, RIGHTSEARCH); 
	}
}

