/*
 * Copyright 1995, 1996, 1997 by Jun Yei.       All Rights Reserved
 *
 * Permission to retain, use, modify, copy, and distribute ChinesePower
 * in source or binary and its documentation (hereafter, the Software)
 * for noncommercial purpose is hereby granted to you without a fee,
 * provided that this entire copyright and permission notice appear in
 * all such copies, that no charge be associated with such copies,
 * that distribution of derivative works (including value-added
 * distributions such as with additional input dictionaries or fonts)
 * include clarification that such added or derived parts are not from
 * the original Software, and that the names of the author(s) not be
 * used to endorse or promote such works. The author(s) of the software
 * makes no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * THE AUTHOR(S) OF THE SOFTWARE DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 * IN NO EVENT SHALL THE AUTHOR(S) OF THE SOFTWARE BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Jun Yei, Ottawa, ON, Canada
 */

#include "chpower.h"

void WorkAreaExposeCB();
void WorkAreaResizeCB();
void RealizeEditWin();
void RevertCB();
void FreeViewport();
void MiscDrawCB();

void PopupPosition(parent, popup)
Widget parent;
Widget popup;
{
	Arg args[MAX_ARGS];
	register int n;

	Position x = 0, y = 0;
	Dimension width, height, pare_width, pare_height;
	
	n = 0;
	XtSetArg(args[n], XmNwidth, &pare_width); n++;
	XtSetArg(args[n], XmNheight, &pare_height); n++;
	XtGetValues(parent, args, n);

	n = 0;
	XtSetArg(args[n], XmNwidth, &width); n++;
	XtSetArg(args[n], XmNheight, &height); n++;
	XtGetValues(popup, args, n);

	XtTranslateCoords(parent, (Position)((pare_width - width)/2), (Position)((pare_height - height)/2), &x, &y);

	n = 0;
	XtSetArg(args[n], XmNx, x); n++;
	XtSetArg(args[n], XmNy, y); n++;
	XtSetValues(popup, args, n);
}

void QuitViewport(viewportlist)
MYviewportlist *viewportlist;
{
	XtRemoveAllCallbacks(viewportlist->myviewport.resource.WorkArea, XmNdestroyCallback);
	RemoveAutoSavedFile(&viewportlist->myviewport);

/* Linux Bug work around */

#ifdef linux
        XtUnrealizeWidget(viewportlist->myviewport.resource.PopUpShell);
#else
	XtDestroyWidget(viewportlist->myviewport.resource.PopUpShell);
#endif

	FreeViewport(viewportlist);
}

void FreeViewportColors(viewport)
VIewportdata *viewport;
{
	XGCValues gc_value;
	int i;

	for(i = 0; i < 8; i++)
	{
		if(viewport->ColorName[i]) 
		{
			free(viewport->ColorName[i]);
			if(i != 0) XGetGCValues(viewport->resource.dpy, viewport->resource.gc[i-1], GCForeground, &gc_value);
			else XGetGCValues(viewport->resource.dpy, viewport->resource.reverse_gc[0],  GCForeground, &gc_value);
			XFreeColors(viewport->resource.dpy,
			    DefaultColormap(viewport->resource.dpy, DefaultScreen(viewport->resource.dpy)),
			    &gc_value.foreground,
			    1, 0);

			viewport->ColorName[i] = NULL;
		}	
	}
}

void FreeViewport(viewportlist)
MYviewportlist *viewportlist;
{
	PAgedata *search, *nextone;
	MYviewportlist *lastvp;

	lastvp = viewportlist->last;
	if(lastvp == NULL)
	{
		if(viewportlist->next == NULL) ViewPortRoot = NULL;
		else
		{
			ViewPortRoot = viewportlist->next;
			ViewPortRoot->last = NULL;
		}	
	}
	else
	{
		lastvp->next = viewportlist->next;
		if(viewportlist->next != NULL) viewportlist->next->last = lastvp;
	}

	if(viewportlist->myviewport.resource.WorkArea) 
		XtRemoveEventHandler(viewportlist->myviewport.resource.WorkArea, EnterWindowMask | LeaveWindowMask | FocusChangeMask, False,
		MiscDrawCB, (caddr_t *)&viewportlist->myviewport);

        if(viewportlist->myviewport.titler.mapped) CancelTitlerCB(NULL, &viewportlist->myviewport.titler, NULL);
        if(viewportlist->myviewport.footer.mapped) CancelTitlerCB(NULL, &viewportlist->myviewport.footer, NULL);
        if(viewportlist->myviewport.pndata.mapped) CancelPageNumCB(NULL, &viewportlist->myviewport.pndata, NULL);
        if(viewportlist->myviewport.GraphRoot) DeletePictureData(&viewportlist->myviewport);

	if(viewportlist->myviewport.cursor.TimerId) XtRemoveTimeOut(viewportlist->myviewport.cursor.TimerId);
	if(viewportlist->myviewport.savetimerId) XtRemoveTimeOut(viewportlist->myviewport.savetimerId);
	if(viewportlist->myviewport.EditFile != NULL) free(viewportlist->myviewport.EditFile);

	if(viewportlist->myviewport.filename != NULL) free(viewportlist->myviewport.filename);
	if(viewportlist->myviewport.savefilename != NULL) free(viewportlist->myviewport.savefilename);
	if(viewportlist->myviewport.undo.ContentPtr != NULL) free(viewportlist->myviewport.undo.ContentPtr);
	if(viewportlist->myviewport.titler.text != NULL) free(viewportlist->myviewport.titler.text);
	if(viewportlist->myviewport.footer.text != NULL) free(viewportlist->myviewport.footer.text);
	if(viewportlist->myviewport.pndata.texta != NULL) free(viewportlist->myviewport.pndata.texta);
	if(viewportlist->myviewport.pndata.textb != NULL) free(viewportlist->myviewport.pndata.textb);

	if(myselect.viewport == (void *)&viewportlist->myviewport) DisownSelection();

	if(active_viewport == &viewportlist->myviewport) active_viewport = NULL;

	FreeViewportColors(&viewportlist->myviewport);

	search = viewportlist->myviewport.page.PageRoot;
	
	while(search != NULL)
	{
		nextone = search->Next;
		free(search);
		search = nextone;
	}

	free(viewportlist);
}

void PopUpInputCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
{
	active_viewport = (VIewportdata *)client_data;

	XtMapWidget(MyHzInMod.PopUpShell);
	XRaiseWindow(XtDisplay(MyHzInMod.PopUpShell), XtWindow(MyHzInMod.PopUpShell));
	MyHzInMod.mapped = 0;
}	 

void QuitCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	MYviewportlist *viewportlist = (MYviewportlist *)client_data;

	if(viewportlist->myviewport.changed) ConfirmQuit(viewportlist, ProSaveCB, ProNoSaveCB);
	else QuitViewport(viewportlist);
}

void MiscDrawCB(w, client_data, event)
Widget w;
XtPointer client_data;
XEvent *event;
{
	VIewportdata *viewport = (VIewportdata *)client_data;
  
	switch(event->type)
	{
		case FocusIn:

		viewport->cursor.Focused = True;

		case EnterNotify:

   		if(viewport->cursor.Focused && viewport->cursor.TimerId == (XtIntervalId) NULL)
   		{
   			viewport->cursor.TimerId = XtAppAddTimeOut(XtWidgetToApplicationContext(viewport->resource.WorkArea),
   			500, CurFlash, viewport);
  		}
		break;
		
		case FocusOut:
		
		viewport->cursor.Focused = False;

		case LeaveNotify:

		DrawMyCursor(viewport, False);

		if(viewport->cursor.TimerId) 
		{
			XtRemoveTimeOut(viewport->cursor.TimerId);
			viewport->cursor.TimerId = (XtIntervalId) NULL;
  		}
	}
}

void DestroyViewCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	DestroySave(&((MYviewportlist *)client_data)->myviewport, False);
	FreeViewport((MYviewportlist *)client_data);
}

void AutoSaveCB(viewport, id)
VIewportdata *viewport;
XtIntervalId *id;
{
	DestroySave(viewport, True);
	viewport->savetimerId = XtAppAddTimeOut(XtWidgetToApplicationContext(viewport->resource.WorkArea),
				(GlobalDefaults.timer_interval + 1)*60000, AutoSaveCB, viewport); 
}

void BorderCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	active_viewport = (VIewportdata *)client_data;

	if(active_viewport->borderOn) 
	{
		active_viewport->borderOn = False;
		MyDrawPage(active_viewport, CLEARDRAW);
	}
	else 
	{
		active_viewport->borderOn = True;
		MyDrawPage(active_viewport, EXPOSEDR);
	}
}

void SymbolCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	active_viewport = (VIewportdata *)client_data;

	if(active_viewport->viewSym) 
	{
		active_viewport->viewSym = False;
		MyDrawPage(active_viewport, CLEARDRAW);
	}
	else 
	{
		active_viewport->viewSym = True;
		MyDrawPage(active_viewport, EXPOSEDR);
	}
}

void Zoom40BtnCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	active_viewport = (VIewportdata *)client_data;

	ZoomBtnZoom((VIewportdata *)client_data, 40);
}

void Zoom70BtnCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	active_viewport = (VIewportdata *)client_data;

	ZoomBtnZoom((VIewportdata *)client_data, 70);
}

void Zoom100BtnCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	active_viewport = (VIewportdata *)client_data;

	ZoomBtnZoom((VIewportdata *)client_data, 100);
}

void Zoom150BtnCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	active_viewport = (VIewportdata *)client_data;

	ZoomBtnZoom((VIewportdata *)client_data, 150);
}

void Zoom200BtnCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	active_viewport = (VIewportdata *)client_data;

	ZoomBtnZoom((VIewportdata *)client_data, 200);
}

void Zoom300BtnCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	active_viewport = (VIewportdata *)client_data;

	ZoomBtnZoom((VIewportdata *)client_data, 300);
}


void Zoom400BtnCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	active_viewport = (VIewportdata *)client_data;

	ZoomBtnZoom((VIewportdata *)client_data, 400);
}

void Zoom3BtnCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
        active_viewport = (VIewportdata *)client_data;
 
        ZoomBtnZoom((VIewportdata *)client_data, 3);
}

void Zoom2BtnCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
        active_viewport = (VIewportdata *)client_data;
 
        ZoomBtnZoom((VIewportdata *)client_data, 2);
}

void Zoom1BtnCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	active_viewport = (VIewportdata *)client_data;

	ZoomBtnZoom((VIewportdata *)client_data, 1);
}

void Zoom0BtnCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	active_viewport = (VIewportdata *)client_data;

	ZoomBtnZoom((VIewportdata *)client_data, 0);
}

void ConvertG2BCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	active_viewport = (VIewportdata *)client_data;

	Convert(active_viewport, GB2BIG5);	
}

void ConvertG2JCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
        active_viewport = (VIewportdata *)client_data;
 
        Convert(active_viewport, GB2JIS); 
}
 
void ConvertG2UCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
        active_viewport = (VIewportdata *)client_data;
 
        Convert(active_viewport, GB2UNI); 
}
 
void ConvertB2GCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
        active_viewport = (VIewportdata *)client_data;
 
        Convert(active_viewport, BIG52GB); 
}
 
void ConvertB2JCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
        active_viewport = (VIewportdata *)client_data;
 
        Convert(active_viewport, BIG52JIS); 
}

void ConvertB2UCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
        active_viewport = (VIewportdata *)client_data;
 
        Convert(active_viewport, BIG52UNI); 
}

void ConvertJ2GCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
        active_viewport = (VIewportdata *)client_data;
 
        Convert(active_viewport, JIS2GB); 
}

void ConvertJ2BCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
        active_viewport = (VIewportdata *)client_data;
 
        Convert(active_viewport, JIS2BIG5); 
}
 
void ConvertJ2UCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
        active_viewport = (VIewportdata *)client_data;
 
        Convert(active_viewport, JIS2UNI); 
}

void ConvertU2BCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
        active_viewport = (VIewportdata *)client_data;
 
        Convert(active_viewport, UNI2BIG5);
}
 
void ConvertU2GCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
        active_viewport = (VIewportdata *)client_data;
 
        Convert(active_viewport, UNI2GB); 
}

void ConvertU2JCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
        active_viewport = (VIewportdata *)client_data;
 
        Convert(active_viewport, UNI2JIS);
}

void ConvertU2KCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
        active_viewport = (VIewportdata *)client_data;
 
	Convert(active_viewport, UNI2KS);
}

void ConvertK2UCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
        active_viewport = (VIewportdata *)client_data;
 
        Convert(active_viewport, KS2UNI);
}

void ToolBarSHCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	Arg args[1];

        active_viewport = (VIewportdata *)client_data;

	if(active_viewport->toolbar.Mapped)
	{
		XtUnmanageChildren(active_viewport->toolbar.Btns, 13);
		active_viewport->toolbar.Mapped = False;
		XtSetArg(args[0], XmNheight, 0);
	}
	else
	{
		XtManageChildren(active_viewport->toolbar.Btns, 13);
		active_viewport->toolbar.Mapped = True;
		XtSetArg(args[0], XmNheight, 50);
	}

	XtSetValues(active_viewport->toolbar.baseWg, args, 1);
}
 
void EditBarSHCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	Arg args[1];

        active_viewport = (VIewportdata *)client_data;

	if(active_viewport->toolbar.EditMapped)
	{
		XtUnmanageChildren(&(active_viewport->toolbar.Btns[13]), 17);
		active_viewport->toolbar.EditMapped = False;
		XtSetArg(args[0], XmNheight, 0);
	}
	else
	{
		XtManageChildren(&(active_viewport->toolbar.Btns[13]), 17);
		active_viewport->toolbar.EditMapped = True;
		XtSetArg(args[0], XmNheight, 50);
	}

	XtSetValues(active_viewport->toolbar.editWg, args, 1);
}

void RefreshCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	active_viewport = (VIewportdata *)client_data;
 
        MyDrawPage(client_data, CLEARDRAW);
}
 
void CreateEditWindow(parent, viewport)
Widget parent;
MYviewportlist *viewport;
{
	Widget main_window;
	Widget menu_bar, seperate;
	Widget pulldowns[10], buttons[15], pbtn[20];
	Widget menubarBtn[10];
	Widget frame, drawwindow, popup;
	Widget horizon_bar, horizon_scroll_bar, vertical_bar, vertical_scroll_bar;
	Widget pageBtn, sizeBtn, zoom_in_Btn, zoom_out_Btn, zoom_Btn, button;
	Widget rightleftcorner, submenu, submenu1, submenu2, submenu3, submenu4;
	long background;

	Arg args[MAX_ARGS];
	register int n;
	
	/* Setup Main Window */
	
	n = 0;
	XtSetArg(args[n], XmNwidth, 600); n++;
	XtSetArg(args[n], XmNheight, 800); n++;
	XtSetArg(args[n], XmNmarginWidth, 0); n++;
	XtSetArg(args[n], XmNmarginHeight,0); n++;
	XtSetArg(args[n], XmNspacing, 0); n++;
	XtSetArg(args[n], XmNshadowThickness, 0); n++;
	XtSetArg(args[n], XmNrubberPositioning, True); n++;
	main_window = XmCreateForm(parent, "main", args, n);
	XtManageChild(main_window);

	/* Setup Menu Bar */
	
	n = 0;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNheight, 16); n++;
	XtSetArg(args[n], XmNresizePolicy, XmRESIZE_NONE); n++;
	XtSetArg(args[n], XmNtraversalOn, False); n++;
	menu_bar = XmCreateMenuBar(main_window, "menu_bar", args, n);
	XtManageChild(menu_bar);

	/* Setup Tool Bar */

	n = 0;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(args[n], XmNtopWidget, menu_bar); n++;
	XtSetArg(args[n], XmNheight, 50); n++;
	XtSetArg(args[n], XmNresizePolicy, XmRESIZE_NONE); n++;
	XtSetArg(args[n], XmNtraversalOn, False); n++;
        XtSetArg(args[n], XmNshadowThickness, 0); n++;
        XtSetArg(args[n], XmNshadowType, XmSHADOW_OUT); n++;
	viewport->myviewport.toolbar.baseWg = XmCreateForm(main_window, "tool_bar", args, n);
	XtManageChild(viewport->myviewport.toolbar.baseWg);
	viewport->myviewport.toolbar.Mapped = True;

	n = 0;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(args[n], XmNtopWidget, viewport->myviewport.toolbar.baseWg); n++;
	XtSetArg(args[n], XmNheight, 50); n++;
	XtSetArg(args[n], XmNresizePolicy, XmRESIZE_NONE); n++;
	XtSetArg(args[n], XmNtraversalOn, False); n++;
	XtSetArg(args[n], XmNshadowThickness, 0); n++;
	XtSetArg(args[n], XmNshadowType, XmSHADOW_OUT); n++;
	viewport->myviewport.toolbar.editWg = XmCreateForm(main_window, "edit_bar", args, n);
	XtManageChild(viewport->myviewport.toolbar.editWg);
	viewport->myviewport.toolbar.EditMapped = True;	

	/*Setup right left corner */

	n = 0;
	XtSetArg(args[n], XmNborderWidth, 0); n++;
	XtSetArg(args[n], XmNshadowThickness, 0); n++;
	XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNheight, 13); n++;
	XtSetArg(args[n], XmNwidth, 13); n++;
	XtSetArg(args[n], XmNresizePolicy, XmRESIZE_NONE); n++;
	rightleftcorner = XmCreateDrawingArea(main_window, "corner", args, n);
	XtManageChild(rightleftcorner);

	/* Setup Horizon Scroll Bars */
	
	n = 0;
	XtSetArg(args[n], XmNmarginHeight, 0); n++;
	XtSetArg(args[n], XmNmarginWidth, 8); n++;
	XtSetArg(args[n], XmNheight, 16); n++;
	XtSetArg(args[n], XmNresizePolicy, XmRESIZE_NONE); n++;
	XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNrightAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(args[n], XmNrightWidget, rightleftcorner); n++;
	XtSetArg(args[n], XmNtraversalOn, False); n++;
	horizon_bar = XmCreateForm(main_window,"horizon_bar", args, n);
	XtManageChild(horizon_bar);
	
	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("page: 100 (of 100)", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNhighlightThickness, 1); n++;
	XtSetArg(args[n], XmNshadowThickness, 1); n++;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNfillOnArm, False); n++;
	XtSetArg(args[n], XmNtraversalOn, False); n++;  
	XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
	pageBtn = XmCreatePushButtonGadget(horizon_bar, "pagenum", args, n);
	XtAddCallback(pageBtn, XmNarmCallback, PageBtnCB, (caddr_t *)&viewport->myviewport);
	viewport->myviewport.resource.PageBtn = pageBtn;

	submenu = (Widget) XmCreatePulldownMenu(horizon_bar, "zoomSel", NULL, 0);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("40%", XmSTRING_DEFAULT_CHARSET)); n++;
	buttons[0] = XmCreatePushButtonGadget(submenu, "zoom40%", args, n);
	XtAddCallback(buttons[0], XmNactivateCallback, Zoom40BtnCB, (caddr_t *)&viewport->myviewport);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("70%", XmSTRING_DEFAULT_CHARSET)); n++;
	buttons[1] = XmCreatePushButtonGadget(submenu, "zoom70%", args, n);
	XtAddCallback(buttons[1], XmNactivateCallback, Zoom70BtnCB, (caddr_t *)&viewport->myviewport);  

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("100%", XmSTRING_DEFAULT_CHARSET)); n++;
	buttons[2] = XmCreatePushButtonGadget(submenu, "zoom100%", args, n);
	XtAddCallback(buttons[2], XmNactivateCallback, Zoom100BtnCB, (caddr_t *)&viewport->myviewport);  

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("150%", XmSTRING_DEFAULT_CHARSET)); n++;
	buttons[3] = XmCreatePushButtonGadget(submenu, "zoom150%", args, n);
	XtAddCallback(buttons[3], XmNactivateCallback, Zoom150BtnCB, (caddr_t *)&viewport->myviewport);  

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("200%", XmSTRING_DEFAULT_CHARSET)); n++;
	buttons[4] = XmCreatePushButtonGadget(submenu, "zoom200%", args, n);
	XtAddCallback(buttons[4], XmNactivateCallback, Zoom200BtnCB, (caddr_t *)&viewport->myviewport);  

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("300%", XmSTRING_DEFAULT_CHARSET)); n++;
	buttons[5] = XmCreatePushButtonGadget(submenu, "zoom300%", args, n);
	XtAddCallback(buttons[5], XmNactivateCallback, Zoom300BtnCB, (caddr_t *)&viewport->myviewport);  

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("400%", XmSTRING_DEFAULT_CHARSET)); n++;
	buttons[6] = XmCreatePushButtonGadget(submenu, "zoom400%", args, n);
	XtAddCallback(buttons[6], XmNactivateCallback, Zoom400BtnCB, (caddr_t *)&viewport->myviewport);  

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Page fit to Window", XmSTRING_DEFAULT_CHARSET)); n++;
	buttons[7] = XmCreatePushButtonGadget(submenu, "ptow", args, n);
	XtAddCallback(buttons[7], XmNactivateCallback, Zoom3BtnCB, (caddr_t *)&viewport->myviewport);

        n = 0;
        XtSetArg(args[n], XmNlabelString, XmStringCreate("Window fit to Margin", XmSTRING_DEFAULT_CHARSET)); n++;
        buttons[8] = XmCreatePushButtonGadget(submenu, "wtop", args, n);
        XtAddCallback(buttons[8], XmNactivateCallback, Zoom2BtnCB, (caddr_t *)&viewport->myviewport);

        n = 0;
        XtSetArg(args[n], XmNlabelString, XmStringCreate("Window fit to Width", XmSTRING_DEFAULT_CHARSET)); n++;
        buttons[9] = XmCreatePushButtonGadget(submenu, "wtop", args, n);
        XtAddCallback(buttons[9], XmNactivateCallback, Zoom1BtnCB, (caddr_t *)&viewport->myviewport);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Window fit to Page", XmSTRING_DEFAULT_CHARSET)); n++;
	buttons[10] = XmCreatePushButtonGadget(submenu, "wtop", args, n);
	XtAddCallback(buttons[10], XmNactivateCallback, Zoom0BtnCB, (caddr_t *)&viewport->myviewport);

	XtManageChildren(buttons, 11);
	
	n = 0;
	XtSetArg(args[n], XmNsubMenuId, submenu); n++;
	XtSetArg(args[n], XmNmenuHistory, buttons[4]); n++;
/*	XtSetArg(args[n], XmNhighlightThickness, 1); n++;
	XtSetArg(args[n], XmNshadowThickness, 1); n++;
*/	XtSetArg(args[n], XmNmarginHeight, 0); n++;
	XtSetArg(args[n], XmNmarginWidth, 0); n++;
/*	XtSetArg(args[n], XmNmarginTop, 0); n++;
	XtSetArg(args[n], XmNmarginBottom, 0); n++;
	XtSetArg(args[n], XmNmarginLeft, 0); n++;
	XtSetArg(args[n], XmNmarginRight, 0); n++;
*/
	XtSetArg(args[n], XmNadjustMargin, False); n++;
	XtSetArg(args[n], XmNheight, 4); n++;
	XtSetArg(args[n], XmNwidth, 40); n++;
	XtSetArg(args[n], XmNpacking, XmPACK_TIGHT); n++;	
	XtSetArg(args[n], XmNspacing, 0); n++;	
/*	XtSetArg(args[n], XmNresizeHeight, False); n++;
	XtSetArg(args[n], XmNresizeWidth, False); n++;
*/	XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(args[n], XmNleftWidget, pageBtn); n++;
	XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNtraversalOn, False); n++;  
	zoom_Btn = XmCreateOptionMenu(horizon_bar, "zoom", args, n);
	viewport->myviewport.resource.ZoomBtn = XmOptionButtonGadget(zoom_Btn);
	XtUnmanageChild(XmOptionLabelGadget(zoom_Btn));

	n = 0;
	XtSetArg(args[n], XmNhighlightThickness, 1); n++;
	XtSetArg(args[n], XmNshadowThickness, 1); n++;
	XtSetArg(args[n], XmNcascadePixmap, XmUNSPECIFIED_PIXMAP); n++;
	XtSetArg(args[n], XmNmarginHeight, 0); n++;
	XtSetArg(args[n], XmNmarginWidth, 0); n++;
	XtSetArg(args[n], XmNmarginTop, 0); n++;
	XtSetArg(args[n], XmNmarginBottom, 0); n++;
	XtSetArg(args[n], XmNmarginLeft, 0); n++;
	XtSetArg(args[n], XmNmarginRight, 0); n++;
	XtSetArg(args[n], XmNheight, 4); n++;        
	XtSetArg(args[n], XmNwidth, 40); n++;           
	XtSetArg(args[n], XmNtraversalOn, False); n++;
	XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
	XtSetValues(viewport->myviewport.resource.ZoomBtn, args, n);
	
	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("z", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNhighlightThickness, 1); n++;
	XtSetArg(args[n], XmNshadowThickness, 1); n++;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(args[n], XmNleftWidget, zoom_Btn); n++;
	XtSetArg(args[n], XmNfillOnArm, False); n++;
	XtSetArg(args[n], XmNtraversalOn, False); n++;	
	XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
	zoom_in_Btn = XmCreatePushButtonGadget(horizon_bar, "zoom_in", args, n);
	XtAddCallback(zoom_in_Btn, XmNarmCallback, ZoomInCB, (caddr_t *)&viewport->myviewport);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Z", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNhighlightThickness, 1); n++;
	XtSetArg(args[n], XmNshadowThickness, 1); n++;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(args[n], XmNleftWidget, zoom_in_Btn); n++;
	XtSetArg(args[n], XmNfillOnArm, False); n++;
	XtSetArg(args[n], XmNtraversalOn, False); n++;  
	XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
	zoom_out_Btn = XmCreatePushButtonGadget(horizon_bar, "zoom_out", args, n);
	XtAddCallback(zoom_out_Btn, XmNarmCallback, ZoomOutCB, (caddr_t *)&viewport->myviewport);  

	n = 0;
	XtSetArg(args[n], XmNorientation, XmHORIZONTAL); n++;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(args[n], XmNleftWidget, zoom_out_Btn); n++;
	XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNtraversalOn, False); n++;
	horizon_scroll_bar = XmCreateScrollBar(horizon_bar, "horizon", args, n);
	XtAddCallback(horizon_scroll_bar, XmNpageDecrementCallback, PageScroLeft, (caddr_t *)&viewport->myviewport);
	XtAddCallback(horizon_scroll_bar, XmNdecrementCallback, LineScroLeft, (caddr_t *)&viewport->myviewport);
	XtAddCallback(horizon_scroll_bar, XmNpageIncrementCallback, PageScroRight, (caddr_t *)&viewport->myviewport);
	XtAddCallback(horizon_scroll_bar, XmNincrementCallback, LineScroRight, (caddr_t *)&viewport->myviewport);
	XtAddCallback(horizon_scroll_bar, XmNvalueChangedCallback, HorScroDraged, (caddr_t *)&viewport->myviewport);


	XtManageChild(pageBtn);
	XtManageChild(zoom_Btn);
	XtManageChild(zoom_in_Btn);
	XtManageChild(zoom_out_Btn);
	XtManageChild(horizon_scroll_bar);
	
	viewport->myviewport.resource.HorScroBar = horizon_scroll_bar;

	/* Setup Vertical Scroll Bars */
	
	n = 0;
	XtSetArg(args[n], XmNmarginHeight, 5); n++;
	XtSetArg(args[n], XmNmarginWidth, 0); n++;
	XtSetArg(args[n], XmNwidth, 16); n++;
	XtSetArg(args[n], XmNresizePolicy, XmRESIZE_NONE); n++;
	XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(args[n], XmNtopWidget, viewport->myviewport.toolbar.editWg); n++;
	XtSetArg(args[n], XmNbottomAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(args[n], XmNbottomWidget, rightleftcorner); n++;
	XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;     
	XtSetArg(args[n], XmNtraversalOn, False); n++;
	vertical_bar = XmCreateForm(main_window,"vertical_bar", args, n);
	XtManageChild(vertical_bar);

	n = 0;
	XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNtraversalOn, False); n++;
	vertical_scroll_bar = XmCreateScrollBar(vertical_bar, "vertical", args, n);
	XtManageChild(vertical_scroll_bar);
	XtAddCallback(vertical_scroll_bar, XmNpageDecrementCallback, PageScroUp, (caddr_t *)&viewport->myviewport);
	XtAddCallback(vertical_scroll_bar, XmNdecrementCallback, LineScroUp, (caddr_t *)&viewport->myviewport);
	XtAddCallback(vertical_scroll_bar, XmNpageIncrementCallback, PageScroDown, (caddr_t *)&viewport->myviewport);
	XtAddCallback(vertical_scroll_bar, XmNincrementCallback, LineScroDown, (caddr_t *)&viewport->myviewport);
	XtAddCallback(vertical_scroll_bar, XmNvalueChangedCallback, VerScroDraged, (caddr_t *)&viewport->myviewport);
	XtAddCallback(vertical_scroll_bar, XmNdragCallback, VerScroDraging, (caddr_t *)&viewport->myviewport);

	viewport->myviewport.resource.VerScroBar = vertical_scroll_bar;
	
	/* Setup Working Area */
	
	n = 0;
	XtSetArg(args[n], XmNmarginWidth, 0); n++;
	XtSetArg(args[n], XmNmarginHeight, 0); n++;
	XtSetArg(args[n], XmNborderWidth, 5); n++;
	XtSetArg(args[n], XmNshadowThickness, 2); n++;    
	XtSetArg(args[n], XmNshadowType, XmSHADOW_IN); n++;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(args[n], XmNtopWidget, viewport->myviewport.toolbar.editWg); n++;
	XtSetArg(args[n], XmNrightAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(args[n], XmNrightWidget, vertical_bar); n++;
	XtSetArg(args[n], XmNbottomAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(args[n], XmNbottomWidget, horizon_bar); n++;
	frame = XmCreateFrame(main_window, "frame", args, n);
	XtManageChild(frame);

	n = 0;
	XtSetArg(args[n], XmNmarginHeight, 0); n++;
	XtSetArg(args[n], XmNmarginWidth, 0); n++;
	drawwindow = XmCreateDrawingArea(frame, "drawwindow", args, n);

	XtManageChild(drawwindow);

	XtOverrideTranslations(drawwindow, XtParseTranslationTable("<Key>Tab: FakeWorkAreaInputCB()"));

	XtAddCallback(drawwindow, XmNinputCallback, WorkAreaInputCB, (caddr_t *)&viewport->myviewport);

	XtAddCallback(drawwindow, XmNdestroyCallback, DestroyViewCB, (caddr_t *)viewport);
	XtAddEventHandler(drawwindow, EnterWindowMask | LeaveWindowMask | FocusChangeMask, False, MiscDrawCB, (caddr_t *)&viewport->myviewport);

	viewport->myviewport.resource.WorkArea = drawwindow;

	n = 0;
	XtSetArg(args[n], XmNbackground, &background); n++;
	XtGetValues(drawwindow, args, n);

	n = 0;
	XtSetArg(args[n], XmNborderColor, background); n++;
	XtSetValues(frame, args, n);

	/* Setup Popup Menu */

	popup = XmCreatePopupMenu(drawwindow, "popup", NULL, 0);
	viewport->myviewport.resource.Popup = popup;

	XtSetArg(args[0], XmNlabelString, XmStringCreate("Edit Actions", XmSTRING_DEFAULT_CHARSET));
	pbtn[0] = XmCreateLabelGadget(popup, "popup", args, 1);
	pbtn[1] = (Widget) XmCreateSeparatorGadget(popup, "popup", NULL, 0);


	/* Setup Menu Buttons */
	
	n = 0;
	pulldowns[0] = XmCreatePulldownMenu(menu_bar, "pulldown1", args, n);

	n = 0;
	XtSetArg(args[n], XmNsubMenuId, pulldowns[0]); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("File", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNmnemonic, 'F'); n++;
	XtSetArg(args[n], XmNtraversalOn, False); n++;
	menubarBtn[0] = XmCreateCascadeButton(menu_bar,"button1", args, n);

	n = 0;
	pulldowns[1] = XmCreatePulldownMenu(menu_bar, "pulldown2", args, n);

	n = 0;
	XtSetArg(args[n], XmNsubMenuId, pulldowns[1]); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Edit", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNmnemonic, 'E'); n++;
	XtSetArg(args[n], XmNtraversalOn, False); n++;
	menubarBtn[1] = XmCreateCascadeButton(menu_bar,"button2", args, n);

	n = 0;
	pulldowns[2] = XmCreatePulldownMenu(menu_bar, "pulldown3", args, n);
	
	n = 0;
	XtSetArg(args[n], XmNsubMenuId, pulldowns[2]); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Format", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNmnemonic, 'm'); n++;
	XtSetArg(args[n], XmNtraversalOn, False); n++;
	menubarBtn[2] = XmCreateCascadeButton(menu_bar,"button3", args, n);

	n = 0;
	pulldowns[3] = XmCreatePulldownMenu(menu_bar, "pulldown4", args, n);

	n = 0;
	XtSetArg(args[n], XmNsubMenuId, pulldowns[3]); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Color", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNmnemonic, 'C'); n++;
	XtSetArg(args[n], XmNtraversalOn, False); n++;
	menubarBtn[3] = XmCreateCascadeButton(menu_bar,"button4", args, n);

	n = 0;
	pulldowns[4] = XmCreatePulldownMenu(menu_bar, "pulldown5", args, n);

	n = 0;
	XtSetArg(args[n], XmNsubMenuId, pulldowns[4]); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("View", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNmnemonic, 'V'); n++;
	XtSetArg(args[n], XmNtraversalOn, False); n++;
	menubarBtn[4] = XmCreateCascadeButton(menu_bar,"button5", args, n);

	n = 0;
	pulldowns[5] = XmCreatePulldownMenu(menu_bar, "pulldown6", args, n);

	n = 0;
	XtSetArg(args[n], XmNsubMenuId, pulldowns[5]); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Convert", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNmnemonic, 'n'); n++;
	XtSetArg(args[n], XmNtraversalOn, False); n++;
	menubarBtn[5] = XmCreateCascadeButton(menu_bar,"button6", args, n);

        n = 0;
        pulldowns[6] = XmCreatePulldownMenu(menu_bar, "pulldown7", args, n);
 
        n = 0;
        XtSetArg(args[n], XmNsubMenuId, pulldowns[6]); n++;
        XtSetArg(args[n], XmNlabelString, XmStringCreate("Input", XmSTRING_DEFAULT_CHARSET)); n++;
        XtSetArg(args[n], XmNmnemonic, 'I'); n++;
        XtSetArg(args[n], XmNtraversalOn, False); n++;
        menubarBtn[6] = XmCreateCascadeButton(menu_bar,"button7", args, n);

	XtManageChildren(menubarBtn, 7);
	
	/* Setup File Pulldown Menu */

	n = 0;
        XtSetArg(args[n], XmNlabelString, XmStringCreate("New", XmSTRING_DEFAULT_CHARSET)); n++;
        XtSetArg(args[n], XmNmnemonic, 'N'); n++;
        buttons[8] = XmCreatePushButtonGadget(pulldowns[0], "button11", args,n);
        XtAddCallback(buttons[8], XmNactivateCallback, MainNewCB, (caddr_t *)NULL);

	n = 0;
        XtSetArg(args[n], XmNlabelString, XmStringCreate("Open", XmSTRING_DEFAULT_CHARSET)); n++;      
        XtSetArg(args[n], XmNmnemonic, 'O'); n++;      
        buttons[9] = XmCreatePushButtonGadget(pulldowns[0], "button11", args,n);       
        XtAddCallback(buttons[9], XmNactivateCallback, MainOpenCB, (caddr_t *)NULL);  
	
	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Save", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNmnemonic, 'S'); n++;
	buttons[0] = XmCreatePushButtonGadget(pulldowns[0], "button11", args,n);
	XtAddCallback(buttons[0], XmNactivateCallback, FileSaveCB, (caddr_t *)&viewport->myviewport);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Save as...", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNmnemonic, 'a'); n++;
	buttons[1] = XmCreatePushButtonGadget(pulldowns[0], "button12", args,n);
	XtAddCallback(buttons[1], XmNactivateCallback, FileSaveAsStartCB, (caddr_t *)&viewport->myviewport);

	seperate = (Widget) XmCreateSeparatorGadget(pulldowns[0], "sep1", NULL, 0);
	XtManageChild(seperate);
	
	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Revert...", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNmnemonic, 'R'); n++;
	buttons[2] = XmCreatePushButtonGadget(pulldowns[0], "button13", args,n);
	XtAddCallback(buttons[2], XmNactivateCallback, RevertCB, (caddr_t *)&viewport->myviewport);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Import...", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNmnemonic, 'I'); n++;
	buttons[3] = XmCreatePushButtonGadget(pulldowns[0], "button14", args,n);
	XtAddCallback(buttons[3], XmNactivateCallback, ImportCB, (caddr_t *)&viewport->myviewport);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Import Image... ", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNmnemonic, 'm'); n++;
	buttons[4] = XmCreatePushButtonGadget(pulldowns[0], "button15", args,n);
	XtAddCallback(buttons[4], XmNactivateCallback, PicImportCB, (caddr_t *)&viewport->myviewport);

        seperate = (Widget) XmCreateSeparatorGadget(pulldowns[0], "sep1", NULL, 0);
        XtManageChild(seperate);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Print...", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNmnemonic, 'P'); n++;
	buttons[5] = XmCreatePushButtonGadget(pulldowns[0], "button16", args,n);
	XtAddCallback(buttons[5], XmNactivateCallback, PrintSetCB, (caddr_t *)&viewport->myviewport);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("To Gif...", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNmnemonic, 'G'); n++;
	buttons[6] = XmCreatePushButtonGadget(pulldowns[0], "button17", args,n);
	XtAddCallback(buttons[6], XmNactivateCallback, GifSetCB, (caddr_t *)&viewport->myviewport);

        seperate = (Widget) XmCreateSeparatorGadget(pulldowns[0], "sep1", NULL, 0);
        XtManageChild(seperate);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Close", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNmnemonic, 'Q'); n++;
	buttons[7] = XmCreatePushButtonGadget(pulldowns[0], "button18", args,n);
	XtAddCallback(buttons[7], XmNactivateCallback, QuitCB, (caddr_t *)viewport);

	XtManageChildren(buttons, 10);
	
	/* Setup Edit Pulldown Menu */
	
	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Undo", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNmnemonic, 'U'); n++;
	buttons[0] = XmCreatePushButtonGadget(pulldowns[1], "button21", args,n);
	pbtn[2] = XmCreatePushButtonGadget(popup, "button21", args,n);
	XtAddCallback(buttons[0], XmNactivateCallback, UndoCB, (caddr_t *)&viewport->myviewport);
	XtAddCallback(pbtn[2], XmNactivateCallback, UndoCB, (caddr_t *)&viewport->myviewport);

        seperate = (Widget) XmCreateSeparatorGadget(pulldowns[1], "sep1", NULL, 0);
        XtManageChild(seperate);

	seperate = (Widget) XmCreateSeparatorGadget(popup, "popup", NULL, 0);
	XtManageChild(seperate);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Cut", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNmnemonic, 'C'); n++;
	buttons[1] = XmCreatePushButtonGadget(pulldowns[1], "button22", args,n);
	pbtn[3] = XmCreatePushButtonGadget(popup, "button22", args,n);
	XtAddCallback(buttons[1], XmNactivateCallback, CutCB, (caddr_t *)&viewport->myviewport);
	XtAddCallback(pbtn[3], XmNactivateCallback, CutCB, (caddr_t *)&viewport->myviewport);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Copy", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNmnemonic, 'o'); n++;
	buttons[2] = XmCreatePushButtonGadget(pulldowns[1], "button23", args,n);
	pbtn[4] = XmCreatePushButtonGadget(popup, "button23", args,n);
	XtAddCallback(buttons[2], XmNactivateCallback, CopyCB, (caddr_t *)&viewport->myviewport);
	XtAddCallback(pbtn[4], XmNactivateCallback, CopyCB, (caddr_t *)&viewport->myviewport);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Paste", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNmnemonic, 'P'); n++;
	buttons[3] = XmCreatePushButtonGadget(pulldowns[1], "button24", args,n);
	pbtn[5] = XmCreatePushButtonGadget(popup, "button24", args,n);
	XtAddCallback(buttons[3], XmNactivateCallback, PasteCB, (caddr_t *)&viewport->myviewport);
	XtAddCallback(pbtn[5], XmNactivateCallback, PasteCB, (caddr_t *)&viewport->myviewport);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Delete", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNmnemonic, 'D'); n++;
	buttons[4] = XmCreatePushButtonGadget(pulldowns[1], "button25", args,n);
	pbtn[6] = XmCreatePushButtonGadget(popup, "button25", args,n);
	XtAddCallback(buttons[4], XmNactivateCallback, DeleteCB, (caddr_t *)&viewport->myviewport);
	XtAddCallback(pbtn[6], XmNactivateCallback, DeleteCB, (caddr_t *)&viewport->myviewport);

        seperate = (Widget) XmCreateSeparatorGadget(pulldowns[1], "sep1", NULL, 0);
        XtManageChild(seperate);

        seperate = (Widget) XmCreateSeparatorGadget(popup, "popup", NULL, 0);
        XtManageChild(seperate);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Select All", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNmnemonic, 'S'); n++;
	buttons[5] = XmCreatePushButtonGadget(pulldowns[1], "button26", args,n);
	pbtn[7] = XmCreatePushButtonGadget(popup, "button26", args,n);	
	XtAddCallback(buttons[5], XmNactivateCallback, SelectAllCB, (caddr_t *)&viewport->myviewport);
	XtAddCallback(pbtn[7], XmNactivateCallback, SelectAllCB, (caddr_t *)&viewport->myviewport);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Page Delete...", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNmnemonic, 'a'); n++;
	buttons[6] = XmCreatePushButtonGadget(pulldowns[1], "button27", args,n);
	XtAddCallback(buttons[6], XmNactivateCallback, PageDelCB, (caddr_t *)&viewport->myviewport);

        seperate = (Widget) XmCreateSeparatorGadget(pulldowns[1], "sep1", NULL, 0);
        XtManageChild(seperate);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Find & Change... ", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNmnemonic, 'F'); n++;
	buttons[7] = XmCreatePushButtonGadget(pulldowns[1], "button28", args,n);
	XtAddCallback(buttons[7], XmNactivateCallback, FindPopupCB, (caddr_t *)&viewport->myviewport);

        seperate = (Widget) XmCreateSeparatorGadget(pulldowns[1], "sep1", NULL, 0);
        XtManageChild(seperate);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Refresh", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNmnemonic, 'R'); n++;
	buttons[8] = XmCreatePushButtonGadget(pulldowns[1], "button29", args,n);
	XtAddCallback(buttons[8], XmNactivateCallback, RefreshCB, (caddr_t *)&viewport->myviewport);

	XtManageChildren(buttons, 9);
	
	/* Setup Format Pulldown Menu */

        seperate = (Widget) XmCreateSeparatorGadget(popup, "popup", NULL, 0);
        XtManageChild(seperate);
	
	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Character...", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNmnemonic, 'C'); n++;
	buttons[0] = XmCreatePushButtonGadget(pulldowns[2], "button31", args,n);
	pbtn[8] = XmCreatePushButtonGadget(popup, "button31", args,n);
	XtAddCallback(buttons[0], XmNactivateCallback, CharSetCB, (caddr_t *)&viewport->myviewport);
	XtAddCallback(pbtn[8], XmNactivateCallback, CharSetCB, (caddr_t *)&viewport->myviewport);

	submenu = (Widget) XmCreatePulldownMenu(pulldowns[2], "Layoutsubmenu", NULL, 0);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Layout", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNmnemonic, 'L'); n++;
	XtSetArg(args[n], XmNsubMenuId, submenu); n++;
	buttons[1] = XmCreateCascadeButtonGadget(pulldowns[2], "button32", args,n);

        seperate = (Widget) XmCreateSeparatorGadget(pulldowns[2], "sep1", NULL, 0);
        XtManageChild(seperate);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Page Set...", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNmnemonic, 'P'); n++;
	buttons[2] = XmCreatePushButtonGadget(pulldowns[2], "button33", args,n);
	XtAddCallback(buttons[2], XmNactivateCallback, PageSetCB, (caddr_t *)&viewport->myviewport);

        seperate = (Widget) XmCreateSeparatorGadget(pulldowns[2], "sep1", NULL, 0);
        XtManageChild(seperate);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Header...", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNmnemonic, 'H'); n++;
	buttons[3] = XmCreatePushButtonGadget(pulldowns[2], "button34", args,n);
	pbtn[9] = XmCreatePushButtonGadget(popup, "button34", args,n);
	XtAddCallback(buttons[3], XmNactivateCallback, TitlerPopupCB, (caddr_t *)&viewport->myviewport);
	XtAddCallback(pbtn[9], XmNactivateCallback, TitlerPopupCB, (caddr_t *)&viewport->myviewport);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Foot Note...", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNmnemonic, 'F'); n++;
	buttons[4] = XmCreatePushButtonGadget(pulldowns[2], "button35", args,n);
	pbtn[10] = XmCreatePushButtonGadget(popup, "button34", args,n);
	XtAddCallback(buttons[4], XmNactivateCallback, FooterPopupCB, (caddr_t *)&viewport->myviewport);
	XtAddCallback(pbtn[10], XmNactivateCallback, FooterPopupCB, (caddr_t *)&viewport->myviewport);

        seperate = (Widget) XmCreateSeparatorGadget(pulldowns[2], "sep1", NULL, 0);
        XtManageChild(seperate);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Page Num...", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNmnemonic, 'N'); n++;
	buttons[5] = XmCreatePushButtonGadget(pulldowns[2], "button35", args,n);
	XtAddCallback(buttons[5], XmNactivateCallback, PageNumSetPopupCB, (caddr_t *)&viewport->myviewport);
	
	XtManageChildren(buttons, 6);

	submenu1 = (Widget) XmCreatePulldownMenu(submenu, "Layoutsubmenu1", NULL, 0);
	submenu2 = (Widget) XmCreatePulldownMenu(submenu, "Layoutsubmenu2", NULL, 0);
	submenu3 = (Widget) XmCreatePulldownMenu(submenu, "Layoutsubmenu3", NULL, 0);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Column", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNsubMenuId, submenu1); n++;
	buttons[0] = XmCreateCascadeButtonGadget(submenu, "sbutton331", args,n);
	pbtn[11] = XmCreateCascadeButtonGadget(popup, "sbutton331", args,n);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Line Alignment", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNsubMenuId, submenu2); n++;
	buttons[1] = XmCreateCascadeButtonGadget(submenu, "sbutton332", args,n);
	pbtn[12]  = XmCreateCascadeButtonGadget(popup, "sbutton332", args,n);


	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Under Line", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNsubMenuId, submenu3); n++;
	buttons[2] = XmCreateCascadeButtonGadget(submenu, "sbutton333", args,n);
	pbtn[13]  = XmCreateCascadeButtonGadget(popup, "sbutton333", args,n);
	XtManageChildren(buttons, 3);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Column 1", XmSTRING_DEFAULT_CHARSET)); n++;
	buttons[0] = XmCreatePushButtonGadget(submenu1, "sbutton3311", args,n);
	XtAddCallback(buttons[0], XmNactivateCallback, ColumnSetCB, (caddr_t *)1);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Column 2", XmSTRING_DEFAULT_CHARSET)); n++;
	buttons[1] = XmCreatePushButtonGadget(submenu1, "sbutton3312", args,n);
	XtAddCallback(buttons[1], XmNactivateCallback, ColumnSetCB, (caddr_t *)2);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Column 3", XmSTRING_DEFAULT_CHARSET)); n++;
	buttons[2] = XmCreatePushButtonGadget(submenu1, "sbutton3313", args,n);
	XtAddCallback(buttons[2], XmNactivateCallback, ColumnSetCB, (caddr_t *)3);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Column 4", XmSTRING_DEFAULT_CHARSET)); n++;
	buttons[3] = XmCreatePushButtonGadget(submenu1, "sbutton3314", args,n);
	XtAddCallback(buttons[3], XmNactivateCallback, ColumnSetCB, (caddr_t *)4);

	XtManageChildren(buttons, 4);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Left Alignment", XmSTRING_DEFAULT_CHARSET)); n++;
	buttons[0] = XmCreatePushButtonGadget(submenu2, "sbutton3321", args,n);
	XtAddCallback(buttons[0], XmNactivateCallback, AlignmentSetCB, (caddr_t *)0);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Center Alignment", XmSTRING_DEFAULT_CHARSET)); n++;
	buttons[1] = XmCreatePushButtonGadget(submenu2, "sbutton3322", args,n);
	XtAddCallback(buttons[1], XmNactivateCallback, AlignmentSetCB, (caddr_t *)1);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Right Alignment", XmSTRING_DEFAULT_CHARSET)); n++;
	buttons[2] = XmCreatePushButtonGadget(submenu2, "sbutton3323", args,n);
	XtAddCallback(buttons[2], XmNactivateCallback, AlignmentSetCB, (caddr_t *)2);

	XtManageChildren(buttons, 3);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("No Line", XmSTRING_DEFAULT_CHARSET)); n++;
	buttons[0] = XmCreatePushButtonGadget(submenu3, "sbutton3331", args,n);
	XtAddCallback(buttons[0], XmNactivateCallback, UnderLineSetCB, (caddr_t *)0);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Middle Line", XmSTRING_DEFAULT_CHARSET)); n++;
	buttons[1] = XmCreatePushButtonGadget(submenu3, "sbutton3322", args,n);
	XtAddCallback(buttons[1], XmNactivateCallback, UnderLineSetCB, (caddr_t *)1);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Under Line", XmSTRING_DEFAULT_CHARSET)); n++;
	buttons[2] = XmCreatePushButtonGadget(submenu3, "sbutton3323", args,n);
	XtAddCallback(buttons[2], XmNactivateCallback, UnderLineSetCB, (caddr_t *)2);

	XtManageChildren(buttons, 3);

	/* Setup Color Pull Down Menu */

	submenu = (Widget) XmCreatePulldownMenu(pulldowns[3], "Colmenu", NULL, 0);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Color", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNmnemonic, 'C'); n++;
	XtSetArg(args[n], XmNsubMenuId, submenu); n++;
	buttons[0] = XmCreateCascadeButtonGadget(pulldowns[3], "button41", args,n);
	pbtn[14] = XmCreateCascadeButtonGadget(popup, "button41", args,n);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Set Colors...", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNmnemonic, 'S'); n++;
	buttons[1] = XmCreatePushButtonGadget(pulldowns[3], "button42", args,n);
	XtAddCallback(buttons[1], XmNactivateCallback, ColorTabPopupCB, (caddr_t *)&viewport->myviewport);
	
	XtManageChildren(buttons, 2);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Standard", XmSTRING_DEFAULT_CHARSET)); n++;
	buttons[0] = XmCreatePushButton(submenu, "sbutton", args,n);
	XtAddCallback(buttons[0], XmNactivateCallback, ColorSetCB, (caddr_t *)0);
	viewport->myviewport.ColorBtns[0] = buttons[0];

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Color 1", XmSTRING_DEFAULT_CHARSET)); n++;
	buttons[1] = XmCreatePushButton(submenu, "sbutton", args,n);
	XtAddCallback(buttons[1], XmNactivateCallback, ColorSetCB, (caddr_t *)1);
	viewport->myviewport.ColorBtns[1] = buttons[1];

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Color 2", XmSTRING_DEFAULT_CHARSET)); n++;
	buttons[2] = XmCreatePushButton(submenu, "sbutton", args,n);
	XtAddCallback(buttons[2], XmNactivateCallback, ColorSetCB, (caddr_t *)2);
	viewport->myviewport.ColorBtns[2] = buttons[2];

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Color 3", XmSTRING_DEFAULT_CHARSET)); n++;
	buttons[3] = XmCreatePushButton(submenu, "sbutton", args,n);
	XtAddCallback(buttons[3], XmNactivateCallback, ColorSetCB, (caddr_t *)3);
	viewport->myviewport.ColorBtns[3] = buttons[3];

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Color 4", XmSTRING_DEFAULT_CHARSET)); n++;
	buttons[4] = XmCreatePushButton(submenu, "sbutton", args,n);
	XtAddCallback(buttons[4], XmNactivateCallback, ColorSetCB, (caddr_t *)4);
	viewport->myviewport.ColorBtns[4] = buttons[4];

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Color 5", XmSTRING_DEFAULT_CHARSET)); n++;
	buttons[5] = XmCreatePushButton(submenu, "sbutton", args,n);
	XtAddCallback(buttons[5], XmNactivateCallback, ColorSetCB, (caddr_t *)5);
	viewport->myviewport.ColorBtns[5] = buttons[5];

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Color 6", XmSTRING_DEFAULT_CHARSET)); n++;
	buttons[6] = XmCreatePushButton(submenu, "sbutton", args,n);
	XtAddCallback(buttons[6], XmNactivateCallback, ColorSetCB, (caddr_t *)6);
	viewport->myviewport.ColorBtns[6] = buttons[6];

	XtManageChildren(buttons, 7);

	/* Setup View Pulldown */

	n = 0;
	XtSetArg(args[0], XmNset, False); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Text Symbol", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNindicatorType, XmN_OF_MANY); n++;
	XtSetArg(args[n], XmNvisibleWhenOff, True); n++;
	XtSetArg(args[n], XmNmnemonic, 'S'); n++;
	buttons[0] = XmCreateToggleButtonGadget(pulldowns[4], "button51", args,n);
	XtAddCallback(buttons[0], XmNvalueChangedCallback, SymbolCB, (caddr_t *)&viewport->myviewport);

	n = 0;
	XtSetArg(args[0], XmNset, False); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Borders", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNindicatorType, XmN_OF_MANY); n++;
	XtSetArg(args[n], XmNvisibleWhenOff, True); n++;
	XtSetArg(args[n], XmNmnemonic, 'B'); n++;
	buttons[1] = XmCreateToggleButtonGadget(pulldowns[4], "button52", args,n);
	XtAddCallback(buttons[1], XmNvalueChangedCallback, BorderCB, (caddr_t *)&viewport->myviewport);

	n = 0;
	XtSetArg(args[0], XmNset, True); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Tool Bar", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNindicatorType, XmN_OF_MANY); n++;
	XtSetArg(args[n], XmNvisibleWhenOff, True); n++;
	XtSetArg(args[n], XmNmnemonic, 'T'); n++;
	buttons[2] = XmCreateToggleButtonGadget(pulldowns[4], "button53", args,n);
	XtAddCallback(buttons[2], XmNvalueChangedCallback, ToolBarSHCB, (caddr_t *)&viewport->myviewport);

	n = 0;
	XtSetArg(args[0], XmNset, True); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Edit Bar", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNindicatorType, XmN_OF_MANY); n++;
	XtSetArg(args[n], XmNvisibleWhenOff, True); n++;
	XtSetArg(args[n], XmNmnemonic, 'E'); n++;
	buttons[3] = XmCreateToggleButtonGadget(pulldowns[4], "button54", args,n);
	XtAddCallback(buttons[3], XmNvalueChangedCallback, EditBarSHCB, (caddr_t *)&viewport->myviewport);

	XtManageChildren(buttons, 4);

	/* Setup Convert Pulldown */

	submenu1 = (Widget) XmCreatePulldownMenu(pulldowns[5], "GBmenu", NULL, 0);
	submenu2 = (Widget) XmCreatePulldownMenu(pulldowns[5], "Big5m", NULL, 0);
	submenu3 = (Widget) XmCreatePulldownMenu(pulldowns[5], "Jismenu", NULL, 0);
	submenu  = (Widget) XmCreatePulldownMenu(pulldowns[5], "Ksmenu", NULL, 0);
	submenu4 = (Widget) XmCreatePulldownMenu(pulldowns[5], "Unimenu", NULL, 0);
	
	n = 0;
        XtSetArg(args[n], XmNlabelString, XmStringCreate("GB -> ...", XmSTRING_DEFAULT_CHARSET)); n++;
        XtSetArg(args[n], XmNmnemonic, 'G'); n++;
        XtSetArg(args[n], XmNsubMenuId, submenu1); n++;
        buttons[0] = XmCreateCascadeButtonGadget(pulldowns[5], "button61", args,n);

	n = 0;
        XtSetArg(args[n], XmNlabelString, XmStringCreate("Big5 -> ...", XmSTRING_DEFAULT_CHARSET)); n++;
        XtSetArg(args[n], XmNmnemonic, 'B'); n++;
        XtSetArg(args[n], XmNsubMenuId, submenu2); n++;
        buttons[1] = XmCreateCascadeButtonGadget(pulldowns[5], "button62", args,n);

	n = 0;
        XtSetArg(args[n], XmNlabelString, XmStringCreate("JIS -> ...", XmSTRING_DEFAULT_CHARSET)); n++;
        XtSetArg(args[n], XmNmnemonic, 'J'); n++;
        XtSetArg(args[n], XmNsubMenuId, submenu3); n++;
        buttons[2] = XmCreateCascadeButtonGadget(pulldowns[5], "button63", args,n);

        n = 0;
        XtSetArg(args[n], XmNlabelString, XmStringCreate("Unicode -> ...", XmSTRING_DEFAULT_CHARSET)); n++;
        XtSetArg(args[n], XmNmnemonic, 'U'); n++;
        XtSetArg(args[n], XmNsubMenuId, submenu4); n++;
        buttons[3] = XmCreateCascadeButtonGadget(pulldowns[5], "button64", args,n);

	n = 0;
        XtSetArg(args[n], XmNlabelString, XmStringCreate("KS -> ...", XmSTRING_DEFAULT_CHARSET)); n++;
        XtSetArg(args[n], XmNmnemonic, 'K'); n++;
        XtSetArg(args[n], XmNsubMenuId, submenu); n++;
        buttons[4] = XmCreateCascadeButtonGadget(pulldowns[5], "button65", args,n);

	XtManageChildren(buttons, 5);
	
	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("GB -> Big5", XmSTRING_DEFAULT_CHARSET)); n++;
	buttons[0] = XmCreatePushButtonGadget(submenu1, "button611", args,n);
	XtAddCallback(buttons[0], XmNactivateCallback, ConvertG2BCB, (caddr_t *)&viewport->myviewport);

	n = 0;
        XtSetArg(args[n], XmNlabelString, XmStringCreate("GB -> JIS", XmSTRING_DEFAULT_CHARSET)); n++;
        buttons[1] = XmCreatePushButtonGadget(submenu1, "button612", args,n);
	XtAddCallback(buttons[1], XmNactivateCallback, ConvertG2JCB, (caddr_t *)&viewport->myviewport);

	n = 0;
        XtSetArg(args[n], XmNlabelString, XmStringCreate("GB -> Unicode", XmSTRING_DEFAULT_CHARSET)); n++;
        buttons[2] = XmCreatePushButtonGadget(submenu1, "button613", args,n);
	XtAddCallback(buttons[2], XmNactivateCallback, ConvertG2UCB, (caddr_t *)&viewport->myviewport);

	XtManageChildren(buttons, 3);

	n = 0;
        XtSetArg(args[n], XmNlabelString, XmStringCreate("Big5 -> GB", XmSTRING_DEFAULT_CHARSET)); n++;
        buttons[0] = XmCreatePushButtonGadget(submenu2, "button621", args,n);
	XtAddCallback(buttons[0], XmNactivateCallback, ConvertB2GCB, (caddr_t *)&viewport->myviewport);
 
        n = 0;
        XtSetArg(args[n], XmNlabelString, XmStringCreate("Big5 -> JIS", XmSTRING_DEFAULT_CHARSET)); n++;
        buttons[1] = XmCreatePushButtonGadget(submenu2, "button622", args,n);
	XtAddCallback(buttons[1], XmNactivateCallback, ConvertB2JCB, (caddr_t *)&viewport->myviewport);
 
        n = 0;
        XtSetArg(args[n], XmNlabelString, XmStringCreate("Big5 -> Unicode", XmSTRING_DEFAULT_CHARSET)); n++;
        buttons[2] = XmCreatePushButtonGadget(submenu2, "button623", args,n);
	XtAddCallback(buttons[2], XmNactivateCallback, ConvertB2UCB, (caddr_t *)&viewport->myviewport);
 
        XtManageChildren(buttons, 3);

	n = 0;
        XtSetArg(args[n], XmNlabelString, XmStringCreate("JIS -> GB", XmSTRING_DEFAULT_CHARSET)); n++;
        buttons[0] = XmCreatePushButtonGadget(submenu3, "button631", args,n);
	XtAddCallback(buttons[0], XmNactivateCallback, ConvertJ2GCB, (caddr_t *)&viewport->myviewport);
 
        n = 0;
        XtSetArg(args[n], XmNlabelString, XmStringCreate("JIS -> Big5", XmSTRING_DEFAULT_CHARSET)); n++;
        buttons[1] = XmCreatePushButtonGadget(submenu3, "button632", args,n);
	XtAddCallback(buttons[1], XmNactivateCallback, ConvertJ2BCB, (caddr_t *)&viewport->myviewport);
 
        n = 0;
        XtSetArg(args[n], XmNlabelString, XmStringCreate("JIS -> Unicode", XmSTRING_DEFAULT_CHARSET)); n++;
        buttons[2] = XmCreatePushButtonGadget(submenu3, "button633", args,n);
	XtAddCallback(buttons[2], XmNactivateCallback, ConvertJ2UCB, (caddr_t *)&viewport->myviewport);
 
        XtManageChildren(buttons, 3);

        n = 0;
        XtSetArg(args[n], XmNlabelString, XmStringCreate("Unicdoe -> GB", XmSTRING_DEFAULT_CHARSET)); n++;
        buttons[0] = XmCreatePushButtonGadget(submenu4, "button641", args,n);
        XtAddCallback(buttons[0], XmNactivateCallback, ConvertU2GCB, (caddr_t *)&viewport->myviewport);
 
        n = 0;
        XtSetArg(args[n], XmNlabelString, XmStringCreate("Unicdoe -> Big5", XmSTRING_DEFAULT_CHARSET)); n++;
        buttons[1] = XmCreatePushButtonGadget(submenu4, "button642", args,n);
        XtAddCallback(buttons[1], XmNactivateCallback, ConvertU2BCB, (caddr_t *)&viewport->myviewport);
 
        n = 0;
        XtSetArg(args[n], XmNlabelString, XmStringCreate("Unicode -> JIS", XmSTRING_DEFAULT_CHARSET)); n++;
        buttons[2] = XmCreatePushButtonGadget(submenu4, "button643", args,n);
        XtAddCallback(buttons[2], XmNactivateCallback, ConvertU2JCB, (caddr_t *)&viewport->myviewport);

	n = 0;
        XtSetArg(args[n], XmNlabelString, XmStringCreate("Unicode -> KS", XmSTRING_DEFAULT_CHARSET)); n++;
        buttons[3] = XmCreatePushButtonGadget(submenu4, "button644", args,n);
        XtAddCallback(buttons[3], XmNactivateCallback, ConvertU2KCB, (caddr_t *)&viewport->myviewport);

	XtManageChildren(buttons, 4);
 
	n = 0;
        XtSetArg(args[n], XmNlabelString, XmStringCreate("KS -> Unicdoe", XmSTRING_DEFAULT_CHARSET)); n++;
        buttons[0] = XmCreatePushButtonGadget(submenu, "button651", args,n);
	XtAddCallback(buttons[0], XmNactivateCallback, ConvertK2UCB, (caddr_t *)&viewport->myviewport);
 
        XtManageChildren(buttons, 1);

	/* Setup Input Pulldown */

        seperate = (Widget) XmCreateSeparatorGadget(popup, "popup", NULL, 0);
        XtManageChild(seperate);

        n = 0;
        XtSetArg(args[0], XmNset, False); n++;
        XtSetArg(args[n], XmNlabelString, XmStringCreate("PopUp Input...", XmSTRING_DEFAULT_CHARSET)); n++;
        XtSetArg(args[n], XmNmnemonic, 'P'); n++;
        buttons[0] = XmCreatePushButtonGadget(pulldowns[6], "button71", args,n);
	pbtn[15] = XmCreatePushButtonGadget(popup, "button71", args,n);
        XtAddCallback(buttons[0], XmNactivateCallback, PopUpInputCB, (caddr_t *)&viewport->myviewport);
	XtAddCallback(pbtn[15], XmNactivateCallback, PopUpInputCB, (caddr_t *)&viewport->myviewport);

	XtManageChildren(buttons, 1);

	/* Setup Tool Bar */

	n = 0;
        XtSetArg(args[n], XmNwidth, 40); n++;
        XtSetArg(args[n], XmNheight, 40); n++;
        XtSetArg(args[n], XmNhighlightThickness, 0); n++;
        XtSetArg(args[n], XmNshadowThickness, 2); n++;
        XtSetArg(args[n], XmNx, 10); n++;
        XtSetArg(args[n], XmNy, 5); n++;
        XtSetArg(args[n], XmNlabelType, XmPIXMAP); n++;
        XtSetArg(args[n], XmNlabelPixmap, ToolBarIcons[NEW_ICON]); n++;
        viewport->myviewport.toolbar.Btns[0] = XmCreatePushButtonGadget(viewport->myviewport.toolbar.baseWg, "tool1", args,n);
        XtAddCallback(viewport->myviewport.toolbar.Btns[0], XmNactivateCallback, MainNewCB, (caddr_t *)NULL);
 
        n = 0;
        XtSetArg(args[n], XmNwidth, 40); n++;
        XtSetArg(args[n], XmNheight, 40); n++;
        XtSetArg(args[n], XmNhighlightThickness, 0); n++;
        XtSetArg(args[n], XmNshadowThickness, 2); n++;
        XtSetArg(args[n], XmNx, 50); n++;
        XtSetArg(args[n], XmNy, 5); n++;
        XtSetArg(args[n], XmNlabelType, XmPIXMAP); n++;
        XtSetArg(args[n], XmNlabelPixmap, ToolBarIcons[OPEN_ICON]); n++;
        viewport->myviewport.toolbar.Btns[1] = XmCreatePushButtonGadget(viewport->myviewport.toolbar.baseWg, "tool1", args,n);
        XtAddCallback(viewport->myviewport.toolbar.Btns[1], XmNactivateCallback, MainOpenCB, (caddr_t *)NULL);
 
	n = 0;
	XtSetArg(args[n], XmNwidth, 40); n++;
	XtSetArg(args[n], XmNheight, 40); n++;
	XtSetArg(args[n], XmNhighlightThickness, 0); n++;
	XtSetArg(args[n], XmNshadowThickness, 2); n++;
	XtSetArg(args[n], XmNx, 90); n++;
	XtSetArg(args[n], XmNy, 5); n++;
	XtSetArg(args[n], XmNlabelType, XmPIXMAP); n++;
        XtSetArg(args[n], XmNlabelPixmap, ToolBarIcons[SAVE_ICON]); n++;
	viewport->myviewport.toolbar.Btns[2] = XmCreatePushButtonGadget(viewport->myviewport.toolbar.baseWg, "tool1", args,n);
	XtAddCallback(viewport->myviewport.toolbar.Btns[2], XmNactivateCallback, FileSaveCB, (caddr_t *)&viewport->myviewport);

	n = 0;
	XtSetArg(args[n], XmNwidth, 40); n++;
	XtSetArg(args[n], XmNheight, 40); n++;
	XtSetArg(args[n], XmNhighlightThickness, 0); n++;
	XtSetArg(args[n], XmNshadowThickness, 2); n++;
	XtSetArg(args[n], XmNx, 140); n++;
	XtSetArg(args[n], XmNy, 5); n++;
	XtSetArg(args[n], XmNlabelType, XmPIXMAP); n++;
        XtSetArg(args[n], XmNlabelPixmap, ToolBarIcons[CUT_ICON]); n++;
	viewport->myviewport.toolbar.Btns[3] = XmCreatePushButtonGadget(viewport->myviewport.toolbar.baseWg, "tool1", args,n);
	XtAddCallback(viewport->myviewport.toolbar.Btns[3], XmNactivateCallback, CutCB, (caddr_t *)&viewport->myviewport);

	n = 0;
	XtSetArg(args[n], XmNwidth, 40); n++;
	XtSetArg(args[n], XmNheight, 40); n++;
	XtSetArg(args[n], XmNhighlightThickness, 0); n++;
	XtSetArg(args[n], XmNshadowThickness, 2); n++;
	XtSetArg(args[n], XmNx, 180); n++;
	XtSetArg(args[n], XmNy, 5); n++;
	XtSetArg(args[n], XmNlabelType, XmPIXMAP); n++;
        XtSetArg(args[n], XmNlabelPixmap, ToolBarIcons[COPY_ICON]); n++;
	viewport->myviewport.toolbar.Btns[4] = XmCreatePushButtonGadget(viewport->myviewport.toolbar.baseWg, "tool1", args,n);
	XtAddCallback(viewport->myviewport.toolbar.Btns[4], XmNactivateCallback, CopyCB, (caddr_t *)&viewport->myviewport);

	n = 0;
	XtSetArg(args[n], XmNwidth, 40); n++;
	XtSetArg(args[n], XmNheight, 40); n++;
	XtSetArg(args[n], XmNhighlightThickness, 0); n++;
	XtSetArg(args[n], XmNshadowThickness, 2); n++;
	XtSetArg(args[n], XmNx, 220); n++;
	XtSetArg(args[n], XmNy, 5); n++;
	XtSetArg(args[n], XmNlabelType, XmPIXMAP); n++;
        XtSetArg(args[n], XmNlabelPixmap, ToolBarIcons[PASTE_ICON]); n++;
	viewport->myviewport.toolbar.Btns[5] = XmCreatePushButtonGadget(viewport->myviewport.toolbar.baseWg, "tool1", args,n);
	XtAddCallback(viewport->myviewport.toolbar.Btns[5], XmNactivateCallback, PasteCB, (caddr_t *)&viewport->myviewport);

	n = 0;
	XtSetArg(args[n], XmNwidth, 40); n++;
	XtSetArg(args[n], XmNheight, 40); n++;
	XtSetArg(args[n], XmNhighlightThickness, 0); n++;
	XtSetArg(args[n], XmNshadowThickness, 2); n++;
	XtSetArg(args[n], XmNx, 260); n++;
	XtSetArg(args[n], XmNy, 5); n++;
	XtSetArg(args[n], XmNlabelType, XmPIXMAP); n++;
        XtSetArg(args[n], XmNlabelPixmap, ToolBarIcons[UNDO_ICON]); n++;
	viewport->myviewport.toolbar.Btns[6] = XmCreatePushButtonGadget(viewport->myviewport.toolbar.baseWg, "tool1", args,n);
	XtAddCallback(viewport->myviewport.toolbar.Btns[6], XmNactivateCallback, UndoCB, (caddr_t *)&viewport->myviewport);

        n = 0;
        XtSetArg(args[n], XmNwidth, 40); n++;
        XtSetArg(args[n], XmNheight, 40); n++;
        XtSetArg(args[n], XmNhighlightThickness, 0); n++;
        XtSetArg(args[n], XmNshadowThickness, 2); n++;
        XtSetArg(args[n], XmNx, 310); n++;
        XtSetArg(args[n], XmNy, 5); n++;
        XtSetArg(args[n], XmNlabelType, XmPIXMAP); n++;
        XtSetArg(args[n], XmNlabelPixmap, ToolBarIcons[PIC_ICON]); n++;
        viewport->myviewport.toolbar.Btns[7] = XmCreatePushButtonGadget(viewport->myviewport.toolbar.baseWg, "tool1", args,n);
        XtAddCallback(viewport->myviewport.toolbar.Btns[7], XmNactivateCallback, PicImportCB, (caddr_t *)&viewport->myviewport);
 
	n = 0;
	XtSetArg(args[n], XmNwidth, 40); n++;
	XtSetArg(args[n], XmNheight, 40); n++;
	XtSetArg(args[n], XmNhighlightThickness, 0); n++;
	XtSetArg(args[n], XmNshadowThickness, 2); n++;
	XtSetArg(args[n], XmNx, 360); n++;
	XtSetArg(args[n], XmNy, 5); n++;
	XtSetArg(args[n], XmNlabelType, XmPIXMAP); n++;
        XtSetArg(args[n], XmNlabelPixmap, ToolBarIcons[PRINT_ICON]); n++;
	viewport->myviewport.toolbar.Btns[8] = XmCreatePushButtonGadget(viewport->myviewport.toolbar.baseWg, "tool1", args,n);
	XtAddCallback(viewport->myviewport.toolbar.Btns[8], XmNactivateCallback, PrintSetCB, (caddr_t *)&viewport->myviewport);

	n = 0;
	XtSetArg(args[n], XmNwidth, 40); n++;
	XtSetArg(args[n], XmNheight, 40); n++;
	XtSetArg(args[n], XmNhighlightThickness, 0); n++;
	XtSetArg(args[n], XmNshadowThickness, 2); n++;
	XtSetArg(args[n], XmNx, 410); n++;
	XtSetArg(args[n], XmNy, 5); n++;
	XtSetArg(args[n], XmNlabelType, XmPIXMAP); n++;
        XtSetArg(args[n], XmNlabelPixmap, ToolBarIcons[FIND_ICON]); n++;
	viewport->myviewport.toolbar.Btns[9] = XmCreatePushButtonGadget(viewport->myviewport.toolbar.baseWg, "tool1", args,n);
	XtAddCallback(viewport->myviewport.toolbar.Btns[9], XmNactivateCallback, FindPopupCB, (caddr_t *)&viewport->myviewport);

	n = 0;
        XtSetArg(args[n], XmNwidth, 40); n++;
        XtSetArg(args[n], XmNheight, 40); n++;
        XtSetArg(args[n], XmNhighlightThickness, 0); n++;
        XtSetArg(args[n], XmNshadowThickness, 2); n++;
        XtSetArg(args[n], XmNx, 460); n++;
        XtSetArg(args[n], XmNy, 5); n++;
        XtSetArg(args[n], XmNlabelType, XmPIXMAP); n++;
        XtSetArg(args[n], XmNlabelPixmap, ToolBarIcons[SYM_ICON]); n++;
        viewport->myviewport.toolbar.Btns[10] = XmCreatePushButtonGadget(viewport->myviewport.toolbar.baseWg, "tool1", args,n);
        XtAddCallback(viewport->myviewport.toolbar.Btns[10], XmNactivateCallback, SymbolCB, (caddr_t *)&viewport->myviewport);

	n = 0;
        XtSetArg(args[n], XmNwidth, 40); n++;
        XtSetArg(args[n], XmNheight, 40); n++;
        XtSetArg(args[n], XmNhighlightThickness, 0); n++;
        XtSetArg(args[n], XmNshadowThickness, 2); n++;
        XtSetArg(args[n], XmNx, 510); n++;
        XtSetArg(args[n], XmNy, 5); n++;
        XtSetArg(args[n], XmNlabelType, XmPIXMAP); n++;
        XtSetArg(args[n], XmNlabelPixmap, ToolBarIcons[INPUT_ICON]); n++;
        viewport->myviewport.toolbar.Btns[11] = XmCreatePushButtonGadget(viewport->myviewport.toolbar.baseWg, "tool1", args,n);
        XtAddCallback(viewport->myviewport.toolbar.Btns[11], XmNactivateCallback, PopUpInputCB, (caddr_t *)&viewport->myviewport);

	n = 0;
	XtSetArg(args[n], XmNwidth, 40); n++;
	XtSetArg(args[n], XmNheight, 40); n++;
	XtSetArg(args[n], XmNhighlightThickness, 0); n++;
	XtSetArg(args[n], XmNshadowThickness, 2); n++;
	XtSetArg(args[n], XmNx, 560); n++;
	XtSetArg(args[n], XmNy, 5); n++;
	XtSetArg(args[n], XmNlabelType, XmPIXMAP); n++;
	XtSetArg(args[n], XmNlabelPixmap, ToolBarIcons[REFRESH_ICON]); n++;
	viewport->myviewport.toolbar.Btns[12] = XmCreatePushButtonGadget(viewport->myviewport.toolbar.baseWg, "tool1", args,n);
	XtAddCallback(viewport->myviewport.toolbar.Btns[12], XmNactivateCallback, RefreshCB, (caddr_t *)&viewport->myviewport);

	XtManageChildren(viewport->myviewport.toolbar.Btns, 13);

	/* Setup Edit Bar */

        n = 0;
        XtSetArg(args[n], XmNwidth, 40); n++;
        XtSetArg(args[n], XmNheight, 40); n++;
        XtSetArg(args[n], XmNhighlightThickness, 0); n++;
        XtSetArg(args[n], XmNshadowThickness, 2); n++;
        XtSetArg(args[n], XmNx, 10); n++;
        XtSetArg(args[n], XmNy, 5); n++;
        XtSetArg(args[n], XmNlabelType, XmPIXMAP); n++;
        XtSetArg(args[n], XmNlabelPixmap, ToolBarIcons[CHAR_ICON]); n++;
        viewport->myviewport.toolbar.Btns[13] = XmCreatePushButtonGadget(viewport->myviewport.toolbar.editWg, "tool1", args,n);
        XtAddCallback(viewport->myviewport.toolbar.Btns[13], XmNactivateCallback, CharSetCB, (caddr_t *)&viewport->myviewport);
 
	n = 0;
	XtSetArg(args[n], XmNwidth, 40); n++;
	XtSetArg(args[n], XmNheight, 40); n++;
	XtSetArg(args[n], XmNhighlightThickness, 0); n++;
	XtSetArg(args[n], XmNshadowThickness, 2); n++;
	XtSetArg(args[n], XmNx, 60); n++;
	XtSetArg(args[n], XmNy, 5); n++;
	XtSetArg(args[n], XmNlabelType, XmPIXMAP); n++;
        XtSetArg(args[n], XmNlabelPixmap, ToolBarIcons[NOLINE_ICON]); n++;
	viewport->myviewport.toolbar.Btns[14] = XmCreatePushButtonGadget(viewport->myviewport.toolbar.editWg, "tool1", args,n);
	XtAddCallback(viewport->myviewport.toolbar.Btns[14], XmNactivateCallback, UnderLineSetCB, (caddr_t *)0);

	n = 0;
	XtSetArg(args[n], XmNwidth, 40); n++;
	XtSetArg(args[n], XmNheight, 40); n++;
	XtSetArg(args[n], XmNhighlightThickness, 0); n++;
	XtSetArg(args[n], XmNshadowThickness, 2); n++;
	XtSetArg(args[n], XmNx, 100); n++;
	XtSetArg(args[n], XmNy, 5); n++;
	XtSetArg(args[n], XmNlabelType, XmPIXMAP); n++;
        XtSetArg(args[n], XmNlabelPixmap, ToolBarIcons[UNDER_ICON]); n++;
	viewport->myviewport.toolbar.Btns[15] = XmCreatePushButtonGadget(viewport->myviewport.toolbar.editWg, "tool1", args,n);
	XtAddCallback(viewport->myviewport.toolbar.Btns[15], XmNactivateCallback, UnderLineSetCB, (caddr_t *)2);

	n = 0;
	XtSetArg(args[n], XmNwidth, 40); n++;
	XtSetArg(args[n], XmNheight, 40); n++;
	XtSetArg(args[n], XmNhighlightThickness, 0); n++;
	XtSetArg(args[n], XmNshadowThickness, 2); n++;
	XtSetArg(args[n], XmNx, 140); n++;
	XtSetArg(args[n], XmNy, 5); n++;
	XtSetArg(args[n], XmNlabelType, XmPIXMAP); n++;
        XtSetArg(args[n], XmNlabelPixmap, ToolBarIcons[MIDDLE_ICON]); n++;
	viewport->myviewport.toolbar.Btns[16] = XmCreatePushButtonGadget(viewport->myviewport.toolbar.editWg, "tool1", args,n);
	XtAddCallback(viewport->myviewport.toolbar.Btns[16], XmNactivateCallback, UnderLineSetCB, (caddr_t *)1);

	n = 0;
	XtSetArg(args[n], XmNwidth, 40); n++;
	XtSetArg(args[n], XmNheight, 40); n++;
	XtSetArg(args[n], XmNhighlightThickness, 0); n++;
	XtSetArg(args[n], XmNshadowThickness, 2); n++;
	XtSetArg(args[n], XmNx, 190); n++;
	XtSetArg(args[n], XmNy, 5); n++;
	XtSetArg(args[n], XmNlabelType, XmPIXMAP); n++;
        XtSetArg(args[n], XmNlabelPixmap, ToolBarIcons[LEFT_ICON]); n++;
	viewport->myviewport.toolbar.Btns[17] = XmCreatePushButtonGadget(viewport->myviewport.toolbar.editWg, "tool1", args,n);
	XtAddCallback(viewport->myviewport.toolbar.Btns[17], XmNactivateCallback, AlignmentSetCB, (caddr_t *)0);

	n = 0;
	XtSetArg(args[n], XmNwidth, 40); n++;
	XtSetArg(args[n], XmNheight, 40); n++;
	XtSetArg(args[n], XmNhighlightThickness, 0); n++;
	XtSetArg(args[n], XmNshadowThickness, 2); n++;
	XtSetArg(args[n], XmNx, 230); n++;
	XtSetArg(args[n], XmNy, 5); n++;
	XtSetArg(args[n], XmNlabelType, XmPIXMAP); n++;
        XtSetArg(args[n], XmNlabelPixmap, ToolBarIcons[CENTER_ICON]); n++;
	viewport->myviewport.toolbar.Btns[18] = XmCreatePushButtonGadget(viewport->myviewport.toolbar.editWg, "tool1", args,n);
	XtAddCallback(viewport->myviewport.toolbar.Btns[18], XmNactivateCallback, AlignmentSetCB, (caddr_t *)1);

	n = 0;
	XtSetArg(args[n], XmNwidth, 40); n++;
	XtSetArg(args[n], XmNheight, 40); n++;
	XtSetArg(args[n], XmNhighlightThickness, 0); n++;
	XtSetArg(args[n], XmNshadowThickness, 2); n++;
	XtSetArg(args[n], XmNx, 270); n++;
	XtSetArg(args[n], XmNy, 5); n++;
	XtSetArg(args[n], XmNlabelType, XmPIXMAP); n++;
        XtSetArg(args[n], XmNlabelPixmap, ToolBarIcons[RIGHT_ICON]); n++;
	viewport->myviewport.toolbar.Btns[19] = XmCreatePushButtonGadget(viewport->myviewport.toolbar.editWg, "tool1", args,n);
	XtAddCallback(viewport->myviewport.toolbar.Btns[19], XmNactivateCallback, AlignmentSetCB, (caddr_t *)2);

	n = 0;
	XtSetArg(args[n], XmNwidth, 40); n++;
	XtSetArg(args[n], XmNheight, 40); n++;
	XtSetArg(args[n], XmNhighlightThickness, 0); n++;
	XtSetArg(args[n], XmNshadowThickness, 2); n++;
	XtSetArg(args[n], XmNx, 320); n++;
	XtSetArg(args[n], XmNy, 5); n++;
	XtSetArg(args[n], XmNlabelType, XmPIXMAP); n++;
        XtSetArg(args[n], XmNlabelPixmap, ToolBarIcons[COL1_ICON]); n++;
	viewport->myviewport.toolbar.Btns[20] = XmCreatePushButtonGadget(viewport->myviewport.toolbar.editWg, "tool1", args,n);
	XtAddCallback(viewport->myviewport.toolbar.Btns[20], XmNactivateCallback, ColumnSetCB, (caddr_t *)1);

	n = 0;
        XtSetArg(args[n], XmNwidth, 40); n++;
        XtSetArg(args[n], XmNheight, 40); n++;
        XtSetArg(args[n], XmNhighlightThickness, 0); n++;
        XtSetArg(args[n], XmNshadowThickness, 2); n++;
        XtSetArg(args[n], XmNx, 360); n++;
        XtSetArg(args[n], XmNy, 5); n++;
        XtSetArg(args[n], XmNlabelType, XmPIXMAP); n++;
        XtSetArg(args[n], XmNlabelPixmap, ToolBarIcons[COL2_ICON]); n++;
        viewport->myviewport.toolbar.Btns[21] = XmCreatePushButtonGadget(viewport->myviewport.toolbar.editWg, "tool1", args,n);
        XtAddCallback(viewport->myviewport.toolbar.Btns[21], XmNactivateCallback, ColumnSetCB, (caddr_t *)2);

	n = 0;
        XtSetArg(args[n], XmNwidth, 40); n++;
        XtSetArg(args[n], XmNheight, 40); n++;
        XtSetArg(args[n], XmNhighlightThickness, 0); n++;
        XtSetArg(args[n], XmNshadowThickness, 2); n++;
        XtSetArg(args[n], XmNx, 410); n++;
        XtSetArg(args[n], XmNy, 5); n++;
        XtSetArg(args[n], XmNlabelType, XmPIXMAP); n++;
        XtSetArg(args[n], XmNlabelPixmap, ToolBarIcons[PALLET_ICON]); n++;
        viewport->myviewport.toolbar.Btns[22] = XmCreatePushButtonGadget(viewport->myviewport.toolbar.editWg, "tool1", args,n);
        XtAddCallback(viewport->myviewport.toolbar.Btns[22], XmNactivateCallback, ColorTabPopupCB, (caddr_t *)&viewport->myviewport);

	n = 0;
	XtSetArg(args[n], XmNwidth, 20); n++;
	XtSetArg(args[n], XmNheight, 20); n++;
	XtSetArg(args[n], XmNhighlightThickness, 0); n++;
	XtSetArg(args[n], XmNshadowThickness, 2); n++;
	XtSetArg(args[n], XmNx, 450); n++;
	XtSetArg(args[n], XmNy, 5); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("C", XmSTRING_DEFAULT_CHARSET)); n++;
	viewport->myviewport.toolbar.Btns[23] = XmCreatePushButton(viewport->myviewport.toolbar.editWg, "tool1", args,n);
	XtAddCallback(viewport->myviewport.toolbar.Btns[23], XmNactivateCallback, ColorSetCB, (caddr_t *)0);

	n = 0;
	XtSetArg(args[n], XmNwidth, 20); n++;
	XtSetArg(args[n], XmNheight, 20); n++;
	XtSetArg(args[n], XmNhighlightThickness, 0); n++;
	XtSetArg(args[n], XmNshadowThickness, 2); n++;
	XtSetArg(args[n], XmNx, 450); n++;
	XtSetArg(args[n], XmNy, 25); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("C", XmSTRING_DEFAULT_CHARSET)); n++;
	viewport->myviewport.toolbar.Btns[24] = XmCreatePushButton(viewport->myviewport.toolbar.editWg, "tool1", args,n);
	XtAddCallback(viewport->myviewport.toolbar.Btns[24], XmNactivateCallback, ColorSetCB, (caddr_t *)1);

	n = 0;
	XtSetArg(args[n], XmNwidth, 20); n++;
	XtSetArg(args[n], XmNheight, 20); n++;
	XtSetArg(args[n], XmNhighlightThickness, 0); n++;
	XtSetArg(args[n], XmNshadowThickness, 2); n++;
	XtSetArg(args[n], XmNx, 470); n++;
	XtSetArg(args[n], XmNy, 5); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("C", XmSTRING_DEFAULT_CHARSET)); n++;
	viewport->myviewport.toolbar.Btns[25] = XmCreatePushButton(viewport->myviewport.toolbar.editWg, "tool1", args,n);
	XtAddCallback(viewport->myviewport.toolbar.Btns[25], XmNactivateCallback, ColorSetCB, (caddr_t *)2);

	n = 0;
	XtSetArg(args[n], XmNwidth, 20); n++;
	XtSetArg(args[n], XmNheight, 20); n++;
	XtSetArg(args[n], XmNhighlightThickness, 0); n++;
	XtSetArg(args[n], XmNshadowThickness, 2); n++;
	XtSetArg(args[n], XmNx, 470); n++;
	XtSetArg(args[n], XmNy, 25); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("C", XmSTRING_DEFAULT_CHARSET)); n++;
	viewport->myviewport.toolbar.Btns[26] = XmCreatePushButton(viewport->myviewport.toolbar.editWg, "tool1", args,n);
	XtAddCallback(viewport->myviewport.toolbar.Btns[26], XmNactivateCallback, ColorSetCB, (caddr_t *)3);

	n = 0;
	XtSetArg(args[n], XmNwidth, 20); n++;
	XtSetArg(args[n], XmNheight, 20); n++;
	XtSetArg(args[n], XmNhighlightThickness, 0); n++;
	XtSetArg(args[n], XmNshadowThickness, 2); n++;
	XtSetArg(args[n], XmNx, 490); n++;
	XtSetArg(args[n], XmNy, 5); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("C", XmSTRING_DEFAULT_CHARSET)); n++;
	viewport->myviewport.toolbar.Btns[27] = XmCreatePushButton(viewport->myviewport.toolbar.editWg, "tool1", args,n);
	XtAddCallback(viewport->myviewport.toolbar.Btns[27], XmNactivateCallback, ColorSetCB, (caddr_t *)4);

	n = 0;
	XtSetArg(args[n], XmNwidth, 20); n++;
	XtSetArg(args[n], XmNheight, 20); n++;
	XtSetArg(args[n], XmNhighlightThickness, 0); n++;
	XtSetArg(args[n], XmNshadowThickness, 2); n++;
	XtSetArg(args[n], XmNx, 490); n++;
	XtSetArg(args[n], XmNy, 25); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("C", XmSTRING_DEFAULT_CHARSET)); n++;
	viewport->myviewport.toolbar.Btns[28] = XmCreatePushButton(viewport->myviewport.toolbar.editWg, "tool1", args,n);
	XtAddCallback(viewport->myviewport.toolbar.Btns[28], XmNactivateCallback, ColorSetCB, (caddr_t *)5);

	n = 0;
	XtSetArg(args[n], XmNwidth, 20); n++;
	XtSetArg(args[n], XmNheight, 20); n++;
	XtSetArg(args[n], XmNhighlightThickness, 0); n++;
	XtSetArg(args[n], XmNshadowThickness, 2); n++;
	XtSetArg(args[n], XmNx, 510); n++;
	XtSetArg(args[n], XmNy, 5); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("C", XmSTRING_DEFAULT_CHARSET)); n++;
	viewport->myviewport.toolbar.Btns[29] = XmCreatePushButton(viewport->myviewport.toolbar.editWg, "tool1", args,n);
	XtAddCallback(viewport->myviewport.toolbar.Btns[29], XmNactivateCallback, ColorSetCB, (caddr_t *)6);


	XtManageChildren(&(viewport->myviewport.toolbar.Btns[13]), 17);
	
	XtManageChildren(pbtn, 16);
}

void StartExposeEdit(viewport)
VIewportdata *viewport;
{
	register int n;
	Arg args[5];
	Dimension a, b;
	char num[10];	
	XWMHints mywmhints;

	XDefineCursor(viewport->resource.dpy, viewport->resource.wind, myxterm);
	
	mywmhints.icon_pixmap = IconPixmap;
	mywmhints.flags = IconPixmapHint;
	XSetWMHints(XtDisplay(viewport->resource.PopUpShell), XtWindow(viewport->resource.PopUpShell), &mywmhints);

	n = 0;
	XtSetArg(args[n], XmNheight, &a); n++;
	XtSetArg(args[n], XmNwidth, &b); n++;
	XtGetValues(viewport->resource.WorkArea, args, n);

	viewport->origpage.ViewPortHeight = XmConvertUnits(viewport->resource.WorkArea, XmVERTICAL, XmPIXELS, (int)a, Xm100TH_POINTS);
	viewport->origpage.ViewPortWidth = XmConvertUnits(viewport->resource.WorkArea, XmHORIZONTAL, XmPIXELS, (int)b, Xm100TH_POINTS);

	if(GlobalDefaults.win_selec == FIT_TO_WIN_PAGE)
	{
		PageFitWindow(viewport);
		ZoomOpr(viewport, 0);
	}

	viewport->viewp.ViewPortHeight = viewport->origpage.ViewPortHeight * 100 / viewport->zoom;
	viewport->viewp.ViewPortWidth = viewport->origpage.ViewPortWidth * 100 / viewport->zoom;

	switch(GlobalDefaults.win_selec)
	{
		case FIT_TO_WIN_PAGE:
                case FIT_TO_PAGE_WIN:

		viewport->viewp.ViewPort_x = 0;
		viewport->viewp.ViewPort_y = 0;
		break;

		case FIT_TO_WIDTH_WIN:

		viewport->viewp.ViewPort_y = 0;

		if(viewport->horOrder)
		{
			viewport->viewp.ViewPort_x = 0;
		}
		else
		{
			viewport->viewp.ViewPort_x = viewport->page.PageWidth - viewport->viewp.ViewPortWidth;
			if(viewport->viewp.ViewPort_x < 0 ) viewport->viewp.ViewPort_x = 0;
		} 
		break;
	
		case FIT_TO_MARGIN_WIN:

		if(viewport->horOrder)
		{
			viewport->viewp.ViewPort_y = 0;
		}
		else
		{
		        viewport->viewp.ViewPort_x = viewport->page.PageWidth - viewport->viewp.ViewPortWidth;
			if(viewport->viewp.ViewPort_x < 0 ) viewport->viewp.ViewPort_x = 0;
		}
		break;

		default:

		viewport->viewp.ViewPort_x = viewport->viewp.ViewPort_x - viewport->viewp.ViewPortWidth;
                if(viewport->viewp.ViewPort_x < 0 ) viewport->viewp.ViewPort_x = 0;

	}
		 
	sprintf(num, "%d%%\0", viewport->zoom);
	
	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate(num, XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetValues(viewport->resource.ZoomBtn, args, n);
	
	VerScroBarExpose(viewport);
	HorScroBarExpose(viewport);

	XtAddCallback(viewport->resource.WorkArea, XmNexposeCallback, WorkAreaExposeCB, (caddr_t *)viewport);
	XtAddCallback(viewport->resource.WorkArea, XmNresizeCallback, WorkAreaResizeCB, (caddr_t *)viewport);

	if(GlobalDefaults.timer_on) viewport->savetimerId = XtAppAddTimeOut(XtWidgetToApplicationContext(viewport->resource.WorkArea),
						(GlobalDefaults.timer_interval + 1)*60000, AutoSaveCB, viewport);
}

void CreateViewportColors(viewport)
VIewportdata *viewport;
{
	XGCValues gc_init;
	register int n;
	Arg args[5];
	unsigned long foreground, background;
	XColor color, exact;
	Status result;
	int i;

	n = 0;
	XtSetArg(args[n], XmNbackground, &gc_init.background); n++;
	XtSetArg(args[n], XmNforeground, &gc_init.foreground); n++;
	XtGetValues(viewport->resource.WorkArea, args, n);

	if(viewport->ColorName[0]) 
	{
		result = XAllocNamedColor(viewport->resource.dpy, DefaultColormap(viewport->resource.dpy, DefaultScreen(viewport->resource.dpy)),
		viewport->ColorName[0], &color, &exact);
		
		if(result) background = color.pixel;
		else 
		{
			background = gc_init.background;
			free(viewport->ColorName[0]);
			viewport->ColorName[0] = NULL;
		}
	} else background = gc_init.background;
	
	if(viewport->ColorName[1]) 
	{
		result = XAllocNamedColor(viewport->resource.dpy, DefaultColormap(viewport->resource.dpy, DefaultScreen(viewport->resource.dpy)),
		viewport->ColorName[1], &color, &exact);
		
		if(result) foreground = color.pixel;
		else 
		{
			foreground = gc_init.foreground;
			free(viewport->ColorName[1]);
			viewport->ColorName[1] = NULL;
		}	
	} else foreground = gc_init.foreground;
	
	
	gc_init.background = foreground;
	gc_init.foreground = background;
	
	viewport->resource.reverse_gc[0] = XCreateGC(viewport->resource.dpy, viewport->resource.wind, GCForeground|GCBackground, &gc_init);

	XtSetArg(args[0], XmNbackground, background);
	XtSetValues(viewport->resource.WorkArea, args, 1);

	XtSetArg(args[0], XmNforeground, foreground);
	XtSetValues(viewport->ColorBtns[0], args, 1);
	XtSetValues(viewport->toolbar.Btns[23], args, 1);

	gc_init.foreground = foreground;
	gc_init.background = background;

	viewport->resource.gc[0] = XCreateGC(viewport->resource.dpy, viewport->resource.wind, GCForeground|GCBackground, &gc_init);
	viewport->resource.dash_gc = XCreateGC(viewport->resource.dpy, viewport->resource.wind, GCForeground|GCBackground,
						&gc_init);
	XSetLineAttributes(viewport->resource.dpy, viewport->resource.dash_gc, 0, LineOnOffDash, CapButt, JoinMiter);

	for(i = 2; i < 8; i++)
	{
		if(viewport->ColorName[i]) 
		{
			result = XAllocNamedColor(viewport->resource.dpy, DefaultColormap(viewport->resource.dpy, DefaultScreen(viewport->resource.dpy)),
		viewport->ColorName[i], &color, &exact);

			if(result)
			{
				gc_init.foreground = color.pixel;      
			}
			else 
			{
				gc_init.foreground = foreground;
				free(viewport->ColorName[i]);
				viewport->ColorName[i] = NULL;
			}
		}
		else gc_init.foreground = foreground;

		gc_init.background = background;

		viewport->resource.gc[i-1] = XCreateGC(viewport->resource.dpy, viewport->resource.wind, GCForeground|GCBackground, &gc_init);

		gc_init.background = gc_init.foreground;
		gc_init.foreground = background;	

		viewport->resource.reverse_gc[i-1] = XCreateGC(viewport->resource.dpy, viewport->resource.wind, GCForeground|GCBackground, &gc_init);

	        XtSetArg(args[0], XmNforeground, gc_init.background);
        	XtSetValues(viewport->ColorBtns[i-1], args, 1);
		XtSetValues(viewport->toolbar.Btns[22+i], args, 1);
	}
}

void ViewPortResize(viewport)
VIewportdata *viewport;
{
	register int n;
	Arg args[5];
	Dimension a, b;

	n = 0;
	XtSetArg(args[n], XmNheight, &a); n++;
	XtSetArg(args[n], XmNwidth, &b); n++;
	XtGetValues(viewport->resource.WorkArea, args, n);

	viewport->origpage.ViewPortHeight = XmConvertUnits(viewport->resource.WorkArea, XmVERTICAL, XmPIXELS, (int)a, Xm100TH_POINTS);
	viewport->origpage.ViewPortWidth = XmConvertUnits(viewport->resource.WorkArea, XmHORIZONTAL, XmPIXELS, (int)b, Xm100TH_POINTS);

	viewport->viewp.ViewPortHeight = viewport->origpage.ViewPortHeight * 100 / viewport->zoom; 
	viewport->viewp.ViewPortWidth = viewport->origpage.ViewPortWidth * 100 / viewport->zoom;
}

void ResetViewport(viewport, pagereset)
VIewportdata *viewport;
int pagereset;
{
	if(viewport->origpage.Portrait)
	{
		viewport->page.PageWidth = PageData[viewport->origpage.PageSetId][0] * 100;
		viewport->page.PageHeight = PageData[viewport->origpage.PageSetId][1] * 100;
	}
	else
	{
		viewport->page.PageWidth = PageData[viewport->origpage.PageSetId][1] * 100;
		viewport->page.PageHeight = PageData[viewport->origpage.PageSetId][0] * 100;
	}

	
	if(pagereset)
	{
		viewport->page.PageMarginWidth *= 100;
		viewport->page.PageMarginHeight *= 100;
		viewport->page.PageColSpacing *= 100;
	}

	viewport->viewp.ViewPort_y = viewport->page.PageMarginHeight - 800;
	viewport->cursor.ViewPortCur_y = viewport->page.PageMarginHeight;

        if(viewport->horOrder) 
        {
		viewport->cursor.ViewPortCur_x = viewport->page.PageMarginWidth;
                viewport->viewp.ViewPort_x = viewport->page.PageMarginWidth - 800;
        }
        else
	{
		viewport->cursor.ViewPortCur_x = viewport->page.PageWidth - viewport->page.PageMarginWidth;
		viewport->viewp.ViewPort_x = viewport->cursor.ViewPortCur_x + 800;
	}

	if(pagereset == 1)
	{
	        if(!viewport->horOrder)
	        {
        	         viewport->viewp.ViewPort_x = viewport->viewp.ViewPort_x - viewport->viewp.ViewPortWidth;
               		 if(viewport->viewp.ViewPort_x < 0 ) viewport->viewp.ViewPort_x = 0;
        	}
	}

	if(pagereset != 2)
	{
		viewport->cursor.ViewPortCurPage = 1;
		viewport->viewp.ViewPortPage = 1;
		viewport->viewp.CurPage = viewport->page.PageRoot;
		viewport->cursor.CurPtr = viewport->EditFile;
		viewport->cursor.StartCurPtr = viewport->EditFile;
		viewport->cursor.EndCurPtr = viewport->EditFile;
		viewport->draw.DrawScr_x = viewport->draw.DrawPos_x = 0;
		viewport->draw.DrawScr_y = viewport->draw.DrawPos_y = 0;
	}
}

void InitPointFont(w, hzdata, fontfile, width, height, zoom)
Widget w;
HZdata *hzdata;
char *fontfile;
int width, height, zoom;
{
	hzdata->point_width = width * 100;
	hzdata->point_height = height * 100;
	hzdata->hbf = NULL;
	hzdata->FontType = HBFONT;

	hzdata->hbf = GetFontHandle(fontfile, &hzdata->FontId, &hzdata->FontType);

	if(hzdata->hbf == NULL) 
	{
		if(FontListRoot != NULL)
		{
			hzdata->hbf = FontListRoot->Hbf;
			hzdata->FontId = FontListRoot->FontId;	
			hzdata->FontType = FontListRoot->FontType;	
		}	
	}
	
	if(hzdata->hbf == NULL)	
	{
		ErrorPopup("No Hz Font Available!");
		return;
	}

	if(hzdata->FontType == HBFONT)
		HBF_GetBitmapBoundingBox(hzdata->hbf, &(hzdata->hz_width), &(hzdata->hz_height), &width, &height);
	else hzdata->hz_width = hzdata->hz_height = 24;

	FontCalc(w, hzdata, zoom);
}

void InitEditWindow(viewport)
VIewportdata *viewport;
{
	int x_disp, y_disp;		
	unsigned char *cp;

	viewport->resource.dpy = XtDisplay(viewport->resource.WorkArea);
	viewport->resource.wind = XtWindow(viewport->resource.WorkArea);
	
	viewport->changed = False;
	viewport->autosavechanged = False;
	viewport->page.PageRoot = (PAgedata *)malloc(sizeof(PAgedata)+1);
	viewport->page.PageRoot->PageStart = viewport->EditFile;
	viewport->page.PageRoot->PageNum = 1;
	viewport->page.PageRoot->Last = NULL;
	viewport->page.PageRoot->Next = NULL;
	viewport->page.PageRoot->ColNum = 1;
	viewport->page.PageRoot->LineAlignment = LeftAlign;	
	viewport->page.PageRoot->ColorNum = 0;	
	viewport->page.PageRoot->UnderLine = 0;
	viewport->hzdata.hbf = NULL;
	viewport->resize = 0;
	viewport->savefilename = NULL;
	viewport->undo.ContentPtr = NULL;
	viewport->undo.lastmode = UDNONE;
	viewport->bar.DragPageNum = 1;
	viewport->borderOn = False;
	viewport->viewSym = False;
	viewport->savetimerId = (XtIntervalId) NULL;

	viewport->titler.mapped = False;
	viewport->titler.viewport = (void *) viewport;

	if(viewport->titler.text)
	{
		cp = DBstrdup(viewport->titler.text);
		if(cp) 
		{
			ReadSetTitler(viewport, cp, &viewport->titler);
			free(cp);	
		}
		else ResetTitler(viewport, &viewport->titler);
	}
	else ResetTitler(viewport, &viewport->titler); 
	
	viewport->footer.mapped = False;
	viewport->footer.viewport = (void *) viewport;

	if(viewport->footer.text)
	{
		cp = DBstrdup(viewport->footer.text);
		if(cp)
		{
			ReadSetTitler(viewport, cp, &viewport->footer); 
			free(cp);
		}
		else ResetTitler(viewport, &viewport->footer);
	}
	else ResetTitler(viewport, &viewport->footer);

	viewport->pndata.mapped = False;
	viewport->pndata.viewport = (void *) viewport;
	
	if(viewport->pndata.texta)
	{
		cp = DBstrdup(viewport->pndata.texta); 
		if(cp)
		{
			ReadSetPageNum(viewport, cp, &viewport->pndata);
			free(cp);
		}
		else ResetPageNum(viewport, &viewport->pndata);
	}
	else ResetPageNum(viewport, &viewport->pndata);

	switch(viewport->filemode)
	{
		case FILE_GB:
		case FILE_HZ:
		cp = (unsigned char *)GlobalDefaults.gbfontname;
		break;

		case FILE_BIG5:
		cp = (unsigned char *)GlobalDefaults.big5fontname;
		break;

		case FILE_EUC_JIS:
		cp = (unsigned char *)GlobalDefaults.jisfontname;
		break;

		case FILE_EUC_KS:
		cp = (unsigned char *)GlobalDefaults.ksfontname;
		break;

		case FILE_UTF_8:
		case FILE_UTF_7:
		case FILE_UTF_2:
		cp = (unsigned char *)GlobalDefaults.unifontname;
		break;

		default:
		cp = (unsigned char *)GlobalDefaults.fontname;
	}

	InitPointFont(viewport->resource.WorkArea, &viewport->hzdata, cp, GlobalDefaults.fontwidth, GlobalDefaults.fontheight, viewport->zoom);

	viewport->page.PageRoot->hzdata = viewport->hzdata;

        viewport->cursor.ViewPortCurPage = 1;
        viewport->viewp.ViewPortPage = 1;
        viewport->viewp.CurPage = viewport->page.PageRoot;
        viewport->cursor.CurPtr = viewport->EditFile;
        viewport->cursor.StartCurPtr = viewport->EditFile;
        viewport->cursor.EndCurPtr = viewport->EditFile;
        viewport->draw.DrawScr_x = viewport->draw.DrawPos_x = 0;
        viewport->draw.DrawScr_y = viewport->draw.DrawPos_y = 0;
	viewport->cursor.hzdata = viewport->hzdata;
	viewport->cursor.LineHeight = GlobalDefaults.fontheight * GlobalDefaults.init_zoom;
	viewport->cursor.Shown = False;
	viewport->cursor.Focused = False;
	viewport->cursor.TimerId = (XtIntervalId) NULL;

	CreateViewportColors(viewport);

	InitialPictures(viewport);

	MyDrawPage(viewport, VIRDRAW);
}

void WorkAreaExposeCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	VIewportdata *viewport = (VIewportdata *)client_data;
	XmDrawingAreaCallbackStruct *mydata = (XmDrawingAreaCallbackStruct *)call_data;
	XEvent *event = mydata->event;	

	if(event->xexpose.count==0)
	{
		MyDrawPage(viewport, EXPOSEDR);
	}
}

void WorkAreaResizeCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	VIewportdata *viewport = (VIewportdata *)client_data;
	
	viewport->resize = 2;	
	ViewPortResize(viewport);
	VerScroBarExpose(viewport);
	HorScroBarExpose(viewport);
}

void RevertCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	YESdoNOreturn(DoRevertCB, client_data, "Are you sure to revert?");
}

void CreateEditRoot(viewportlist)
MYviewportlist *viewportlist;
{
	char *cp;
	int i;

	if(viewportlist->myviewport.filename)
	{
		i = strlen(viewportlist->myviewport.filename);
		cp = viewportlist->myviewport.filename + i;
		for(; *cp != '/' && cp > viewportlist->myviewport.filename; cp--);
		if(*cp == '/') cp++;

		viewportlist->myviewport.resource.PopUpShell = XtAppCreateShell(cp, "XMdemos1", topLevelShellWidgetClass,
		XtDisplay(toplevel), NULL, 0);
	}
	else
	{
		viewportlist->myviewport.resource.PopUpShell = XtAppCreateShell("Unamed", "XMdemos1", topLevelShellWidgetClass, XtDisplay(toplevel),
		NULL, 0);
	}
}

void InitWinSize(viewport)
VIewportdata *viewport;
{
	register int n;
	Arg args[5];

	viewport->zoom = GlobalDefaults.init_zoom;
	ResetViewport(viewport, 2);

	switch(GlobalDefaults.win_selec)
	{
		case FIT_TO_DATA_WIN:
		case FIT_TO_WIN_PAGE:

		n = 0;
		XtSetArg(args[n], XmNheight, GlobalDefaults.win_height); n++;
		XtSetArg(args[n], XmNwidth, GlobalDefaults.win_width); n++;
		XtSetValues(viewport->resource.PopUpShell, args, n);

		break;

		case FIT_TO_MARGIN_WIN:
		case FIT_TO_WIDTH_WIN:
		case FIT_TO_PAGE_WIN:

		WindowFitPage(viewport, GlobalDefaults.win_selec, True);
		break;	
	}
}	

void RealizeEditWin(viewportlist)
MYviewportlist *viewportlist;
{
	BusySomeWindows();
	CreateEditRoot(viewportlist);	
	CreateEditWindow(viewportlist->myviewport.resource.PopUpShell, viewportlist);
	InitWinSize(&viewportlist->myviewport);
	XtRealizeWidget(viewportlist->myviewport.resource.PopUpShell);
	InitEditWindow(&viewportlist->myviewport);
	StartExposeEdit(&viewportlist->myviewport);
}

void IniViewport(parent, filename)
Widget parent;
char *filename;
{
	MYviewportlist *viewport, *temp;
	FILE *fp;
	char *cp;
	int i;

	viewport = (MYviewportlist *)malloc(sizeof(MYviewportlist));
	active_viewport = &viewport->myviewport;

	if(viewport == NULL) 
	{
		ErrorPopup("Can Allocate Memory!");
		return;
	}
	else
	{
		if(ViewPortRoot == NULL) 
		{
			ViewPortRoot = viewport;
			viewport->next = NULL;
			viewport->last = NULL;
		}
		else
		{
			for(temp = ViewPortRoot; temp->next != NULL; temp = temp->next);
			temp->next = viewport;
			viewport->next = NULL; 
			viewport->last = temp;
		}

		viewport->myviewport.savefilename = NULL;
		viewport->myviewport.undo.ContentPtr = NULL;
		viewport->myviewport.titler.text = NULL;
		viewport->myviewport.footer.text = NULL;
		viewport->myviewport.titler.mapped = False;
		viewport->myviewport.footer.mapped = False;
		viewport->myviewport.page.PageRoot = NULL;
		viewport->myviewport.EditFile = NULL;
		viewport->myviewport.pndata.mapped = False;
		viewport->myviewport.pndata.texta = NULL;
		viewport->myviewport.pndata.textb = NULL;
		viewport->myviewport.filemode = fileset.Encoding;
		viewport->myviewport.GraphRoot = NULL;
		viewport->myviewport.resource.PopUpShell = NULL;
		viewport->myviewport.resource.dpy = NULL;

		ResetPageInfo(&viewport->myviewport);
		InitColorInfo(&viewport->myviewport);
	}

	switch(ReadFile(viewport, filename))
	{
		case 1:
		
		RealizeEditWin(viewport);
		break;
		
		case -1:
	
		FreeViewport(viewport);
		break;

	}
}

