/*
 * Copyright 1995, 1996, 1997 by Jun Yei.       All Rights Reserved
 *
 * Permission to retain, use, modify, copy, and distribute ChinesePower 1.0
 * in source or binary and its documentation (hereafter, the Software)
 * for noncommercial purpose is hereby granted to you without a fee,
 * provided that this entire copyright and permission notice appear in
 * all such copies, that no charge be associated with such copies,
 * that distribution of derivative works (including value-added
 * distributions such as with additional input dictionaries or fonts)
 * include clarification that such added or derived parts are not from
 * the original Software, and that the names of the author(s) not be
 * used to endorse or promote such works. The author(s) of the software
 * makes no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * THE AUTHOR(S) OF THE SOFTWARE DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 * IN NO EVENT SHALL THE AUTHOR(S) OF THE SOFTWARE BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Jun Yei, Ottawa, ON, Canada
 */

#ifndef _XMHZVIEW_H_
#define _XMHZVIEW_H_

#include        <stdio.h>
#include        <ctype.h>
#include        <X11/Intrinsic.h>
#include        <X11/StringDefs.h>
#include        <X11/Xlib.h>
#include        <X11/Shell.h>
#include <sys/types.h>
#ifdef BSD
#include <sys/dir.h>
#else
#include <dirent.h>
#endif

#include <X11/Intrinsic.h>
#include <X11/IntrinsicP.h>
#include <X11/CoreP.h>
#include <X11/Shell.h>
#include <X11/cursorfont.h>
#include <X11/keysym.h>
#include <Xm/Xm.h>
#include <Xm/BulletinB.h>
#include <Xm/CascadeB.h>
#include <Xm/CascadeBG.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/LabelG.h>
#include <Xm/MainW.h>
#include <Xm/MessageB.h>
#include <Xm/PushB.h>
#include <Xm/PushBG.h>
#include <Xm/Form.h>
#include <Xm/FormP.h>
#include <Xm/RowColumn.h>
#include <Xm/ScrollBar.h>
#include <Xm/ScrolledW.h>
#include <Xm/Separator.h>
#include <Xm/DrawingA.h>
#include <Xm/DrawingAP.h>
#include <Xm/List.h>
#include <Xm/Text.h>
#include <Xm/FileSB.h>
#include <Xm/FileSBP.h>
#include <Xm/DialogS.h>
#include <Xm/DialogSP.h>
#include <Xm/ToggleBG.h>
#include <Xm/PanedW.h>

#include        "hbf.h"
#include 	"HZinput.h"
#include	"fonts.h"
#include        "simplehz.h"

#define MAX_ARGS 20


#define CANCELCOUNTER			25
#define FILE_SIZE_UNIT			8*1024
#define NUMOFPAGESET			31
#define NUMOFENCODING			9
#define HZ_TOTAL_NUM			0xffff
#define MAX_FONT_SIZE			1000
#define MIN_FONT_SIZE                   4
#define DEFAULT_FONT_SIZE		9
#define DEFAULT_FONT			"s16"
#define DEFAULT_ENGLISH_FONT		"8x16.hbf"
#define ENGLISH_FONT_WIDTH		8
#define ENGLISH_FONT_HEIGHT		16
#define CONFIGURE_FILE			"chpower.cfg"
#define INPUT_TABLE_FILE		"chpowertab."
#define CONF_FILE_PATH			"CHPOWERPATH"
#define PIC_FILE_PATH			"CHPPICPATH"
#define PRINT_CMD_FILE			".chpower.prtcmd"
#define COLOR_MAP_FILE			"/usr/lib/X11/rgb.txt"
#define FONTID				"font:"
#define SOFTKEYID			"softkey"
#define TYPE1				"type1"
#define TYPE2				"type2"
#define BEGIN				"begin"
#define END				"end"
#define BASIC				"BASIC"
#define HEIGHT				"height"
#define MAX_IM_IN_LIST  		25
#define MAX_SOFTKEY			100
#define KEYTYPE1			1
#define KEYTYPE2			2
#define DEFAULT_PAGE_MARGIN_W		72
#define DEFAULT_PAGE_MARGIN_H		90
#define DEFAULT_COL_SPACING		30
#define DEFAULT_LINE_SPACING		50 
#define DEFAULT_CHAR_SPACING		5
#define GB_SQUARE			0xA1F5
#define ZOOM_UNIT			10
#define LEAST_ZOOM			20
#define MOST_ZOOM			400
#define DEFAULT_ZOOM			150
#define MAXINPUT			100
#define MAXFONTNUM			30
#define DEFAULT_INPUTWIN_HZSIZE		16
#define DEFAULT_INPUTBTN_HZSIZE         16
#define SHW_HZSIZE			16
#define EDIT_FILE_SIZE			6400
#define TAB_SIZE			4
#define PAGE_NUM_WIDTH			9
#define PAGE_NUM_HEIGHT			9
#define MAXIDNUM			15
#define ENGLISHPSPARA			119
#define LETTERPAGESETID 		5
#define MAX_TAB_NUM			30
#define PICSCALE			72

typedef int             BOOL;
typedef unsigned int    UINT;

/* values 'picType' can take */

#define PIC8  0
#define PIC24 1

/* Draw Mode */

#define TRUEDRAW	0
#define CURSEARCH	1
#define VIRDRAW		2
#define UPDOWNAR	3
#define LEFTRIGHTAR	4
#define SCANEND		5
#define ASIS		6
#define INVIRDRAW	7 
#define HIGHLITE	8
#define UNHIGHLITE	9
#define MOVESEARCH	10
#define NONEACT		11
#define SMALLAFF	12
#define BIGAFF		13
#define EXPOSEDR	14
#define PSDRAW		15
#define REVCURSEARCH	16
#define CLEARDRAW	17
#define GIFLINEDRAW	18
#define GIFPAGEDRAW	19
#define ONLYHILITE	20

/* Input Mode */

#define KEYINPUT	0
#define CHARSETINS1	1
#define CHARSETINS2	2
#define CHARSETINS3	3
#define BTN2PRESS	4
#define VIRINPUT	5
#define BLOCKDELETED    6

/* Input Window ID */

#define HZINPUT 0
#define WAINPUT 1
#define SHINPUT 2

/* Cursor Action ID */

#define LEFT		0
#define RIGHT		1
#define UP		2
#define DOWN		3
#define NOCURM		4
#define ERROR		5
#define RIGHTSHIFTED 	6

/* Search Mode */

#define LEFTSEARCH	0
#define RIGHTSEARCH	1
#define WHOLESEARCH	2
#define LEFTANY		3
#define RIGHTANY	4
#define WHOLESEARCHCL	5

/* Control Info Mask */

#define SizeMask	(1<<0)
#define FontMask	(1<<1)
#define ColumnMask	(1<<2)
#define AlignmentMask	(1<<3) 
#define ColorMask	(1<<4)
#define UnderLineMask	(1<<5)
#define GraphicsMask	(1<<6)

/* ALignment Mode */

#define LeftAlign	0
#define CenterAlign	1
#define RightAlign	2

/* Create Viewport Action ID */

#define VIEWOPEN 	0
#define VIEWNEW		1	

/* Undo Mode */

#define UDDELETE	1
#define UDINPUT		2
#define UDBOTH		3
#define UDNONE		0

/* UnderLine Mode */

#define LINENONE	0
#define LINEMIDDLE	1
#define LINEUNDER	2

/* Window Fit Page Mode */
#define FIT_TO_PAGE_WIN         0
#define FIT_TO_WIDTH_WIN        1
#define FIT_TO_MARGIN_WIN	2
#define FIT_TO_WIN_PAGE		3
#define FIT_TO_DATA_WIN         4
#define ASIS_WIN                5

/* Convert Mode */
#define GB2BIG5		0
#define GB2JIS		1
#define GB2KS		2
#define GB2UNI          16
#define BIG52GB		3
#define BIG52JIS	4
#define BIG52KS		5
#define BIG52UNI        17
#define JIS2GB		6
#define JIS2BIG5	7
#define JIS2KS		8
#define JIS2UNI         18
#define KS2GB		9
#define KS2BIG5		10
#define KS2JIS		11
#define KS2UNI          19
#define UNI2GB          12
#define UNI2BIG5        13
#define UNI2JIS         14
#define UNI2KS          15

/* File Encoding */
#define FILE_DEFAULT	0
#define FILE_GB		1
#define FILE_HZ		2
#define FILE_BIG5	3
#define FILE_EUC_JIS	4
#define FILE_EUC_KS	5
#define FILE_UTF_2	6
#define FILE_UTF_7	7
#define FILE_UTF_8	8

/* Tool Bar Icons */
#define SAVE_ICON	0
#define OPEN_ICON	1
#define CUT_ICON	2
#define COPY_ICON	3
#define PASTE_ICON	4
#define FIND_ICON	5
#define CENTER_ICON	6
#define UNDO_ICON	7
#define LEFT_ICON	8
#define RIGHT_ICON	9
#define COL1_ICON	10
#define COL2_ICON	11
#define SYM_ICON	12
#define UNDER_ICON	13
#define PRINT_ICON	14
#define MIDDLE_ICON	15
#define NOLINE_ICON	16
#define INPUT_ICON	17
#define CHAR_ICON	18
#define PALLET_ICON	19
#define REFRESH_ICON	20
#define NEW_ICON	21
#define PIC_ICON	22

/* Color Option for Graph */
#define REVERSE_COLOR	0 
#define NORMAL_COLOR	1

/* return values from ReadFileType()
 * positive values are *definitely* readable formats (HAVE_*** is defined)
 * negative values are random files that XV can't read, but display as
 *   different icons in the visual browser
 */
#define RFT_ERROR    -1    /* couldn't open file, or whatever... */
#define RFT_UNKNOWN   0
#define RFT_GIF       1
#define RFT_PM        2
#define RFT_PBM       3
#define RFT_XBM       4
#define RFT_SUNRAS    5
#define RFT_BMP       6
#define RFT_UTAHRLE   7
#define RFT_IRIS      8
#define RFT_PCX       9
#define RFT_JFIF     10
#define RFT_TIFF     11
#define RFT_PDSVICAR 12
#define RFT_COMPRESS 13
#define RFT_PS       14
#define RFT_IFF      15
#define RFT_TARGA    16
#define RFT_XPM      17
#define RFT_XWD      18
#define RFT_FITS     19
 
/* Macros */

#define MIN(a,b)        (((a)<(b)) ? (a) : (b))
#define MAX(a,b)        (((a)>(b)) ? (a) : (b))
#define ISHZ(c)         ((c) > 0x00ff)
#define ISACHZ(c)         ((c) > 0xa0)
#define SKIPIDS(tp)	if(fileset.TextOnly && *tp == '@' && *(tp+1) == '[') \
			{ int i;  unsigned short *tmp = tp;\
			  for(i = 0, tp += 2; *tp != ']' && *tp != 0 && i < MAXIDNUM; tp++, i++);\
			  if(*tp != ']') tp = tmp;\
			  else { tp++; continue; }\
			}

#define CONVERT_X	viewport->horOrder ? XmHORIZONTAL : XmVERTICAL
#define CONVERT_Y	viewport->horOrder ? XmVERTICAL : XmHORIZONTAL

/* MONO returns total intensity of r,g,b triple (i = .33R + .5G + .17B) */
#define MONO(rd,gn,bl) ( ((int)(rd)*11 + (int)(gn)*16 + (int)(bl)*5) >> 5)

typedef unsigned char byte;

typedef struct SOFTKEYMEM
{
	char *KeyMem;
	int NumofKey;
	struct	SOFTKEYMEM *Next;
} SoftKeyMem;

typedef struct SOFTKEY
{
	char *KeyName;
	SoftKeyMem *MemRoot;
	int height;	
	struct SOFTKEY *Next;
} SoftKey;

typedef struct PAGEDATA
{
	HZdata hzdata;
	int LineAlignment;
	int ColNum;
	int PageNum;
	int ColorNum;
	int UnderLine;
	unsigned short *PageStart;
	struct PAGEDATA *Last;
	struct PAGEDATA *Next;
}PAgedata;

typedef struct
{
	unsigned short *text;
	HZdata hzdata;
	int underline;
	int ColorNum;
	int Position;
	int mapped;
	void *viewport;
}MYViewSHW;

typedef struct
{
	HZdata hzdata;
	int ColorNum;
	int Align;
	int Position;
	int ShowIt;
	int FirstShow;
	int mapped;
	unsigned short *textb;
	unsigned short *texta;
	void *viewport;
} PAgeNumData;

typedef struct GRAPHE
{
	struct GRAPHE *next;
	char *fileName;
	int w;
	int h;
	int curW;
	int curH;
	int zoomNum;
	int picType;                 /* PIC8 or PIC24 */
        byte r[256], g[256], b[256];
	byte *pic;
	XImage *Image;
	int used;
} GRaphe;

typedef struct
{
	struct
	{
		int ViewPortPage;
		int ViewPort_x;
		int ViewPort_y;
		int ViewPortHeight;
		int ViewPortWidth;
		PAgedata *CurPage;
	} viewp;

	struct
	{
		int ViewPortHeight;
		int ViewPortWidth;
		int PageSetId;
		int Portrait;
	} origpage;
	
	struct
	{
		int PageWidth;
		int PageHeight;
		int PageMarginWidth;
		int PageMarginHeight;
		int PageColSpacing;      
		int PageLineSpacing; 
		int CharSpacing;	
		int MaxPageNum;
		PAgedata *PageRoot;
	} page;

	struct
	{
		int ColNum;
		int LineAlignment;
		int UnderLine;
		int GraphHeight;
	} layout;

	struct
	{
		int VerBarInc;
		int HorBarInc;
		int DragPageNum;	
	} bar;
	
	struct
	{
		int DrawPos_x;
		int DrawPos_y;
		int DrawLen_x;
		int DrawLen_y;
		int DrawScr_x;
		int DrawScr_y;
		int LineHeight;	
		int LineWidth;	
	} draw;
	
	struct
	{
		unsigned short *RedrawCurPtr;	
		unsigned short *CurPtr;
		unsigned short *newcurptr;
		HZdata hzdata;
		int Shown;
		int Focused;
		XtIntervalId TimerId;	
		int LineHeight;
		int ViewPortCurPage;
		int ViewPortCur_x;
		int ViewPortCur_y;
		int ColNum;
		int UnderLine;
		int LineAlignment;
		int ColorNum;
		unsigned short *MoveCurPtr;
		HZdata Mohzdata;
		int MoveCurPage;
		int MoveCur_x;
		int MoveCur_y;
		int MoveColNum;
		int MoveUnderLine;
		int MoveDir;
		int MoveLineAlignment;
		int MoveColorNum;
		unsigned short *StartCurPtr;
		unsigned short *EndCurPtr;
		unsigned short *LastStartCurPtr;
		unsigned short *LastEndCurPtr;
		int CurActionId;
	} cursor;

	struct
	{
		unsigned short *StartPtr;
		unsigned short *EndPtr;
		unsigned short *TmpSPtr;
		unsigned short *TmpEPtr;	
	} set;

	struct
	{
		int Btn1Pressed;
		int Pointer_x;
		int Pointer_y;
		int MovePo_x;
		int MovePo_y;
		int DrawCur_x;	
	} pointer;

	struct
	{
		unsigned short *ContentPtr;
		unsigned short *InsertPtr;
		int numofinput;	
		int numofbyteinput;
		int lastmode;
	} undo;

	struct
	{
		Widget PopUpShell;
		Widget VerScroBar;
		Widget HorScroBar;
		Widget WorkArea;
		Widget ZoomBtn;
		Widget PageBtn;	
		Widget Popup;	
		Display *dpy;
		Window  wind;                   /* drawing toplevel window */
		GC gc[7];                          /* default GC */
		GC reverse_gc[7];
		GC dash_gc;
	} resource;

	struct
	{
		int Mapped;
		int EditMapped;
		Widget baseWg;
		Widget editWg;
		Widget Btns[30];
	} toolbar;
		
	HZdata hzdata;
	MYViewSHW titler;
	MYViewSHW footer;
	PAgeNumData pndata;
	char *ColorName[8];	
	int ColorNum;
	int resize;
	int zoom;
	int oldzoom;
	unsigned short *EditFile;
	FILE *fp;
	char *filename;
	char *savefilename;
	long int sizeunit;	
	int filemode;
	int changed;	
	int autosavechanged;
	int virdrawn;
	int borderOn;
	int viewSym;
	int horOrder;
	XtIntervalId savetimerId;
	Widget ColorBtns[7];
	GRaphe *GraphRoot;
} VIewportdata;

typedef struct MYVIEWPORTLIST
{
	VIewportdata myviewport;
	struct MYVIEWPORTLIST *next;
	struct MYVIEWPORTLIST *last;
} MYviewportlist;

typedef struct 
{
	XtermWidgetRec InputWin;
	CXtermInputModule cxtermInput;
	Widget Root;	
	Widget PopUpShell;
	Widget Type1Wgd, Type1KeyWgd;
	Widget Type2Wgd, Type2KeyWgd;
	Widget AutoSelWgd[3];
	Widget AutoSegWgd;
	Widget AssociaWgd;
	Widget PlInputMtdWg;	
	Widget SubInputMtdWg[MAX_IM_IN_LIST];
	Widget PlSoftKeyWg;	
	Widget SubSoftKeyWg[MAX_TAB_NUM];

	int mapped;
	int assolstchg;
	int topheight, key1height, key2height;
	String im_list[MAX_IM_IN_LIST];

	struct
	{
		int NumOfInput;
		unsigned short InputPtr[MAXINPUT];
		unsigned char HzInputPtr[MAXINPUT];	
		unsigned short *buffer;
		char *FontId;
		char *EndFontId;
		int ColNum;
		int EndColNum;
		int LineAlignment;
		int EndLineAlignment;
		int ColorNum;
		int EndColorNum;
		int UnderLine;
		int EndUnderLine;
		int point_width;
		int Endpoint_width;
		int point_height;
		int Endpoint_height;
	} input;

} HZinputmod;

typedef struct
{
	Widget PopUpWg;
	Widget Root;
	Widget WidthWg;
	Widget HeightWg;
	Widget HzFontLstWg;
	Widget EgFontLstWg;
	Widget ApplyBtn;
	Widget CloseBtn;
	int charsetmapped;
	XmString *fontlist;
} CharSetClass;

typedef struct
{
	Widget PopUpWg;
	Widget ImgLstWg;
	Widget ImBtn;
	XmString *picnamelist;
	int picnum;
} PIcImportSet;

typedef struct
{
	Widget PopUpWg;
	Widget FileList;
	int Encoding;
	int TextOnly;
	Widget DefaultEncWg;
	Widget TextOnlyWg;
	Widget OptionWg;
} FileSelList;

typedef struct
{
	Widget PopUpWg;
	Widget Root;
	Display *dpy;
	Window  wind;
	GC gc;
	int mapped;
} TitleShow;

typedef struct
{
	Widget AsBit;
	Widget Text;
	int Size;
	Boolean BitMap;
} TTFWig;

typedef struct
{
	Widget PopUpWg;
	Widget Root;
	TTFWig *Font;
} TTFMenu;

typedef struct
{
	int AllPage;
	int StartPage;
	int EndPage;
	int Printer;
	int ColorPS;	
	char *PrCmd;
	FILE *psfile;	
	Widget PopUpWg;
	Widget Root;
	Widget AllPgBtn;
	Widget PartPgBtn;
	Widget StartPgWg;
	Widget EndPgWg;
	Widget PrinterBtn;
	Widget PrintCmdWg;
	Widget FileBtn;
	Widget PsFileWg;
	Widget PrintBtn;
	Widget CancelBtn;
} PrintMenu;

typedef struct
{
	int AllPage;
	int StartPage;
	int EndPage;
	int PageGif;
	int Transparent;
	int Interlace;
	int ColorGif;
	char *giffilename;
	Widget PopUpWg;
	Widget Root;
	Widget AllPgBtn;
	Widget PartPgBtn;
	Widget StartPgWg;
	Widget EndPgWg;
	Widget PageGifBtn;
	Widget LineGifBtn;
	Widget GifFileWg;	
	Widget PrintBtn;
	Widget CancelBtn;
	Pixmap pixmap;

	struct
	{
		XImage *image;
		int width;
		int height;
		unsigned long pixel[8];
		unsigned char red[8];
		unsigned char green[8];
		unsigned char blue[8];
	} draw;
} GifMenu;

typedef struct
{
	int Portrait;
	int PageSetId;
	int horizonOrder;
	Widget PopUpWg;
	Widget Root;
	Widget PageSize;
	Widget LandBtn;
	Widget PortBtn;
	Widget LHorBtn;
	Widget LVerBtn;
	Widget VerMarWg;
	Widget HorMarWg;
	Widget LineSpWg;
	Widget CharSpWg;	
	Widget ColSpWg;
	Widget SetBtn;
	Widget CloseBtn;
	Widget PageBtn[NUMOFPAGESET];
} PageSetMenu;

typedef struct
{
	Widget PopUpWg;
	Widget Root;
	Widget StrPgWg;
	Widget EndPgWg;
	Widget DelBtn;
	Widget CancelBtn;
} PageDelMenu;

typedef struct
{
	Widget PopUpWg;
	Widget Root;
	Widget ColorSelWg;
	Widget DrawWg;
	Widget OkBtn;
	Widget CancelBtn;
	int ColorNum;
	int AllocedColor;	
	Display *dpy;
	Window  wind;
	GC gc;
	XmString *ColorList;
	int TotalColor;
} COlorMenu;

typedef struct
{
	Widget PopUpWg;
	Widget Root;
	Widget Title;
	Widget HzFontLstWg;
	Widget WidthWg;
	Widget HeightWg;
	Widget UnderLineWg;
	Widget TextWg;
	Widget OkBtn;
	Widget CancelBtn;
	Widget ColorOpWg;
	Widget ColorBtn[7];
	Widget AlignOpWg;
	Widget AlignBtn[3];
	int mapped;
	int underline;
	int ColorNum;
	int Position;
	SImpleHzWin sviewport;
} SHwPopup;

typedef struct
{
	Widget PopUpWg;
	Widget Root;
	Widget HzFontLstWg;
	Widget WidthWg;
	Widget HeightWg;
	Widget TextBWg;
	Widget TextAWg;
	Widget ColorOpWg;
	Widget ColorBtn[7];
	Widget AlignOpWg;
	Widget AlignBtn[3];
	Widget PosOpWg;
	Widget PosBtn[2];
	Widget ShowNumTgWd;
	Widget FirstTgWd;
	int mapped;
	int ColorNum;
	int Align;
	int Position;
	int ShowIt;
	int FirstShow;
	PAgeNumData *Ptrpn;
	SImpleHzWin textb;
	SImpleHzWin texta;
} PAgeNumPopup;

typedef struct
{
	Widget PopUpWg;
	Widget Root;
	Widget FindWg;
	Widget ChangeWg;
	SImpleHzWin Findviewport;
	SImpleHzWin Changeviewport;
	int    mapped;
	unsigned short *startpoint;
	int startedfind;
} FIndPopup;

typedef struct
{
	Widget PopUpWg;
	Widget Root;
	Widget PageNumWg;
	Widget TotalLWg;
} PageButton;

typedef struct
{
	Widget main_window, newBtn, openBtn, inputBtn, infoBtn, exitBtn;
} TOplevelWg;

typedef struct
{
	char *fontname;
	int fontwidth;
	int fontheight;
	int margin_width;
	int margin_height;
	int page_type;
	int line_spacing;
	int char_spacing;
	int column_spacing;
	int init_zoom;
	int page_orien;
	int pagenum_width;	
	int pagenum_height;	
	char *color[8];
	char *topBtnColor[5];
	char comma[2];
	char period[2];
	char space[2];
	char *gbfontname;
	char *big5fontname;
	char *jisfontname;
	char *ksfontname;
	char *unifontname;
	int hz_file_width; 
	int horOrder;
	int timer_on;
	int timer_interval;
	int win_width;
	int win_height;
	int win_selec;
	int ttf_as_bitmap;
	int hz_enter_escape;
} MYdefault;

typedef struct
{
	Atom *atoms;
	int atoms_size;
	int count;
	Time time;
	unsigned char *sel;
	unsigned long length;
	void *viewport;
	int VPType;
	Widget w;
} MYSelection;

extern Cursor myxterm;
extern Cursor mywatch;
extern HZinputmod MyHzInMod;
extern VIewportdata *active_viewport;
extern TitleShow title;
extern PrintMenu printset;
extern GifMenu gifset;
extern PageSetMenu pageset;
extern MYfontlist *FontListRoot;
extern MYviewportlist *ViewPortRoot;
extern CharSetClass charset;
extern Widget toplevel;
extern XmStringCharSet chardef;
extern HBF *EnglishFontHbf;
extern PageButton pagebtn;
extern PageDelMenu pagedel;
extern SImpleHzWin *active_simphzwin;
extern int simple_active;
extern XmString fontlist[];
extern FIndPopup findset;
extern SHwPopup titlerset;
extern COlorMenu colortab;
extern char *HZmthdParam[][2];
extern char *HZmthAct[];
extern char *PageInfo[];
extern char *EncodingInfo[];
extern int PageData[][2];
extern char *HZautoSlect[];
extern char *Default_Colors[];
extern char *Default_TopBtnColors[];
extern Pixmap IconPixmap;
extern MYdefault GlobalDefaults;
extern int Global_Big;
extern TTFMenu ttfset;
extern MYSelection myselect;
extern XtIntervalId HzTimerId;
extern unsigned short delID[];
extern unsigned short enterID[];
extern FileSelList fileset;
extern Pixmap ToolBarIcons[];
extern char *TBIconsName[];
extern int LongProcCancelFlg;
extern XtAppContext app;
extern int CriticalGoing;

extern void InputTableInitial();
extern void FontDefaultInitial();
extern void DestroyMyFontList();
extern void PageScroUp();
extern void PageScroDown();
extern void VerScroBarExpoHandler();
extern void LineScroUp();
extern void LineScroDown();
extern void VerScroDraged();
extern void PageScroLeft();
extern void PageScroRight();
extern void HorScroBarExpoHandler();
extern void LineScroLeft();
extern void LineScroRight();
extern void HorScroDraged();
extern void MyDrawCharacter();
extern void WorkAreaInputCB();
extern void FakeWorkAreaInputCB();
extern int TotalNumOfFont();
extern char *GetFontId();
extern void FileOpenCB();
extern void FileSaveAsCB();
extern void FileSaveAsStartCB();
extern void FileSaveCB();
extern void HzInputAreaInputCB();
extern void FontCalc();
extern HBF *GetFontHandle();
extern int Input();
extern int SearchFontList();
extern HBF *GetFontHandleFrIndex();
extern void MyDrawPage();
extern void ResetViewport();
extern void ErrorPopup();
extern void IniHzInMod();
extern void InitMyFontList();
extern void DestroyMyFontList();
extern void DestroyCXtermInput();
extern void CreateFileSet();
extern void IniViewport();
extern void FileOpen();
extern void HzInput();
extern void OpenTitleCB();
extern void ShowTitle();
extern void CloseTitle();
extern void DestroySoftKey();
extern void CreatePrintSet();
extern void CreatePageSet();
extern void CreateErrorMsg();
extern void CreatePromptMsg();
extern void CreatePageNum();
extern void PageBtnCB();
extern void HzBtnInputCB();
extern void PrintSetCB();
extern void ColumnSetCB();
extern void AlignmentSetCB();
extern void CutCB();
extern void CopyCB();
extern void DeleteCB();
extern void PasteCB();
extern void PageSetCB();
extern void ImportCB();
extern void SelectAllCB();
extern void ConfirmQuit();
extern void YESdoNOreturn();
extern void DoRevertCB();
extern void ZoomInCB();
extern void ZoomOutCB();
extern void CreateCharSet();
extern void CharSetCB();
extern void HorScroBarExpose();
extern void VerScroBarExpose();
extern void ZoomRtBtnCB();
extern int rmanysamecomponent();
extern int rmtypedcomponent();
extern unsigned short *insert();
extern unsigned short *delete();
extern unsigned short *escbackhead();
extern unsigned short *escforehead();
extern void LeftRightAroInput();
extern void RenewCharSet();
extern void VerScroBarPos();
extern void HorScroBarPos();
extern void DrawMyCursor();
extern int PageMoveUp();
extern int PageMoveDown();
extern int PageMoveRight();
extern int PageMoveLeft();
extern void DeleteCharSet();
extern void SetHZinMethod();
extern void ZoomBtnZoom();
extern int CheckCurrentPage();
extern void HzDataZoom();
extern void MainExitYesCB();
extern void ProSaveCB();
extern void ProNoSaveCB();
extern void ExitSaveCB();
extern void ExitNoSaveCB();
extern void NewFileSaveAndExit();
extern void UndoCB();
extern void PageDelCB();
extern void CreatePageDel();
extern FILE *My_path_open();
extern void PopupPosition();
extern void ZoomOpr();
extern void MyPutPixmapToGif();
extern void GifSetCB();
extern void CreateGifSet();
extern int GIFEncode();
extern void CreateCursor();
extern void BusySomeWindows();
extern void BusyWindows();
extern void UnBusySomeWindows();
extern void UnBusyWindows();
extern void VerScroDraging();
extern int Input();
extern void SimpleHzWinInputCB();
extern void InitialSimpleHZWin();
extern void InitSingleFont();
extern void DrawSimpleHzWin();
extern void TitlerPopupCB();
extern void FooterPopupCB();
extern void AddSHwCallbacks();
extern void ShwDrawCursor();
extern int SimpleHzWinMoveLeft();
extern int SimpleHzWinMoveRight();
extern void CancelTitlerCB();
extern void FindPopupCB();
extern void CreateFinder();
extern void GifToggleCB();
extern void ColorTabPopupCB();
extern void DestroyColor();
extern void DestroyCharSet();
extern void ColorSetCB();
extern void UnderLineSetCB();
extern void InitPointFont();
extern void ReadSetTitler();
extern void FileReadSetTitler();
extern void ResetTitler();
extern void WriteTitlerToFile();
extern void DestroyDefaults();
extern void Init_TTF_Env();
extern void *Open_TTF();
extern void Close_TTF();
extern void DestroyTTF();
extern int TTF_GetBitmap();
extern int GetFontType();
extern int PS_TTF_GetChar();
extern void TTFSetCB();
extern void CreateTTFSet();
extern void DestroyTTFSet();
extern void ResetPageNum();
extern void CancelPageNumCB();
extern void CreatePageNumSet();
extern void PageNumSetPopupCB();
extern void GetSelection();
extern void OwnSelection();
extern void DisownSelection();      
extern void DistroySelection();
extern void FileReadSetPageNum();
extern void ReadSetPageNum();
extern void WritePageNumToFile();
extern void WritePageInfoToFile();
extern void ReadSetPageInfo();
extern void ResetPageInfo();
extern void WriteColorInfoToFile();
extern void ReadSetColorInfo();
extern void ResetColorInfo();
extern void CurFlash();
extern void SimCurFlash();
extern void HzCurFlash();
extern void HzSimCurFlash();
extern void InitColorInfo();
extern void ClearUpDefaults();
extern void InitDefault();
extern void InitialSimpleHZWin();
extern void DBfileWrite();
extern long DBfileSize();
extern void DBfileRead();
extern int DBlen();
extern int DBbytelen();
extern int byteDBlen();
extern unsigned short *DBdup();
extern void DBcat();
extern void DBstrcat();
extern unsigned short *strDBdup();
extern unsigned char *DBstrdup();
extern unsigned char *strDBcpy();
extern unsigned short *DBcpy();
extern int DBbytenlen();
extern int MatchAssocList();
extern void ModifyAssocList();
extern int SaveAssocList();
extern void ConfirmSaveHZAssoc();
extern void MainDestroyCB();
extern void DestroySave();
extern void YESdoNOdo();
extern void RemoveAutoSavedFile();
extern void SelectionReceived();
extern void PhraseOperation();
extern void PhraseAddSelReceived();
extern void PhraseDelSelReceived();
extern void PhraseChkSelReceived();
extern void WindowFitPage();
extern void PageFitWindow();
extern void FreeViewport();
extern void CreateViewportColors();
extern void FreeViewportColors();
extern void ErrorFuncPopup();
extern void ErrOkCB();
extern void Convert();
extern void U8fileRead();
extern void U8fileWrite();
extern void U7fileRead();
extern void U7fileWrite();
extern void U2fileRead();
extern void U2fileWrite();
extern void GenerateZoomedPic();
extern GRaphe *SearchPicture();
extern void xvbzero();
extern void GenerateFSGamma();
extern void InitialPictures();
extern void ReadPictureFilename();
extern void WritePictureToFile();
extern int UpdatePicture();
extern void PicImportCB();
extern int PictureCount();
extern void PictureToPS();
extern void ClearUpPic();
extern void MainNewCB();
extern void MainOpenCB();
extern void LongProcCancelCB();
extern void CreatePicImport();
extern void PicImportDestroy();
extern void ErrorChgMsg();

extern Widget CreateSimpleHzWin(
#if NeedFunctionPrototypes
	Widget,
	char *,
	Arg *,
	int,
	SImpleHzWin *
#endif
);

#endif /*_XMHZVIEW_H_*/

