/*
 * Copyright 1995, 1996, 1997 by Jun Yei.       All Rights Reserved
 *
 * Permission to retain, use, modify, copy, and distribute ChinesePower
 * in source or binary and its documentation (hereafter, the Software)
 * for noncommercial purpose is hereby granted to you without a fee,
 * provided that this entire copyright and permission notice appear in
 * all such copies, that no charge be associated with such copies,
 * that distribution of derivative works (including value-added
 * distributions such as with additional input dictionaries or fonts)
 * include clarification that such added or derived parts are not from
 * the original Software, and that the names of the author(s) not be
 * used to endorse or promote such works. The author(s) of the software
 * makes no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * THE AUTHOR(S) OF THE SOFTWARE DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 * IN NO EVENT SHALL THE AUTHOR(S) OF THE SOFTWARE BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Jun Yei, Ottawa, ON, Canada
 */

#include "chpower.h"

void ColorTabCB();
void CancelColorTabCB();
void ShowColorCB();
void OrigColorCB();
void OkColorTabCB();
void CheckFreeColor();

COlorMenu colortab;

void DestroyColor()
{
	int i;

	for(i = 0; i < colortab.TotalColor; i++) XmStringFree(colortab.ColorList[i]);
	
	free(colortab.ColorList);
}

void ColorTabExposeCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	XFillRectangle(colortab.dpy, colortab.wind, colortab.gc, 0, 0, 100, 150);
}

void CreateColorTab(parent)
Widget parent;
{
	Arg args[MAX_ARGS];
	register int n;
	int i, total;	
	char line[256], *cp, *tmp;
	Widget label, submenu, ColorBtn[8];
	FILE *fp;
	XGCValues gc_init;

	colortab.PopUpWg = XtCreatePopupShell("Color Window", transientShellWidgetClass, parent, NULL, 0);

	n = 0;
	XtSetArg(args[n], XmNheight,290); n++;
	XtSetArg(args[n], XmNwidth, 300); n++;
	XtSetArg(args[n], XmNhorizontalSpacing, 0); n++;
	XtSetArg(args[n], XmNverticalSpacing, 0); n++;
	XtSetArg(args[n], XmNrubberPositioning, True); n++;
	colortab.Root = XmCreateForm(colortab.PopUpWg, "CharSet", args, n);
	XtManageChild(colortab.Root);
	XtAddCallback(colortab.Root, XmNdestroyCallback, MainDestroyCB, NULL);

	colortab.TotalColor = 0;

	if((fp = fopen(COLOR_MAP_FILE, "r")))
	{
		while((cp=fgets(line, 256, fp)) != NULL)
		{
			while(*cp == ' ' || *cp == '\t' || ( *cp >= '0' && *cp <= '9')) cp++;
			
			if(*cp == '\n' || *cp == '\0') continue;
			
			colortab.TotalColor++;
		}
		
		rewind(fp);
	
		if(colortab.TotalColor)
		{
			colortab.ColorList = (XmString *)malloc(colortab.TotalColor*sizeof(XmString));
			total = 0;

			while((cp=fgets(line, 256, fp)) != NULL)
			{
				while(*cp == ' ' || *cp == '\t' || ( *cp >= '0' && *cp <= '9')) cp++;
			
				if(*cp == '\n' || *cp == '\0') continue;
			
				for(tmp = cp; *tmp != '\n' && *tmp != '\0'; tmp++);

				*tmp = '\0';

				colortab.ColorList[total] = XmStringCreateLtoR(cp, XmSTRING_DEFAULT_CHARSET);

				total ++;
				if(total > colortab.TotalColor) break;	
			}
		}	
	
		fclose(fp);	
	}
	
	if(colortab.TotalColor == 0)
	{
		colortab.TotalColor = 1;
		colortab.ColorList = (XmString *)malloc(sizeof(XmString));

		colortab.ColorList[0] = XmStringCreateLtoR("No Color Available", XmSTRING_DEFAULT_CHARSET);
	}
	
	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Colors:", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNx, 15); n++;
	XtSetArg(args[n], XmNy, 45); n++;
	label = XmCreateLabelGadget(colortab.Root, "Ftype", args, n);
	XtManageChild(label);

	n = 0;
	XtSetArg(args[n], XmNitems, colortab.ColorList); n++;
	XtSetArg(args[n], XmNitemCount, colortab.TotalColor); n++;
	XtSetArg(args[n], XmNx, 15); n++;
	XtSetArg(args[n], XmNy, 65); n++;
	XtSetArg(args[n], XmNheight, 200); n++;
	XtSetArg(args[n], XmNwidth, 150); n++;
	colortab.ColorSelWg = XmCreateScrolledList(colortab.Root, "HZfontlist", args, n);
	XtAddCallback(colortab.ColorSelWg, XmNbrowseSelectionCallback, ShowColorCB, NULL);	
	XtManageChild(colortab.ColorSelWg);

	submenu = (Widget) XmCreatePulldownMenu(colortab.Root, "Pull", NULL, 0);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("BackGround", XmSTRING_DEFAULT_CHARSET)); n++;
	ColorBtn[0] = XmCreatePushButtonGadget(submenu, "Co", args, n);
	XtAddCallback(ColorBtn[0], XmNactivateCallback, OrigColorCB, (caddr_t *)0);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Standard", XmSTRING_DEFAULT_CHARSET)); n++;
	ColorBtn[1] = XmCreatePushButtonGadget(submenu, "Co", args, n);
	XtAddCallback(ColorBtn[1], XmNactivateCallback, OrigColorCB, (caddr_t *)1);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Color 1", XmSTRING_DEFAULT_CHARSET)); n++;
	ColorBtn[2] = XmCreatePushButtonGadget(submenu, "Co", args, n);
	XtAddCallback(ColorBtn[2], XmNactivateCallback, OrigColorCB, (caddr_t *)2);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Color 2", XmSTRING_DEFAULT_CHARSET)); n++;
	ColorBtn[3] = XmCreatePushButtonGadget(submenu, "Co", args, n);
	XtAddCallback(ColorBtn[3], XmNactivateCallback, OrigColorCB, (caddr_t *)3);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Color 3", XmSTRING_DEFAULT_CHARSET)); n++;
	ColorBtn[4] = XmCreatePushButtonGadget(submenu, "Co", args, n);
	XtAddCallback(ColorBtn[4], XmNactivateCallback, OrigColorCB, (caddr_t *)4);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Color 4", XmSTRING_DEFAULT_CHARSET)); n++;
	ColorBtn[5] = XmCreatePushButtonGadget(submenu, "Co", args, n);
	XtAddCallback(ColorBtn[5], XmNactivateCallback, OrigColorCB, (caddr_t *)5);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Color 5", XmSTRING_DEFAULT_CHARSET)); n++;
	ColorBtn[6] = XmCreatePushButtonGadget(submenu, "Co", args, n);
	XtAddCallback(ColorBtn[6], XmNactivateCallback, OrigColorCB, (caddr_t *)6);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Color 6", XmSTRING_DEFAULT_CHARSET)); n++;
	ColorBtn[7] = XmCreatePushButtonGadget(submenu, "Co", args, n);
	XtAddCallback(ColorBtn[7], XmNactivateCallback, OrigColorCB, (caddr_t *)7);

	XtManageChildren(ColorBtn, 8);

	colortab.ColorNum = 2;

	n = 0;
	XtSetArg(args[n], XmNsubMenuId, submenu); n++;
	XtSetArg(args[n], XmNmenuHistory, ColorBtn[2]); n++;
	XtSetArg(args[n], XmNmarginHeight, 0); n++;
	XtSetArg(args[n], XmNmarginWidth, 0); n++;
	XtSetArg(args[n], XmNspacing, 0); n++;
	XtSetArg(args[n], XmNpacking, XmPACK_TIGHT); n++;
	XtSetArg(args[n], XmNx, 70); n++;
	XtSetArg(args[n], XmNy, 15); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Set Color:", XmSTRING_DEFAULT_CHARSET)); n++;
	label = XmCreateOptionMenu(colortab.Root, "Pull", args, n);
	XtManageChild(label);

	n = 0;
	XtSetArg(args[n], XmNmarginHeight, 0); n++;
	XtSetArg(args[n], XmNmarginHeight, 0); n++;
	XtSetArg(args[n], XmNwidth, 100); n++;
	XtSetArg(args[n], XmNheight, 100); n++;
	XtSetArg(args[n], XmNshadowThickness, 3); n++;
	XtSetArg(args[n], XmNshadowType, XmSHADOW_IN); n++;
	XtSetArg(args[n], XmNx, 180); n++;
	XtSetArg(args[n], XmNy, 85); n++;
	label  = XmCreateFrame(colortab.Root, "Frame", args, n);
	XtManageChild(label);
	
	n = 0;
	XtSetArg(args[n], XmNmarginHeight, 0); n++;
	XtSetArg(args[n], XmNmarginWidth, 0); n++;
	colortab.DrawWg = XmCreateDrawingArea(label, "Draw", args, n);
	XtAddCallback(colortab.DrawWg, XmNexposeCallback, ColorTabExposeCB, NULL);
	XtManageChild(colortab.DrawWg);

	n = 0;
	XtSetArg(args[n], XmNwidth, 50); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Set", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNx, 180); n++;
	XtSetArg(args[n], XmNy, 215); n++;
	colortab.OkBtn = XmCreatePushButtonGadget(colortab.Root, "applyBtn", args, n);
	XtAddCallback(colortab.OkBtn, XmNactivateCallback, OkColorTabCB, NULL);
	XtManageChild(colortab.OkBtn);

	n = 0;
	XtSetArg(args[n], XmNwidth, 50); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Close", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(args[n], XmNleftWidget, colortab.OkBtn); n++;
	XtSetArg(args[n], XmNy, 215); n++;
	colortab.CancelBtn = XmCreatePushButtonGadget(colortab.Root, "closeBtn", args, n);
	XtManageChild(colortab.CancelBtn);
	XtAddCallback(colortab.CancelBtn, XmNactivateCallback, CancelColorTabCB, NULL);

	XtRealizeWidget(colortab.PopUpWg);

	colortab.dpy = XtDisplay(colortab.DrawWg);
	colortab.wind = XtWindow(colortab.DrawWg);

	n = 0;
	XtSetArg(args[n], XmNbackground, &gc_init.background); n++;
	XtSetArg(args[n], XmNforeground, &gc_init.foreground); n++;
	XtGetValues(colortab.DrawWg, args, n);

	colortab.gc = XCreateGC(colortab.dpy, colortab.wind, GCForeground|GCBackground, &gc_init);
	colortab.AllocedColor = False;
}

void ShowColorCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	Arg args[1];
	char *name = NULL;
	XmString *newcolor = NULL;
	XColor color, exact;
	Status result;
	XGCValues gc_value;

	XtSetArg(args[0], XmNselectedItems, &newcolor);
	XtGetValues(colortab.ColorSelWg, args, 1);
	XmStringGetLtoR(*newcolor, XmSTRING_DEFAULT_CHARSET, &name);

	result = XAllocNamedColor(colortab.dpy, DefaultColormap(colortab.dpy, DefaultScreen(colortab.dpy)), name, &color, &exact);
	
	if(name) XtFree(name);
	
	if(result)
	{
		CheckFreeColor();	
		colortab.AllocedColor = True;
		XSetForeground(colortab.dpy, colortab.gc, color.pixel);	
		XFillRectangle(colortab.dpy, colortab.wind, colortab.gc, 0, 0, 100, 150);
	}
	else ErrorPopup("Can not Get Color!");
}

void OkColorTabCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	VIewportdata *viewport = active_viewport;
	Arg args[1];
	char *name = NULL;
	XmString *newcolor = NULL;
	XColor color, exact;
	XGCValues gc_value;
	Status result;
	int i;

	XtSetArg(args[0], XmNselectedItems, &newcolor);
	XtGetValues(colortab.ColorSelWg, args, 1);
	XmStringGetLtoR(*newcolor, XmSTRING_DEFAULT_CHARSET, &name);

	if(name)
	{
		result = XAllocNamedColor(viewport->resource.dpy, DefaultColormap(viewport->resource.dpy, DefaultScreen(viewport->resource.dpy)), name, &color, &exact);
	
		if(result)
		{
			viewport->changed = True;	
			viewport->autosavechanged = True;
	
			if(viewport->ColorName[colortab.ColorNum])
			{
				free(viewport->ColorName[colortab.ColorNum]);

				if(colortab.ColorNum != 0) XGetGCValues(viewport->resource.dpy, viewport->resource.gc[colortab.ColorNum - 1], GCForeground, &gc_value);
				else XGetGCValues(viewport->resource.dpy, viewport->resource.reverse_gc[0], GCForeground, &gc_value);

				XFreeColors(viewport->resource.dpy, 
					    DefaultColormap(viewport->resource.dpy, DefaultScreen(viewport->resource.dpy)),
					    &gc_value.foreground,
					    1, 0);
			}
			
			viewport->ColorName[colortab.ColorNum] = strdup(name);

			XtFree(name);
			
			switch(colortab.ColorNum) 
			{
				case 1:
				
				XSetForeground(viewport->resource.dpy, viewport->resource.dash_gc, color.pixel);
				case 2:
				case 3:
				case 4:
				case 5:
				case 6:
				case 7:
	
				XSetForeground(viewport->resource.dpy, viewport->resource.gc[colortab.ColorNum - 1], color.pixel);
				XSetBackground(viewport->resource.dpy, viewport->resource.reverse_gc[colortab.ColorNum - 1], color.pixel);
			        XtSetArg(args[0], XmNforeground, color.pixel);
			        XtSetValues(viewport->ColorBtns[colortab.ColorNum - 1], args, 1);
				XtSetValues(viewport->toolbar.Btns[colortab.ColorNum + 19], args, 1);
	
				break;
			
				case 0:

				for(i = 0; i < 7; i++)
				{
					XSetBackground(viewport->resource.dpy, viewport->resource.gc[i], color.pixel);
					XSetForeground(viewport->resource.dpy, viewport->resource.reverse_gc[i], color.pixel);
				}
	
				XSetBackground(viewport->resource.dpy, viewport->resource.dash_gc, color.pixel);

				XtSetArg(args[0], XmNbackground, color.pixel); 
				XtSetValues(viewport->resource.WorkArea, args, 1);
	
				return;
			}	
		}
		else 
		{
			XtFree(name);
			ErrorPopup("Can not Get Color!");
			return;
		}
	}

	MyDrawPage(viewport, EXPOSEDR);
}

void CancelColorTabCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	
	XtPopdown(colortab.PopUpWg);
}

void CheckFreeColor()
{
	XGCValues gc_value;

	if(colortab.AllocedColor)
	{
		XGetGCValues(colortab.dpy, colortab.gc, GCForeground, &gc_value);
		XFreeColors(colortab.dpy, DefaultColormap(colortab.dpy, DefaultScreen(colortab.dpy)),
		&gc_value.foreground, 1, 0);
	}

	colortab.AllocedColor = False;
}

void OrigColorCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	XGCValues gc_value;
	
	colortab.ColorNum = (int) client_data;

	CheckFreeColor();
	
	if(colortab.ColorNum != 0) XGetGCValues(active_viewport->resource.dpy, active_viewport->resource.gc[colortab.ColorNum - 1], GCForeground, &gc_value);
	else XGetGCValues(active_viewport->resource.dpy, active_viewport->resource.reverse_gc[0], GCForeground, &gc_value);

	XSetForeground(colortab.dpy, colortab.gc, gc_value.foreground);

	XFillRectangle(colortab.dpy, colortab.wind, colortab.gc, 0, 0, 100, 150);
}

	
void ColorTabPopupCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	XGCValues gc_value;
	
	active_viewport = (VIewportdata *)client_data;

	CheckFreeColor();

	if(colortab.ColorNum != 0) XGetGCValues(active_viewport->resource.dpy, active_viewport->resource.gc[colortab.ColorNum - 1], GCForeground, &gc_value);
	else XGetGCValues(active_viewport->resource.dpy, active_viewport->resource.reverse_gc[0], GCForeground, &gc_value);

	XSetForeground(colortab.dpy, colortab.gc, gc_value.foreground);

	PopupPosition(active_viewport->resource.PopUpShell, colortab.PopUpWg);
	XtPopup(colortab.PopUpWg, XtGrabExclusive);
}

void InitColorInfo(viewport)
VIewportdata *viewport;
{
	int i;

	for(i = 0; i < 8; i++)
	{
		if(GlobalDefaults.color[i]) viewport->ColorName[i] = strdup(GlobalDefaults.color[i]);
		else viewport->ColorName[i] = NULL;
	}
}

void ResetColorInfo(viewport)
VIewportdata *viewport;
{
	int i;
	XGCValues gc_value;

	for(i = 0; i < 8; i++)
	{
		if(viewport->ColorName[i]) 
		{
			free(viewport->ColorName[i]);
			viewport->ColorName[i] = NULL;
			if(i != 0) XGetGCValues(viewport->resource.dpy, viewport->resource.gc[i-1], GCForeground, &gc_value);
			else XGetGCValues(viewport->resource.dpy, viewport->resource.reverse_gc[0], GCForeground, &gc_value);
			XFreeColors(viewport->resource.dpy,
				    DefaultColormap(viewport->resource.dpy, DefaultScreen(viewport->resource.dpy)),
				    &gc_value.foreground,
				    1, 0);
		}
	}
	InitColorInfo(viewport);
}

void ReadSetColorInfo(viewport, cp)
VIewportdata *viewport;
char *cp;
{
	char i, *searchprt;
	XGCValues gc_value;

	for(searchprt = cp; *searchprt != '\n' && *searchprt != '\0'; searchprt++);

	*searchprt = '\0';

	while(*cp == '@' && *(cp+1) == '[')
	{
		cp += 2;
	
		for(i = 0; i < 8; i++)
		{
			if(*cp == '0' + i && *(cp+1) == ':')
			{
				cp += 2;
			
				for(searchprt = cp; *searchprt != '\0' && *searchprt != ']'; searchprt++);
				
				if(*searchprt == '\0') break;

				*searchprt = '\0';

				if(viewport->ColorName[i]) free(viewport->ColorName[i]);
				viewport->ColorName[i] = strdup(cp);
			
				cp = searchprt + 1;
	
				break;
			}
		}

		if(i == 8) break;
	}
}

void WriteColorInfoToFile(viewport, note)
VIewportdata *viewport;
char *note;
{
	int i, nochange = True;
	
	for(i = 0; i < 8; i++)
	{
		if(GlobalDefaults.color[i])
		{
			if(strcmp(GlobalDefaults.color[i], viewport->ColorName[i]) != 0) 
			{
				nochange = False; break;
			}	
		}
		else if(viewport->ColorName[i]) { nochange = False; break; }
	}

	if(nochange) return;
	
	fprintf(viewport->fp, "%s", note);
	
	for(i = 0; i < 8; i++) if(viewport->ColorName[i]) fprintf(viewport->fp, "@[%d:%s]", i, viewport->ColorName[i]);
	fprintf(viewport->fp, "\n");
}

