/*
 * Copyright 1995, 1996, 1997 by Jun Yei.       All Rights Reserved
 *
 * Permission to retain, use, modify, copy, and distribute ChinesePower
 * in source or binary and its documentation (hereafter, the Software)
 * for noncommercial purpose is hereby granted to you without a fee,
 * provided that this entire copyright and permission notice appear in
 * all such copies, that no charge be associated with such copies,
 * that distribution of derivative works (including value-added
 * distributions such as with additional input dictionaries or fonts)
 * include clarification that such added or derived parts are not from
 * the original Software, and that the names of the author(s) not be
 * used to endorse or promote such works. The author(s) of the software
 * makes no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * THE AUTHOR(S) OF THE SOFTWARE DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 * IN NO EVENT SHALL THE AUTHOR(S) OF THE SOFTWARE BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Jun Yei, Ottawa, ON, Canada
 */

#include "chpower.h"

char *convertTableName[]= {
"gb2big.t",
"gb2jis.t",
"gb2ks.t",
"big2gb.t",
"big2jis.t",
"big2ks.t",
"jis2gb.t",
"jis2big.t",
"jis2ks.t",
"ks2gb.t",
"ks2big.t",
"ks2jis.t",
"uni2gb.t",
"uni2big.t",
"uni2jis.t",
"uni2ks.t",
"gb2uni.t",
"big2uni.t",
"jis2uni.t",
"ks2uni.t"
};

void Convert(viewport, mode)
VIewportdata *viewport;
int mode;
{
	unsigned short *ptr, *newfont, *convert_table;
	char *oldfontId, *newfontId;
	int hilite, i, loc, hi, lo, memsize;
	PAgedata *curpage;
	FILE *fp;
	int filesize;

	switch(mode)
	{
		case BIG52GB:
		case JIS2GB:
		case KS2GB:
                case UNI2GB:

		if(SearchFontList(GlobalDefaults.gbfontname) == 0)
		{
			ErrorPopup("No default GB font!");
			return;
		}
		break;

		case GB2BIG5:
		case JIS2BIG5:
		case KS2BIG5:
                case UNI2BIG5:

		if(SearchFontList(GlobalDefaults.big5fontname) == 0)
		{
			ErrorPopup("No default Big5 font!");
			return;
		}
		break;

		case GB2JIS:
		case BIG52JIS:
		case KS2JIS:
                case UNI2JIS:

		if(SearchFontList(GlobalDefaults.jisfontname) == 0)
		{
			ErrorPopup("No default JIS font!");
			return;
		}
		break;

		case GB2KS:
		case BIG52KS:
		case JIS2KS:
                case UNI2KS:

		if(SearchFontList(GlobalDefaults.ksfontname) == 0)
		{
			ErrorPopup("No default KS font!");
			return;
		}
		break;

                case GB2UNI:
                case BIG52UNI:
                case JIS2UNI:
                case KS2UNI:

                if(SearchFontList(GlobalDefaults.unifontname) == 0)
		{
                        ErrorPopup("No default Unicode font!");
			return;
		}
		break;

	}

	switch(mode)
	{
		case GB2KS:
		case JIS2KS:
		case KS2GB:
		case KS2BIG5:
		case KS2JIS:
		case GB2BIG5:
		case GB2JIS:
		case JIS2GB:
		case JIS2BIG5:
                case GB2UNI:
                case JIS2UNI:
                case KS2UNI:

		memsize = (0xf7 - 0xa0) * (0xfe - 0xa0);
		break;

		case BIG52GB:
		case BIG52JIS:
		case BIG52KS:
                case BIG52UNI:

		memsize = (0xfe - 0xa0) * ((0xfe - 0xa0) + (0x7f - 0x40));
		break;

                case UNI2GB:
                case UNI2BIG5:
                case UNI2JIS:
                case UNI2KS:

                memsize = 0xffff;
                break;
	}

	fp = My_path_open(getenv(CONF_FILE_PATH), convertTableName[mode], "r");


	if(!fp)
	{
		ErrorPopup("Can not open Convert Table!");
		return;
	}

	convert_table = (unsigned short *) malloc(memsize*sizeof(unsigned short));

	if(!convert_table)
	{
		ErrorPopup("Error: Can not locate memory!");
		fclose(fp);
		return;
	}

	fseek(fp, 0, 2);
	filesize = (int) ftell(fp)/2;
	rewind(fp);

	U2fileRead(fp, convert_table);
	fclose(fp);

	if(filesize < memsize)
	{
		for(ptr = convert_table + filesize; filesize < memsize; filesize++, *ptr++ = 0xa1a1);
	} 

	viewport->cursor.RedrawCurPtr = viewport->cursor.StartCurPtr;

	if(viewport->cursor.StartCurPtr == viewport->cursor.EndCurPtr)
	{
		hilite = 0;
		
		oldfontId = viewport->cursor.hzdata.FontId;

		viewport->cursor.StartCurPtr = viewport->EditFile;

		for(curpage = viewport->page.PageRoot; curpage->Next != NULL; curpage = curpage->Next);

		viewport->cursor.EndCurPtr = curpage->PageStart + DBlen(curpage->PageStart);
	}
	else if(viewport->cursor.CurPtr == viewport->cursor.StartCurPtr)
	{
		hilite = 1;
		oldfontId = viewport->cursor.Mohzdata.FontId;
	}
	else
	{
		hilite = 2;
		oldfontId = viewport->cursor.hzdata.FontId;
	}

	for(ptr = viewport->cursor.StartCurPtr; ptr < viewport->cursor.EndCurPtr; ptr++)
	{
		if(!ISHZ(*ptr)) continue;

		lo = *ptr & 0xff;
		hi = *ptr >> 8;

		switch(mode)
		{
			case GB2BIG5:
                	case GB2JIS:
			case GB2KS:
                	case JIS2GB:
                	case JIS2BIG5:
			case JIS2KS:
			case KS2GB:
			case KS2BIG5:
			case KS2JIS:
                        case GB2UNI:
                        case JIS2UNI:
                        case KS2UNI:

			loc = (hi - 0xa1) * (0xfe - 0xa0) + (lo - 0xa1);
			break;

			case BIG52GB:
	                case BIG52JIS:
			case BIG52KS:
                        case BIG52UNI:

			loc = (hi - 0xa1) * ((0xfe - 0xa0) + (0x7f - 0x40));

			if(lo < 0xa0) loc += lo - 0x40;
			else loc += (lo  - 0xa1 ) + (0x7f - 0x40);
			break;

                        case UNI2GB:
                        case UNI2BIG5:
                        case UNI2JIS:
                        case UNI2KS:

                        loc = *ptr;
                        break;
		}
		*ptr = convert_table[loc];
	}

	free(convert_table);

	ptr = viewport->cursor.CurPtr;
	viewport->cursor.CurPtr = viewport->cursor.StartCurPtr;

	switch(mode)
	{
		case BIG52GB:
		case JIS2GB:
                case KS2GB:
                case UNI2GB:

		newfont = (unsigned short *) calloc(4 + strlen(GlobalDefaults.gbfontname), sizeof(unsigned short));
		*newfont = 0;
		DBstrcat(newfont, "@[");
		DBstrcat(newfont, GlobalDefaults.gbfontname);
		DBstrcat(newfont, "]");
		newfontId = GetFontId(SearchFontList(GlobalDefaults.gbfontname));
		break;

		case GB2BIG5:
		case JIS2BIG5:
                case KS2BIG5:
                case UNI2BIG5:

		newfont = (unsigned short *) calloc(4 + strlen(GlobalDefaults.big5fontname), sizeof(unsigned short));
		*newfont = 0;
		DBstrcat(newfont, "@[");
		DBstrcat(newfont, GlobalDefaults.big5fontname);
		DBstrcat(newfont, "]");
		newfontId = GetFontId(SearchFontList(GlobalDefaults.big5fontname));
		break;

		case GB2JIS:
		case BIG52JIS:
                case KS2JIS:
                case UNI2JIS:

		newfont = (unsigned short *) calloc(4 + strlen(GlobalDefaults.jisfontname), sizeof(unsigned short));
		*newfont = 0;
		DBstrcat(newfont, "@[");
		DBstrcat(newfont, GlobalDefaults.jisfontname);
		DBstrcat(newfont, "]");
		newfontId = GetFontId(SearchFontList(GlobalDefaults.jisfontname));
		break;

                case GB2KS:
                case BIG52KS:
                case JIS2KS:
                case UNI2KS:

                newfont = (unsigned short *) calloc(4 + strlen(GlobalDefaults.ksfontname), sizeof(unsigned short));
		*newfont = 0;
		DBstrcat(newfont, "@[");
                DBstrcat(newfont, GlobalDefaults.ksfontname);
		DBstrcat(newfont, "]");
                newfontId = GetFontId(SearchFontList(GlobalDefaults.ksfontname));
		break;

                case GB2UNI:
                case BIG52UNI:
                case JIS2UNI:
                case KS2UNI:

                newfont = (unsigned short *) calloc(4 + strlen(GlobalDefaults.unifontname), sizeof(unsigned short));
		*newfont = 0;
		DBstrcat(newfont, "@[");
                DBstrcat(newfont, GlobalDefaults.unifontname);
		DBstrcat(newfont, "]");
                newfontId = GetFontId(SearchFontList(GlobalDefaults.unifontname));
		break;
	}

	Input(viewport, CHARSETINS1, newfont, DBlen(newfont));

	free(newfont);

	viewport->cursor.EndCurPtr += DBlen(newfont);
	viewport->cursor.StartCurPtr = viewport->set.TmpEPtr;

	i = rmtypedcomponent(viewport->set.TmpEPtr, viewport->cursor.EndCurPtr, FontMask);

	viewport->cursor.EndCurPtr -= i;

	viewport->cursor.CurPtr =  viewport->cursor.EndCurPtr;

	newfont = (unsigned short *) calloc(4 + strlen(oldfontId), sizeof(unsigned short));
	*newfont = 0;
	DBstrcat(newfont, "@[");
	DBstrcat(newfont, oldfontId);
	DBstrcat(newfont, "]");

	Input(viewport, CHARSETINS1, newfont, DBlen(newfont));
	free(newfont);

	viewport->cursor.EndCurPtr = viewport->cursor.CurPtr;

	switch(hilite)
	{
		case 1:
		MyDrawPage(viewport, INVIRDRAW);
		viewport->cursor.CurPtr = viewport->cursor.StartCurPtr;
		viewport->cursor.MoveCurPtr = viewport->cursor.EndCurPtr;
		viewport->cursor.hzdata.FontId = newfontId;
		MyDrawPage(viewport, ONLYHILITE);
		break;

		case 2:
		MyDrawPage(viewport, INVIRDRAW);
		viewport->cursor.CurPtr = viewport->cursor.EndCurPtr;
		viewport->cursor.MoveCurPtr = viewport->cursor.StartCurPtr;
		viewport->cursor.Mohzdata.FontId = newfontId;
		MyDrawPage(viewport, ONLYHILITE);
		break;

		default:
		MyDrawPage(viewport, VIRDRAW);
		viewport->cursor.CurPtr = ptr;
		MyDrawPage(viewport, REVCURSEARCH);
		viewport->cursor.StartCurPtr = viewport->cursor.EndCurPtr = viewport->cursor.CurPtr;
		MyDrawPage(active_viewport, CLEARDRAW);
	}
}

