/*
 * Copyright 1995, 1996, 1997 by Jun Yei.       All Rights Reserved
 *
 * Permission to retain, use, modify, copy, and distribute ChinesePower
 * in source or binary and its documentation (hereafter, the Software)
 * for noncommercial purpose is hereby granted to you without a fee,
 * provided that this entire copyright and permission notice appear in
 * all such copies, that no charge be associated with such copies,
 * that distribution of derivative works (including value-added
 * distributions such as with additional input dictionaries or fonts)
 * include clarification that such added or derived parts are not from
 * the original Software, and that the names of the author(s) not be
 * used to endorse or promote such works. The author(s) of the software
 * makes no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * THE AUTHOR(S) OF THE SOFTWARE DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 * IN NO EVENT SHALL THE AUTHOR(S) OF THE SOFTWARE BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Jun Yei, Ottawa, ON, Canada
 */

#include "chpower.h"

void DBfileWrite(fp, blkprt)
FILE *fp;
unsigned short *blkprt;
{
	unsigned short tmp;

	while(*blkprt != 0)
	{
		SKIPIDS(blkprt);

		if(ISHZ(*blkprt))
		{
			tmp = *blkprt >> 8;
			fputc((unsigned char) tmp, fp);
			fputc((unsigned char) *blkprt, fp);
		}
		else
		{
			fputc((unsigned char) *blkprt, fp);
		}
		blkprt++;
	}
}

long DBfileSize(fp)
FILE *fp;
{
	long filesize = 0;
	int sample;

	while((sample = getc(fp)) != EOF)
	{
		filesize++;
		
		if(ISACHZ(sample))
		{
			sample = getc(fp);
			if(sample == EOF) break;
		}
	}
	return(filesize);
}

void DBfileRead(fp, tp)
FILE *fp;
unsigned short *tp;
{
	int sample;

	while((sample = getc(fp)) != EOF)
	{
		*tp = sample;

		if(ISACHZ(sample))
		{
			sample = getc(fp);
			if(sample == EOF) break;
			*tp <<= 8;
			*tp |= sample;
		}

		tp++;
	}
	*tp = 0;	
}

int DBlen(cp)
unsigned short *cp;
{
	int count = 0;
	
	for(; *cp != 0; cp++) count++;
	
	return(count);
}

int DBbytelen(cp)
unsigned short *cp;
{
	int count = 0;
	for(; *cp != 0; cp++) 
		if(ISHZ(*cp)) count += 2;
		else	count ++;
	return(count);
}

int DBbytenlen(cp, num)
unsigned short *cp;
int num;
{
	int i, count = 0;

	for(i = 0; i < num; i++, cp++)
		if(ISHZ(*cp)) count += 2;
		else    count ++;
	return(count);
}

int byteDBlen(cp)
unsigned char *cp;
{
	int count = 0;
	
	for(; *cp != '\0'; cp++)
	{	count++; if(ISACHZ(*cp)) cp++; }
	
	return(count);
}

unsigned short *DBdup(cp)
unsigned short *cp;
{
	unsigned short *tp;

	tp = (unsigned short *) malloc((DBlen(cp)+1)*sizeof(unsigned short));

	*tp = 0;

	DBcat(tp, cp);
	
	return(tp);
}

void DBcat(target, copy)
unsigned short *target, *copy;
{
	while(*target != 0) target++;

	while(*copy != 0)
	{
		*target++ = *copy++;
	}
	
	*target = 0;
}

void DBstrcat(target, copy)
unsigned short *target;
unsigned char *copy;
{
	while(*target != 0) target++;

	while(*copy != '\0')
	{
		*target = *copy;
	
		if(ISACHZ(*copy)) 
		{
			copy ++;
			*target <<= 8;	
			*target |= *copy;
		}
		target++;	
		copy++;	
	}

	*target = 0;
}

unsigned short *strDBdup(copy)
unsigned char *copy;
{
	unsigned short *tp;

	tp = (unsigned short *) malloc((byteDBlen(copy) + 1) * sizeof(unsigned short));

	*tp = 0;
	
	DBstrcat(tp, copy);

	return(tp);
}

unsigned char *DBstrdup(copy)
unsigned short *copy;
{
	unsigned char *tp, *tmp;

	tmp = tp = (unsigned char *) malloc((DBbytelen(copy) + 1));

	while(*copy)
	{
		if(ISHZ(*copy))
		{
			*tmp++ = 0x00ff & (*copy >> 8);
		}
	
		*tmp++ = (char) *copy;

		copy++;
	}

	*tmp = '\0';

	return(tp);
}

unsigned char *strDBcpy(target, copy, n)
unsigned char *target; 
unsigned short *copy;
int n;
{
	int i;

	for(i=0; i < n; i++)
	{
		*target++ = *copy++;
	}

	*target = 0;

	return(target);
}

unsigned short *DBcpy(target, copy, n)
unsigned short *target, *copy;
int n;
{
	int i;

	for(i=0; i < n; i++)
	{
		*target++ = *copy++;
	}
	
	*target = 0;
	
	return(target);
}

