/*
 * Copyright 1995, 1996, 1997 by Jun Yei.       All Rights Reserved
 *
 * Permission to retain, use, modify, copy, and distribute ChinesePower
 * in source or binary and its documentation (hereafter, the Software)
 * for noncommercial purpose is hereby granted to you without a fee,
 * provided that this entire copyright and permission notice appear in
 * all such copies, that no charge be associated with such copies,
 * that distribution of derivative works (including value-added
 * distributions such as with additional input dictionaries or fonts)
 * include clarification that such added or derived parts are not from
 * the original Software, and that the names of the author(s) not be
 * used to endorse or promote such works. The author(s) of the software
 * makes no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * THE AUTHOR(S) OF THE SOFTWARE DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 * IN NO EVENT SHALL THE AUTHOR(S) OF THE SOFTWARE BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Jun Yei, Ottawa, ON, Canada
 */

#include 	"chpower.h"
#include	"default.h"

MYdefault GlobalDefaults = { NULL, DEFAULT_FONT_SIZE, DEFAULT_FONT_SIZE, DEFAULT_PAGE_MARGIN_W, DEFAULT_PAGE_MARGIN_H, LETTERPAGESETID, DEFAULT_LINE_SPACING,
DEFAULT_CHAR_SPACING, DEFAULT_COL_SPACING, DEFAULT_ZOOM, True, PAGE_NUM_WIDTH, PAGE_NUM_HEIGHT, { NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL }, { NULL, NULL,
NULL, NULL, NULL}, {0xa3, 0xac}, {0xa1, 0xa3}, {0xa1, 0xa1}, NULL, NULL, NULL, NULL, NULL, 
76, True, False, 10, 600, 800, 5, False, False};

void DestroyDefaults()
{
	int i;

	if(GlobalDefaults.gbfontname) free(GlobalDefaults.gbfontname);
	if(GlobalDefaults.big5fontname) free(GlobalDefaults.big5fontname);
	if(GlobalDefaults.jisfontname) free(GlobalDefaults.jisfontname);
	if(GlobalDefaults.ksfontname) free(GlobalDefaults.ksfontname);
	if(GlobalDefaults.unifontname) free(GlobalDefaults.unifontname);

	for(i = 0; i < 8; i++)
	{
		if(GlobalDefaults.color[i]) free(GlobalDefaults.color[i]);
	}

	for(i = 0; i < 5; i++)
	{
		if(GlobalDefaults.topBtnColor[i]) free(GlobalDefaults.topBtnColor[i]);
	}
}

void ClearUpDefaults()
{
	int i;
	
	for(i = 2; i < 8; i++)
	{
		if(!GlobalDefaults.color[i]) GlobalDefaults.color[i] = strdup(Default_Colors[i-2]);
	}

	for(i = 0; i < 5; i++)
	{
		if(!GlobalDefaults.topBtnColor[i]) GlobalDefaults.topBtnColor[i] = strdup(Default_TopBtnColors[i]);
	}
}

void InitDefault(cp)
char *cp;
{
	char *tmp;
	int i;

	for(tmp = cp; *tmp != '\0' && *tmp != '\n'; tmp++);
	*tmp = '\0';

	if(strncmp(GB_FONT_FLAG, cp, strlen(GB_FONT_FLAG)) == 0)
	{
		cp += strlen(GB_FONT_FLAG);
		GlobalDefaults.gbfontname = strdup(cp);
	}
	else if(strncmp(BIG5_FONT_FLAG, cp, strlen(BIG5_FONT_FLAG)) == 0)
	{
		cp += strlen(BIG5_FONT_FLAG);
		GlobalDefaults.big5fontname = strdup(cp);
	}
	else if(strncmp(JIS_FONT_FLAG, cp, strlen(JIS_FONT_FLAG)) == 0)
	{
		cp += strlen(JIS_FONT_FLAG);
		GlobalDefaults.jisfontname = strdup(cp);
	}
	else if(strncmp(KS_FONT_FLAG, cp, strlen(KS_FONT_FLAG)) == 0)
	{
		cp += strlen(KS_FONT_FLAG);
		GlobalDefaults.ksfontname = strdup(cp);
	}
        else if(strncmp(UNI_FONT_FLAG, cp, strlen(UNI_FONT_FLAG)) == 0)
        {
                cp += strlen(UNI_FONT_FLAG);
                GlobalDefaults.unifontname = strdup(cp);
        }
	else if(strncmp(FONT_WIDTH_FLAG, cp, strlen(FONT_WIDTH_FLAG)) == 0)
	{
		cp += strlen(FONT_WIDTH_FLAG);
		sscanf(cp, "%d", &GlobalDefaults.fontwidth);
	}
	else if(strncmp(FONT_HEIGHT_FLAG, cp, strlen(FONT_HEIGHT_FLAG)) == 0)
	{
       		cp += strlen(FONT_HEIGHT_FLAG);
		sscanf(cp, "%d", &GlobalDefaults.fontheight);
	}
	else if(strncmp(MARGIN_WIDTH_FLAG, cp, strlen(MARGIN_WIDTH_FLAG)) == 0)
	{
		cp += strlen(MARGIN_WIDTH_FLAG);
		sscanf(cp, "%d", &GlobalDefaults.margin_width);
	}
	else if(strncmp(MARGIN_HEIGHT_FLAG, cp, strlen(MARGIN_HEIGHT_FLAG)) == 0)
	{
		cp += strlen(MARGIN_HEIGHT_FLAG);
		sscanf(cp, "%d", &GlobalDefaults.margin_height);
	}
	else if(strncmp(HZ_FILE_WIDTH_FLAG, cp, strlen(HZ_FILE_WIDTH_FLAG)) == 0)
	{
		cp += strlen(HZ_FILE_WIDTH_FLAG);
		sscanf(cp, "%d", &GlobalDefaults.hz_file_width);
	}
	else if(strncmp(PAGE_TYPE_FLAG, cp, strlen(PAGE_TYPE_FLAG)) == 0)
	{
		cp += strlen(PAGE_TYPE_FLAG);

		for(i = 0; i < NUMOFPAGESET; i++)
		{
			if(strcmp(cp, PageInfo[i]) == 0) 
			{
				GlobalDefaults.page_type = i;
				return;
			}
		}
	}
	else if(strncmp(LINE_SPACING_FLAG, cp, strlen(LINE_SPACING_FLAG)) == 0)
	{
		cp += strlen(LINE_SPACING_FLAG);
		sscanf(cp, "%d", &GlobalDefaults.line_spacing);
	}
	else if(strncmp(CHAR_SPACING_FLAG, cp, strlen(CHAR_SPACING_FLAG)) == 0)
	{
		cp += strlen(CHAR_SPACING_FLAG);
		sscanf(cp, "%d", &GlobalDefaults.char_spacing);
	}
	else if(strncmp(COL_SPACING_FLAG, cp, strlen(COL_SPACING_FLAG)) == 0)
	{
		cp += strlen(COL_SPACING_FLAG);
		sscanf(cp, "%d", &GlobalDefaults.column_spacing);
	}
	else if(strncmp(ZOOM_FLAG, cp, strlen(ZOOM_FLAG)) == 0)
	{
		cp += strlen(ZOOM_FLAG);
		sscanf(cp, "%d", &GlobalDefaults.init_zoom);
	}
	else if(strncmp(PAGE_OR_FLAG, cp, strlen(PAGE_OR_FLAG)) == 0)
	{
		cp += strlen(PAGE_OR_FLAG);
		if(strncmp(cp, LANDSCAPE, strlen(LANDSCAPE)) == 0) GlobalDefaults.page_orien = False;
	}
	else if(strncmp(PAGE_NUM_WIDTH_FLAG, cp, strlen(PAGE_NUM_WIDTH_FLAG)) == 0)
	{
		cp += strlen(PAGE_NUM_WIDTH_FLAG);
		sscanf(cp, "%d", &GlobalDefaults.pagenum_width);
	}
	else if(strncmp(PAGE_NUM_HEIGHT_FLAG, cp, strlen(PAGE_NUM_HEIGHT_FLAG)) == 0)
	{
		cp += strlen(PAGE_NUM_HEIGHT_FLAG);
		sscanf(cp, "%d", &GlobalDefaults.pagenum_height);
	}
	else if(strncmp(LINE_ORIENTATION_FLAG, cp, strlen(LINE_ORIENTATION_FLAG)) == 0)
        {
                cp += strlen(LINE_ORIENTATION_FLAG);
                if(strncmp(cp, VERTICAL, strlen(VERTICAL)) == 0) GlobalDefaults.horOrder = False;
        }
	else if(strncmp(FOREGROUND_FLAG, cp, strlen(FOREGROUND_FLAG)) == 0)
	{
		cp += strlen(FOREGROUND_FLAG);
		GlobalDefaults.color[1] = strdup(cp);
	}
	else if(strncmp(BACKGROUND_FLAG, cp, strlen(BACKGROUND_FLAG)) == 0)
	{
		cp += strlen(BACKGROUND_FLAG);
		GlobalDefaults.color[0] = strdup(cp);
	}
	else if(strncmp(COLOR_1_FLAG, cp, strlen(COLOR_1_FLAG)) == 0)
	{
		cp += strlen(COLOR_1_FLAG);
		GlobalDefaults.color[2] = strdup(cp);
	}
	else if(strncmp(COLOR_2_FLAG, cp, strlen(COLOR_2_FLAG)) == 0)
	{
		cp += strlen(COLOR_2_FLAG);    
		GlobalDefaults.color[3] = strdup(cp);
	}
	else if(strncmp(COLOR_3_FLAG, cp, strlen(COLOR_3_FLAG)) == 0)
	{
		cp += strlen(COLOR_3_FLAG);    
		GlobalDefaults.color[4] = strdup(cp);
	}
	else if(strncmp(COLOR_4_FLAG, cp, strlen(COLOR_4_FLAG)) == 0)
	{
		cp += strlen(COLOR_4_FLAG);    
		GlobalDefaults.color[5] = strdup(cp);
	}
	else if(strncmp(COLOR_5_FLAG, cp, strlen(COLOR_5_FLAG)) == 0)
	{
		cp += strlen(COLOR_5_FLAG);    
		GlobalDefaults.color[6] = strdup(cp);
	}
	else if(strncmp(COLOR_6_FLAG, cp, strlen(COLOR_6_FLAG)) == 0)
	{
		cp += strlen(COLOR_6_FLAG);    
		GlobalDefaults.color[7] = strdup(cp);
	}
	else if(strncmp(BTNCOLOR_1_FLAG, cp, strlen(BTNCOLOR_1_FLAG)) == 0)
	{
		cp += strlen(BTNCOLOR_1_FLAG);
		GlobalDefaults.topBtnColor[0] = strdup(cp);
	}
	else if(strncmp(BTNCOLOR_2_FLAG, cp, strlen(BTNCOLOR_2_FLAG)) == 0)
	{
		cp += strlen(BTNCOLOR_2_FLAG);
		GlobalDefaults.topBtnColor[1] = strdup(cp);
	}
	else if(strncmp(BTNCOLOR_3_FLAG, cp, strlen(BTNCOLOR_3_FLAG)) == 0)
	{
		cp += strlen(BTNCOLOR_3_FLAG);
		GlobalDefaults.topBtnColor[2] = strdup(cp);
	}
	else if(strncmp(BTNCOLOR_4_FLAG, cp, strlen(BTNCOLOR_4_FLAG)) == 0)
	{
		cp += strlen(BTNCOLOR_4_FLAG);
		GlobalDefaults.topBtnColor[3] = strdup(cp);
	}
	else if(strncmp(BTNCOLOR_5_FLAG, cp, strlen(BTNCOLOR_5_FLAG)) == 0)
	{
		cp += strlen(BTNCOLOR_5_FLAG);
		GlobalDefaults.topBtnColor[4] = strdup(cp);
	}
	else if(strncmp(ENCODING_FLAG, cp, strlen(ENCODING_FLAG)) == 0)
	{
		cp += strlen(ENCODING_FLAG);
		MyHzInMod.InputWin.misc.hz_encoding = strdup(cp);
	}
	else if(strncmp(AUTO_SAVE_FLAG, cp, strlen(AUTO_SAVE_FLAG)) == 0)
	{
		cp += strlen(AUTO_SAVE_FLAG);
		if(strncmp(ON, cp, strlen(ON)) == 0) GlobalDefaults.timer_on = True;
	}
	else if(strncmp(AUTO_SAVE_INTERVAL_FLAG, cp, strlen(AUTO_SAVE_INTERVAL_FLAG)) == 0)
	{
		cp += strlen(AUTO_SAVE_INTERVAL_FLAG);
		sscanf(cp, "%d", &GlobalDefaults.timer_interval);
	}
	else if(strncmp(EDIT_WIN_WIDTH_FLAG, cp, strlen(EDIT_WIN_WIDTH_FLAG)) == 0)
	{
		cp += strlen(EDIT_WIN_WIDTH_FLAG);
		sscanf(cp, "%d", &GlobalDefaults.win_width);
	}
	else if(strncmp(EDIT_WIN_HEIGHT_FLAG, cp, strlen(EDIT_WIN_HEIGHT_FLAG)) == 0)
	{
		cp += strlen(EDIT_WIN_HEIGHT_FLAG);
		sscanf(cp, "%d", &GlobalDefaults.win_height);
	}
	else if(strncmp(EDIT_WIN_SELEC_FLAG, cp, strlen(EDIT_WIN_SELEC_FLAG)) == 0)
	{
		cp += strlen(EDIT_WIN_SELEC_FLAG);
		if(strncmp(ASIS_FIT, cp, strlen(ASIS_FIT)) == 0) GlobalDefaults.win_selec = ASIS_WIN;
		else if(strncmp(WINDOW_FIT_TO_MARGIN, cp, strlen(WINDOW_FIT_TO_MARGIN)) == 0) GlobalDefaults.win_selec = FIT_TO_MARGIN_WIN;
		else if(strncmp(WINDOW_FIT_TO_DATA, cp, strlen(WINDOW_FIT_TO_DATA)) == 0) GlobalDefaults.win_selec = FIT_TO_DATA_WIN;
		else if(strncmp(WINDOW_FIT_TO_WIDTH, cp, strlen(WINDOW_FIT_TO_WIDTH)) == 0) GlobalDefaults.win_selec = FIT_TO_WIDTH_WIN;
		else if(strncmp(WINDOW_FIT_TO_PAGE, cp, strlen(WINDOW_FIT_TO_PAGE)) == 0) GlobalDefaults.win_selec = FIT_TO_PAGE_WIN;
		else if(strncmp(PAGE_FIT_TO_WINDOW, cp, strlen(PAGE_FIT_TO_WINDOW)) == 0) GlobalDefaults.win_selec = FIT_TO_WIN_PAGE;
	}
	else if(strncmp(USER_PAGE_WIDTH_FLAG, cp, strlen(USER_PAGE_WIDTH_FLAG)) == 0)
	{
		cp += strlen(USER_PAGE_WIDTH_FLAG);
		sscanf(cp, "%d", &PageData[30][0]);
	}
	else if(strncmp(USER_PAGE_HEIGHT_FLAG, cp, strlen(USER_PAGE_HEIGHT_FLAG)) == 0)
	{
		cp += strlen(USER_PAGE_HEIGHT_FLAG);
		sscanf(cp, "%d", &PageData[30][1]);
	}
	else if(strncmp(TTF_AS_BITMAP_FLAG, cp, strlen(TTF_AS_BITMAP_FLAG)) == 0)
        {
                cp += strlen(TTF_AS_BITMAP_FLAG);
                if(strncmp(ON, cp, strlen(ON)) == 0) GlobalDefaults.ttf_as_bitmap = True;
        } 
	else if(strncmp(HZ_ENTER_ESCAPE_FLAG, cp, strlen(HZ_ENTER_ESCAPE_FLAG)) == 0)
	{
		cp += strlen(HZ_ENTER_ESCAPE_FLAG);
		if(strncmp(ON, cp, strlen(ON)) == 0) GlobalDefaults.hz_enter_escape = True;
	}
}

