/*
 * Copyright 1995, 1996, 1997 by Jun Yei.       All Rights Reserved
 *
 * Permission to retain, use, modify, copy, and distribute ChinesePower
 * in source or binary and its documentation (hereafter, the Software)
 * for noncommercial purpose is hereby granted to you without a fee,
 * provided that this entire copyright and permission notice appear in
 * all such copies, that no charge be associated with such copies,
 * that distribution of derivative works (including value-added
 * distributions such as with additional input dictionaries or fonts)
 * include clarification that such added or derived parts are not from
 * the original Software, and that the names of the author(s) not be
 * used to endorse or promote such works. The author(s) of the software
 * makes no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * THE AUTHOR(S) OF THE SOFTWARE DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 * IN NO EVENT SHALL THE AUTHOR(S) OF THE SOFTWARE BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Jun Yei, Ottawa, ON, Canada
 */

#include "chpower.h"

void CreateHeaderTrailer();

void UndoCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	VIewportdata *viewport = (VIewportdata *)client_data;
	unsigned short *tmp;

	if(viewport->undo.lastmode == UDNONE) 
	{
		ErrorPopup("Nothing to Undo!");
		return;
	}

	if(viewport->cursor.StartCurPtr != viewport->cursor.EndCurPtr) MyDrawPage(viewport, UNHIGHLITE);

	switch(viewport->undo.lastmode)
	{
		case UDINPUT:
		case UDBOTH:

		viewport->cursor.CurPtr = viewport->undo.InsertPtr;
		viewport->cursor.MoveCurPtr = viewport->undo.InsertPtr + viewport->undo.numofinput;
		viewport->cursor.StartCurPtr = viewport->cursor.CurPtr;
		viewport->cursor.EndCurPtr = viewport->cursor.MoveCurPtr;
		viewport->cursor.Mohzdata = viewport->cursor.hzdata;
		viewport->cursor.MoveColNum = viewport->cursor.ColNum;
		viewport->cursor.MoveLineAlignment = viewport->cursor.LineAlignment;
		viewport->cursor.MoveColorNum = viewport->cursor.ColorNum;
		viewport->cursor.MoveUnderLine = viewport->cursor.UnderLine;
	
		if(viewport->undo.lastmode == UDINPUT)
		{	
			MyDrawPage(viewport, Input(viewport, KEYINPUT, delID, 1));
			break;
		}
		
		tmp = (unsigned short*)DBdup(viewport->undo.ContentPtr);
		MyDrawPage(viewport, Input(viewport, KEYINPUT, tmp, DBlen(tmp)));
		free(tmp);
		break;

		case UDDELETE:
		
		tmp = (unsigned short*)DBdup(viewport->undo.ContentPtr);	
		viewport->cursor.StartCurPtr = viewport->cursor.EndCurPtr = viewport->cursor.MoveCurPtr = viewport->cursor.CurPtr = viewport->undo.InsertPtr;	
		MyDrawPage(viewport, Input(viewport, KEYINPUT, tmp, DBlen(tmp)));
		free(tmp);	
	}
}

void DeleteCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	VIewportdata *viewport = (VIewportdata *)client_data;

	if(viewport->cursor.StartCurPtr != viewport->cursor.EndCurPtr)
	{
		MyDrawPage(viewport, Input(viewport, KEYINPUT, delID, 1));
	}
	else
	{
		ErrorPopup("Make Selection First!");
	}
}

void PasteCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	VIewportdata *viewport = (VIewportdata *)client_data;
	unsigned short header[40], trailer[40], *InputPtr = NULL;
	unsigned short *head, *tail;
	int i;

	if(MyHzInMod.input.buffer != NULL)
	{
		if(viewport->cursor.StartCurPtr != viewport->cursor.EndCurPtr)
		{
			MyDrawPage(viewport, Input(viewport, KEYINPUT, delID, 1));
		}
		
		CreateHeaderTrailer(header, trailer, &viewport->cursor.hzdata, viewport->cursor.ColNum, viewport->cursor.LineAlignment,
		viewport->cursor.ColorNum, viewport->cursor.UnderLine);
	
		InputPtr = (unsigned short *) malloc((DBlen(header) + DBlen(MyHzInMod.input.buffer) + DBlen(trailer) + 1)*sizeof(unsigned short));
		InputPtr[0] = '\0';

		DBcat(InputPtr, header);
		DBcat(InputPtr, MyHzInMod.input.buffer);
		DBcat(InputPtr, trailer);
		
		i = DBlen(InputPtr);
		head = viewport->cursor.CurPtr;
		tail = head + i;
		
		i = Input(viewport, KEYINPUT, InputPtr, i);
		viewport->cursor.newcurptr -= rmanysamecomponent(tail, WHOLESEARCH); 
		viewport->cursor.newcurptr -= rmanysamecomponent(head, WHOLESEARCH);
		MyDrawPage(viewport, i);

		free(InputPtr);	
	}
	else
	{
		ErrorPopup("Nothing to Paste!");
	}	
}

void CreateHeaderTrailer(header, trailer, hzdata, colnum, align, colrnum, underline)
unsigned short *header, *trailer;
HZdata *hzdata;
int colnum, align, colrnum, underline;
{
	char line[20];
	
	*header = '\0'; *trailer = '\0';
	
	if(strcmp(MyHzInMod.input.FontId, hzdata->FontId) != 0)
	{
		DBstrcat(header, "@[");
		DBstrcat(header, MyHzInMod.input.FontId);
		DBstrcat(header, "]");
	}
	
	if(strcmp(MyHzInMod.input.EndFontId, hzdata->FontId) != 0)
	{
		DBstrcat(trailer, "@[");
		DBstrcat(trailer, hzdata->FontId);
		DBstrcat(trailer, "]");
	}

	if(MyHzInMod.input.point_width != hzdata->point_width || MyHzInMod.input.point_height != hzdata->point_height)
	{
		if(MyHzInMod.input.point_width != MyHzInMod.input.point_height)
			sprintf(line, "@[%d,%d]", MyHzInMod.input.point_width/100, MyHzInMod.input.point_height/100);
		else
			sprintf(line, "@[%d]", MyHzInMod.input.point_width/100);

		DBstrcat(header, line);
	}

	if(MyHzInMod.input.Endpoint_width != hzdata->point_width || MyHzInMod.input.Endpoint_height != hzdata->point_height)
	{
		if(hzdata->point_width != hzdata->point_height)
			sprintf(line, "@[%d,%d]", hzdata->point_width/100, hzdata->point_height/100);
		else
			sprintf(line, "@[%d]", hzdata->point_width/100);

		DBstrcat(trailer, line);   
	}

	if(MyHzInMod.input.ColNum != colnum)
	{
		sprintf(line, "@[C%d]", MyHzInMod.input.ColNum);
		DBstrcat(header, line);   
	}

	if(MyHzInMod.input.EndColNum != colnum)   
	{
		sprintf(line, "@[C%d]", colnum);
		DBstrcat(trailer, line);   
	}

	if(MyHzInMod.input.LineAlignment != align)
	{
		sprintf(line, "@[A%d]", MyHzInMod.input.LineAlignment);
		DBstrcat(header, line);   
	}

	if(MyHzInMod.input.EndLineAlignment != align)
	{
		sprintf(line, "@[A%d]", align);
		DBstrcat(trailer, line);   
	}

	if(MyHzInMod.input.ColorNum != colrnum)
	{
		sprintf(line, "@[Z%d]", MyHzInMod.input.ColorNum);
		DBstrcat(header, line);  
	}

	if(MyHzInMod.input.EndColorNum != colrnum)
	{
		sprintf(line, "@[Z%d]", colrnum);
		DBstrcat(trailer, line);
	}

	if(MyHzInMod.input.UnderLine != underline)
	{
		sprintf(line, "@[U%d]", MyHzInMod.input.UnderLine);
		DBstrcat(header, line);
	}

	if(MyHzInMod.input.EndUnderLine != underline)
	{
		sprintf(line, "@[U%d]", underline);
		DBstrcat(trailer, line);
	}
}

void CreateControlData(hzdata, colnum, align, colrnum, underline, endhzdata, endcolnum, endalign, endcolrnum, endunderline)
HZdata *hzdata, *endhzdata;
int colnum, endcolnum, colrnum, align, endalign, endcolrnum, underline, endunderline;
{
	MyHzInMod.input.FontId = hzdata->FontId;
	MyHzInMod.input.point_width = hzdata->point_width;
	MyHzInMod.input.point_height = hzdata->point_height;

	MyHzInMod.input.EndFontId = endhzdata->FontId; 
	MyHzInMod.input.Endpoint_width = endhzdata->point_width;
	MyHzInMod.input.Endpoint_height = endhzdata->point_height;   

	MyHzInMod.input.ColNum = colnum;
	MyHzInMod.input.EndColNum = endcolnum;

	MyHzInMod.input.LineAlignment = align;
	MyHzInMod.input.EndLineAlignment = endalign;

	MyHzInMod.input.ColorNum = colrnum;
	MyHzInMod.input.EndColorNum = endcolrnum;

	MyHzInMod.input.UnderLine = underline;
	MyHzInMod.input.EndUnderLine = endunderline;
}

void CopyHighLightToBuffer(viewport)
VIewportdata *viewport;
{
	unsigned short charmem, *tmp;
	
	if(MyHzInMod.input.buffer != NULL) free(MyHzInMod.input.buffer);

	if(viewport->cursor.StartCurPtr == viewport->cursor.CurPtr)
		CreateControlData(&viewport->cursor.hzdata, viewport->cursor.ColNum, viewport->cursor.LineAlignment,
		viewport->cursor.ColorNum, viewport->cursor.UnderLine, &viewport->cursor.Mohzdata, viewport->cursor.MoveColNum, viewport->cursor.MoveLineAlignment,
		viewport->cursor.MoveColorNum, viewport->cursor.MoveUnderLine);	
	else
		CreateControlData(&viewport->cursor.Mohzdata, viewport->cursor.MoveColNum, viewport->cursor.MoveLineAlignment,
		viewport->cursor.MoveColorNum, viewport->cursor.MoveUnderLine, &viewport->cursor.hzdata, viewport->cursor.ColNum, viewport->cursor.LineAlignment,
		viewport->cursor.ColorNum, viewport->cursor.UnderLine);

	charmem = *viewport->cursor.EndCurPtr;
	*viewport->cursor.EndCurPtr = '\0';

	MyHzInMod.input.buffer = (unsigned short *)malloc((DBlen(viewport->cursor.StartCurPtr) + 1)*sizeof(unsigned short));
		
	if(MyHzInMod.input.buffer == NULL) return;

	*MyHzInMod.input.buffer = '\0';

	DBcat(MyHzInMod.input.buffer, viewport->cursor.StartCurPtr);

	*viewport->cursor.EndCurPtr = charmem;
}

void CopyCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	VIewportdata *viewport = (VIewportdata *)client_data;

	if(viewport->cursor.StartCurPtr != viewport->cursor.EndCurPtr)
	{
		CopyHighLightToBuffer(viewport);

	}
	else
	{
		ErrorPopup("Make Selection First!");
	}
}

void CutCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	VIewportdata *viewport = (VIewportdata *)client_data;

	if(viewport->cursor.StartCurPtr != viewport->cursor.EndCurPtr)
	{
		CopyHighLightToBuffer(viewport);
		MyDrawPage(viewport, Input(viewport, KEYINPUT, delID, 1));
	}
	else
	{
		ErrorPopup("Make Selection First!");
	}
}

void SelectAllCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	VIewportdata *viewport = (VIewportdata *)client_data;
	PAgedata *curpage;

	if(viewport->virdrawn == False) MyDrawPage(viewport, VIRDRAW);
	
	active_viewport = viewport;

	viewport->cursor.CurPtr = viewport->EditFile;
	viewport->cursor.StartCurPtr = viewport->cursor.CurPtr;
	viewport->cursor.LastStartCurPtr = viewport->cursor.CurPtr;
	curpage = viewport->page.PageRoot;
	viewport->cursor.hzdata = curpage->hzdata;
	viewport->cursor.ColNum = curpage->ColNum;
	viewport->cursor.LineAlignment = curpage->LineAlignment;
	viewport->cursor.ColorNum = curpage->ColorNum;
	viewport->cursor.UnderLine = curpage->UnderLine;

	for(; curpage->Next != NULL; curpage = curpage->Next);

	viewport->cursor.MoveCurPtr = curpage->PageStart + DBlen(curpage->PageStart);
	viewport->cursor.EndCurPtr = viewport->cursor.MoveCurPtr;
	viewport->cursor.Mohzdata = viewport->cursor.hzdata;
	viewport->cursor.LastEndCurPtr = viewport->cursor.MoveCurPtr;
	viewport->cursor.MoveColNum = viewport->cursor.ColNum;
	viewport->cursor.MoveLineAlignment = viewport->cursor.LineAlignment;
	viewport->cursor.MoveColorNum = viewport->cursor.ColorNum;
	viewport->cursor.MoveUnderLine = viewport->cursor.UnderLine;

	MyDrawPage(viewport, EXPOSEDR);
}
	
