/*
 * Copyright 1995, 1996, 1997 by Jun Yei.       All Rights Reserved
 *
 * Permission to retain, use, modify, copy, and distribute ChinesePower
 * in source or binary and its documentation (hereafter, the Software)
 * for noncommercial purpose is hereby granted to you without a fee,
 * provided that this entire copyright and permission notice appear in
 * all such copies, that no charge be associated with such copies,
 * that distribution of derivative works (including value-added
 * distributions such as with additional input dictionaries or fonts)
 * include clarification that such added or derived parts are not from
 * the original Software, and that the names of the author(s) not be
 * used to endorse or promote such works. The author(s) of the software
 * makes no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * THE AUTHOR(S) OF THE SOFTWARE DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 * IN NO EVENT SHALL THE AUTHOR(S) OF THE SOFTWARE BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Jun Yei, Ottawa, ON, Canada
 */

#include "chpower.h"
#include "fileformat.h"

extern void IniViewport();
extern void MyDrawPage();
extern void QuitViewport();
extern void YesHzFormatImportCB();
extern void NoHzFormatImportCB();
extern void YesHzFormatCB();
extern void NoHzFormatCB();
extern void YESdoNOreturn();

extern Widget toplevel;

void OpenFileCB();
void CancelFileCB();
void SaveNewFileCB();
void ReadAllFile();
int SaveFile();
void HzfileWrite();
void SaveFileExistCB();
void NewFileSaveAndQuit();
void FileSaveAsErrCB();
void NewFileSaveAndExitErrCB();
void NewFileSaveAndQuitErrCB();
void ImportAllFile();
void HZfileRead();

FileSelList fileset;

void CodingFileRead(viewport, fp, tp)
VIewportdata *viewport;
FILE *fp;
unsigned short *tp;
{

	switch(viewport->filemode)
	{
		case FILE_UTF_8:
		U8fileRead(fp, tp);
		break;

		case FILE_UTF_7:
		U7fileRead(fp, tp);
		break;

		case FILE_UTF_2:
		U2fileRead(fp, tp);
		break;

		case FILE_HZ:
		HZfileRead(fp, tp);
		break;

		default:
		DBfileRead(fp, tp);
	}
}

void CodingFileWrite(viewport, fp, tp)
VIewportdata *viewport;
FILE *fp;
unsigned short *tp;
{
 
        switch(fileset.Encoding)
        {
                case FILE_UTF_8:
                U8fileWrite(fp, tp);
                break;
 
                case FILE_UTF_7:
                U7fileWrite(fp, tp);
                break;
 
                case FILE_UTF_2:
                U2fileWrite(fp, tp);
                break;
 
                case FILE_HZ:
	        HzfileWrite(fp, tp);
                break;
 
                case FILE_DEFAULT:

	        switch(viewport->filemode)
	        {
                	case FILE_UTF_8:
                	U8fileWrite(fp, tp);
                	break;
 
                	case FILE_UTF_7:
                	U7fileWrite(fp, tp);
                	break;
 
                	case FILE_UTF_2:
                	U2fileWrite(fp, tp);
                	break;

			case FILE_HZ:
			HzfileWrite(fp, tp);
			break;

                	default:
                	DBfileWrite(fp, tp);
		}
		break;

	        default:
                DBfileWrite(fp, tp);
        }

}

void FileSetRecover()
{
	Arg args[2];

	XtSetArg(args[0], XmNmenuHistory, fileset.DefaultEncWg);
	XtSetValues(fileset.OptionWg, args, 1);

	XtSetArg(args[0], XmNset, False);
	XtSetValues(fileset.TextOnlyWg, args, 1);

	fileset.TextOnly = False;
	fileset.Encoding = FILE_DEFAULT;
}

void FileEncodingCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
        fileset.Encoding = (int) client_data;
}

void FileTextOnlyCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
        fileset.TextOnly = !fileset.TextOnly;
}

void CreateFileSet(parent)
Widget parent;
{
	Arg args[MAX_ARGS];
	register int n;
	Widget tmpWg, submenu, baseWg;
	int i;

	fileset.Encoding = FILE_DEFAULT;
	fileset.TextOnly = False;

	fileset.PopUpWg = XtCreatePopupShell("FileWin", transientShellWidgetClass,
	 parent, NULL, 0);

	n = 0;
	XtSetArg(args[n], XmNresizePolicy, XmRESIZE_NONE); n++;
	fileset.FileList = XmCreateFileSelectionBox(fileset.PopUpWg, "FileSet", args, n);
	XtAddCallback(fileset.FileList, XmNcancelCallback, CancelFileCB, NULL);
	XtAddCallback(fileset.FileList, XmNdestroyCallback, MainDestroyCB, NULL);

	tmpWg = XmFileSelectionBoxGetChild(fileset.FileList, XmDIALOG_HELP_BUTTON);
	XtUnmanageChild(tmpWg);

	n = 0;
	XtSetArg(args[n], XmNheight,40); n++;
	XtSetArg(args[n], XmNwidth, 400); n++;
	XtSetArg(args[n], XmNshadowThickness, 2); n++;
	XtSetArg(args[n], XmNshadowType, XmSHADOW_ETCHED_OUT); n++;
	XtSetArg(args[n], XmNhorizontalSpacing, 0); n++;
	XtSetArg(args[n], XmNverticalSpacing, 0); n++;
	XtSetArg(args[n], XmNrubberPositioning, True); n++;
	baseWg = XmCreateForm(fileset.FileList, "fileSel", args, n);
	XtManageChild(baseWg);

	submenu = (Widget) XmCreatePulldownMenu(baseWg, "fileEncoding", NULL, 0);
 
        for(i = 0; i < NUMOFENCODING; i++)
        {
                n = 0;
                XtSetArg(args[n], XmNlabelString, XmStringCreate(EncodingInfo[i], XmSTRING_DEFAULT_CHARSET)); n++;
		tmpWg = XmCreatePushButtonGadget(submenu, EncodingInfo[i], args, n);
                XtAddCallback(tmpWg, XmNactivateCallback, FileEncodingCB, (caddr_t *)i);
		XtManageChild(tmpWg);
		if(i == 0) fileset.DefaultEncWg = tmpWg;
        }
 
        n = 0;
	XtSetArg(args[n], XmNx, 5); n++;
	XtSetArg(args[n], XmNy, 2); n++;
        XtSetArg(args[n], XmNlabelString, XmStringCreate("Encoding of File:", XmSTRING_DEFAULT_CHARSET)); n++;
        XtSetArg(args[n], XmNsubMenuId, submenu); n++;
        XtSetArg(args[n], XmNmenuHistory, fileset.DefaultEncWg); n++;
        fileset.OptionWg = XmCreateOptionMenu(baseWg, "EncSel", args, n);
        XtManageChild(fileset.OptionWg);

        n = 0;
        XtSetArg(args[n], XmNindicatorType, XmONE_OF_MANY); n++;
        XtSetArg(args[n], XmNvisibleWhenOff, True); n++;
        XtSetArg(args[n], XmNlabelString, XmStringCreate("Pure Text Only", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNset, False); n++;
        XtSetArg(args[n], XmNx, 250); n++;
        XtSetArg(args[n], XmNy, 7); n++;
        fileset.TextOnlyWg = XmCreateToggleButtonGadget(baseWg, "TxtO", args,n);
        XtManageChild(fileset.TextOnlyWg);
        XtAddCallback(fileset.TextOnlyWg, XmNvalueChangedCallback, FileTextOnlyCB, NULL);
}

void FileOpen(w)
Widget w;
{
	Arg args[1];

	FileSetRecover();

	XtSetArg(args[0], XmNokLabelString, XmStringCreate("Open", XmSTRING_DEFAULT_CHARSET));
	XtSetValues(fileset.FileList, args, 1);

	XtRemoveAllCallbacks(fileset.FileList, XmNokCallback);	
	XtAddCallback(fileset.FileList, XmNokCallback, OpenFileCB, NULL);	
	XtManageChild(fileset.FileList);
	PopupPosition(w, fileset.PopUpWg);
	XtPopup(fileset.PopUpWg, XtGrabExclusive);
}

void FileSaveAsStartCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	FileSetRecover();

	FileSaveAsCB(w, client_data, call_data);
}

void FileSaveAsCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	VIewportdata *viewport;
	Arg args[1];

	viewport = (VIewportdata *)client_data;

	XtSetArg(args[0], XmNokLabelString, XmStringCreate("Save", XmSTRING_DEFAULT_CHARSET));
        XtSetValues(fileset.FileList, args, 1);

	XtRemoveAllCallbacks(fileset.FileList, XmNokCallback);

	XtAddCallback(fileset.FileList, XmNokCallback, SaveNewFileCB, (caddr_t *)viewport);

	XtManageChild(fileset.FileList);
	PopupPosition(viewport->resource.PopUpShell, fileset.PopUpWg);
	XtPopup(fileset.PopUpWg, XtGrabExclusive);
}

void WriteForeWord(viewport)
VIewportdata *viewport;
{
	ClearUpPic(viewport);

	if(!fileset.TextOnly && 
	   ((fileset.Encoding != FILE_DEFAULT && 
	     fileset.Encoding != FILE_UTF_2) ||
	    (fileset.Encoding == FILE_DEFAULT &&
	    viewport->filemode != FILE_UTF_2)))
	{
		WriteTitlerToFile(viewport, &viewport->titler, HEADER);
		WriteTitlerToFile(viewport, &viewport->footer, FOOTNOTE);
		WritePageNumToFile(viewport, &viewport->pndata, PAGENUMF);
		WritePageInfoToFile(viewport, PAGEINFO);
		WriteColorInfoToFile(viewport, COLORINFO);
		WritePictureToFile(viewport, PICINFO);
	}
}

void FileSaveCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	VIewportdata *viewport;

	viewport = (VIewportdata *)client_data;

        fileset.TextOnly = False;
        fileset.Encoding = FILE_DEFAULT;

	if(viewport->filename != NULL)
	{
		if(strncmp(viewport->filename, "crash.save", 10) == 0)
                {
                        viewport->fp = fopen(viewport->filename, "a+");
                }
                else
                {
                        viewport->fp = fopen(viewport->filename, "w");
                }
 
                if(viewport->fp)
                {
			WriteForeWord(viewport);
			CodingFileWrite(viewport, viewport->fp, viewport->EditFile);
			fclose(viewport->fp);
			viewport->changed = False;     
			viewport->autosavechanged = False;
		}
		else
		{
			ErrorPopup("File Open Error for Write!");
		}
	}
	else
	{
		FileSaveAsStartCB(NULL, (caddr_t *)viewport, NULL);
	}
}

int ChkFileName(filename)
char *filename;
{
	char *cp;
	FILE *fp;

	if(filename == NULL) return 1;
	else
	{
		cp = filename;
		cp += strlen(filename)-1;

		if(*cp == '/') return 1;

		if((fp = fopen(filename, "r"))) fclose(fp);
		else return 2;
	}
	return 0;
}

void OpenFileErrCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	ErrOkCB(w, client_data, call_data);
	FileOpen(toplevel);
}
	
void OpenFileCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	Arg args[MAX_ARGS];
	register int n;
	char *filename = NULL;
	XmString select = NULL;

	n = 0;
	XtSetArg(args[n], XmNtextString, &select); n++;
	XtGetValues(fileset.FileList, args, n);
	XmStringGetLtoR(select, XmSTRING_DEFAULT_CHARSET, &filename);
	
	XtUnmanageChild(fileset.FileList);
	XtRemoveAllCallbacks(fileset.FileList, XmNokCallback);
	XtPopdown(fileset.PopUpWg);

	switch(ChkFileName(filename))
	{
		case 1:
		ErrorFuncPopup("No Filename!", OpenFileErrCB,  client_data);
		break;

		case 2:
		ErrorFuncPopup("File Open Error!", OpenFileErrCB,  client_data);
		break;
	
		default:
		IniViewport(toplevel, filename, VIEWOPEN);
	}

	if(filename) XtFree(filename);
	if(select) XmStringFree(select);
}

int SaveNewFile(viewport, func)
VIewportdata *viewport;
int (*func)();
{
	Arg args[MAX_ARGS];
	register int n;
	char *filename, *cp;
	XmString select = NULL;

	XtUnmanageChild(fileset.FileList);
	XtRemoveAllCallbacks(fileset.FileList, XmNokCallback);
	XtPopdown(fileset.PopUpWg);

	n = 0;
	XtSetArg(args[n], XmNtextString, &select); n++;
	XtGetValues(fileset.FileList, args, n);
	XmStringGetLtoR(select, XmSTRING_DEFAULT_CHARSET, &filename);

	if(select) XmStringFree(select);
	
	if(filename == NULL) return 1; 

	cp = filename;
	cp += strlen(filename)-1;
	
	if(*cp == '/') 
	{ 
		if(filename) XtFree(filename); 
		return 1; 
	}

	if(viewport->savefilename != NULL) free(viewport->savefilename);
	viewport->savefilename = strdup(filename);

	if(filename) XtFree(filename);
	
	if((viewport->fp = fopen(viewport->savefilename, "r"))) 
	{
		fclose(viewport->fp);
		return 2;
	}

	return((*func)(viewport));
}

void SaveFileExistCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	if(SaveFile((VIewportdata *)client_data))
	{
		ErrorFuncPopup("File Open Error for Write!", FileSaveAsErrCB, client_data);
	}
}

int SaveFile(viewport)
VIewportdata *viewport;
{
	Display *dpy;
	Window  wind;
	char *cp;
	int i;

	if((viewport->fp = fopen(viewport->savefilename, "w")))
	{
		WriteForeWord(viewport);
		CodingFileWrite(viewport, viewport->fp, viewport->EditFile);
		fclose(viewport->fp);
	}
	else 
	{
		return 3;
	}

	if(!fileset.TextOnly)
	{
		viewport->changed = False;
		viewport->autosavechanged = False;

		if(fileset.Encoding != FILE_DEFAULT) viewport->filemode = fileset.Encoding;

		if(viewport->filename != NULL) free(viewport->filename);
	
		viewport->filename = strdup(viewport->savefilename);

		dpy = XtDisplay(viewport->resource.PopUpShell);
		wind = XtWindow(viewport->resource.PopUpShell);

		i = strlen(viewport->filename);
		cp = viewport->filename + i;
		for(; *cp != '/' && cp > viewport->filename; cp--);
		if(*cp == '/') cp++;

		XStoreName(dpy, wind, cp);
		XSetIconName(dpy, wind, cp);
	}

	return 0;
}

void HzfileWrite(fp, tp)
FILE *fp;
unsigned short *tp;
{
	int length, inhzmod, hzcount, tail;
	unsigned short tmp;

	if(GlobalDefaults.hz_enter_escape)
		tail = 3;
	else
		tail = 2;
				
	inhzmod = False;
	length = 0;

	while(*tp != 0)
	{
		SKIPIDS(tp);

		if(inhzmod)
		{
			if(!ISHZ(*tp))
			{
				inhzmod = False;
				fputs("~}", fp);
				length += 2;	
		
				switch(*tp)
				{
					case '\n':
					
					fputs("\n", fp);
					length = 0;
					break;
				
					case '~':
						
					fputs("~~", fp);
					length += 2;
					break;
					
					case '\r':
					
					break;
					
					default:
				
					fputc(*tp, fp);
					length++;
				}	
			}
			else 
			{
				tmp = *tp >> 8;
				fputc(((unsigned char)tmp) & 0x7f, fp);
				fputc((unsigned char)*tp & 0x7f, fp);

				length += 2;
			}
		}
		else
		{
			if(ISHZ(*tp))
			{
				inhzmod = True;
				fputs("~{", fp);
				tmp = *tp >> 8;
				fputc(((unsigned char)tmp) & 0x7f, fp);
				fputc((unsigned char)*tp & 0x7f, fp);
				length += 4;
			}
			else
			{
				switch(*tp)
				{
					case '\n':
					
					fputs("\n", fp);
					length = 0;
					break;
				
					case '~':
						
					fputs("~~", fp);
					length += 2;
					break;
				
					case '\r':

					break;

					default:
				
					fputc(*tp, fp);
					length++;
				}	
			}
		}
	
		if(((!inhzmod) && length >= GlobalDefaults.hz_file_width + 2 - tail) ||
		   ((inhzmod) && ((length >= GlobalDefaults.hz_file_width - tail) ||
		    (length >= GlobalDefaults.hz_file_width - 1 - tail))))
		{
			if(GlobalDefaults.hz_enter_escape && *(tp+1) != '\n')
			{
				if(inhzmod) fputs("~}~\n", fp);
				else fputs("~\n", fp);
			}
			else
			{
				if(inhzmod) fputs("~}\n", fp);
				else fputs("\n", fp);

				if(*(tp+1) == '\n') tp++;
			}

			inhzmod = False;
			length = 0;
		}

		tp++;
	}

	if(inhzmod) fputs("~}\n", fp);
	else fputs("\n", fp);
}

void FileSaveAsErrCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	ErrOkCB(w, client_data, call_data);
	FileSaveAsCB(w, client_data, call_data);
}

void SaveNewFileCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	switch(SaveNewFile((VIewportdata *)client_data, SaveFile))
	{
		case 1: 
		ErrorFuncPopup("No Filename!", FileSaveAsErrCB, client_data);
		return;

		case 2:
		YESdoNOdo(SaveFileExistCB, client_data, FileSaveAsCB, client_data, "Overwrite existing file?");
		return;

		case 3:
		ErrorFuncPopup("File Open Error for Write!", FileSaveAsErrCB, client_data);
		return;
	}
}

void SaveFileExistQuitCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	MYviewportlist *viewportlist = (MYviewportlist *)client_data;

	if(SaveFile(&viewportlist->myviewport))
	{
		ErrorFuncPopup("File Open Error!", NewFileSaveAndQuitErrCB, viewportlist); 
		return;
	}
	QuitViewport(viewportlist);
}

void SaveFileExistExitCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	MYviewportlist *viewportlist = (MYviewportlist *)client_data;

	if(SaveFile(&viewportlist->myviewport))
	{
		ErrorFuncPopup("File Open Error!", NewFileSaveAndExitErrCB, viewportlist);
		return;
	}
	QuitViewport(viewportlist);
	MainExitYesCB(NULL, NULL, NULL);
}

void NewFileSaveAndQuitErrCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	ErrOkCB(w, client_data, call_data);
	NewFileSaveAndQuit(w, client_data, call_data);
}

void SaveNewFileQuitCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	MYviewportlist *viewportlist = (MYviewportlist *)client_data;

	switch(SaveNewFile(&viewportlist->myviewport, SaveFile))
	{
		case 1:
		ErrorFuncPopup("No Filename!", NewFileSaveAndQuitErrCB, viewportlist);
		return;

		case 2:
		YESdoNOdo(SaveFileExistQuitCB, viewportlist, NewFileSaveAndQuit, viewportlist, "Overwrite existing file?");
		return;

		case 3:
		ErrorFuncPopup("File Open Error for Write!", NewFileSaveAndQuitErrCB, viewportlist);
		return;
	}
	QuitViewport(viewportlist);
}

void NewFileSaveAndExitErrCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
        ErrOkCB(w, client_data, call_data);
	NewFileSaveAndExit(w, client_data, call_data);
}

void SaveNewFileExitCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	MYviewportlist *viewportlist = (MYviewportlist *)client_data;

	switch(SaveNewFile(&viewportlist->myviewport, SaveFile))
	{
		case 1:
		ErrorFuncPopup("No Filename!", NewFileSaveAndExitErrCB, viewportlist);
		return;

		case 2:
		
		YESdoNOdo(SaveFileExistExitCB, viewportlist, NewFileSaveAndExit, viewportlist, "Overwrite existing file?");
		return;

		case 3:
		ErrorFuncPopup("File Open Error for Write!", NewFileSaveAndExitErrCB, viewportlist);
		return;
	}
	QuitViewport(viewportlist);
	MainExitYesCB(NULL, NULL, NULL);
}

void CancelFileCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	XtUnmanageChild(fileset.FileList);
	XtPopdown(fileset.PopUpWg);
}

int ImportFile(viewport, filename)
VIewportdata *viewport;
char *filename;
{
	char *cp;
	long int filesize;
	unsigned short *tp;
	int filemode;

	if(filename == NULL) return 1;

	cp = filename;
	cp += strlen(filename)-1;
	if(*cp == '/') return 1;

	if(!(viewport->fp = fopen(filename, "r"))) return 2;

	if( fileset.Encoding == FILE_DEFAULT )
	{	
		if(IsHzFormat(viewport->fp, viewport, YesHzFormatImportCB, NoHzFormatImportCB)) return 0;
	}

	rewind(viewport->fp);
	filesize = DBfileSize(viewport->fp);
	tp = (unsigned short*)malloc((filesize+1)*sizeof(unsigned short));
	if(tp == NULL) { ErrorPopup("Can't allocate enough memory!"); fclose(viewport->fp); return 0;}

	filemode = viewport->filemode;

	viewport->filemode = fileset.Encoding;

	ReadAllFile(viewport, tp, 2);

	fclose(viewport->fp);

	viewport->filemode = filemode;

	viewport->cursor.RedrawCurPtr = viewport->cursor.CurPtr;

	Input(viewport, KEYINPUT, tp, DBlen(tp));
	free(tp);

	MyDrawPage(viewport, INVIRDRAW);
	MyDrawPage(viewport, CLEARDRAW);

	return 0;
}

void ImportFileErrCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	ErrOkCB(w, client_data, call_data);
	ImportCB(w, client_data, call_data);
}

void ImportFileCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	Arg args[MAX_ARGS];
	register int n;
	char *filename;
	XmString select = NULL;

	n = 0;
	XtSetArg(args[n], XmNtextString, &select); n++;
	XtGetValues(fileset.FileList, args, n);
	XmStringGetLtoR(select, XmSTRING_DEFAULT_CHARSET, &filename);

	XtUnmanageChild(fileset.FileList);
	XtRemoveAllCallbacks(fileset.FileList, XmNokCallback);
	XtPopdown(fileset.PopUpWg);

	switch(ImportFile((VIewportdata *)client_data, filename))
	{
		case 1:
		ErrorFuncPopup("No Filename!", ImportFileErrCB, client_data);
		break;

		case 2:
		ErrorFuncPopup("File Open Error!", ImportFileErrCB, client_data);
		break;
	}

	if(filename) XtFree(filename);
	if(select) XmStringFree(select);
}

void ImportCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	VIewportdata *viewport;
	Arg args[1];

	viewport = (VIewportdata *)client_data;

	FileSetRecover();

	XtSetArg(args[0], XmNokLabelString, XmStringCreate("Import", XmSTRING_DEFAULT_CHARSET));
        XtSetValues(fileset.FileList, args, 1);

	XtRemoveAllCallbacks(fileset.FileList, XmNokCallback);
	XtAddCallback(fileset.FileList, XmNokCallback, ImportFileCB, (caddr_t *)viewport);
	XtManageChild(fileset.FileList);
	PopupPosition(viewport->resource.PopUpShell, fileset.PopUpWg);
	XtPopup(fileset.PopUpWg, XtGrabExclusive);
}

void NewFileSaveAndQuitStart(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	FileSetRecover();
	NewFileSaveAndQuit(w, client_data, call_data);
}
	
void NewFileSaveAndQuit(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	MYviewportlist *viewportlist = (MYviewportlist *) client_data;
	Arg args[1];

	XtSetArg(args[0], XmNokLabelString, XmStringCreate("Save", XmSTRING_DEFAULT_CHARSET));
        XtSetValues(fileset.FileList, args, 1);

	XtRemoveAllCallbacks(fileset.FileList, XmNokCallback);
	XtAddCallback(fileset.FileList, XmNokCallback, SaveNewFileQuitCB, (caddr_t *)viewportlist);
	XtManageChild(fileset.FileList);
	PopupPosition(viewportlist->myviewport.resource.PopUpShell, fileset.PopUpWg);
	XtPopup(fileset.PopUpWg, XtGrabExclusive);
}

void NewFileSaveAndExitStart(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	FileSetRecover();
	NewFileSaveAndExit(w, client_data, call_data);
}

void NewFileSaveAndExit(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	MYviewportlist *viewportlist = (MYviewportlist *) client_data;
	Arg args[1];

	XtSetArg(args[0], XmNokLabelString, XmStringCreate("Save", XmSTRING_DEFAULT_CHARSET));
        XtSetValues(fileset.FileList, args, 1);

	XtRemoveAllCallbacks(fileset.FileList, XmNokCallback);
	XtAddCallback(fileset.FileList, XmNokCallback, SaveNewFileExitCB, (caddr_t *)viewportlist);
	XtManageChild(fileset.FileList);
	PopupPosition(viewportlist->myviewport.resource.PopUpShell, fileset.PopUpWg);      
	XtPopup(fileset.PopUpWg, XtGrabExclusive);
}

int OpenRealFile(viewportlist)
MYviewportlist *viewportlist;
{
	VIewportdata *viewport = &viewportlist->myviewport;
	long filesize;

	if(viewport->fp)
	{
		filesize = DBfileSize(viewport->fp);
		while(viewport->sizeunit*(long)FILE_SIZE_UNIT <= filesize) viewport->sizeunit++;
		viewport->EditFile = (unsigned short *)malloc(viewport->sizeunit*(long)FILE_SIZE_UNIT*sizeof(unsigned short));

		if(viewport->EditFile == NULL)
		{
			ErrorPopup("Can't allocate enough memory!");
			return -1;
		}
		
		if( fileset.Encoding == FILE_DEFAULT )
		{	
			if(IsHzFormat(viewport->fp, viewportlist, YesHzFormatCB, NoHzFormatCB)) return 0;
		}

		ReadAllFile(viewport, viewport->EditFile, 0);
	}
	else 
	{
		ErrorPopup("File Open Error!");
		return -1;
	}

	return 1;
}

void OpenCrashCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	VIewportdata *viewport = &((MYviewportlist *)client_data)->myviewport;
	char *crashfile = malloc(strlen(viewport->filename) + 2);

	strcpy(crashfile, viewport->filename);
	strcat(crashfile, "@");

	viewport->fp = fopen(crashfile, "r");

	switch(OpenRealFile(client_data))
	{
		case 1:

		RealizeEditWin(client_data);
		viewport->changed = True;
		break;

		case -1:

		FreeViewport(client_data);
		break;
	} 

	unlink(crashfile);
	free(crashfile);
}

void OpenOrigCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	VIewportdata *viewport = &((MYviewportlist *)client_data)->myviewport;
	char *crashfile = malloc(strlen(viewport->filename) + 2);

	strcpy(crashfile, viewport->filename);
	strcat(crashfile, "@");
	unlink(crashfile);
	free(crashfile);

	viewport->fp = fopen(viewport->filename, "r");

	switch(OpenRealFile(client_data))
	{
		case 1:

		RealizeEditWin(client_data);
		break;

		case -1:

		FreeViewport(client_data);
		break;
	}
}

int ReadFile(viewportlist, file)
MYviewportlist *viewportlist;
char *file;
{
	VIewportdata *viewport = &viewportlist->myviewport;
	char *crashfile;
	FILE *crashfp;

	viewport->filename = NULL;
	viewport->sizeunit = 1;

	if(file == NULL) 
	{
		viewport->EditFile = (unsigned short *)malloc(viewport->sizeunit*FILE_SIZE_UNIT*sizeof(unsigned short));
		
		if(viewport->EditFile == NULL)
		{
			ErrorPopup("Can't allocate enough memory!");
			return -1;
		}
		
		viewport->fp = NULL;
		*(viewport->EditFile) = '\0';	
	}
	else
	{
		crashfile = malloc(strlen(file) + 2); 
		viewport->filename = strdup(file);
		strcpy(crashfile, file);
		strcat(crashfile, "@");

		crashfp = fopen(crashfile, "r");

		free(crashfile);

		if(crashfp)
		{
			fclose(crashfp);
			YESdoNOdo(OpenCrashCB, viewportlist, OpenOrigCB, viewportlist, "Use Crash Recovery?");
			return 0;
		}

		viewport->fp = fopen(file, "r");

		return(OpenRealFile(viewportlist));
	}
	
	return 1;
}

void HZfileRead(fp, tp)
FILE *fp;
unsigned short *tp;
{
	unsigned char *cp, line[101];
	int ishzform, noenter;

	ishzform = False;
	noenter = False;

	while((cp=(unsigned char *)fgets(line,100, fp)) != NULL)
	{
		for(; *cp != '\n' && *cp != '\0'; cp++)
		{
			if(*cp == '~')
			{
				switch(*(cp+1))
				{
					case '{':
					
					cp++; 
					ishzform = True;
					continue;	

					case '}':
					
					cp++; 
					ishzform = False;
					continue;      
			
					case '~':

					*tp++ = *cp++;
					continue;

					case '\r':
					case '\n':
					
					noenter = True;
					continue;	
				}
			}
			
			if(ishzform) 
			{
				*tp = 0x80 | *cp++;
				*tp <<= 8;
				*tp++ |= 0x80 | *cp;
			}
			else	if(*cp != '\r') *tp++ = *cp;
		}
	
		if(*cp == '\n') 
		{
			if(!noenter)  *tp++ = '\n'; 
	
			ishzform = False;
			noenter = False;
		}	
	}

	*tp = 0;

	fclose(fp);
}

void ReadAllFile(viewport, tp, actid)
VIewportdata *viewport;
unsigned short *tp;
int actid;
{
	char *cp = NULL, line[256], *tmp;
	int titlerneedset = True, footerneedset = True, pageinfoset = True, pagenumset = True, colorinfoset = True;

	rewind(viewport->fp);

	if(fileset.Encoding != FILE_UTF_2)
	{
		while((cp=fgets(line, 255, viewport->fp)))
		{
			if(strncmp(HEADER, cp, strlen(HEADER)) == 0)
			{
				cp += strlen(HEADER);
				if(actid != 2) FileReadSetTitler(viewport, cp, &viewport->titler, actid);
				titlerneedset = False;		
			}
			else if(strncmp(FOOTNOTE, cp, strlen(FOOTNOTE)) == 0)
			{
				cp += strlen(FOOTNOTE);
				if(actid != 2) FileReadSetTitler(viewport, cp, &viewport->footer, actid);
				footerneedset = False;	
			}
			else if(strncmp(PAGENUMF, cp, strlen(PAGENUMF)) == 0)
			{
				cp += strlen(PAGENUMF);
				if(actid != 2) FileReadSetPageNum(viewport, cp, &viewport->pndata, actid);
				pagenumset = False;
			}
			else if(strncmp(PAGEINFO, cp, strlen(PAGEINFO)) == 0)
			{
				cp += strlen(PAGEINFO);
				if(actid != 2) ReadSetPageInfo(viewport, cp);
				pageinfoset = False;
			}
			else if(strncmp(COLORINFO, cp, strlen(COLORINFO)) == 0)     
			{
				cp += strlen(COLORINFO);
				if(actid == 1) ResetColorInfo(viewport);
				if(actid != 2) ReadSetColorInfo(viewport, cp);
				colorinfoset = False;
			}
			else if(strncmp(PICINFO, cp, strlen(PICINFO)) == 0)
			{
				if(actid == 0)
				{
					cp += strlen(PICINFO);
					for( tmp = cp; *tmp != '\n' && *tmp != '\0'; tmp++);
					*tmp = '\0';
					ReadPictureFilename(viewport, cp);
				}
			}	
			else
			{
				break;
			}
		}	
	}

	if(titlerneedset && (actid == 1)) 
	{
		ResetTitler(viewport, &viewport->titler);
	}
	
	if(footerneedset && (actid == 1))
	{
		ResetTitler(viewport, &viewport->footer);
	}

	if(pagenumset && (actid == 1))
	{
		ResetPageNum(viewport, &viewport->pndata);
	}

	if(pageinfoset && (actid == 1))
	{
		ResetPageInfo(viewport);
	}

	if(colorinfoset && (actid == 1))
	{
		ResetColorInfo(viewport);
	}

	if(cp) fseek(viewport->fp, -strlen(cp), 1); 
		
	CodingFileRead(viewport, viewport->fp, tp);

	fclose(viewport->fp);
}

void YesHzFormatCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	MYviewportlist *viewportlist = (MYviewportlist *)client_data;
	VIewportdata *viewport = &viewportlist->myviewport;
	
	viewport->filemode = FILE_HZ;
	ReadAllFile(viewport, viewport->EditFile, 0);
	RealizeEditWin(viewportlist);
}

void NoHzFormatCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	MYviewportlist *viewportlist = (MYviewportlist *)client_data;
	VIewportdata *viewport = &viewportlist->myviewport;
	
	ReadAllFile(viewport, viewport->EditFile, 0);
	RealizeEditWin(viewportlist);
}

void YesHzFormatRevertCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	VIewportdata *viewport = (VIewportdata *)client_data;

	viewport->filemode = FILE_HZ;
	ReadAllFile(viewport, viewport->EditFile, 1);
	viewport->changed = False;
	viewport->autosavechanged = False;
	ResetViewport(viewport, 1);

	CreateViewportColors(viewport);
        MyDrawPage(viewport, VIRDRAW);
        MyDrawPage(viewport, TRUEDRAW);
}

void NoHzFormatRevertCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	VIewportdata *viewport = (VIewportdata *)client_data;

	ReadAllFile(viewport, viewport->EditFile, 1);
	viewport->changed = False;
	viewport->autosavechanged = False;
	ResetViewport(viewport, 1);

	CreateViewportColors(viewport); 
        MyDrawPage(viewport, VIRDRAW);
        MyDrawPage(viewport, TRUEDRAW);
}

void DoRevertCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	VIewportdata *viewport = (VIewportdata *)client_data;

	if(viewport->filename == NULL) { ErrorPopup("No Filename!"); return; }
	
	if((viewport->fp = fopen(viewport->filename, "r")))
	{
		if( fileset.Encoding == FILE_DEFAULT )
		{
			if(IsHzFormat(viewport->fp, viewport, YesHzFormatRevertCB, NoHzFormatRevertCB)) return;
		}

		ReadAllFile(viewport, viewport->EditFile, 1);
		
		ResetViewport(viewport, 1);
		viewport->changed = False;
		viewport->autosavechanged = False;

		CreateViewportColors(viewport);
		MyDrawPage(viewport, VIRDRAW);
		MyDrawPage(viewport, TRUEDRAW);
	}
	else ErrorPopup("Can not open file!");
}

void YesHzFormatImportCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	VIewportdata *viewport = (VIewportdata *)client_data;

	ImportAllFile(viewport, FILE_HZ);
}

void NoHzFormatImportCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	VIewportdata *viewport = (VIewportdata *)client_data;

	ImportAllFile(viewport, fileset.Encoding);
}

void ImportAllFile(viewport, encoding)
VIewportdata *viewport;
int encoding;
{
	long filesize;
	unsigned short *cp;
	int filemode;

	rewind(viewport->fp);
	filesize = DBfileSize(viewport->fp);
	cp = (unsigned short *)malloc((filesize+1)*sizeof(unsigned short));

	if(cp == NULL) { ErrorPopup("Fatal error: can't allocate enough memory"); fclose(viewport->fp); return;}

	filemode = viewport->filemode;
	viewport->filemode = encoding;
	
	ReadAllFile(viewport, cp, 2);

	viewport->filemode = filemode;

	viewport->cursor.RedrawCurPtr = viewport->cursor.CurPtr;

	Input(viewport, KEYINPUT, cp, DBlen(cp));

	free(cp);
	
	MyDrawPage(viewport, VIRDRAW);
	MyDrawPage(viewport, CLEARDRAW);
}

void DestroySave(viewport, keepChange)
VIewportdata *viewport;
int keepChange;
{
	if((keepChange && viewport->autosavechanged) || (viewport->changed && !keepChange))
	{
		char *crashfile, *oirgfile = viewport->filename;
		int filemode = viewport->filemode;

		if(viewport->filename == NULL)
		{
			if(keepChange)
			{
				viewport->autosavechanged = False;
				return;
			}

			crashfile = strdup("crash.save");
		}
		else
		{
			crashfile = malloc(strlen(viewport->filename) + 2);
			strcpy(crashfile, viewport->filename);
			strcat(crashfile, "@");
		}

		viewport->filename = crashfile;

		FileSaveCB(NULL, viewport, NULL);

		viewport->filemode = filemode;
		viewport->filename = oirgfile;

		if(keepChange) 
		{
			viewport->changed = True;
			viewport->autosavechanged = False;
		}

		free(crashfile);
	}
}

void RemoveAutoSavedFile(viewport)
VIewportdata *viewport;
{
        char *crashfile;

	if(viewport->filename == NULL) return;

	crashfile = malloc(strlen(viewport->filename) + 2);
	strcpy(crashfile, viewport->filename);
	strcat(crashfile, "@");

	unlink(crashfile);

	free(crashfile);
}

