/*
 * Copyright 1995, 1996, 1997 by Jun Yei.       All Rights Reserved
 *
 * Permission to retain, use, modify, copy, and distribute ChinesePower
 * in source or binary and its documentation (hereafter, the Software)
 * for noncommercial purpose is hereby granted to you without a fee,
 * provided that this entire copyright and permission notice appear in
 * all such copies, that no charge be associated with such copies,
 * that distribution of derivative works (including value-added
 * distributions such as with additional input dictionaries or fonts)
 * include clarification that such added or derived parts are not from
 * the original Software, and that the names of the author(s) not be
 * used to endorse or promote such works. The author(s) of the software
 * makes no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * THE AUTHOR(S) OF THE SOFTWARE DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 * IN NO EVENT SHALL THE AUTHOR(S) OF THE SOFTWARE BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Jun Yei, Ottawa, ON, Canada
 */

#include "ttffont.h"

/*
   Function that translate two-byte character code into a
   location offset in Glyph Data.

   int Font_getFontOffset(TTFFont *f,BYTE *cc);
   You can use
      off = Font_getFontOffset(f,cc);
      Font_load_GLYF(f,fp,off);
   To fetch data in f->glyf
*/

void Font_select_CMAP(f, platId, encId)
TTFFont *f;
USHORT platId;
USHORT encId;
{
  int i;
  for(i=0;i<f->numberOfEncoding;i++)
    {
      if ((f->cmap[i].PlateformID == platId) ||(f->cmap[i].EncodingID==encId))
	break;
    }
  if (i == f->numberOfEncoding)
    f->curmap = &f->cmap[0];
  else
    f->curmap = &f->cmap[i];
}

int Font_getFontOffset(f, cc)
TTFFont *f;
unsigned short cc;
{
  int i;

  switch(f->curmap->format)
    {
    case 0:
      return Font_getFontOffset0(f,cc);
    case 2:
      return Font_getFontOffset2(f,cc);
    case 4:
      return Font_getFontOffset4(f,cc);
      /*  case 6:
	 return Font_getFontOffset6(f,cc);
	 */
    default:
      return -1;
    }
}

int Font_getFontOffset0(f, cc)
TTFFont *f;
unsigned short cc;
{
  BYTE index;

  index = f->curmap->map.cmap0->glyphIdArray[cc];
  return f->indexToLocation[index];
}

int Font_getFontOffset2(f, value)
TTFFont *f;
unsigned short value;
{
    USHORT index;
    USHORT index1,firstCode,entryCount;
    SHORT idDelta;
    USHORT idRangeOffset;
    CMAP *cm;
    BYTE cc[2];

    cc[1] = value;
    cc[0] = (value >> 8);

    cm = f->curmap;
    index1 = cm->map.cmap2->subHeaderKeys[*cc];
    
    if (index1 == 0)
	index = cm->map.cmap2->glyphIdArray[*cc];
    else
    {
	firstCode = cm->map.cmap2->data[index1*4];
	entryCount = cm->map.cmap2->data[index1*4+1];
	idDelta = ((SHORT *)cm->map.cmap2->data)[index1*4+2];
	idRangeOffset = cm->map.cmap2->data[index1*4+3];

	if (cc[1] < firstCode)
	    return -1;
	
	if (cc[1] >= firstCode+entryCount)
	    return -1;
	index = cm->map.cmap2->glyphIdArray[idRangeOffset/2+cc[1]-firstCode]+idDelta;
    }
    if (index > f->profile->numglyphs)
    {
	return -1;
     }
    return f->indexToLocation[index];
}
  
int Font_getFontOffset4(f, value)
TTFFont *f;
unsigned short value;
{
    USHORT index;
    CMAP4 *cm;
    int found, i;

    cm = f->curmap->map.cmap4;
    
    found = 0;
    
    for(i = 0; i < cm->segCount; i++)
    {
	if(value >= cm->startCount[i] && value <= cm->endCount[i]) 
	{
		found = 1;
		break;
	}
    }
    
    if(found)
    {
	if(cm->idRangeOffset[i] != 0)
	{
		index = cm->glyphIdArray[cm->idRangeOffset[i]/2 + (value - cm->startCount[i]) - cm->segCount + i];
		if(index) index += cm->idDelta[i];	
	}
	else
	{
		index = value + cm->idDelta[i];
   	} 
    }
    else
    {
	index = 0;
    }
    
    if (index > f->profile->numglyphs)
    {
	return -1;
     }
    
    return f->indexToLocation[index];
}
  
