/*
 * Copyright 1995, 1996, 1997 by Jun Yei.       All Rights Reserved
 *
 * Permission to retain, use, modify, copy, and distribute ChinesePower
 * in source or binary and its documentation (hereafter, the Software)
 * for noncommercial purpose is hereby granted to you without a fee,
 * provided that this entire copyright and permission notice appear in
 * all such copies, that no charge be associated with such copies,
 * that distribution of derivative works (including value-added
 * distributions such as with additional input dictionaries or fonts)
 * include clarification that such added or derived parts are not from
 * the original Software, and that the names of the author(s) not be
 * used to endorse or promote such works. The author(s) of the software
 * makes no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * THE AUTHOR(S) OF THE SOFTWARE DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 * IN NO EVENT SHALL THE AUTHOR(S) OF THE SOFTWARE BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Jun Yei, Ottawa, ON, Canada
 */

#include 	"chpower.h"

MYfontlist *FontListRoot = NULL;
HBF *EnglishFontHbf = NULL;

void FontDefaultInitial()
{
	FILE *fp;
	char line[100];
	char *cp, *fontname, *tmp;
	MYfontlist *fontlist, *search;
	int i, TTFExist = False;

	MyHzInMod.InputWin.misc.hz_encoding = NULL;
	MyHzInMod.InputWin.misc.it_dirs = NULL;

	if(HBF_OpenFont(DEFAULT_ENGLISH_FONT, &EnglishFontHbf) != 0)
	{
		fprintf(stderr, "Can't open font '%s'\n", DEFAULT_ENGLISH_FONT);
		EnglishFontHbf = NULL;
	}

	fp = My_path_open(getenv(CONF_FILE_PATH), CONFIGURE_FILE, "r");

	if(fp)
	{
		while((cp=fgets(line,99, fp)))
		{
			if(strncmp(FONTID, cp, strlen(FONTID)) == 0)
			{
				cp += strlen(FONTID);
				for(fontname = cp; *fontname != '\n' && *fontname != '\0'; fontname++);
				*fontname = '\0';
				for(fontname = cp; *fontname != '=' && *fontname != '\0'; fontname++);
				if(*fontname == '=')
				{
					*fontname = '\0';
					fontname++;
					fontlist = (MYfontlist *)malloc(sizeof(MYfontlist));
					if (fontlist != NULL)
					{
						fontlist->FontName = strdup(fontname);
						fontlist->FontId = strdup(cp);
						fontlist->FontType = HBFONT;
						fontlist->Next = NULL;
					
						i = 1;

						i = HBF_OpenFont(fontlist->FontName, &(fontlist->Hbf));

						if(i)
						{
							fontlist->Hbf = (HBF *)Open_TTF(fontlist->FontName);
							
							if(fontlist->Hbf)
							{
								TTFExist = True;	
								fontlist->FontType = TTFONT;
								i = 0;
							}	
						}

						if (i)
						{
							fprintf(stderr, "Can't open font '%s'\n", fontlist->FontName);
							free(fontlist->FontName);
							free(fontlist->FontId);
							free(fontlist);
						}
						else
						{
							if(FontListRoot == NULL) FontListRoot = fontlist;
							else
							{
								for(search = FontListRoot; search->Next != NULL; search = search->Next);
								search->Next = fontlist;
							}
						}
					}
				}	
			}
			else
			{
				InitDefault(cp);	
			}
		}

		fclose(fp);
	}
	else
	{
		fprintf(stderr, "Configure File %s Not Found!\n", CONFIGURE_FILE);
	}

	ClearUpDefaults();

	if(TTFExist) Init_TTF_Env();
}

int TotalNumOfFont()
{
	int i = 0;
	MYfontlist *search;

	for(search = FontListRoot; search != NULL; search = search->Next)
		i++;
	return(i);
}

char *GetFontId(num)
int num;
{
	int i = 1;
	MYfontlist *search;
	
	for(search = FontListRoot; search != NULL; search = search->Next)
	{
		if(num == i) return(search->FontId);
		i++;	
	}
	return(NULL);
}

int GetFontType(num)
int num;
{
	int i = 1;
	MYfontlist *search;

	for(search = FontListRoot; search != NULL; search = search->Next)
	{
		if(num == i) return(search->FontType);
		i++;
	}
	return(HBFONT);
}

HBF *GetFontHandleFrIndex(num)
int num;
{
	int i = 1;
	MYfontlist *search;

	for(search = FontListRoot; search != NULL; search = search->Next)
	{
		if(num == i) return(search->Hbf);
		i++;
	}
	return(NULL);
}

int SearchFontList(fontId)
char *fontId;
{
	MYfontlist *search;
	int i = 0;

	if(fontId)
	{
		for(search = FontListRoot; search != NULL; search = search->Next) 
		{
			i ++;	
			if (strcmp(fontId, search->FontId) == 0) return i;
		}
	}	
	return 0;
}

HBF *GetFontHandle(fontId, cp, fonttype)
char *fontId, **cp;
int *fonttype;
{
	MYfontlist *search;
	
	if(fontId)
	{
		for(search = FontListRoot; search != NULL; search = search->Next)
		{
			if (strcmp(fontId, search->FontId) == 0) 
			{
				*cp = search->FontId;
				*fonttype = search->FontType;	
				return(search->Hbf);
			}
		}
	}
	return(NULL);
}

void DestroyMyFontList()
{
	MYfontlist *search, *nextone;
	
	search = FontListRoot;
	
	if(search == NULL) return;
	
	do
	{
		nextone = search->Next;
		
		if(search->FontType == HBFONT) HBF_CloseFont(search->Hbf);
		else Close_TTF((void *)search->Hbf);

		free(search->FontName);	
		free(search->FontId);
		free(search);
		search = nextone;
	} 
	while(search != NULL);

	DestroyTTF();
	DestroyTTFSet();
}

