/*
 * Copyright 1995, 1996, 1997 by Jun Yei.       All Rights Reserved
 *
 * Permission to retain, use, modify, copy, and distribute ChinesePower
 * in source or binary and its documentation (hereafter, the Software)
 * for noncommercial purpose is hereby granted to you without a fee,
 * provided that this entire copyright and permission notice appear in
 * all such copies, that no charge be associated with such copies,
 * that distribution of derivative works (including value-added
 * distributions such as with additional input dictionaries or fonts)
 * include clarification that such added or derived parts are not from
 * the original Software, and that the names of the author(s) not be
 * used to endorse or promote such works. The author(s) of the software
 * makes no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * THE AUTHOR(S) OF THE SOFTWARE DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 * IN NO EVENT SHALL THE AUTHOR(S) OF THE SOFTWARE BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Jun Yei, Ottawa, ON, Canada
 */

#include "chpower.h"

void CancelGifCB();
void DoGifCB();
int GifPrint();

GifMenu gifset;

void GifPopup(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
        XtPopup(gifset.PopUpWg, XtGrabExclusive);
}
 
int MyReadXImage(x, y)
int x, y;
{
	long pixel;
	int i;

	pixel = XGetPixel(gifset.draw.image, x, y);

	if(gifset.ColorGif)
	{
		for(i = 0; i < 8; i++) { if(pixel == gifset.draw.pixel[i]) return i; }
	}
	else
	{
		if(pixel != gifset.draw.pixel[0]) return 1;
	}
	
	return 0;
}

void MyPutPixmapToGif(viewport, filenum)
VIewportdata *viewport;
int filenum;
{
	char myfilename[60];	
	FILE *fp;
	int i;
	XGCValues gc_value;
	XColor color;

	if( LongProcCancelFlg ) return;

	if(gifset.draw.width == 0 || gifset.draw.height == 0) return;

	sprintf(myfilename, "%s.%d.gif", gifset.giffilename, filenum);
	fp = fopen(myfilename, "w");

	if(fp)
	{
		gifset.draw.image = XGetImage(viewport->resource.dpy, gifset.pixmap, 0, 0, gifset.draw.width, gifset.draw.height, -1, XYPixmap);

		if(gifset.ColorGif)
		{
			for(i = 0; i < 8; i++)
			{
				if(i != 0) XGetGCValues(viewport->resource.dpy, viewport->resource.gc[i-1], GCForeground, &gc_value);
				else XGetGCValues(viewport->resource.dpy, viewport->resource.reverse_gc[0], GCForeground, &gc_value);
				gifset.draw.pixel[i] = gc_value.foreground;
				
				color.pixel = gifset.draw.pixel[i];	
				XQueryColor(viewport->resource.dpy,
					    DefaultColormap(viewport->resource.dpy, DefaultScreen(viewport->resource.dpy)),
					    &color);
				
				gifset.draw.red[i] = (char) color.red;
				gifset.draw.green[i] = (char)color.green;
				gifset.draw.blue[i] = (char)color.blue;
			}

			i = 3;

		}
		else
		{
			XGetGCValues(viewport->resource.dpy, viewport->resource.reverse_gc[0], GCForeground, &gc_value);
			gifset.draw.pixel[0] = gc_value.foreground;

			gifset.draw.red[0] = gifset.draw.green[0] = gifset.draw.blue[0] = 0xff;
			gifset.draw.red[1] = gifset.draw.green[1] = gifset.draw.blue[1] = 0;

			i = 1;
		}
	
		if(!GIFEncode(fp, gifset.draw.width, gifset.draw.height, gifset.Interlace, gifset.Transparent, i, gifset.draw.red, gifset.draw.green, gifset.draw.blue, MyReadXImage))
		{
			ErrorFuncPopup("Failed to Generate GIF", GifPopup, NULL);
			LongProcCancelFlg = 29;
		}
		
		XDestroyImage(gifset.draw.image);
		fclose(fp);	

		if( LongProcCancelFlg )
		{
			unlink(myfilename);
		}
	}
	else
	{
		ErrorFuncPopup("Can not open Gif file for write!", GifPopup, NULL);
		LongProcCancelFlg = 29;
	}
}

int GifPrint(viewport)
VIewportdata *viewport;
{	
	char *line = NULL;
	char msg[64];
	Arg args[1];
	FILE *fp;

	if(!gifset.AllPage)
	{
		XtSetArg(args[0], XmNvalue, &line);
		XtGetValues(gifset.StartPgWg, args, 1);

		sscanf(line, "%d", &gifset.StartPage);
	
		XtFree(line);

		XtSetArg(args[0], XmNvalue, &line);
		XtGetValues(gifset.EndPgWg, args, 1);

		sscanf(line, "%d", &gifset.EndPage);

		XtFree(line);
	
		if(gifset.StartPage < 1 || gifset.EndPage > viewport->page.MaxPageNum || gifset.StartPage > gifset.EndPage)
		{
			ErrorFuncPopup("Wrong Page Number!", GifPopup, NULL);
			return(True);
		}
	}
	
	if(gifset.giffilename) free(gifset.giffilename);
	
	gifset.giffilename = NULL;

	XtSetArg(args[0], XmNvalue, &line);
	XtGetValues(gifset.GifFileWg, args, 1);
		
	if(line)
	{
		gifset.giffilename = strdup(line);
		if(strlen(gifset.giffilename) == 0)
		{
			ErrorFuncPopup("Gif File Name Missing!", GifPopup, NULL);
                        return(True);
		}

		XtFree(line);
		if(strlen(gifset.giffilename) == 0)
		{
			ErrorFuncPopup("Gif File Name Missing!", GifPopup, NULL);
                        return(True);
                }
	}
	else
	{
		ErrorFuncPopup("Gif File Name Missing!", GifPopup, NULL);
                return(True);
	}


        if(viewport->virdrawn == False)
        {
                viewport->cursor.RedrawCurPtr = viewport->cursor.CurPtr;
                MyDrawPage(viewport, INVIRDRAW);
        }

	LongProcCancelFlg = False;

	if(gifset.AllPage)
		sprintf(msg, "Cancel GIF Generation? (Page 1)");
	else
		sprintf(msg, "Cancel GIF Generation? (Page %d)", gifset.StartPage);

	ErrorFuncPopup(msg, LongProcCancelCB, &LongProcCancelFlg);

	if(gifset.PageGif)
	{
		MyDrawPage(viewport, GIFPAGEDRAW);
	}
	else
	{
		MyDrawPage(viewport, GIFLINEDRAW);
	}

	CriticalGoing = False;

	if( LongProcCancelFlg == True ) return(False);
	else if( LongProcCancelFlg == 29 ) return(True);

	ErrOkCB();

	return(True);
}

void GifPageCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	Arg args[1];

	gifset.PageGif = True;
		
	XtSetArg(args[0], XmNset, True);
	XtSetValues(gifset.PageGifBtn, args, 1);
		
	XtSetArg(args[0], XmNset, False);
	XtSetValues(gifset.LineGifBtn, args, 1);
}

void GifLineCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	Arg args[1];

	gifset.PageGif = False;

	XtSetArg(args[0], XmNset, False);
	XtSetValues(gifset.PageGifBtn, args, 1);

	XtSetArg(args[0], XmNset, True);
	XtSetValues(gifset.LineGifBtn, args, 1);
}

void GifToggleCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	int *mydata = (int *)client_data;
	
	if(*mydata == True)
	{
		*mydata = False;
	}
	else
	{
		*mydata = True;
	}
}

void GifPartPageTextCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	Arg args[1];

	gifset.AllPage = False;

	XtSetArg(args[0], XmNset, True);
	XtSetValues(gifset.PartPgBtn, args, 1);

	XtSetArg(args[0], XmNset, False);
	XtSetValues(gifset.AllPgBtn, args, 1);
}

void GifAllPageCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	Arg args[1];

	gifset.AllPage = True;
		
	XtSetArg(args[0], XmNset, True);
	XtSetValues(gifset.AllPgBtn, args, 1);
		
	XtSetArg(args[0], XmNset, False);
	XtSetValues(gifset.PartPgBtn, args, 1);
}

void CreateGifSet(parent)
Widget parent;
{
	Widget label;
	
	Arg args[MAX_ARGS];
	register int n;
	char *cp, line[50];

	gifset.PopUpWg = XtCreatePopupShell("Gif Window", transientShellWidgetClass, parent, NULL, 0);
	gifset.AllPage = True;
	gifset.PageGif = True;
	gifset.ColorGif = False;
	gifset.Transparent = False;
	gifset.Interlace = False;	
	gifset.giffilename = NULL;

	n = 0;
	XtSetArg(args[n], XmNheight, 175); n++;
	XtSetArg(args[n], XmNwidth, 340); n++;
	XtSetArg(args[n], XmNhorizontalSpacing, 0); n++;
	XtSetArg(args[n], XmNverticalSpacing, 0); n++;
	XtSetArg(args[n], XmNrubberPositioning, True); n++;
	gifset.Root = XmCreateForm(gifset.PopUpWg, "GifSet", args, n);
	XtManageChild(gifset.Root);
	XtAddCallback(gifset.Root, XmNdestroyCallback, MainDestroyCB, NULL);

	n = 0;
	XtSetArg(args[n], XmNset, True);
	XtSetArg(args[n], XmNindicatorType, XmONE_OF_MANY); n++;
	XtSetArg(args[n], XmNvisibleWhenOff, True); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("All Pages", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNx, 15); n++;
	XtSetArg(args[n], XmNy, 15); n++;
	gifset.AllPgBtn = XmCreateToggleButtonGadget(gifset.Root, "AllPg", args,n);
	XtManageChild(gifset.AllPgBtn);
	XtAddCallback(gifset.AllPgBtn, XmNvalueChangedCallback, GifAllPageCB, NULL);

	n = 0;
	XtSetArg(args[n], XmNset, False);
	XtSetArg(args[n], XmNindicatorType, XmONE_OF_MANY); n++;
	XtSetArg(args[n], XmNvisibleWhenOff, True); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("From:", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNx, 110); n++;
	XtSetArg(args[n], XmNy, 15); n++;
	gifset.PartPgBtn = XmCreateToggleButtonGadget(gifset.Root, "FromPg", args,n);
	XtManageChild(gifset.PartPgBtn);
	XtAddCallback(gifset.PartPgBtn, XmNvalueChangedCallback, GifPartPageTextCB, NULL);

	n = 0;
	XtSetArg(args[n], XmNwidth, 50); n++;
	XtSetArg(args[n], XmNscrollHorizontal, False); n++;
	XtSetArg(args[n], XmNscrollVertical, False); n++;
	XtSetArg(args[n], XmNresizeHeight, False); n++;
	XtSetArg(args[n], XmNresizeWidth, False); n++;
	XtSetArg(args[n], XmNmarginHeight, 0); n++;
	XtSetArg(args[n], XmNmarginWidth, 0); n++;
	XtSetArg(args[n], XmNmaxLength, 3); n++;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(args[n], XmNleftWidget, gifset.PartPgBtn); n++;
	XtSetArg(args[n], XmNy, 15); n++;
	gifset.StartPgWg = XmCreateText(gifset.Root, "StartPg", args, n);
	XtManageChild(gifset.StartPgWg);
	XtAddCallback(gifset.StartPgWg, XmNfocusCallback, GifPartPageTextCB, NULL);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("To:", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNx, 142); n++;
	XtSetArg(args[n], XmNy, 40); n++;
	label = XmCreateLabelGadget(gifset.Root, "ToLabel", args, n);
	XtManageChild(label);

	n = 0;
	XtSetArg(args[n], XmNwidth, 50); n++;
	XtSetArg(args[n], XmNscrollHorizontal, False); n++;
	XtSetArg(args[n], XmNscrollVertical, False); n++;
	XtSetArg(args[n], XmNresizeHeight, False); n++;
	XtSetArg(args[n], XmNresizeWidth, False); n++;
	XtSetArg(args[n], XmNmarginHeight, 0); n++;
	XtSetArg(args[n], XmNmarginWidth, 0); n++;
	XtSetArg(args[n], XmNmaxLength, 3); n++;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(args[n], XmNleftWidget, label); n++;
	XtSetArg(args[n], XmNy, 40); n++;
	gifset.EndPgWg = XmCreateText(gifset.Root, "EndPg", args, n); 
	XtManageChild(gifset.EndPgWg);
	XtAddCallback(gifset.EndPgWg, XmNfocusCallback, GifPartPageTextCB, NULL);

        n = 0;
        XtSetArg(args[n], XmNset, False); n++;
        XtSetArg(args[n], XmNindicatorType, XmN_OF_MANY); n++;
        XtSetArg(args[n], XmNvisibleWhenOff, True); n++;
        XtSetArg(args[n], XmNlabelString, XmStringCreate("Color", XmSTRING_DEFAULT_CHARSET)); n++;
        XtSetArg(args[n], XmNx, 15); n++;
        XtSetArg(args[n], XmNy, 65); n++;
        label = XmCreateToggleButtonGadget(gifset.Root, "ColorWg", args,n);
        XtManageChild(label);
        XtAddCallback(label, XmNvalueChangedCallback, GifToggleCB, &gifset.ColorGif);

	n = 0;
	XtSetArg(args[n], XmNset, False);
	XtSetArg(args[n], XmNindicatorType, XmN_OF_MANY); n++;
	XtSetArg(args[n], XmNvisibleWhenOff, True); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Transparent", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNx, 100); n++;
	XtSetArg(args[n], XmNy, 65); n++;
	label = XmCreateToggleButtonGadget(gifset.Root, "TansWg", args,n);
	XtManageChild(label);
	XtAddCallback(label, XmNvalueChangedCallback, GifToggleCB, &gifset.Transparent);   

	n = 0;
	XtSetArg(args[n], XmNset, False);
	XtSetArg(args[n], XmNindicatorType, XmN_OF_MANY); n++;
	XtSetArg(args[n], XmNvisibleWhenOff, True); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Interlace", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNx, 210); n++;
	XtSetArg(args[n], XmNy, 65); n++;
	label = XmCreateToggleButtonGadget(gifset.Root, "InterlWg", args,n);
	XtManageChild(label);
	XtAddCallback(label, XmNvalueChangedCallback, GifToggleCB, &gifset.Interlace);

	n = 0;
	XtSetArg(args[n], XmNset, True);
	XtSetArg(args[n], XmNindicatorType, XmONE_OF_MANY); n++;
	XtSetArg(args[n], XmNvisibleWhenOff, True); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Page Gif", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNx, 235); n++;
	XtSetArg(args[n], XmNy, 15); n++;
	gifset.PageGifBtn = XmCreateToggleButtonGadget(gifset.Root, "PageGifWg", args,n);
	XtManageChild(gifset.PageGifBtn);
	XtAddCallback(gifset.PageGifBtn, XmNvalueChangedCallback, GifPageCB, NULL);   

	n = 0;
	XtSetArg(args[n], XmNset, False);
	XtSetArg(args[n], XmNindicatorType, XmONE_OF_MANY); n++;
	XtSetArg(args[n], XmNvisibleWhenOff, True); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Line Gif", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNx, 235); n++;
	XtSetArg(args[n], XmNy, 40); n++;
	gifset.LineGifBtn = XmCreateToggleButtonGadget(gifset.Root, "PageGifWg", args,n);
	XtManageChild(gifset.LineGifBtn);
	XtAddCallback(gifset.LineGifBtn, XmNvalueChangedCallback, GifLineCB, NULL);   

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Gif File Name:", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNx, 15); n++;
	XtSetArg(args[n], XmNy, 100); n++;
	label = XmCreateLabelGadget(gifset.Root, "GifL", args, n);
	XtManageChild(label);

	n = 0;
	XtSetArg(args[n], XmNwidth, 180); n++;
	XtSetArg(args[n], XmNscrollHorizontal, False); n++;
	XtSetArg(args[n], XmNscrollVertical, False); n++;
	XtSetArg(args[n], XmNresizeHeight, False); n++;
	XtSetArg(args[n], XmNresizeWidth, False); n++;
	XtSetArg(args[n], XmNmarginHeight, 2); n++;
	XtSetArg(args[n], XmNmarginWidth, 2); n++;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(args[n], XmNleftWidget, label); n++;
	XtSetArg(args[n], XmNy, 97); n++;
	gifset.GifFileWg = XmCreateText(gifset.Root, "PsFile", args, n); 
	XtManageChild(gifset.GifFileWg);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate(".n.gif", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(args[n], XmNleftWidget, gifset.GifFileWg); n++;
	XtSetArg(args[n], XmNy, 100); n++;
	label = XmCreateLabelGadget(gifset.Root, "GifR", args, n);
	XtManageChild(label);

	n = 0;
	XtSetArg(args[n], XmNwidth, 155); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Create", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNx, 15); n++;
	XtSetArg(args[n], XmNy, 135); n++;
	gifset.PrintBtn = XmCreatePushButtonGadget(gifset.Root, "printBtn", args, n);
	XtManageChild(gifset.PrintBtn);
	XtAddCallback(gifset.PrintBtn, XmNactivateCallback, DoGifCB, NULL);

	n = 0;
	XtSetArg(args[n], XmNwidth, 155); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Cancel", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(args[n], XmNleftWidget, gifset.PrintBtn); n++;
	XtSetArg(args[n], XmNy, 135); n++;
	gifset.CancelBtn = XmCreatePushButtonGadget(gifset.Root, "closeBtn", args, n);
	XtManageChild(gifset.CancelBtn);
	XtAddCallback(gifset.CancelBtn, XmNactivateCallback, CancelGifCB, NULL);

	XtRealizeWidget(gifset.PopUpWg);
}

void GifSetCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	Arg args[3];
	char line[80];
	long curpos;

	active_viewport = (VIewportdata *)client_data;	
	
	if(gifset.AllPage)
	{
		XtSetArg(args[0], XmNset, True);
		XtSetValues(gifset.AllPgBtn, args, 1);

		XtSetArg(args[0], XmNset, False);
		XtSetValues(gifset.PartPgBtn, args, 1);
	}
	else
	{
		XtSetArg(args[0], XmNset, True);
		XtSetValues(gifset.PartPgBtn, args, 1);

		XtSetArg(args[0], XmNset, False);
		XtSetValues(gifset.AllPgBtn, args, 1);
	}

	if(gifset.PageGif)
	{
		XtSetArg(args[0], XmNset, True);
		XtSetValues(gifset.PageGifBtn, args, 1);

		XtSetArg(args[0], XmNset, False);
		XtSetValues(gifset.LineGifBtn, args, 1);
	}
	else
	{
		XtSetArg(args[0], XmNset, True);
		XtSetValues(gifset.LineGifBtn, args, 1);

		XtSetArg(args[0], XmNset, False);
		XtSetValues(gifset.PageGifBtn, args, 1);
	}

	curpos = 0;	
	if(active_viewport->filename) 
	{
		sprintf(line, "%s\0", active_viewport->filename);
		if(strlen(line) > 28) curpos = strlen(line) - 28;
	}
	else line[0] = '\0';
	
	XtSetArg(args[0], XmNvalue, line);
	XtSetArg(args[1], XmNtopCharacter, curpos);
	XtSetArg(args[2], XmNcursorPosition, (long) strlen(line));	
	XtSetValues(gifset.GifFileWg, args, 3);

	sprintf(line, "%d\0", active_viewport->viewp.CurPage->PageNum);
	XtSetArg(args[0], XmNvalue, line);
	XtSetValues(gifset.StartPgWg, args, 1);
	XtSetValues(gifset.EndPgWg, args, 1);

	PopupPosition(active_viewport->resource.PopUpShell, gifset.PopUpWg);
	XtPopup(gifset.PopUpWg, XtGrabExclusive); 
}

void DoGifCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	XtPopdown(gifset.PopUpWg);

	if(!GifPrint(active_viewport))
	{
		XtPopup(gifset.PopUpWg, XtGrabExclusive);
	}
}

void CancelGifCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	
	XtPopdown(gifset.PopUpWg);
}

