/*
 * Copyright 1995, 1996, 1997 by Jun Yei.       All Rights Reserved
 *
 * Permission to retain, use, modify, copy, and distribute ChinesePower
 * in source or binary and its documentation (hereafter, the Software)
 * for noncommercial purpose is hereby granted to you without a fee,
 * provided that this entire copyright and permission notice appear in
 * all such copies, that no charge be associated with such copies,
 * that distribution of derivative works (including value-added
 * distributions such as with additional input dictionaries or fonts)
 * include clarification that such added or derived parts are not from
 * the original Software, and that the names of the author(s) not be
 * used to endorse or promote such works. The author(s) of the software
 * makes no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * THE AUTHOR(S) OF THE SOFTWARE DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 * IN NO EVENT SHALL THE AUTHOR(S) OF THE SOFTWARE BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Jun Yei, Ottawa, ON, Canada
 */

/*****************************************************************
 * Portions of this code Copyright (C) 1989 by Michael Mauldin.
 * Permission is granted to use this file in whole or in
 * part for any purpose, educational, recreational or commercial,
 * provided that this copyright notice is retained unchanged.
 * This software is available to all free of charge by anonymous
 * FTP and in the UUNET archives.
 *
 *
 * Authors:  Michael Mauldin (mlm@cs.cmu.edu)
 *           David Rowley (mgardi@watdcsu.waterloo.edu)
 *
 * Based on: compress.c - File compression ala IEEE Computer, June 1984.
 *
 *	Spencer W. Thomas       (decvax!harpo!utah-cs!utah-gr!thomas)
 *	Jim McKie               (decvax!mcvax!jim)
 *	Steve Davies            (decvax!vax135!petsd!peora!srd)
 *	Ken Turkowski           (decvax!decwrl!turtlevax!ken)
 *	James A. Woods          (decvax!ihnp4!ames!jaw)
 *	Joe Orost               (decvax!vax135!petsd!joe)
 *****************************************************************/
#include <stdio.h>
#include "chpower.h"

extern void compress();

typedef long int        count_int;
typedef int (* ifunptr)();

static int  width, height;
static int  curx, cury;
static long CountDown;
static int  Interlace;
static int  Pass;
static int counter;

static void putword();

/*************************************************************/
int GIFEncode(fp, w, h, GInterlace, GTransparent, BitsPerPixel, Red, Green, Blue, GetPixel)
     FILE *fp;
     int   w,h, GInterlace, GTransparent, BitsPerPixel;
     unsigned char Red[], Green[], Blue[];
     ifunptr GetPixel;
{
  int   i, RWidth, RHeight;
  int   LeftOfs, TopOfs;
  int   ColorMapSize, InitCodeSize, Background;
  int   Resolution, B;

  Interlace = GInterlace;
  ColorMapSize = 1 << BitsPerPixel;
  counter = 0;

  Resolution = BitsPerPixel;

  Background = 0;

  RWidth  = width  = w;
  RHeight = height = h;
  LeftOfs = TopOfs = 0;
	
  CountDown = (long) w * (long) h;    /* # of pixels we'll be doing */

  Pass = 0;

  if (BitsPerPixel <= 1) InitCodeSize = 2;
                    else InitCodeSize = BitsPerPixel;

  curx = cury = 0;

  if(GTransparent)
    fwrite("GIF89a", 1, 6, fp);    /* the GIF magic number */
  else
    fwrite("GIF87a", 1, 6, fp);    /* the GIF magic number */

  putword(RWidth, fp);           /* screen descriptor */
  putword(RHeight, fp);

  B = 0x80;       /* Yes, there is a color map */

  B |= (Resolution - 1) << 5;  /* Resolution */;

  B |= (BitsPerPixel - 1); /* Bits per Pixel */

  fputc( B, fp );

  fputc(Background, fp);         /* background color */

  fputc(0, fp);                  /* future expansion byte */


  for( i=0; i<ColorMapSize; i++ ) {  /* Write out Coloe Map */
	fputc( Red[i], fp );
	fputc( Green[i], fp );
	fputc( Blue[i], fp );
  }
  
/*  if (comment && strlen(comment) > (size_t) 0) {
    char *sp;
    int   i, blen;

    fputc(0x21, fp);
    fputc(0xFE, fp);

    sp = comment;
    while ( (blen=strlen(sp)) > 0) {
      if (blen>255) blen = 255;
      fputc(blen, fp);
      for (i=0; i<blen; i++, sp++) fputc(*sp, fp);
    }
    fputc(0, fp); 
  }
*/

 if (GTransparent) {

    fputc(0x21, fp); /* Extension Introducer */
    fputc(0xF9, fp); /* Graphic Control Label */

    fputc(0x04, fp); /* Block Size = 4 */
    fputc(0x01, fp); /* No Disposal Method, No User Input, Transparent Color Set */
    fputc(0x00, fp); /* Delay Time = 0 */
    fputc(0x00, fp); 
    fputc(0x00, fp); /* Back Ground Color = colormap[0] */

    fputc(0x00, fp); /* Block Terminator */
  }


  fputc( ',', fp );              /* image separator */

  /* Write the Image header */

  putword(LeftOfs, fp);
  putword(TopOfs,  fp);
  putword(width,   fp);
  putword(height,  fp);

  if (Interlace) fputc(0x40, fp);   /* Use Global Colormap, maybe Interlace */
            else fputc(0x00, fp);

  fputc(InitCodeSize, fp);

  compress(InitCodeSize+1, fp, GetPixel);

  fputc(0,fp);                      /* Write out a Zero-length packet (EOF) */
  fputc(';',fp);                    /* Write GIF file terminator */

  if (ferror(fp)) return 0;

  return (1);
}

/******************************/
static void putword(w, fp)
int w;
FILE *fp;
{
  /* writes a 16-bit integer in GIF order (LSB first) */
  fputc(w & 0xff, fp);
  fputc((w>>8)&0xff, fp);
}

/********************************************************
  Bump the 'curx' and 'cury' to point to the next pixel */

static
int BumpPixel()
{
	curx++;

	if( curx == width ) {
		curx = 0;

		counter++;

		if( counter > CANCELCOUNTER )
		{
			while( XtAppPending(app) )
			{
				XtAppProcessEvent(app, XtIMAll);
			}
			counter = 0;

		}
 
	        if( !Interlace ) 
			cury++;
		else {
		     switch( Pass ) {
	     
	               case 0:
        	          cury += 8;
                	  if( cury >= height ) {
		  		Pass++;
				cury = 4;
		  	  }
                          break;
		  
	               case 1:
        	          cury += 8;
                	  if( cury >= height ) {
		  		Pass++;
				cury = 2;
		  	  }
			  break;
		  
	               case 2:
	                  cury += 4;
	                  if( cury >= height ) {
	                     Pass++;
	                     cury = 1;
	                  }
	                  break;
			  
	               case 3:
	                  cury += 2;
	                  break;
			}
		}
	}
}

/*
 * Return the next pixel from the image
 */
int GIFNextPixel( getpixel )
ifunptr getpixel;
{
	int r;

	if( CountDown == 0 )
		return EOF;

	CountDown--;

	r = ( * getpixel )( curx, cury );

	BumpPixel();

	return r;
}

