/*
 * xvxbm.c - load routine for X11 Bitmap format pictures
 *
 * LoadXBM(fname)  -  loads an X11 Bitmap file\
 * WriteXBM(fp, pic, w, h)
 */

#include "chpower.h"



/*
 * File Format:
 *   (format identifier:  "#define" as first couple chars in file)
 *
 * looks for first line beginning with '#define'
 *   reads "#define identifier width"  (identifier is ignored)
 * looks for next line beginning with '#define'
 *   reads "#define identifier height" (identifier is ignored)
 * looks for next occurence of characters '0x'
 *   read next two chars as two hex digits
 *   move forward to next occurence of '0x'
 *   repeat
 */
 

static int xbmError();


/*******************************************/
int LoadXBM(fname, pinfo)
     char    *fname;
     GRaphe  *pinfo;
{
  /* returns '1' on success */

  FILE  *fp;
  int    c, c1;
  int    i, j, k, bit, w, h;
  byte  *pix, *pic8;
  long   filesize;
  char   line[256], name[256];
  byte   hex[256];

  k = 0;

  fp = My_path_open(getenv(PIC_FILE_PATH), fname, "r");
  if (!fp) return (xbmError(fname, "couldn't open file"));

  fseek(fp, 0L, 2);
  filesize = ftell(fp);
  fseek(fp, 0L, 0);


  /* read width:  skip lines until we hit a #define */
  while (1) {
    if (!fgets(line,256,fp)) 
      return(xbmError(fname, "EOF reached in header info."));

    if (strncmp(line,"#define",    (size_t) 7)==0 &&
	sscanf(line,"#define %s %d", name, &w)==2 &&
	strstr(name, "_width") != NULL) break;
  }


  /* read height:  skip lines until we hit another #define */
  while (1) {
    if (!fgets(line,256,fp)) 
      return(xbmError(fname, "EOF reached in header info."));
    
    if (strncmp(line,"#define",    (size_t) 7)==0 &&
	sscanf(line,"#define %s %d", name, &h)==2 &&
	strstr(name, "_height") != NULL) break;
  }



  /* scan forward until we see the first '0x' */
  c = getc(fp);  c1 = getc(fp);
  while (c1!=EOF && !(c=='0' && c1=='x') ) { c = c1;  c1 = getc(fp); }

  if (c1==EOF) 
    return(xbmError(fname, "No bitmap data found"));

  if (w<1 || h<1 || w>10000 || h>10000) 
    return(xbmError(fname, "not an XBM file"));
  
  pic8 = (byte *) calloc((size_t) w*h, (size_t) 1);
  if (!pic8) return(xbmError(fname, "couldn't malloc 'pic8'"));

  /* load up the pinfo structure */
  pinfo->pic = pic8;
  pinfo->w = w;     
  pinfo->h = h;
  pinfo->picType = PIC8;

  /* B/W bitmaps have a two entry colormap */
  pinfo->r[0] = pinfo->g[0] = pinfo->b[0] = 255;     /* 0 = white */
  pinfo->r[1] = pinfo->g[1] = pinfo->b[1] = 0;       /* 1 = black */


  /* initialize the 'hex' array for zippy ASCII-hex -> int conversion */

  for (i=0; i<256; i++) hex[i] = 255;   /* flag 'undefined' chars */
  for (i='0'; i<='9'; i++) hex[i] = i - '0';
  for (i='a'; i<='f'; i++) hex[i] = i + 10 - 'a';
  for (i='A'; i<='F'; i++) hex[i] = i + 10 - 'A';

  /* read/convert the image data */

  for (i=0, pix=pic8; i<h; i++)
    for (j=0,bit=0; j<w; j++, pix++, bit = (++bit)&7) {

      if (!bit) {
	/* get next byte from file.  we're already positioned at it */
	c = getc(fp);  c1 = getc(fp);
	if (c<0 || c1<0) { 
	  /* EOF: break out of loop */	  
	  c=c1='0'; i=h; j=w;
	  xbmError(fname, "The file would appear to be truncated.");
	}

	if (hex[c1] == 255) {  
	  if (hex[c] == 255) k = 0;   /* no digits after the '0x' ... */
	  else k = hex[c];
	}
	else k = (hex[c] << 4) + hex[c1];

	/* advance to next '0x' */
	c = getc(fp);  c1 = getc(fp);
	while (c1!=EOF && !(c=='0' && c1=='x') ) { c = c1;  c1 = getc(fp); }
      }

      *pix = (k&1) ? 1 : 0;
      k = k >> 1;
    }

  fclose(fp);

  return 1;
}  



/*******************************************/
static int xbmError(fname, st)
     char *fname, *st;
{
  char msg[100];

  sprintf(msg,"%s: %s", fname, st);
  ErrorPopup(msg);
  return 0;
}

