/*
 * Copyright 1995, 1996, 1997 by Jun Yei.       All Rights Reserved
 *
 * Permission to retain, use, modify, copy, and distribute ChinesePower
 * in source or binary and its documentation (hereafter, the Software)
 * for noncommercial purpose is hereby granted to you without a fee,
 * provided that this entire copyright and permission notice appear in
 * all such copies, that no charge be associated with such copies,
 * that distribution of derivative works (including value-added
 * distributions such as with additional input dictionaries or fonts)
 * include clarification that such added or derived parts are not from
 * the original Software, and that the names of the author(s) not be
 * used to endorse or promote such works. The author(s) of the software
 * makes no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * THE AUTHOR(S) OF THE SOFTWARE DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 * IN NO EVENT SHALL THE AUTHOR(S) OF THE SOFTWARE BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Jun Yei, Ottawa, ON, Canada
 */

#include "chpower.h"

void InputAreaExposeCB();
void InputAreaResizeCB();
void MiscInputCB();
void InputModeChangeCB();
void InputModeChange();

XtIntervalId HzTimerId = (XtIntervalId) NULL;

SoftKey *SoftKeyType1 = NULL;
SoftKey *SoftKeyType2 = NULL;

HZinputmod MyHzInMod;
VIewportdata *active_viewport=NULL;

char *KeySysType1Name=NULL;
char *KeySysType2Name=NULL;

HZdata KeySysHzdata;

void SetInFocusToHz()
{

	XmProcessTraversal(MyHzInMod.InputWin.screen.inputwidg, XmTRAVERSE_CURRENT);
}

int InitOneSoftKey(fp, softkeylist)
FILE *fp;
SoftKey *softkeylist;  
{
	char line[200];
	char *cp, *tmp;
	SoftKeyMem *softkeymem, *search;

	if((cp=fgets(line,199, fp)) != NULL)
	{
		if(strncmp(HEIGHT, cp, strlen(HEIGHT)) == 0)
		{
			cp += strlen(HEIGHT) + 1;
			for(tmp = cp; *tmp != '\n' && *tmp != '\0'; tmp++);
			*tmp = '\0';
			sscanf(cp, "%d", &softkeylist->height);
			if((cp=fgets(line,199, fp)) == NULL) return;
		}
		else softkeylist->height = 4;
	
		if(strncmp(BEGIN, cp, strlen(BEGIN)) == 0)
		{
			while((cp=fgets(line,199, fp)) != NULL)
			{
				if(strncmp(END, cp, strlen(END)) == 0) break;
				
				softkeymem = (SoftKeyMem *)malloc(sizeof(SoftKeyMem));
				if(softkeymem != NULL)
				{
					for(tmp = cp; *tmp != '\n' && *tmp != '\0'; tmp++);
					*tmp = '\0';
					softkeymem->KeyMem = strdup(cp);
					softkeymem->Next = NULL;
					softkeymem->NumofKey = 0;
					tmp = softkeymem->KeyMem;

					while(*tmp != '\0')
					{
						if(*tmp == ' ')
						{
							softkeymem->NumofKey++;
							*tmp = '\0';
						}
						
						tmp++;	
					}
				
					softkeymem->NumofKey++;

					if(softkeylist->MemRoot == NULL) softkeylist->MemRoot = softkeymem;
					else
					{
						for(search = softkeylist->MemRoot; search->Next != NULL; search =
						search->Next);
						search->Next = softkeymem;
					}
				}
			}
			return 0;	
		}
	}
	return 1;
}

void InitSoftKey(fp, keyname)
FILE *fp;
char *keyname;
{
	char line[100];
	char *cp;
	SoftKey *softkeylist, *search;

	if((cp=fgets(line,99, fp)) != NULL)
	{
		if(strncmp(TYPE1, cp, strlen(TYPE1)) == 0 || strncmp(TYPE2, cp, strlen(TYPE2)) == 0)
		{
			softkeylist = (SoftKey *)malloc(sizeof(SoftKey));
			if(softkeylist != NULL)
			{
				softkeylist->KeyName = strdup(keyname);
				softkeylist->MemRoot = NULL;
				softkeylist->Next = NULL;
				if(InitOneSoftKey(fp, softkeylist) == 0)
				{
					if(strncmp(TYPE1, cp, strlen(TYPE1)) == 0)
					{
						if(SoftKeyType1 == NULL) SoftKeyType1 = softkeylist;
						else
						{
							for(search = SoftKeyType1; search->Next; search = search->Next);
							search->Next = softkeylist;
						}
					}
					else
					{
						if(SoftKeyType2 == NULL) SoftKeyType2 = softkeylist;
						else
						{
							for(search = SoftKeyType2; search->Next; search = search->Next);
							search->Next = softkeylist;
						}
					}
				}
			}
		}
	}
}

void DestroyTypedSoftKey(softkeyroot)
SoftKey *softkeyroot;
{
	SoftKey *search, *nextone;
	SoftKeyMem *memsearch, *memnextone;      

	search = softkeyroot;
	
	if(search == NULL) return;

	do
	{
		nextone = search->Next;
		free(search->KeyName);	
		memsearch = search->MemRoot;
		
		do
		{
			memnextone = memsearch->Next;
			free(memsearch->KeyMem);
			free(memsearch);
			memsearch = memnextone;
		} while(memsearch != NULL);

		free(search);
		search = nextone;	
	} while(search != NULL);
}

void DestroySoftKey()
{
	DestroyTypedSoftKey(SoftKeyType1);
	DestroyTypedSoftKey(SoftKeyType2);
	XtRemoveEventHandler(MyHzInMod.Root, EnterWindowMask | LeaveWindowMask, False, MiscInputCB, (caddr_t *)NULL);
	if(MyHzInMod.input.buffer != NULL) free(MyHzInMod.input.buffer);
	if(MyHzInMod.InputWin.misc.hz_encoding) free(MyHzInMod.InputWin.misc.hz_encoding);
	if(MyHzInMod.InputWin.misc.it_dirs) free(MyHzInMod.InputWin.misc.it_dirs);
	if(MyHzInMod.InputWin.misc.assoc_files) free(MyHzInMod.InputWin.misc.assoc_files);
}


void HzBtnInput_deleteone_CB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	MyHzInMod.input.HzInputPtr[0] = '\b';
	HZParseInput(&MyHzInMod.cxtermInput, &MyHzInMod.InputWin.screen, 1, MyHzInMod.input.HzInputPtr);
	SetInFocusToHz();
}

void HzBtnInput_delete_CB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	int i;

	if(MyHzInMod.cxtermInput.blockdelnum <= 0) MyHzInMod.cxtermInput.blockdelnum = 1;
	for(i = 0; i < MyHzInMod.cxtermInput.blockdelnum; i++) MyHzInMod.input.HzInputPtr[i] = '\b';
	HZParseInput(&MyHzInMod.cxtermInput, &MyHzInMod.InputWin.screen, MyHzInMod.cxtermInput.blockdelnum, MyHzInMod.input.HzInputPtr);
	MyHzInMod.cxtermInput.blockdelnum = 1;
	SetInFocusToHz();
}

void SoftTypeOneInputCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	MyHzInMod.cxtermInput.blockdelnum = strlen((char *)client_data);
	
	strcpy(MyHzInMod.input.HzInputPtr, (char *)client_data);
	MyHzInMod.input.NumOfInput = HZParseInput(&MyHzInMod.cxtermInput, &MyHzInMod.InputWin.screen, MyHzInMod.cxtermInput.blockdelnum, MyHzInMod.input.HzInputPtr);

	MyHzInMod.input.HzInputPtr[MyHzInMod.input.NumOfInput] = 0;
	MyHzInMod.input.InputPtr[0] = 0;
	DBstrcat(MyHzInMod.input.InputPtr, MyHzInMod.input.HzInputPtr);
	MyHzInMod.input.NumOfInput = DBlen(MyHzInMod.input.InputPtr);

	if(MyHzInMod.input.NumOfInput > 0)
	{
		if(simple_active)
		{
			if(active_simphzwin != NULL) DrawSimpleHzWin(active_simphzwin, SimpleHzWinInput(active_simphzwin, KEYINPUT,
			MyHzInMod.input.InputPtr, MyHzInMod.input.NumOfInput));
		}
		else
		{
			if(active_viewport != NULL) MyDrawPage(active_viewport, Input(active_viewport, KEYINPUT, MyHzInMod.input.InputPtr,
			MyHzInMod.input.NumOfInput));
		}
	}
	SetInFocusToHz();
}

void SKBtnInput_sig_CB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	
	MyHzInMod.input.InputPtr[0] = (unsigned short)client_data;
	
	if(simple_active)
	{
		if(active_simphzwin != NULL) DrawSimpleHzWin(active_simphzwin, SimpleHzWinInput(active_simphzwin, KEYINPUT,
			MyHzInMod.input.InputPtr, 1));
	}
	else
	{
		if(active_viewport != NULL) MyDrawPage(active_viewport, Input(active_viewport, KEYINPUT, MyHzInMod.input.InputPtr, 1));
	}
	SetInFocusToHz();
}

void SKBtnInput_two_CB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	unsigned char *cp = (unsigned char *)client_data;
	
	MyHzInMod.input.InputPtr[0] = *cp++;
	MyHzInMod.input.InputPtr[0] <<= 8;
	MyHzInMod.input.InputPtr[0] |= *cp;
	
	if(simple_active)
	{
		if(active_simphzwin != NULL) DrawSimpleHzWin(active_simphzwin, SimpleHzWinInput(active_simphzwin, KEYINPUT,
			MyHzInMod.input.InputPtr, 1));
	}
	else
	{
		if(active_viewport != NULL) MyDrawPage(active_viewport, Input(active_viewport, KEYINPUT, MyHzInMod.input.InputPtr, 1));
	}
	SetInFocusToHz();
}

void HzBtnInput_clear_CB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
        MyHzInMod.input.HzInputPtr[0] = 0x15;
        MyHzInMod.cxtermInput.blockdelnum = 1;
        HZParseInput(&MyHzInMod.cxtermInput, &MyHzInMod.InputWin.screen, 1, MyHzInMod.input.HzInputPtr);
        SetInFocusToHz();
}

void HzBtnInput_last_CB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	MyHzInMod.input.HzInputPtr[0] = 0x10;
	MyHzInMod.cxtermInput.blockdelnum = 1;
	HZParseInput(&MyHzInMod.cxtermInput, &MyHzInMod.InputWin.screen, 1, MyHzInMod.input.HzInputPtr);
	SetInFocusToHz();
}

void HzBtnInput_dis_enable_CB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	HZimSetParam(&MyHzInMod.cxtermInput, HZmthAct[0]);
	SetInFocusToHz();
}

void SetSoftKeyTyped(keyname, keytype)
char *keyname;
int keytype;
{
	SoftKey *search;
	SoftKeyMem *memsearch;
	int grpnum;
	char *grpchar;
	Widget hzinputBtn, main_window, msgwindow;
	Display *dpy;
	Pixmap pixmap;
	unsigned short sap;

	Arg args[MAX_ARGS];
	register int n;

	if(keytype == KEYTYPE1)
	{
		if(SoftKeyType1 == NULL) return;

		if(KeySysType1Name == NULL) grpnum = 1;
		else grpnum = strcmp(KeySysType1Name, keyname);
	}
	else
	{
		if(SoftKeyType2 == NULL) return;
		
		if(KeySysType2Name == NULL) grpnum = 1;
		else grpnum = strcmp(KeySysType2Name, keyname);
	}

	if(grpnum)
	{
		if(keytype == KEYTYPE1)
		{
			if(KeySysType1Name != NULL) XtDestroyWidget(MyHzInMod.Type1KeyWgd);
			
			KeySysType1Name = NULL;

			for(search = SoftKeyType1; strcmp(search->KeyName, keyname) != 0 && search->Next != NULL; search = search->Next);
			if(strcmp(search->KeyName, keyname) != 0)
			{
				for(search = SoftKeyType1; strcmp(search->KeyName, BASIC) != 0 && search->Next != NULL; search =
				search->Next);
				if(strcmp(search->KeyName, BASIC) != 0) return;
			}	
		}
		else
		{
			if(KeySysType2Name != NULL) XtDestroyWidget(MyHzInMod.Type2KeyWgd);

			KeySysType2Name = NULL;

			for(search = SoftKeyType2; strcmp(search->KeyName, keyname) != 0 && search->Next != NULL; search =
			search->Next);

			if(strcmp(search->KeyName, keyname) != 0) return;
		}

		memsearch = search->MemRoot;

		if(memsearch == NULL) return;
		
		BusyWindows();
		
		n = 0;
		XtSetArg(args[n], XmNmarginWidth, 0); n++;
		XtSetArg(args[n], XmNmarginHeight,0); n++;
		XtSetArg(args[n], XmNspacing, 0); n++;
		XtSetArg(args[n], XmNshadowThickness, 0); n++;
		
		if(keytype == KEYTYPE1)
		{
			main_window = XmCreateForm(MyHzInMod.Type1Wgd, "main", args, n);
			MyHzInMod.Type1KeyWgd = main_window;
			KeySysType1Name = search->KeyName;	
			
			MyHzInMod.key1height = search->height * 20 + 10;

			n = 0;
			XtSetArg(args[n], XmNheight, MyHzInMod.topheight + MyHzInMod.key1height + MyHzInMod.key2height); n++;
			XtSetValues(MyHzInMod.PopUpShell, args, n);
			
			n = 0;
			XtSetArg(args[n], XmNheight, MyHzInMod.key1height); n++;
			XtSetValues(MyHzInMod.Type1Wgd, args, n);
		}
		else
		{
			main_window = XmCreateForm(MyHzInMod.Type2Wgd, "main", args, n);
			MyHzInMod.Type2KeyWgd = main_window;
			KeySysType2Name = search->KeyName;	
	
			MyHzInMod.key2height = search->height * 20 + 10;        

			n = 0;
			XtSetArg(args[n], XmNheight, MyHzInMod.topheight + MyHzInMod.key1height + MyHzInMod.key2height); n++;
			XtSetValues(MyHzInMod.PopUpShell, args, n);

			n = 0;
			XtSetArg(args[n], XmNheight, MyHzInMod.key2height); n++;
			XtSetValues(MyHzInMod.Type2Wgd, args, n);
		}
		
		XtManageChild(main_window);

		n = 0;
		XtSetArg(args[n], XmNmarginWidth, 0); n++;
		XtSetArg(args[n], XmNmarginHeight, 0); n++;
		XtSetArg(args[n], XmNorientation, XmHORIZONTAL); n++;
		XtSetArg(args[n], XmNpacking, XmPACK_COLUMN); n++;
		XtSetArg(args[n], XmNspacing, 0); n++;
		XtSetArg(args[n], XmNnumColumns, search->height); n++;
		XtSetArg(args[n], XmNshadowThickness, 0); n++;
		XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
		XtSetArg(args[n], XmNresizePolicy, XmRESIZE_NONE); n++;
		XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
		XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); n++;
		msgwindow = XmCreateRowColumn(main_window, "twocharwindow", args, n);
		XtManageChild(msgwindow);

		do
		{
			grpnum = memsearch->NumofKey;
			grpchar = memsearch->KeyMem;

			for(;grpnum > 0; grpnum--)
			{
				n = 0;
				
				if(keytype == KEYTYPE1 || ISACHZ((unsigned char)*grpchar)) 
				{	
					XtSetArg(args[n], XmNlabelString, XmStringCreate(grpchar, XmSTRING_DEFAULT_CHARSET)); 
					n++;
				}

				XtSetArg(args[n], XmNhighlightThickness, 0); n++;
				XtSetArg(args[n], XmNmarginWidth, 2); n++;
				XtSetArg(args[n], XmNmarginHeight, 2); n++;
				XtSetArg(args[n], XmNfillOnArm, False); n++;
				hzinputBtn = XmCreatePushButtonGadget(msgwindow, grpchar, args, n);
				XtManageChild(hzinputBtn);

				if(keytype != KEYTYPE1 && ISACHZ((unsigned char)*grpchar))
				{
					dpy = XtDisplay(hzinputBtn);
					pixmap = XCreatePixmap(dpy, XtWindow(hzinputBtn), DEFAULT_INPUTBTN_HZSIZE,
					DEFAULT_INPUTBTN_HZSIZE, DisplayPlanes(dpy, DefaultScreen(dpy)));
			
					sap = (unsigned char) *grpchar;
					sap <<= 8;
					sap |= (unsigned char) *(grpchar+1);

					MyDrawCharacter(dpy, pixmap, MyHzInMod.InputWin.screen.reverseGC, 0, 0, sap, &KeySysHzdata, 1);

					n = 0;
					XtSetArg(args[n], XmNlabelType, XmPIXMAP); n++;
					XtSetArg(args[n], XmNlabelPixmap, pixmap); n++;
					XtSetValues(hzinputBtn, args, n);
				
					XtAddCallback(hzinputBtn, XmNactivateCallback, SKBtnInput_two_CB, grpchar);
				}
				else XtAddCallback(hzinputBtn, XmNactivateCallback, SoftTypeOneInputCB, grpchar);
				
				for(; *grpchar != '\0'; grpchar++);
				grpchar++;
			}

			n = 0;
			XtSetArg(args[n], XmNorientation, XmVERTICAL); n++;
			XtSetArg(args[n], XmNwidth, 10); n++;
			XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
			XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
			XtSetArg(args[n], XmNleftWidget, msgwindow); n++;
			XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); n++;
			hzinputBtn = XmCreateSeparator(main_window, "s1", args, n);
			XtManageChild(hzinputBtn);

			n = 0;
			XtSetArg(args[n], XmNmarginWidth, 0); n++;
			XtSetArg(args[n], XmNmarginHeight, 0); n++;
			XtSetArg(args[n], XmNorientation, XmHORIZONTAL); n++;
			XtSetArg(args[n], XmNpacking, XmPACK_COLUMN); n++;
			XtSetArg(args[n], XmNspacing, 0); n++;
			XtSetArg(args[n], XmNnumColumns, search->height); n++;
			XtSetArg(args[n], XmNshadowThickness, 0); n++;
			XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
			XtSetArg(args[n], XmNleftWidget, hzinputBtn); n++;
			XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
			XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); n++;
			msgwindow = XmCreateRowColumn(main_window, "twocharwindow", args, n);
			XtManageChild(msgwindow);
	
			memsearch = memsearch->Next;

		} while(memsearch != NULL);

		n = 0;
		XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
		XtSetValues(msgwindow, args, n);

		if(keytype == KEYTYPE1)
		{
			n = 0;
			XtSetArg(args[n], XmNlabelString, XmStringCreate("<-", XmSTRING_DEFAULT_CHARSET)); n++;
			XtSetArg(args[n], XmNhighlightThickness, 0); n++;
			XtSetArg(args[n], XmNfillOnArm, False); n++;
			hzinputBtn = XmCreatePushButtonGadget(msgwindow, "<-", args, n);
			XtManageChild(hzinputBtn);
			XtAddCallback(hzinputBtn, XmNactivateCallback, HzBtnInput_delete_CB, NULL);

			n = 0;
			XtSetArg(args[n], XmNlabelString, XmStringCreate("Clear", XmSTRING_DEFAULT_CHARSET)); n++;
			XtSetArg(args[n], XmNhighlightThickness, 0); n++;
			XtSetArg(args[n], XmNfillOnArm, False); n++;
			hzinputBtn = XmCreatePushButtonGadget(msgwindow, "clear", args, n);
			XtManageChild(hzinputBtn);
			XtAddCallback(hzinputBtn, XmNactivateCallback, HzBtnInput_clear_CB, NULL);

			n = 0;
			XtSetArg(args[n], XmNlabelString, XmStringCreate("Last", XmSTRING_DEFAULT_CHARSET)); n++;
			XtSetArg(args[n], XmNhighlightThickness, 0); n++;
			XtSetArg(args[n], XmNfillOnArm, False); n++;
			hzinputBtn = XmCreatePushButtonGadget(msgwindow, "last", args, n);
			XtManageChild(hzinputBtn);
			XtAddCallback(hzinputBtn, XmNactivateCallback, HzBtnInput_last_CB, NULL);

			n = 0;
			XtSetArg(args[n], XmNlabelString, XmStringCreate("Dis/Enable", XmSTRING_DEFAULT_CHARSET)); n++;
			XtSetArg(args[n], XmNhighlightThickness, 0); n++;
			XtSetArg(args[n], XmNfillOnArm, False); n++;
			hzinputBtn = XmCreatePushButtonGadget(msgwindow, "dis/enable", args, n);
			XtManageChild(hzinputBtn);
			XtAddCallback(hzinputBtn, XmNactivateCallback, HzBtnInput_dis_enable_CB, NULL);
		}
		else
		{
			n = 0;
			XtSetArg(args[n], XmNlabelString, XmStringCreate("<-", XmSTRING_DEFAULT_CHARSET)); n++;
			XtSetArg(args[n], XmNhighlightThickness, 0); n++;
			XtSetArg(args[n], XmNfillOnArm, False); n++;
			hzinputBtn = XmCreatePushButtonGadget(msgwindow, "<-", args, n);
			XtManageChild(hzinputBtn);
			XtAddCallback(hzinputBtn, XmNactivateCallback, SKBtnInput_sig_CB, (caddr_t *)'\b');

			n = 0;
			XtSetArg(args[n], XmNlabelString, XmStringCreate("Space", XmSTRING_DEFAULT_CHARSET)); n++;
			XtSetArg(args[n], XmNhighlightThickness, 0); n++;
			XtSetArg(args[n], XmNfillOnArm, False); n++;
			hzinputBtn = XmCreatePushButtonGadget(msgwindow, "space", args, n);
			XtManageChild(hzinputBtn);
			XtAddCallback(hzinputBtn, XmNactivateCallback, SKBtnInput_sig_CB, (caddr_t *)' ');

			n = 0;
			XtSetArg(args[n], XmNlabelString, XmStringCreate("HzSpace", XmSTRING_DEFAULT_CHARSET)); n++;
			XtSetArg(args[n], XmNhighlightThickness, 0); n++;
			XtSetArg(args[n], XmNfillOnArm, False); n++;
			hzinputBtn = XmCreatePushButtonGadget(msgwindow, "space", args, n);
			XtManageChild(hzinputBtn);
			XtAddCallback(hzinputBtn, XmNactivateCallback, SKBtnInput_two_CB, (caddr_t *)GlobalDefaults.space);

			n = 0;
			XtSetArg(args[n], XmNlabelString, XmStringCreate("Enter", XmSTRING_DEFAULT_CHARSET)); n++;
			XtSetArg(args[n], XmNhighlightThickness, 0); n++;
			XtSetArg(args[n], XmNfillOnArm, False); n++;
			hzinputBtn = XmCreatePushButtonGadget(msgwindow, "enter", args, n);
			XtManageChild(hzinputBtn);
			XtAddCallback(hzinputBtn, XmNactivateCallback, SKBtnInput_sig_CB, (caddr_t *)'\n');
		}
	
		UnBusyWindows();
	}
}


static void MakeIMList(cxin, hzinputdir, list, pnum)
CXtermInputModule *cxin;
char *hzinputdir;
String list[];
int *pnum;
{
	static char listbuffer[150];
	char *ptr;
	int i;

	*pnum = 0;
	ptr = listbuffer;

	for (i = 0; i < cxin->numHZim; i++) 
	{
		if (cxin->imtbl[i].type == im_type_Builtin)
		{
			strcpy( ptr, cxin->imtbl[i].name );
			list[(*pnum)++] = ptr;
			ptr += strlen(ptr) + 1;
		}
	}

	if (! HZfindsuffix(CIT_SUFFIX, hzinputdir, ptr)) return;
	
	list[(*pnum)++] = ptr;
	ptr += strlen(ptr) + 1;
	
	while ( HZfindsuffix(NULL, NULL, ptr) )
	{
		list[(*pnum)++] = ptr;
		ptr += strlen(ptr) + 1;
	}
}

void SetHZtogBtn()
{
	int i, j;
	Arg args[1];

	switch(MyHzInMod.cxtermInput.chzim->m.auto_select)
	{
		case HZ_AUTOSELECT_ALWAYS: j = 0; break;
		case HZ_AUTOSELECT_WHENNOMATCH: j = 1; break;
		default: j = 2;
	}

	for(i = 0; i < 3; i++)
	{
		if(i == j) XtSetArg(args[0], XmNset, True);
		else XtSetArg(args[0], XmNset, False);
		XtSetValues(MyHzInMod.AutoSelWgd[i], args, 1);
	}

	if(MyHzInMod.cxtermInput.chzim->m.do_auto_segment) XtSetArg(args[0], XmNset, True);
	else XtSetArg(args[0], XmNset, False);
	XtSetValues(MyHzInMod.AutoSegWgd, args, 1);

	if(MyHzInMod.cxtermInput.chzim->m.do_ps_assoc) XtSetArg(args[0], XmNset, True);
	else XtSetArg(args[0], XmNset, False);
	XtSetValues(MyHzInMod.AssociaWgd, args, 1);
}

void HzSetMthdCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	int i = (int) client_data;

	SetHZinMethod(&MyHzInMod.cxtermInput, &MyHzInMod.InputWin, MyHzInMod.im_list[i], True);
	SetHZtogBtn();	
	SetSoftKeyTyped(MyHzInMod.im_list[i], KEYTYPE1);
}

void HzsoftKeyCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	SetSoftKeyTyped((char *)client_data, KEYTYPE2);
}


void HzInputCloseCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	if(MyHzInMod.mapped == 0)
	{
		XtUnmapWidget(MyHzInMod.PopUpShell);
		MyHzInMod.mapped = 1;
	}
}

void HzSetAutoSeCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	char line[30];
	int i, j;
	Arg args[1];

	sprintf(line, "auto-select=%s", (char *)client_data);	
	HZimSetParam(&MyHzInMod.cxtermInput, line);

	switch(MyHzInMod.cxtermInput.chzim->m.auto_select)
	{
		case HZ_AUTOSELECT_ALWAYS: j = 0; break;
		case HZ_AUTOSELECT_WHENNOMATCH: j = 1; break;
		default: j = 2;
	}

	for(i = 0; i < 3; i++)
	{
		if(i == j) XtSetArg(args[0], XmNset, True);
		else XtSetArg(args[0], XmNset, False);
		XtSetValues(MyHzInMod.AutoSelWgd[i], args, 1);
	}
}

void HzSetAutoSegCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	Arg args[1];

	if(MyHzInMod.cxtermInput.chzim->m.do_auto_segment)
		HZimSetParam(&MyHzInMod.cxtermInput, "auto-segment=No");
	else	HZimSetParam(&MyHzInMod.cxtermInput, "auto-segment=Yes");
	
	if(MyHzInMod.cxtermInput.chzim->m.do_auto_segment) XtSetArg(args[0], XmNset, True);
	else XtSetArg(args[0], XmNset, False);
	
	XtSetValues(MyHzInMod.AutoSegWgd, args, 1);
}

void HzSetAssociaCB(w, client_data, call_data)  
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	Arg args[1];

	if(MyHzInMod.cxtermInput.chzim->m.do_ps_assoc)
		HZimSetParam(&MyHzInMod.cxtermInput, "association=No");
	else	HZimSetParam(&MyHzInMod.cxtermInput, "association=Yes");
	
	if(MyHzInMod.cxtermInput.chzim->m.do_ps_assoc) XtSetArg(args[0], XmNset, True);
	else XtSetArg(args[0], XmNset, False);
	
	XtSetValues(MyHzInMod.AssociaWgd, args, 1);
}

void MiscInputCB(w, client_data, event)
Widget w;
XtPointer client_data;
XEvent *event;
{
	VIewportdata *viewport;
	SImpleHzWin *sviewport;

	viewport = active_viewport;
	sviewport = active_simphzwin;

	if(simple_active && sviewport == NULL) return;
	if(!simple_active && viewport == NULL) return;

	if(event->xcrossing.detail == NotifyInferior) return;

	switch(event->type)
	{
		case EnterNotify:
		
		if(simple_active)
		{
			if(HzTimerId == (XtIntervalId) NULL)
			{
				HzTimerId =
				XtAppAddTimeOut(XtWidgetToApplicationContext(sviewport->resource.WorkArea),500,
				HzSimCurFlash, sviewport);
			}
		}
		else
		{
			if(HzTimerId == (XtIntervalId) NULL)
			{
				HzTimerId =
				XtAppAddTimeOut(XtWidgetToApplicationContext(viewport->resource.WorkArea),
				500, HzCurFlash, viewport);
			}	
		}
		break;

		case LeaveNotify:
	
		if(simple_active) ShwDrawCursor(sviewport, False);
		else DrawMyCursor(viewport, False);
		
		if(HzTimerId)
		{
			XtRemoveTimeOut(HzTimerId);
			HzTimerId = (XtIntervalId) NULL;
		}	
	}
}

void InputPhraseOpCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	int i;

	if((int)client_data == 3)
	{
		if(MyHzInMod.assolstchg) {
			if(SaveAssocList(&MyHzInMod.cxtermInput, MyHzInMod.InputWin.misc.assoc_files, MyHzInMod.InputWin.misc.it_dirs))
			{
				MyHzInMod.assolstchg = False;
				ErrorPopup("Phrase File is saved!");
			}
			else
			{
				ErrorPopup("Error while Saving!");
			}
		}
		else
		{
			ErrorPopup("Phrase is not changed!");
		}
		return;
	}

	if(MyHzInMod.cxtermInput.hzal == NULL || MyHzInMod.cxtermInput.hzal->phrases == NULL)
	{
		ErrorPopup("No Phrase File Defined!"); return;
	}

	switch((int)client_data)
	{
		case 0:
		GetSelection(MyHzInMod.PopUpShell, CurrentTime, 2, PhraseAddSelReceived);
		break;

		case 1:
		GetSelection(MyHzInMod.PopUpShell, CurrentTime, 2, PhraseDelSelReceived);
		break;

		case 2:
		GetSelection(MyHzInMod.PopUpShell, CurrentTime, 2, PhraseChkSelReceived);
		break;
	}
}


void PhraseOperation(selection, length, opid)
unsigned char *selection;
int length;
int opid;
{
	int i;

	if(selection == NULL || length <= 0) 
	{
		ErrorPopup("No Selection!"); return;
	}

	if(length < 4)
	{
		ErrorPopup("Phrase Please!"); return;
	}

	if(length % 2 != 0)
	{
		ErrorPopup("No English Please!"); return;
	}

	for(i = 0; i < length; i += 2)
	{
		if(selection[i] < 0x80) 
		{
			ErrorPopup("No English Please!"); return;
		}
	}

	i = MatchAssocList(MyHzInMod.cxtermInput.hzal, selection);

	switch(opid)
	{
		case 0:
		if(i < 0)
		{
			MyHzInMod.assolstchg = True;
			ModifyAssocList(&MyHzInMod.cxtermInput, selection, i, length, 1);
			ErrorPopup("It is added!");
		}
		else
		{
			ErrorPopup("It already exists!");
		}
		break;

		case 1:
		if(i < 0)
		{
			ErrorPopup("Can not find!");
		}
		else
		{
			MyHzInMod.assolstchg = True;
			ModifyAssocList(&MyHzInMod.cxtermInput, selection, i, length, 0);
			ErrorPopup("It is removed!");
		}
		break;

		case 2:
		if(i < 0) 
		{
			ErrorPopup("Can not find!");
		}
		else
		{
			ErrorPopup("It is found!");
		}
		break;
	}
}

void CreateHzInWindow(parent)
Widget parent;
{
	Widget panewindow, main_window, button;
	Widget msgwindow, hzinputwin, hzinputfr;
	Widget cmdwindow, hzinputBtn, menu_bar;
	Widget submenu, buttons[4], menubarBtn[4], pulldowns[4];
	int i, num;
	long background;
	SoftKey *keysys;

	Arg args[MAX_ARGS];
	register int n;
	
	/* Pane Window */

	n = 0;
	XtSetArg(args[n], XmNmarginWidth, 0); n++;
	XtSetArg(args[n], XmNmarginHeight,0); n++;
	XtSetArg(args[n], XmNborderWidth, 2); n++;
	XtSetArg(args[n], XmNallowResize, True); n++;
	XtSetArg(args[n], XmNinitialResourcesPersistent, False); n++;	
	panewindow = XmCreatePanedWindow(parent, "Paned", args, n);
	XtManageChild(panewindow);
	XtAddCallback(panewindow, XmNdestroyCallback, MainDestroyCB, NULL);
	XtAddEventHandler(panewindow, EnterWindowMask | LeaveWindowMask, False, MiscInputCB, (caddr_t *)NULL);
	MyHzInMod.Root = panewindow;
	
	/* Setup Main Window */
	
	n = 0;
	XtSetArg(args[n], XmNwidth, 600); n++;
	XtSetArg(args[n], XmNheight, 78); n++;
	XtSetArg(args[n], XmNmarginWidth, 0); n++;
	XtSetArg(args[n], XmNmarginHeight,0); n++;
	XtSetArg(args[n], XmNspacing, 0); n++;
	XtSetArg(args[n], XmNshadowThickness, 0); n++;
	XtSetArg(args[n], XmNresizePolicy, XmRESIZE_NONE); n++;
	main_window = XmCreateForm(panewindow, "main", args, n);
	XtManageChild(main_window);

	/* Setup Menu Bar */

	n = 0;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNheight, 16); n++;
	XtSetArg(args[n], XmNresizePolicy, XmRESIZE_NONE); n++;
	menu_bar = XmCreateMenuBar(main_window, "menu_bar", args, n);
	XtManageChild(menu_bar);

	n = 0;
	pulldowns[0] = XmCreatePulldownMenu(menu_bar, "pulldown1", args, n);

	n = 0;
	XtSetArg(args[n], XmNsubMenuId, pulldowns[0]); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Close", XmSTRING_DEFAULT_CHARSET)); n++;
	menubarBtn[0] = XmCreateCascadeButton(menu_bar,"button1", args, n);

	n = 0;
	pulldowns[1] = XmCreatePulldownMenu(menu_bar, "pulldown2", args, n);

	n = 0;
	XtSetArg(args[n], XmNsubMenuId, pulldowns[1]); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Options", XmSTRING_DEFAULT_CHARSET)); n++;
	menubarBtn[1] = XmCreateCascadeButton(menu_bar,"button2", args, n);

	n = 0;
	pulldowns[2] = XmCreatePulldownMenu(menu_bar, "pulldown3", args, n);

	n = 0;
	XtSetArg(args[n], XmNsubMenuId, pulldowns[2]); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Phrases", XmSTRING_DEFAULT_CHARSET)); n++;
	menubarBtn[2] = XmCreateCascadeButton(menu_bar,"button3", args, n);

	n = 0;
	pulldowns[3] = XmCreatePulldownMenu(menu_bar, "pulldown4", args, n);

	n = 0;
	XtSetArg(args[n], XmNsubMenuId, pulldowns[3]); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Encoding", XmSTRING_DEFAULT_CHARSET)); n++;
	menubarBtn[3] = XmCreateCascadeButton(menu_bar,"button4", args, n);

	XtManageChildren(menubarBtn, 4);

	/* Setup Close Pulldown Menu */

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Close", XmSTRING_DEFAULT_CHARSET)); n++;
	buttons[0] = XmCreatePushButtonGadget(pulldowns[0], "button11", args,n);
	XtAddCallback(buttons[0], XmNactivateCallback, HzInputCloseCB, NULL);

	XtManageChildren(buttons, 1);

	/* Setup Phrases Pulldown Menu */

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Add Phrase", XmSTRING_DEFAULT_CHARSET)); n++;
	buttons[0] = XmCreatePushButtonGadget(pulldowns[2], "button41", args,n);
	XtAddCallback(buttons[0], XmNactivateCallback, InputPhraseOpCB, (caddr_t)0);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Delete Phrase", XmSTRING_DEFAULT_CHARSET)); n++;
	buttons[1] = XmCreatePushButtonGadget(pulldowns[2], "button42", args,n);
	XtAddCallback(buttons[1], XmNactivateCallback, InputPhraseOpCB, (caddr_t)1);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Check Phrase", XmSTRING_DEFAULT_CHARSET)); n++;
	buttons[2] = XmCreatePushButtonGadget(pulldowns[2], "button43", args,n);
	XtAddCallback(buttons[2], XmNactivateCallback, InputPhraseOpCB, (caddr_t)2);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Save Phrase", XmSTRING_DEFAULT_CHARSET)); n++;
	buttons[3] = XmCreatePushButtonGadget(pulldowns[2], "button44", args,n);
	XtAddCallback(buttons[3], XmNactivateCallback, InputPhraseOpCB, (caddr_t)3);

	XtManageChildren(buttons, 4);

	/* Setup Encoding Pulldown Menu */
	
	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("GB Encoding", XmSTRING_DEFAULT_CHARSET)); n++;
	buttons[0] = XmCreatePushButtonGadget(pulldowns[3], "button31", args,n);
	XtAddCallback(buttons[0], XmNactivateCallback, InputModeChangeCB, "gb");

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Big5 Encoding", XmSTRING_DEFAULT_CHARSET)); n++;
	buttons[1] = XmCreatePushButtonGadget(pulldowns[3], "button32", args,n);
	XtAddCallback(buttons[1], XmNactivateCallback, InputModeChangeCB, "big5");

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("JIS Encoding", XmSTRING_DEFAULT_CHARSET)); n++;
	buttons[2] = XmCreatePushButtonGadget(pulldowns[3], "button33", args,n);
	XtAddCallback(buttons[2], XmNactivateCallback, InputModeChangeCB, "jis");

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("KS Encoding", XmSTRING_DEFAULT_CHARSET)); n++;
	buttons[3] = XmCreatePushButtonGadget(pulldowns[3], "button34", args,n);
	XtAddCallback(buttons[3], XmNactivateCallback, InputModeChangeCB, "ks");

	XtManageChildren(buttons, 4);

	/* Setup Option Pulldown Menu */

	submenu = (Widget) XmCreatePulldownMenu(pulldowns[1], "Input", NULL, 0);
	
	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Input Method", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNsubMenuId, submenu); n++;
	buttons[0] = XmCreateCascadeButtonGadget(pulldowns[1], "button21", args,n);

	XtManageChild(buttons[0]);

	MakeIMList(&MyHzInMod.cxtermInput, MyHzInMod.InputWin.misc.it_dirs, MyHzInMod.im_list, &num );
	
	MyHzInMod.PlInputMtdWg = submenu;

	for(i = 0; i < num; i++)
	{
		n = 0;
		XtSetArg(args[n], XmNlabelString, XmStringCreate(MyHzInMod.im_list[i], XmSTRING_DEFAULT_CHARSET)); n++;
		button = XmCreatePushButtonGadget(submenu, MyHzInMod.im_list[i], args,n);
		XtAddCallback(button, XmNactivateCallback, HzSetMthdCB, (caddr_t *)i);
		XtManageChild(button);
		MyHzInMod.SubInputMtdWg[i] = button;
	}
	MyHzInMod.SubInputMtdWg[i] = NULL;

	submenu = (Widget) XmCreatePulldownMenu(pulldowns[1], "Softkey", NULL, 0);
	
	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Soft Key", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNsubMenuId, submenu); n++;
	buttons[0] = XmCreateCascadeButtonGadget(pulldowns[1], "button22", args,n);

	XtManageChild(buttons[0]);

	keysys = SoftKeyType2;

	MyHzInMod.PlSoftKeyWg = submenu;
	i = 0;

	while(keysys)
	{
		n = 0;
		XtSetArg(args[n], XmNlabelString, XmStringCreate(keysys->KeyName, XmSTRING_DEFAULT_CHARSET)); n++;
		button = XmCreatePushButtonGadget(submenu, keysys->KeyName, args,n);
		XtAddCallback(button, XmNactivateCallback, HzsoftKeyCB, keysys->KeyName);
		XtManageChild(button);
		keysys = keysys->Next;
		MyHzInMod.SubSoftKeyWg[i] = button;
		i++;	
	}
	MyHzInMod.SubSoftKeyWg[i] = NULL;

	submenu = (Widget) XmCreatePulldownMenu(pulldowns[1], "Auto Selection", NULL, 0);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Auto Selection", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNsubMenuId, submenu); n++;
	buttons[0] = XmCreateCascadeButtonGadget(pulldowns[1], "button23", args,n);

	XtManageChild(buttons[0]);

	num = 2;

	for(i = 0; i < 3; i++)
	{
		n = 0;
		if(i == num) XtSetArg(args[n], XmNset, True);
		else XtSetArg(args[n], XmNset, False);
		n++;
		XtSetArg(args[n], XmNindicatorType, XmONE_OF_MANY); n++;
		XtSetArg(args[n], XmNvisibleWhenOff, True); n++;
		XtSetArg(args[n], XmNlabelString, XmStringCreate(HZautoSlect[i], XmSTRING_DEFAULT_CHARSET)); n++;
		MyHzInMod.AutoSelWgd[i] = XmCreateToggleButtonGadget(submenu, HZautoSlect[i], args,n);
		XtAddCallback(MyHzInMod.AutoSelWgd[i], XmNvalueChangedCallback, HzSetAutoSeCB, HZautoSlect[i]);
	}
	
	XtManageChildren(MyHzInMod.AutoSelWgd, 3);

	n = 0;
	XtSetArg(args[0], XmNset, True); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Auto Seg", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNindicatorType, XmN_OF_MANY); n++;  
	XtSetArg(args[n], XmNvisibleWhenOff, True); n++;  
	MyHzInMod.AutoSegWgd = XmCreateToggleButtonGadget(pulldowns[1], "button24", args,n);
	XtAddCallback(MyHzInMod.AutoSegWgd, XmNvalueChangedCallback, HzSetAutoSegCB, NULL);
	XtManageChild(MyHzInMod.AutoSegWgd);
	
	n = 0;
	XtSetArg(args[0], XmNset, True); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Association", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNindicatorType, XmN_OF_MANY); n++;  
	XtSetArg(args[n], XmNvisibleWhenOff, True); n++; 
	MyHzInMod.AssociaWgd = XmCreateToggleButtonGadget(pulldowns[1], "button25", args,n);
	XtAddCallback(MyHzInMod.AssociaWgd, XmNvalueChangedCallback, HzSetAssociaCB, NULL);
	XtManageChild(MyHzInMod.AssociaWgd);

	/* Setup HZ input window */

	n = 0;
	XtSetArg(args[n], XmNmarginWidth, 0); n++;
	XtSetArg(args[n], XmNmarginHeight, 0); n++;
	XtSetArg(args[n], XmNborderWidth, 2); n++;
	XtSetArg(args[n], XmNshadowThickness, 2); n++;    
	XtSetArg(args[n], XmNshadowType, XmSHADOW_IN); n++;
	XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(args[n], XmNtopWidget, menu_bar); n++;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNheight, 55); n++;
	XtSetArg(args[n], XmNresizePolicy, XmRESIZE_NONE); n++;
	hzinputfr = XmCreateFrame(main_window, "hzinputframe", args, n);
	XtManageChild(hzinputfr);

	n = 0;
	XtSetArg(args[n], XmNmarginWidth, 0); n++;
	XtSetArg(args[n], XmNmarginHeight, 0); n++;
	XtSetArg(args[n], XmNborderWidth, 0); n++;
	XtSetArg(args[n], XmNshadowThickness, 0); n++;
	XtSetArg(args[n], XmNshadowType, XmSHADOW_IN); n++;
	XtSetArg(args[n], XmNheight, 40); n++;
	XtSetArg(args[n], XmNresizePolicy, XmRESIZE_NONE); n++;
	hzinputwin = XmCreateDrawingArea(hzinputfr, "hzinputwindow", args, n);
	XtManageChild(hzinputwin);
	MyHzInMod.InputWin.screen.inputwidg = hzinputwin;
	XtAddCallback(hzinputwin, XmNinputCallback, HzInputAreaInputCB, NULL);
	XtOverrideTranslations(hzinputwin, XtParseTranslationTable("<Key>Tab: FakeWorkAreaInputCB()"));

	MyHzInMod.topheight = 80;

	/* Setup Type1 Softkey Window*/

	n = 0;
	XtSetArg(args[n], XmNwidth, 600); n++;
	XtSetArg(args[n], XmNheight,100); n++;
	XtSetArg(args[n], XmNmarginWidth, 0); n++;
	XtSetArg(args[n], XmNmarginHeight,0); n++;
	XtSetArg(args[n], XmNborderWidth, 0); n++;
	XtSetArg(args[n], XmNshadowThickness, 0); n++;
	XtSetArg(args[n], XmNallowResize, True); n++;  
	MyHzInMod.Type1Wgd = XmCreateFrame(panewindow, "Type1Key", args, n);
	XtManageChild(MyHzInMod.Type1Wgd);
	MyHzInMod.key1height = 110;

	/* Setup Type2 Softkey Window*/

	n = 0;
	XtSetArg(args[n], XmNwidth, 600); n++;
	XtSetArg(args[n], XmNheight, 5); n++;
	XtSetArg(args[n], XmNmarginWidth, 0); n++;
	XtSetArg(args[n], XmNmarginHeight,0); n++;
	XtSetArg(args[n], XmNborderWidth, 0); n++;
	XtSetArg(args[n], XmNshadowThickness, 0); n++;
	XtSetArg(args[n], XmNallowResize, True); n++;  
	MyHzInMod.Type2Wgd = XmCreateFrame(panewindow, "Type2Key", args, n);
	XtManageChild(MyHzInMod.Type2Wgd);
	MyHzInMod.key2height = 10;    

	n = 0;
	XtSetArg(args[n], XmNbackground, &background); n++;
	XtGetValues(hzinputwin, args, n);

	n = 0;
	XtSetArg(args[n], XmNborderColor, background); n++;
	XtSetValues(hzinputfr, args, n);
}

void StartExposeHzIn()
{
	XGCValues gc_init;
	register int n;
	int i;
	Arg args[5];
	Dimension a, b;
	unsigned long foreground, background;
	XColor color, exact;
	Status result;

	n = 0;
	XtSetArg(args[n], XmNbackground, &gc_init.background); n++;
	XtSetArg(args[n], XmNforeground, &gc_init.foreground); n++;
	XtSetArg(args[n], XmNheight, &a); n++;
	XtSetArg(args[n], XmNwidth, &b); n++;
	XtGetValues(MyHzInMod.InputWin.screen.inputwidg, args, n);

	MyHzInMod.InputWin.screen.reverseGC = XtGetGC(MyHzInMod.InputWin.screen.inputwidg,
							GCForeground|GCBackground,
							&gc_init);
	if(GlobalDefaults.color[0])
	{
		result = XAllocNamedColor(MyHzInMod.InputWin.screen.display, 
			 DefaultColormap(MyHzInMod.InputWin.screen.display,
			 DefaultScreen(MyHzInMod.InputWin.screen.display)),
			 GlobalDefaults.color[0],
			 &color,
			 &exact);
			 
		if(result) background = color.pixel;
		else background = gc_init.background;
	} 
	else background = gc_init.background;
	
	if(GlobalDefaults.color[1])
	{
		result = XAllocNamedColor(MyHzInMod.InputWin.screen.display, 
			 DefaultColormap(MyHzInMod.InputWin.screen.display,
			 DefaultScreen(MyHzInMod.InputWin.screen.display)),
			 GlobalDefaults.color[1],
			 &color,
			 &exact);
			 
		if(result) foreground = color.pixel;
		else foreground = gc_init.foreground;
	} 
	else foreground = gc_init.foreground;
	
	gc_init.foreground = foreground;
	gc_init.background = background;

	MyHzInMod.InputWin.screen.normalGC = XCreateGC(MyHzInMod.InputWin.screen.display,
							MyHzInMod.InputWin.screen.window,
							GCForeground|GCBackground, 
							&gc_init);

	XtSetArg(args[0], XmNbackground, background);
	XtSetValues(MyHzInMod.InputWin.screen.inputwidg,  args, 1);

	MyHzInMod.InputWin.screen.height = (int)a;
	MyHzInMod.InputWin.screen.width  = (int)b;

	RealizeCXtermInput(&MyHzInMod.cxtermInput, &MyHzInMod.InputWin);

	XtAddCallback(MyHzInMod.InputWin.screen.inputwidg, XmNexposeCallback, InputAreaExposeCB, NULL);
	XtAddCallback(MyHzInMod.InputWin.screen.inputwidg, XmNresizeCallback, InputAreaResizeCB, NULL);
}


void InputAreaPortExpose()
{
	RefreshCXtermInput(&MyHzInMod.cxtermInput, &MyHzInMod.InputWin.screen);
}

void InputAreaPortResize()
{
	register int n;
	Arg args[5];
	Dimension a, b;

	n = 0;
	XtSetArg(args[n], XmNheight, &a); n++;
	XtSetArg(args[n], XmNwidth, &b); n++;
	XtGetValues(MyHzInMod.InputWin.screen.inputwidg, args, n);

	MyHzInMod.InputWin.screen.height = (int)a;
	MyHzInMod.InputWin.screen.width  = (int)b;


	ResizeCXtermInput(&MyHzInMod.cxtermInput, &MyHzInMod.InputWin.screen);
}

void InitSingleFont(hzdata, fontID, width, height)
HZdata *hzdata;
char *fontID;
int width, height;
{
	hzdata->orighz_swidth = width;
	hzdata->orighz_sheight = height;
	hzdata->hbf = NULL;
	hzdata->FontType = HBFONT;

	hzdata->hbf = GetFontHandle(fontID, &hzdata->FontId, &hzdata->FontType);
	
	if(hzdata->hbf == NULL) 
	{
		if(FontListRoot)
		{
			hzdata->hbf = FontListRoot->Hbf;
			hzdata->FontId = FontListRoot->FontId;
			hzdata->FontType = FontListRoot->FontType;	
		}	
	}

	if(hzdata->hbf == NULL)
	{
		fprintf(stderr, "No Hz Font Available\n");
		hzdata->hz_width = 16;
		hzdata->hz_height = 16;

	}
	else 
	{
		if(hzdata->FontType == HBFONT) HBF_GetBitmapBoundingBox(hzdata->hbf, &(hzdata->hz_width), &(hzdata->hz_height), &width, &height);
		else hzdata->hz_width = hzdata->hz_height = 16;
	}

	HzDataZoom(hzdata, 100);
}

void InitHzdataType2Key()
{
	InitSingleFont(&KeySysHzdata, GlobalDefaults.fontname, DEFAULT_INPUTBTN_HZSIZE, DEFAULT_INPUTBTN_HZSIZE);
}
		
void PreInitHzInWindow()
{
	
	MyHzInMod.mapped = 1;
	MyHzInMod.assolstchg = False;
	InitSingleFont(&MyHzInMod.InputWin.screen.hzdata, GlobalDefaults.fontname, DEFAULT_INPUTWIN_HZSIZE, DEFAULT_INPUTWIN_HZSIZE);
	MyHzInMod.InputWin.screen.height = 32;
	MyHzInMod.InputWin.screen.width  = 400;
	MyHzInMod.input.buffer = NULL;
	InitHzdataType2Key();
	CreateCXtermInput(&MyHzInMod.cxtermInput, &MyHzInMod.InputWin);
}

void InitHzInWindow()
{
	MyHzInMod.InputWin.screen.display = XtDisplay(MyHzInMod.InputWin.screen.inputwidg);
	MyHzInMod.InputWin.screen.window = XtWindow(MyHzInMod.InputWin.screen.inputwidg);

	/* Setup Type1 Softkey for ASCII */

	SetSoftKeyTyped("ASCII", KEYTYPE1);
}

void InputAreaExposeCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	XmDrawingAreaCallbackStruct *mydata = (XmDrawingAreaCallbackStruct *)call_data;
	XEvent *event = mydata->event;
	
	if (event->xexpose.count==0)
		InputAreaPortExpose();
}

void InputAreaResizeCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	InputAreaPortResize();
}

void IniHzInMod(parent)
Widget parent;
{
	MyHzInMod.PopUpShell = XtAppCreateShell("Hz Input Window", "XMdemos1", topLevelShellWidgetClass, XtDisplay(toplevel),
	NULL, 0);
	PreInitHzInWindow();
	CreateHzInWindow(MyHzInMod.PopUpShell);
	XtSetMappedWhenManaged(MyHzInMod.PopUpShell, False);
	XtRealizeWidget(MyHzInMod.PopUpShell);
	InitHzInWindow();
	StartExposeHzIn();
	InputAreaPortResize();
}

void InputTableInitial(Redo)
int Redo;
{
	FILE *fp;
	char line[100];
	char *cp, *tmp, tab_file[30];

	tab_file[0] = '\0';

	strcat(tab_file, INPUT_TABLE_FILE);

	if(MyHzInMod.InputWin.misc.hz_encoding == NULL) MyHzInMod.InputWin.misc.hz_encoding = strdup("GB");
	if(Redo && MyHzInMod.InputWin.misc.it_dirs) 
		free(MyHzInMod.InputWin.misc.it_dirs);
	MyHzInMod.InputWin.misc.it_dirs = NULL;

	cp = (char *) getenv ("HZINPUTDIR");

	if(cp != NULL)
	{
		MyHzInMod.InputWin.misc.it_dirs = (char *) malloc (strlen(cp) + 8);
		*(MyHzInMod.InputWin.misc.it_dirs) = '\0';
		strcat(MyHzInMod.InputWin.misc.it_dirs, cp);
	}

	switch(HZencode(MyHzInMod.InputWin.misc.hz_encoding))
	{
		case GB_ENCODE:
			
		strcat(tab_file, "GB");
		if(MyHzInMod.InputWin.misc.it_dirs) strcat(MyHzInMod.InputWin.misc.it_dirs, "/gb");
		GlobalDefaults.fontname = GlobalDefaults.gbfontname;		
		GlobalDefaults.comma[0] = 0xa3;
		GlobalDefaults.comma[1] = 0xac;	
		GlobalDefaults.period[0] = 0xa1;
		GlobalDefaults.period[1] = 0xa3;
		GlobalDefaults.space[0] = 0xa1;
		GlobalDefaults.space[1] = 0xa1;
		break;
		
		case BIG5_ENCODE:
		
		strcat(tab_file, "BIG5");
		if(MyHzInMod.InputWin.misc.it_dirs) strcat(MyHzInMod.InputWin.misc.it_dirs, "/big5");
		GlobalDefaults.fontname = GlobalDefaults.big5fontname;	
		GlobalDefaults.comma[0] = 0xa1;
		GlobalDefaults.comma[1] = 0x41;
		GlobalDefaults.period[0] = 0xa1;
		GlobalDefaults.period[1] = 0x43;
		GlobalDefaults.space[0] = 0xa1;
		GlobalDefaults.space[1] = 0x40;
		break;
		
		case JIS_ENCODE:
		
		strcat(tab_file, "JIS");
		if(MyHzInMod.InputWin.misc.it_dirs) strcat(MyHzInMod.InputWin.misc.it_dirs, "/jis");
		GlobalDefaults.fontname = GlobalDefaults.jisfontname;	
		GlobalDefaults.comma[0] = 0xa1;
		GlobalDefaults.comma[1] = 0xa4;
		GlobalDefaults.period[0] = 0xa1;
		GlobalDefaults.period[1] = 0xa3;
		GlobalDefaults.space[0] = 0xa1;
		GlobalDefaults.space[1] = 0xa1;
		break;
		
		case KS_ENCODE:
		
		strcat(tab_file, "KS");
		if(MyHzInMod.InputWin.misc.it_dirs) strcat(MyHzInMod.InputWin.misc.it_dirs, "/ks");
		GlobalDefaults.fontname = GlobalDefaults.ksfontname;  
		GlobalDefaults.comma[0] = 0xa3;
		GlobalDefaults.comma[1] = 0xac;
		GlobalDefaults.period[0] = 0xa3;
		GlobalDefaults.period[1] = 0xae;
		GlobalDefaults.space[0] = 0xa1;
		GlobalDefaults.space[1] = 0xa1;
		break;
		
		default:
		
		fprintf(stderr, "Wrong default_encoding value '%s' in the file %s\n", MyHzInMod.InputWin.misc.hz_encoding, CONFIGURE_FILE); 
		if(MyHzInMod.InputWin.misc.hz_encoding) free(MyHzInMod.InputWin.misc.hz_encoding);
		MyHzInMod.InputWin.misc.hz_encoding = strdup("GB");
		strcat(tab_file, ".GB");
		if(MyHzInMod.InputWin.misc.it_dirs) strcat(MyHzInMod.InputWin.misc.it_dirs, "/gb");
		GlobalDefaults.fontname = GlobalDefaults.gbfontname;	
		GlobalDefaults.comma[0] = 0xa3;
		GlobalDefaults.comma[1] = 0xac;
		GlobalDefaults.period[0] = 0xa1;
		GlobalDefaults.period[1] = 0xa3;
		GlobalDefaults.space[0] = 0xa1;
		GlobalDefaults.space[1] = 0xa1;
		break;
	}

	fp = My_path_open(getenv(CONF_FILE_PATH), tab_file, "r");


	if(fp)
	{
		while((cp=fgets(line, 99, fp)))
		{
		
			if(strncmp(SOFTKEYID, cp, strlen(SOFTKEYID)) == 0)
			{
				cp += strlen(SOFTKEYID);
				for(; *cp == ' '; cp++);
				if(*cp == '\n' || *cp == '\0') continue;
			
				for(tmp = cp; *tmp != '\n' && *tmp != '\0'; tmp++);
				*tmp = '\0';
			
				InitSoftKey(fp, cp);
			}
		}

		fclose(fp);
	}
	else
	{
		if(Redo)
		{
			sprintf(line, "Input Table File %s Not Found!", tab_file);
			ErrorPopup(line);
		}
		else
		{
			fprintf(stderr, "Input Table File %s Not Found!\n", tab_file);
		}	
	}
}

void YesSaveMChgCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	if(SaveAssocList(&MyHzInMod.cxtermInput, MyHzInMod.InputWin.misc.assoc_files, MyHzInMod.InputWin.misc.it_dirs))
	{
		ErrorPopup("Phrase is saved!");
	}
	else
	{
		ErrorPopup("Error wile Saving!");
	}

	InputModeChange((char *)client_data);
}

void NoSaveMChgCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	InputModeChange((char *)client_data);
}

void InputModeChangeCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	if(HZencode(MyHzInMod.InputWin.misc.hz_encoding) == HZencode(client_data)) return;

	if(MyHzInMod.assolstchg) 
	{
		MyHzInMod.assolstchg = False;
		ConfirmSaveHZAssoc(client_data, YesSaveMChgCB, NoSaveMChgCB); 
	}
	else
	{
		InputModeChange((char *)client_data);
	}
}

void InputModeChange(newhzmode)
char *newhzmode;
{
	int i, num;
	SoftKey *keysys;
	Widget button;

	Arg args[MAX_ARGS];
	register int n;

	BusyWindows();
	
	DestroyTypedSoftKey(SoftKeyType1);
	DestroyTypedSoftKey(SoftKeyType2);
	SoftKeyType1 = NULL;
	SoftKeyType2 = NULL;
	
	if(KeySysType1Name != NULL) XtDestroyWidget(MyHzInMod.Type1KeyWgd);
	KeySysType1Name = NULL;

	if(KeySysType2Name != NULL) XtDestroyWidget(MyHzInMod.Type2KeyWgd);

	for(i = 0; MyHzInMod.SubInputMtdWg[i] != NULL; i++)
	{
		XtDestroyWidget(MyHzInMod.SubInputMtdWg[i]);
	}

	for(i = 0;  MyHzInMod.SubSoftKeyWg[i] != NULL; i++)
	{
		XtDestroyWidget(MyHzInMod.SubSoftKeyWg[i]);    
	}

	if(MyHzInMod.InputWin.misc.hz_encoding) free(MyHzInMod.InputWin.misc.hz_encoding);
	MyHzInMod.InputWin.misc.hz_encoding = strdup(newhzmode);
	
	InputTableInitial(True);

	InitSingleFont(&MyHzInMod.InputWin.screen.hzdata, GlobalDefaults.fontname, DEFAULT_INPUTWIN_HZSIZE, DEFAULT_INPUTWIN_HZSIZE);
	InitialSimpleHZWin();
	InitHzdataType2Key();
	
	ResetCXtermInput(&MyHzInMod.cxtermInput, &MyHzInMod.InputWin);

	MakeIMList(&MyHzInMod.cxtermInput, MyHzInMod.InputWin.misc.it_dirs, MyHzInMod.im_list, &num );

	for(i = 0; i < num; i++)
	{
		n = 0;
		XtSetArg(args[n], XmNlabelString, XmStringCreate(MyHzInMod.im_list[i], XmSTRING_DEFAULT_CHARSET)); n++;
		button = XmCreatePushButtonGadget(MyHzInMod.PlInputMtdWg, MyHzInMod.im_list[i], args,n);
		XtAddCallback(button, XmNactivateCallback, HzSetMthdCB, (caddr_t *)i);
		XtManageChild(button);
		MyHzInMod.SubInputMtdWg[i] = button;
	}
	MyHzInMod.SubInputMtdWg[i] = NULL;

	keysys = SoftKeyType2;
	i = 0;

	while(keysys)
	{
		n = 0;
		XtSetArg(args[n], XmNlabelString, XmStringCreate(keysys->KeyName, XmSTRING_DEFAULT_CHARSET)); n++;
		button = XmCreatePushButtonGadget(MyHzInMod.PlSoftKeyWg, keysys->KeyName, args,n);
		XtAddCallback(button, XmNactivateCallback, HzsoftKeyCB, keysys->KeyName);
		XtManageChild(button);
		keysys = keysys->Next;
		MyHzInMod.SubSoftKeyWg[i] = button;
		i++;	
	}
	MyHzInMod.SubSoftKeyWg[i] = NULL;

	HzSetMthdCB(MyHzInMod.Root, 0, NULL);

	if(KeySysType2Name)
	{
		KeySysType2Name	= NULL;
		SetSoftKeyTyped(SoftKeyType2->KeyName, KEYTYPE2);
	}

	UnBusyWindows();
}


