/*
 * Copyright 1995, 1996, 1997 by Jun Yei.       All Rights Reserved
 *
 * Permission to retain, use, modify, copy, and distribute ChinesePower
 * in source or binary and its documentation (hereafter, the Software)
 * for noncommercial purpose is hereby granted to you without a fee,
 * provided that this entire copyright and permission notice appear in
 * all such copies, that no charge be associated with such copies,
 * that distribution of derivative works (including value-added
 * distributions such as with additional input dictionaries or fonts)
 * include clarification that such added or derived parts are not from
 * the original Software, and that the names of the author(s) not be
 * used to endorse or promote such works. The author(s) of the software
 * makes no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * THE AUTHOR(S) OF THE SOFTWARE DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 * IN NO EVENT SHALL THE AUTHOR(S) OF THE SOFTWARE BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Jun Yei, Ottawa, ON, Canada
 */

#include "chpower.h"

void checkMotionNotify();
void checkShwMotionNotify();
int checkShwLeaveNotify();
int checkLeaveNotify();
void EditWindowInput();

void HzInputAreaInputCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	EditWindowInput(w, client_data, call_data, HZINPUT);
}

void WorkAreaInputCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	EditWindowInput(w, client_data, call_data, WAINPUT);
}


void FakeWorkAreaInputCB(w, event, params, nparams)
Widget w;
String *params;
Cardinal *nparams;
XEvent *event;
{
	XmDrawingAreaCallbackStruct fakedata;

	fakedata.event = event;
	fakedata.reason = XmCR_INPUT;

	if(active_viewport == NULL) return;

	EditWindowInput(w, active_viewport, &fakedata, WAINPUT);
}

void SimpleHzWinInputCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	EditWindowInput(w, client_data, call_data, SHINPUT);
}

void EditWindowInput(w, client_data, call_data, actid)
Widget w;
caddr_t client_data;
caddr_t call_data;
int actid;
{
	XmDrawingAreaCallbackStruct *mydata;
	VIewportdata *viewport;
	SImpleHzWin *sviewport;
	XEvent *myprt, current;	
	static char buffer[20];
	int bufsize = 20;
	KeySym key;
	int i, count, moved, movedlast;
	unsigned char *btncp;
	unsigned short *tmp, chmem;

	switch(actid)
	{
		case WAINPUT:
		
		viewport = (VIewportdata *)client_data;
		if (active_viewport != viewport) active_viewport = viewport;
		simple_active = False;
		break;

		case SHINPUT:

		sviewport = (SImpleHzWin *)client_data;
		if(active_simphzwin != sviewport) active_simphzwin = sviewport;
		simple_active = True;
		break;

		default:

		if(simple_active) sviewport = active_simphzwin;
		else viewport = active_viewport;
	}

	mydata = (XmDrawingAreaCallbackStruct *)call_data;
	myprt = (XEvent *)mydata->event;

	if(mydata->reason == XmCR_INPUT)
	{
		switch(myprt->type)
		{
			case ButtonPress:
				if(myprt->xbutton.button == Button1)
				{
					switch(actid)
					{
						case WAINPUT:
						
						viewport->pointer.Btn1Pressed = True;
						viewport->pointer.Pointer_x = XmConvertUnits(viewport->resource.WorkArea, XmHORIZONTAL,
						XmPIXELS, myprt->xbutton.x, Xm100TH_POINTS) * 100 / viewport->zoom;
						viewport->pointer.Pointer_y = XmConvertUnits(viewport->resource.WorkArea, XmVERTICAL, XmPIXELS,
						myprt->xbutton.y, Xm100TH_POINTS) * 100 / viewport->zoom;

						if(!viewport->horOrder) 
						{
							i = viewport->pointer.Pointer_x;
							viewport->pointer.Pointer_x = viewport->pointer.Pointer_y;
							viewport->pointer.Pointer_y = i;
						}
 
						if(viewport->cursor.StartCurPtr != viewport->cursor.EndCurPtr) 
						{
							MyDrawPage(viewport, UNHIGHLITE);
						}
						MyDrawPage(viewport, CURSEARCH);
						viewport->cursor.MoveCurPage = viewport->cursor.ViewPortCurPage;
						viewport->cursor.MoveCur_x = viewport->cursor.ViewPortCur_x;
						viewport->pointer.DrawCur_x =  (viewport->horOrder ? viewport->cursor.ViewPortCur_x : viewport->cursor.ViewPortCur_y); 
						viewport->cursor.MoveCur_y = viewport->cursor.ViewPortCur_y;
						viewport->cursor.MoveCurPtr = viewport->cursor.CurPtr;
						viewport->cursor.StartCurPtr = viewport->cursor.CurPtr;
						viewport->cursor.EndCurPtr = viewport->cursor.CurPtr;
						viewport->cursor.LastStartCurPtr = viewport->cursor.CurPtr;
						viewport->cursor.LastEndCurPtr = viewport->cursor.CurPtr;
						viewport->cursor.Mohzdata = viewport->cursor.hzdata;
						viewport->cursor.MoveColNum = viewport->cursor.ColNum;
						viewport->cursor.MoveLineAlignment = viewport->cursor.LineAlignment;
						viewport->cursor.MoveColorNum = viewport->cursor.ColorNum;
						viewport->cursor.MoveUnderLine = viewport->cursor.UnderLine;
						findset.startedfind = False;

						while(True)
						{
							moved = False;
							
							do 
							{ 
							  movedlast = moved;
							  moved = XCheckTypedEvent(viewport->resource.dpy, MotionNotify, &current); 
							}
							while(moved);

							if(movedlast) checkMotionNotify(viewport, &current);
							
							if(XCheckTypedEvent(viewport->resource.dpy, LeaveNotify, &current))
							{
								viewport->cursor.MoveDir = DOWN;
								
								if(current.xmotion.y <= 0) viewport->cursor.MoveDir = UP;
								else 
								{
									if(current.xmotion.x <= 0) viewport->cursor.MoveDir = LEFT;
									else 
									{
										if(current.xmotion.x >= XmConvertUnits(viewport->resource.WorkArea,
										XmHORIZONTAL, Xm100TH_POINTS, viewport->origpage.ViewPortWidth,
										XmPIXELS)) viewport->cursor.MoveDir = RIGHT;
									}
								}

								if(checkLeaveNotify(viewport) == ButtonRelease) break;
							}
						
							if(XCheckTypedEvent(viewport->resource.dpy, ButtonRelease, &current)) break;	
						}

						break;

						case SHINPUT:

						if(sviewport->cursor.StartPos != sviewport->cursor.EndPos) 
						{
							DrawSimpleHzWin(sviewport, UNHIGHLITE);
						}
						ShwDrawCursor(sviewport, True);
						sviewport->cursor.CurPos = myprt->xbutton.x * 2 / SHZW_FONT_SIZE + sviewport->viewp.firstnum;
						DrawSimpleHzWin(sviewport, CURSEARCH);	
						ShwDrawCursor(sviewport, False);
						sviewport->cursor.MovePos = sviewport->cursor.CurPos;
						sviewport->cursor.StartPos = sviewport->cursor.CurPos;
						sviewport->cursor.EndPos = sviewport->cursor.CurPos;
						sviewport->cursor.DBMovePos = sviewport->cursor.DBCurPos;
						sviewport->cursor.DBStartPos = sviewport->cursor.DBCurPos;
						sviewport->cursor.DBEndPos = sviewport->cursor.DBCurPos;
						sviewport->cursor.LastStartPos = sviewport->cursor.CurPos;
						sviewport->cursor.LastEndPos = sviewport->cursor.CurPos;

						while(True)
						{
							moved = False;
							do
							{
								movedlast = moved;
								moved = XCheckTypedEvent(sviewport->resource.dpy, MotionNotify, &current);
							} while(moved);

							if(movedlast) checkShwMotionNotify(sviewport, &current);

							if(XCheckTypedEvent(sviewport->resource.dpy, LeaveNotify, &current))
							{
								if(current.xmotion.y >= 0 && current.xmotion.y <= sviewport->viewp.height)
								{
									sviewport->cursor.MoveDir = RIGHT;

									if(current.xmotion.x <= 0) sviewport->cursor.MoveDir = LEFT;

									if(checkShwLeaveNotify(sviewport) == ButtonRelease) break;
								}
							}

							if(XCheckTypedEvent(sviewport->resource.dpy, ButtonRelease, &current)) break;
						}

						break;
					}							
				}
				else 
				{
					if(myprt->xbutton.button == Button2 && actid != HZINPUT) 
					{
						if(viewport != NULL && simple_active == False)
							GetSelection(viewport->resource.WorkArea, myprt->xbutton.time, 2, SelectionReceived);
						else
						{
							if(sviewport != NULL && simple_active == True)
								GetSelection(sviewport->resource.WorkArea, myprt->xbutton.time, 2, SelectionReceived);
						}
					
					}
					else
					{
						if(actid == WAINPUT && myprt->xbutton.button == Button3 && viewport != NULL)
						{
							XmMenuPosition(viewport->resource.Popup, (XButtonPressedEvent *)myprt);
							XtManageChild(viewport->resource.Popup);
						}
					}
					break;
				}
				myprt = &current;

			case ButtonRelease:
				if(myprt->xbutton.button == Button1)
				{
					switch(actid)
					{
						case WAINPUT:
						
						viewport->pointer.Btn1Pressed = False;
						if(viewport->cursor.StartCurPtr != viewport->cursor.EndCurPtr)
						{
							unsigned short *DBprt;

							RenewCharSet(viewport);
							
							chmem = *viewport->cursor.EndCurPtr;
							*viewport->cursor.EndCurPtr = '\0';	
							
							DBprt = (unsigned short *)DBdup(viewport->cursor.StartCurPtr);
							
							*viewport->cursor.EndCurPtr = chmem;
						
							if(DBprt != NULL)
							{
								tmp = viewport->cursor.EndCurPtr;
								viewport->cursor.EndCurPtr = viewport->cursor.StartCurPtr;
								
								DisownSelection();
								
								viewport->cursor.EndCurPtr = tmp;
								
								i = rmtypedcomponent(DBprt, DBprt + (int)(viewport->cursor.EndCurPtr -
								viewport->cursor.StartCurPtr), UnderLineMask | SizeMask | FontMask | ColumnMask | AlignmentMask | ColorMask | GraphicsMask);
								btncp = (unsigned char *) DBstrdup(DBprt);
								free(DBprt);
								myselect.sel = btncp;
								myselect.length = strlen(btncp);
								myselect.viewport = (void *) viewport;
								myselect.VPType = True;
								myselect.time = myprt->xbutton.time;

								OwnSelection(viewport->resource.WorkArea, 2);
							}	
						
						}
						break;

						case SHINPUT:

						if(sviewport->cursor.DBStartPos != sviewport->cursor.DBEndPos)
						{
							chmem = *(sviewport->EditFile + sviewport->cursor.DBEndPos);
							*(sviewport->EditFile + sviewport->cursor.DBEndPos) = '\0';

							btncp = (unsigned char *)DBstrdup(sviewport->EditFile +
							sviewport->cursor.DBStartPos);

							*(sviewport->EditFile + sviewport->cursor.DBEndPos) = chmem;

							if(btncp != NULL)
							{
								count = sviewport->cursor.EndPos;
								sviewport->cursor.EndPos = sviewport->cursor.StartPos;

								DisownSelection();
							
								sviewport->cursor.EndPos = count;

								myselect.sel = btncp;
								myselect.length = strlen(btncp);
								myselect.viewport = (void *) sviewport;
								myselect.VPType = False;
								myselect.time = myprt->xbutton.time;
								
								OwnSelection(sviewport->resource.WorkArea, 2);
							}	
						}
						break;
						
						case HZINPUT:	
						
						HZiaClickAction(myprt->xbutton.x, myprt->xbutton.y);
						break;
					}	
				}	
				break;

			case KeyPress:
				count = XLookupString((XKeyEvent *)myprt, buffer, bufsize, &key, NULL);
			
				switch(key)
				{
					case XK_Left:
						switch(actid)
						{
							case WAINPUT:
							
							if(viewport->horOrder)
							{
								viewport->cursor.CurActionId = LEFT;
								MyDrawPage(viewport, LEFTRIGHTAR);
							}
							else
							{
								viewport->cursor.CurActionId = DOWN;
								MyDrawPage(viewport, UPDOWNAR);
							}
							break;

							case SHINPUT:

							sviewport->cursor.CurActionId = LEFT;
							DrawSimpleHzWin(sviewport, LEFTRIGHTAR);
							break;

							case HZINPUT:

							if(MyHzInMod.mapped == 0)
							{
								buffer[0] = ',';
								HZParseInput(&MyHzInMod.cxtermInput, &MyHzInMod.InputWin.screen, 1, buffer);
							}	
						}
						break;
					case XK_Right:
						switch(actid)
						{
							case WAINPUT:
							
							if(viewport->horOrder)
							{
								viewport->cursor.CurActionId = RIGHT;
								MyDrawPage(viewport, LEFTRIGHTAR);
							}
							else
							{
								viewport->cursor.CurActionId = UP;
								MyDrawPage(viewport, UPDOWNAR);
							}
							break;

							case SHINPUT:

							sviewport->cursor.CurActionId = RIGHT;
							DrawSimpleHzWin(sviewport, LEFTRIGHTAR);
							break;

							case HZINPUT:

							if(MyHzInMod.mapped == 0)
							{
								buffer[0] = '.';
								HZParseInput(&MyHzInMod.cxtermInput, &MyHzInMod.InputWin.screen, 1, buffer);
							}
						}
						break;
					case XK_Up:
						if(actid == WAINPUT)
						{
							if(viewport->horOrder)
							{
								viewport->cursor.CurActionId = UP;
								MyDrawPage(viewport, UPDOWNAR);
							}
							else
							{
								viewport->cursor.CurActionId = LEFT;
								MyDrawPage(viewport, LEFTRIGHTAR);
							}
						}
						break;
					case XK_Down:
						if(actid == WAINPUT)
						{
							if(viewport->horOrder)
							{
								viewport->cursor.CurActionId = DOWN;
								MyDrawPage(viewport, UPDOWNAR);
							}
							else
							{
								viewport->cursor.CurActionId = RIGHT;
								MyDrawPage(viewport, LEFTRIGHTAR);
							}
						}
						break;
					case XK_Prior:
						if(actid == WAINPUT)
						{
							PageMoveUp(viewport, (long) viewport->viewp.ViewPortHeight);
							MyDrawPage(viewport, TRUEDRAW);
						}
						break;
					case XK_Next:
						if(actid == WAINPUT)
						{
							PageMoveDown(viewport, (long) viewport->viewp.ViewPortHeight);
							MyDrawPage(viewport, TRUEDRAW);
								
						}
						break;
					case XK_Escape:
						if(MyHzInMod.mapped) break;
						HZimSetParam(&MyHzInMod.cxtermInput, HZmthAct[1]);
						break;
					case XK_F1:
						if(MyHzInMod.mapped) break;
						if(myprt->xkey.state & ShiftMask)
							SetHZinMethod(&MyHzInMod.cxtermInput, &MyHzInMod.InputWin, HZmthdParam[0][1], True);
						else
							HZimSetParam(&MyHzInMod.cxtermInput, HZmthAct[0]);
						break;
					case XK_F2:
						if(MyHzInMod.mapped) break;
						if(myprt->xkey.state & ShiftMask)
							SetHZinMethod(&MyHzInMod.cxtermInput, &MyHzInMod.InputWin, HZmthdParam[1][1], True);
						else
							SetHZinMethod(&MyHzInMod.cxtermInput, &MyHzInMod.InputWin, HZmthdParam[1][0], True);
						break;
					case XK_F3:
						if(MyHzInMod.mapped) break;
						if(myprt->xkey.state & ShiftMask)
							SetHZinMethod(&MyHzInMod.cxtermInput, &MyHzInMod.InputWin, HZmthdParam[2][1], True);
						else
							SetHZinMethod(&MyHzInMod.cxtermInput, &MyHzInMod.InputWin, HZmthdParam[2][0], True);
						break;
					case XK_F4:
						if(MyHzInMod.mapped) break;
						if(myprt->xkey.state & ShiftMask)
							SetHZinMethod(&MyHzInMod.cxtermInput, &MyHzInMod.InputWin, HZmthdParam[3][1], True);
						else
							SetHZinMethod(&MyHzInMod.cxtermInput, &MyHzInMod.InputWin, HZmthdParam[3][0], True);
						break;
					case XK_F5:
						if(MyHzInMod.mapped) break;
						if(myprt->xkey.state & ShiftMask)
							SetHZinMethod(&MyHzInMod.cxtermInput, &MyHzInMod.InputWin, HZmthdParam[4][1], True);
						else
							SetHZinMethod(&MyHzInMod.cxtermInput, &MyHzInMod.InputWin, HZmthdParam[4][0], True);
						break;
					case XK_F6:
						if(MyHzInMod.mapped) break;
						if(myprt->xkey.state & ShiftMask)
							SetHZinMethod(&MyHzInMod.cxtermInput, &MyHzInMod.InputWin, HZmthdParam[5][1], True);
						else
							SetHZinMethod(&MyHzInMod.cxtermInput, &MyHzInMod.InputWin, HZmthdParam[5][0], True);
						break;
					case XK_F7:
						if(MyHzInMod.mapped) break;
						if(myprt->xkey.state & ShiftMask)
							SetHZinMethod(&MyHzInMod.cxtermInput, &MyHzInMod.InputWin, HZmthdParam[6][1], True);
						else
							SetHZinMethod(&MyHzInMod.cxtermInput, &MyHzInMod.InputWin, HZmthdParam[6][0], True);
						break;
					case XK_F8:
						if(MyHzInMod.mapped) break;
						if(myprt->xkey.state & ShiftMask)
							SetHZinMethod(&MyHzInMod.cxtermInput, &MyHzInMod.InputWin, HZmthdParam[7][1], True);
						else
							SetHZinMethod(&MyHzInMod.cxtermInput, &MyHzInMod.InputWin, HZmthdParam[7][0], True);
						break;
					case XK_F9:
						if(MyHzInMod.mapped) break;
						if(myprt->xkey.state & ShiftMask)
							SetHZinMethod(&MyHzInMod.cxtermInput, &MyHzInMod.InputWin, HZmthdParam[8][1], True);
						else
							SetHZinMethod(&MyHzInMod.cxtermInput, &MyHzInMod.InputWin, HZmthdParam[8][0], True);
						break;  
					default:
						if(MyHzInMod.mapped == 0)
						{
							count = HZParseInput(&MyHzInMod.cxtermInput, &MyHzInMod.InputWin.screen, count, buffer);
						}
						
						buffer[count] = '\0';	
						MyHzInMod.input.InputPtr[0] = 0;        
						DBstrcat(MyHzInMod.input.InputPtr, buffer);
						count = DBlen(MyHzInMod.input.InputPtr);       

						if(simple_active)
						{
							if(count > 0 && sviewport!=NULL)
							{
								sviewport->cursor.CurActionId = (int) key;
								DrawSimpleHzWin(sviewport, SimpleHzWinInput(sviewport, KEYINPUT, MyHzInMod.input.InputPtr, count));
							}	
						}
						else
						{
							if(count > 0 && viewport!=NULL)
							{
								viewport->cursor.CurActionId = (int) key;
								MyDrawPage(viewport, Input(viewport, KEYINPUT, MyHzInMod.input.InputPtr, count));
							}
						}
						break;	
				}
				break;

			case KeyRelease:
				break;
		}
	}
}

void checkMotionNotify(viewport, current)
XEvent *current;
VIewportdata *viewport;
{
	int MoveCur_x, MoveCur_y, x, y;

	if(current->type == MotionNotify)
	{
		if(viewport->pointer.Btn1Pressed)
		{
			MoveCur_x = viewport->cursor.MoveCur_x;
			MoveCur_y = viewport->cursor.MoveCur_y;

			current->xmotion.x = XmConvertUnits(viewport->resource.WorkArea, XmHORIZONTAL, XmPIXELS, current->xmotion.x, Xm100TH_POINTS) * 100 / viewport->zoom;
			current->xmotion.y = XmConvertUnits(viewport->resource.WorkArea, XmVERTICAL, XmPIXELS, current->xmotion.y, Xm100TH_POINTS) * 100 / viewport->zoom;

			if(viewport->horOrder)
			{
				x = viewport->pointer.MovePo_x - current->xmotion.x;
				y = viewport->pointer.MovePo_y - current->xmotion.y;
			}
			else
			{
				x = viewport->pointer.MovePo_x - current->xmotion.y;
				y = viewport->pointer.MovePo_y - current->xmotion.x;
			}

			if(x < 0) x = -x; 
			if(y < 0) y = -y;

			if(x >= viewport->cursor.Mohzdata.point_width/2 || y >= viewport->cursor.Mohzdata.point_height)
			{
				if(viewport->horOrder)
				{
					viewport->pointer.MovePo_x = current->xmotion.x;
					viewport->pointer.MovePo_y = current->xmotion.y;
				}
				else
				{
					viewport->pointer.MovePo_x = current->xmotion.y;
					viewport->pointer.MovePo_y = current->xmotion.x;
				}

				MyDrawPage(viewport, MOVESEARCH);

				if(viewport->cursor.MoveCur_x !=  MoveCur_x || viewport->cursor.MoveCur_y !=  MoveCur_y)
				{
					viewport->cursor.LastStartCurPtr = viewport->cursor.StartCurPtr;
					viewport->cursor.LastEndCurPtr = viewport->cursor.EndCurPtr;

					if(viewport->cursor.MoveCurPtr > viewport->cursor.CurPtr)
					{
						viewport->cursor.StartCurPtr = viewport->cursor.CurPtr;
						viewport->cursor.EndCurPtr = viewport->cursor.MoveCurPtr;
					}
					else
					{
						viewport->cursor.StartCurPtr = viewport->cursor.MoveCurPtr;
						viewport->cursor.EndCurPtr = viewport->cursor.CurPtr;
					}

					MyDrawPage(viewport, HIGHLITE);
				}
			}
		}
	}
}

int checkLeaveNotify(viewport)
VIewportdata *viewport;
{
	XEvent current;
	int i;

	while(TRUE)
	{
		if(XCheckTypedEvent(viewport->resource.dpy, EnterNotify, &current))
		{
			return(current.type);
		}
		if(XCheckTypedEvent(viewport->resource.dpy, ButtonRelease, &current))
		{
			return(current.type);
		}

		switch(viewport->cursor.MoveDir)
		{
			case UP: 
			viewport->pointer.MovePo_y = 1;
			i = PageMoveUp(viewport, viewport->viewp.ViewPortHeight/3);
			break;

			case DOWN:
			viewport->pointer.MovePo_y = viewport->viewp.ViewPortHeight - 1;
			i = PageMoveDown(viewport, viewport->viewp.ViewPortHeight/3);
			break;  

			case LEFT:
			viewport->pointer.MovePo_x = 1;
			i = PageMoveLeft(viewport, viewport->viewp.ViewPortWidth/3);
			break;  

			case RIGHT:
			viewport->pointer.MovePo_x = viewport->viewp.ViewPortWidth - 1;
			i = PageMoveRight(viewport, viewport->viewp.ViewPortWidth/3);
			break;  
		}

		if(i)
		{
			MyDrawPage(viewport, TRUEDRAW);
			MyDrawPage(viewport, MOVESEARCH);

			viewport->cursor.LastStartCurPtr = viewport->cursor.StartCurPtr;
			viewport->cursor.LastEndCurPtr = viewport->cursor.EndCurPtr;

			if(viewport->cursor.MoveCurPtr > viewport->cursor.CurPtr)
			{
				viewport->cursor.StartCurPtr = viewport->cursor.CurPtr;
				viewport->cursor.EndCurPtr = viewport->cursor.MoveCurPtr;
			}
			else
			{
				viewport->cursor.StartCurPtr = viewport->cursor.MoveCurPtr;
				viewport->cursor.EndCurPtr = viewport->cursor.CurPtr;
			}

			MyDrawPage(viewport, HIGHLITE);
		}
	}
}

void checkShwMotionNotify(viewport, current)
XEvent *current;
SImpleHzWin *viewport;
{
	int MoveCur_x, x;

	if(current->type == MotionNotify)
	{

		x = current->xmotion.x * 2 / SHZW_FONT_SIZE + viewport->viewp.firstnum;

		if(x != viewport->cursor.MovePos)
		{
			MoveCur_x = viewport->cursor.MovePos;
			viewport->cursor.MovePos = x;

			DrawSimpleHzWin(viewport, MOVESEARCH);

			if(MoveCur_x != viewport->cursor.MovePos)
			{
				viewport->cursor.LastStartPos = viewport->cursor.StartPos;
				viewport->cursor.LastEndPos = viewport->cursor.EndPos;

				if(viewport->cursor.MovePos > viewport->cursor.CurPos)
				{
					viewport->cursor.StartPos = viewport->cursor.CurPos;
					viewport->cursor.DBStartPos = viewport->cursor.DBCurPos;
					viewport->cursor.EndPos = viewport->cursor.MovePos;
					viewport->cursor.DBEndPos = viewport->cursor.DBMovePos;
				}
				else
				{
					viewport->cursor.StartPos = viewport->cursor.MovePos;
					viewport->cursor.DBStartPos = viewport->cursor.DBMovePos;
					viewport->cursor.EndPos = viewport->cursor.CurPos;
					viewport->cursor.DBEndPos = viewport->cursor.DBCurPos;
				}

				DrawSimpleHzWin(viewport, HIGHLITE);
			}
		}
	}
}

int checkShwLeaveNotify(viewport)
SImpleHzWin *viewport;
{
	XEvent current;
	int i = False;

	while(TRUE)
	{
		if(XCheckTypedEvent(viewport->resource.dpy, EnterNotify, &current))
		{
			return(current.type);
		}

		if(XCheckTypedEvent(viewport->resource.dpy, ButtonRelease, &current))
		{
			return(current.type);
		}

		i = False;

		switch(viewport->cursor.MoveDir)
		{
			case LEFT:
			i = SimpleHzWinMoveLeft(viewport, 2);
			break;

			case RIGHT:
			i = SimpleHzWinMoveRight(viewport, 2);
			break;
		}

		if(i)
		{
			viewport->cursor.LastStartPos = viewport->cursor.StartPos;
			viewport->cursor.LastEndPos = viewport->cursor.EndPos;

			if(viewport->cursor.MovePos > viewport->cursor.CurPos)
			{
				viewport->cursor.StartPos = viewport->cursor.CurPos;
				viewport->cursor.DBStartPos = viewport->cursor.DBCurPos;
				viewport->cursor.EndPos = viewport->cursor.MovePos;
				viewport->cursor.DBEndPos = viewport->cursor.DBMovePos;
			}
			else
			{
				viewport->cursor.StartPos = viewport->cursor.MovePos;
				viewport->cursor.DBStartPos = viewport->cursor.DBMovePos;
				viewport->cursor.EndPos = viewport->cursor.CurPos;
				viewport->cursor.DBEndPos = viewport->cursor.DBCurPos;
			}

			DrawSimpleHzWin(viewport, TRUEDRAW);
		}

/*		sleep(1);
*/	}
}

