/*
 * Copyright 1995, 1996, 1997 by Jun Yei.       All Rights Reserved
 *
 * Permission to retain, use, modify, copy, and distribute ChinesePower
 * in source or binary and its documentation (hereafter, the Software)
 * for noncommercial purpose is hereby granted to you without a fee,
 * provided that this entire copyright and permission notice appear in
 * all such copies, that no charge be associated with such copies,
 * that distribution of derivative works (including value-added
 * distributions such as with additional input dictionaries or fonts)
 * include clarification that such added or derived parts are not from
 * the original Software, and that the names of the author(s) not be
 * used to endorse or promote such works. The author(s) of the software
 * makes no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * THE AUTHOR(S) OF THE SOFTWARE DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 * IN NO EVENT SHALL THE AUTHOR(S) OF THE SOFTWARE BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Jun Yei, Ottawa, ON, Canada
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "ttf.h"
#include "ttffont.h"

#define SEEK_SET 0
#define max(a,b) ((a)>(b)? (a):(b))
/*
   This modual is very long.But it just read the data file describe by the
   True Type Font Manual.Many parts of file does not read now.
*/

extern FILE *My_path_open();

typedef struct _dir_chuck {
    char tag[4];
    ULONG cksum;
    ULONG off;
    ULONG len;
} dir_chuck;

void Close_TTF(f)
TTFFont *f;
{
	int i;
	
	if(!f) return;

	if(f->name) free(f->name);
	if(f->fp) fclose(f->fp);
	
	if(f->glyf)
	{
		if(f->glyf->endPtsOfContours) free(f->glyf->endPtsOfContours);
		if(f->glyf->xCoordinates) free(f->glyf->xCoordinates);
		if(f->glyf->yCoordinates) free(f->glyf->yCoordinates);
		if(f->glyf->instruction) free(f->glyf->instruction);	
		if(f->glyf->flags) free(f->glyf->flags);
		free(f->glyf);
	}

	if(f->cmap)
	{
		for(i=0;i<f->numberOfEncoding;i++)
		{
			switch(f->cmap[i].format)
			{
				case 0:
			
				if(f->cmap[i].map.cmap0) free(f->cmap[i].map.cmap0);
				break;

				case 2:

				if(f->cmap[i].map.cmap2)
				{
					if(f->cmap[i].map.cmap2->data) free(f->cmap[i].map.cmap2->data);
					if(f->cmap[i].map.cmap2->glyphIdArray) free(f->cmap[i].map.cmap2->glyphIdArray);
					free(f->cmap[i].map.cmap2);
				}
				break;
		
				case 4:

				if(f->cmap[i].map.cmap4)
				{
					if(f->cmap[i].map.cmap4->endCount) free(f->cmap[i].map.cmap4->endCount);
					if(f->cmap[i].map.cmap4->startCount) free(f->cmap[i].map.cmap4->startCount);
					if(f->cmap[i].map.cmap4->idDelta) free(f->cmap[i].map.cmap4->idDelta);
					if(f->cmap[i].map.cmap4->idRangeOffset) free(f->cmap[i].map.cmap4->idRangeOffset);
					if(f->cmap[i].map.cmap4->glyphIdArray) free(f->cmap[i].map.cmap4->glyphIdArray);
					free(f->cmap[i].map.cmap4);
				}
				break;
			}
		}
		free(f->cmap);
	}

	if(f->head) free(f->head);

	if(f->horz) 
	{
		if(f->horz->longHorMetrics) free(f->horz->longHorMetrics);
		if(f->horz->leftSideBearing) free(f->horz->leftSideBearing);	
		free(f->horz);
	}

	if(f->indexToLocation) free(f->indexToLocation);
	if(f->profile) free(f->profile);
	if(f->cvt) free(f->cvt);
	if(f->fpgm) free(f->fpgm);
	if(f->prep) free(f->prep);

	free(f);
}

void Alloc_TTF(f)
TTFFont *f;
{
	int i;
	
	f->glyf = NULL;
	f->cmap = NULL;
	f->head = NULL;
	f->horz = NULL;
	f->indexToLocation = NULL;
	f->profile = NULL;
	f->cvt = NULL;
	f->fpgm = NULL;
	f->prep = NULL;
}

TTFFont *TTF_Open_Font(name)
char *name;
{
    FILE *fp;
    TTFFont *f;
    int ntabs,i;
    char *tag;
    dir_chuck *ch;

    fp = My_path_open(getenv("TTFPATH"), name, "r");
    
    if (fp == NULL)
    {
	  return NULL;
    }
    
    if ((f = TTFALLOC(TTFFont))==NULL)
    {
	fprintf(stderr, "Can not locate memory!\n");
	return NULL;
    }

	Alloc_TTF(f);
    f->name = strdup(name);
    f->fp = fp;
    f->version = get_Fixed(fp);
   
   if (f->version != 0x00010000) goto return_error;
    
    ntabs = get_USHORT(fp);
    
    /* skip search optimize data,I don't use it now */
    fseek(fp,12L,SEEK_SET);

    /* load ntabs tables pointer */
    ch = (dir_chuck *) malloc(sizeof(dir_chuck)*ntabs);
    
    for(i=0;i<ntabs;i++)
    {
	  ch[i].tag[0] = get_BYTE(fp);
	  ch[i].tag[1] = get_BYTE(fp);
	  ch[i].tag[2] = get_BYTE(fp);
	  ch[i].tag[3] = get_BYTE(fp);
	  ch[i].cksum = get_ULONG(fp);
	  ch[i].off = get_ULONG(fp);
	  ch[i].len = get_ULONG(fp);
    }

    /* 
       load MAXP &HEAD table first.Because they will beused by other
       table.
       */
    
    for(i=0;i<ntabs;i++)
    {
	tag = ch[i].tag;
	if (*tag == 'm' && strncmp(tag,"maxp",4)==0)
	  {
	    if (Font_load_MAXP(f,fp,ch[i].off,ch[i].len))
	      goto return_error;
	  }
	else if (*tag =='h' && strncmp(tag,"head",4)==0)
	  {
	    if (Font_load_HEAD(f,fp,ch[i].off,ch[i].len))
	      goto return_error;
	  }
      }	 
    
    /* allocate outline data from MAXP table */
    f->glyf = TTFALLOC(OUTLINE);
    f->glyf->endPtsOfContours = (USHORT *) 
      malloc(sizeof(USHORT)*max(f->profile->maxContours,f->profile->maxCompositeContours));
    f->glyf->xCoordinates = (LONG *) 
      malloc(sizeof(LONG)*max(f->profile->maxPoints,f->profile->maxCompositePoints));
    f->glyf->yCoordinates = (LONG *) 
      malloc(sizeof(LONG)*max(f->profile->maxPoints,f->profile->maxCompositePoints));
    f->glyf->flags = (BYTE *) 
      malloc(sizeof(BYTE)*max(f->profile->maxPoints,f->profile->maxCompositePoints));
    f->glyf->instruction = NULL;
    f->glyf->instructionLength = 0; 
    /*(BYTE *)
      malloc(sizeof(BYTE)*max(1, f->profile->maxSizeOfInstructions));
   */ 
    /* 
       read the others,but the GLYF table will not read now.It is
       read when demand
       */
    
    for(i=0;i<ntabs;i++)
      {
	  tag = ch[i].tag;
	  if (*tag == 'c' && !strncmp(tag,"cmap",4))
	    {
		if (Font_load_CMAP(f,fp,ch[i].off,ch[i].len))
		  goto return_error;
	    }
	  else if (*tag == 'g' && !strncmp(tag,"glyf",4))
	    {
		f->glyfoff = ch[i].off;
	    }
	  else if (*tag == 'h' && !strncmp(tag,"head",4))
	    {
	    }
	  else if (*tag == 'h' && !strncmp(tag,"hhea",4))
	    {
		if (Font_load_HHEA(f,fp,ch[i].off,ch[i].len))
		  goto return_error;
	    }
	  else if (*tag == 'h' && !strncmp(tag,"hmtx",4))
	    {
		if (Font_load_HMTX(f,fp,ch[i].off,ch[i].len))
		  goto return_error;
	    }
	  else if (*tag == 'l' && !strncmp(tag,"loca",4))
	    {
		if (Font_load_LOCA(f,fp,ch[i].off,ch[i].len))
		  goto return_error;
	    }
	  else if (*tag == 'm' && !strncmp(tag,"maxp",4))
	    {
	    }
	  else if (*tag == 'n' && !strncmp(tag,"name",4))
	    {
		if (Font_load_NAME(f,fp,ch[i].off,ch[i].len))
		  goto return_error;
	    }
	  else if (*tag == 'p' && !strncmp(tag,"post",4))
	    {
		/*
		if (Font_load_POST(f,fp,ch[i].off,ch[i].len))
		  goto return_error;
		  */
	    }
	  else if (*tag == 'f' && !strncmp(tag,"fpgm",4))
	    {
		if (Font_load_FPGM(f,fp,ch[i].off,ch[i].len))
		  goto return_error;
	    }
	  else if (*tag == 'O' && !strncmp(tag,"OS/2",4))
	    {
		/*
		if (Font_load_OS2(f,fp,ch[i].off,ch[i].len))
		  goto return_error;
		  */
	    }
	  else if (*tag == 'c' && !strncmp(tag,"cvt",3))
	    {
		if (Font_load_CVT(f,fp,ch[i].off,ch[i].len))
		  goto return_error;
	    }
	  else if (*tag == 'h' && !strncmp(tag,"hdmx",4))
	    {
		if (Font_load_HDMX(f,fp,ch[i].off,ch[i].len))
		  goto return_error;
	    }
	  else if (*tag == 'k' && !strncmp(tag,"kern",4))
	    {
		if (Font_load_KERN(f,fp,ch[i].off,ch[i].len))
		  goto return_error;
	    }
	  else if (*tag == 'L' && !strncmp(tag,"LTSH",4))
	    {
		if (Font_load_LTSH(f,fp,ch[i].off,ch[i].len))
		  goto return_error;
	    }
	  else if (*tag == 'p' && !strncmp(tag,"prep",4))
	    {
		if (Font_load_PREP(f,fp,ch[i].off,ch[i].len))
		  goto return_error;
	    }
	  else if (*tag == 'W' && !strncmp(tag,"WIN",3))
	    {
		/*
		if (Font_load_WIN(f,fp,ch[i].off,ch[i].len))
		  goto return_error;
		  */
	    }
	  else if (*tag == 'V' && !strncmp(tag,"VDMX",4))
	    {
		/*
		if (Font_load_VDMX(f,fp,ch[i].off,ch[i].len))
		  goto return_error;
		  */
	    }
	  else if (*tag == 'F' && !strncmp(tag,"FOCA",4))
	    {
		/*
		if (Font_load_FOCA(f,fp,ch[i].off,ch[i].len))
		  goto return_error;
		  */
	    }
	  else if (*tag == 'P' && !strncmp(tag,"PCLT",4))
	    {
		if (Font_load_PCLT(f,fp,ch[i].off,ch[i].len))
		  goto return_error;
	    }
	  else
	    {
		fprintf(stderr, "Incorrect format of %s in TTF %s\n", tag, f->name);
	    }
      }
    free(ch); 
    return(f);

  return_error:
      Close_TTF(f);
    return NULL;
}

    
int Font_load_CMAP(f, fp, off, len)
TTFFont *f;
FILE *fp;
int off, len;
{
    CMAP *cm;
    CMAP0 *m0;
    CMAP2 *m2;
    CMAP4 *m4;
    CMAP6 *m6;
    int i,j,length,num_var,k;
    int version;
    USHORT type;
    ULONG *offs;
    int NotLoad = 1;

    fseek(fp,off,SEEK_SET);
    /* ignore version */
    version = get_USHORT(fp);
    f->numberOfEncoding = get_USHORT(fp);
    f->cmap = (CMAP *) malloc(sizeof(CMAP)*f->numberOfEncoding);
    offs = (ULONG *) malloc(sizeof(ULONG)*f->numberOfEncoding);
    
    for(i=0;i<f->numberOfEncoding;i++)
    {
	f->cmap[i].PlateformID = get_USHORT(fp);
	f->cmap[i].EncodingID = get_USHORT(fp);
	offs[i] = get_ULONG(fp);
    }
    
    for(i=0;i<f->numberOfEncoding;i++)
    {
	fseek(fp,off+offs[i],SEEK_SET);
	f->cmap[i].format = get_USHORT(fp);
	if((f->cmap[i].PlateformID == 3 || f->cmap[i].EncodingID == 4) && NotLoad)
	{
		switch(f->cmap[i].format)
		{
	  		case 0:
	    		m0 = f->cmap[i].map.cmap0 = TTFALLOC(CMAP0);
	    		fread(m0->glyphIdArray,1,256,fp);
	    		NotLoad = 0;
			break;
	  
	  		case 2:
	    		m2 = f->cmap[i].map.cmap2 = TTFALLOC(CMAP2);
	    		num_var = 0;
	       	    	length = get_USHORT(fp);
	    		get_USHORT(fp);
	    
	    		for(j=0;j<256;j++)
	    		{
		  		m2->subHeaderKeys[j] = get_USHORT(fp)/8;
		  		if (num_var < m2->subHeaderKeys[j])
		    		num_var = m2->subHeaderKeys[j];
	    		}
	    
	    		length -= (256+3)*2+num_var*8;
	    		/* ignore version */
	    		m2->data = (USHORT *) malloc(sizeof(USHORT)*num_var*8);
	    
	    		if (m2->data == NULL)
	    		{
				free(m2);
				free(f->cmap);
				f->cmap = NULL;
				return 1;
	    		}
	    
	    		for(j=0;j<=num_var;j++)
	    		{
		  		m2->data[j*4] = get_USHORT(fp);
		  		m2->data[j*4+1] = get_USHORT(fp);
		  		m2->data[j*4+2] = get_USHORT(fp);
		  		m2->data[j*4+3] = get_USHORT(fp)-(num_var-j)*8-2;
	    		}
	    
	    		m2->glyphIdArray = (USHORT *) malloc(length);
	    
	    		if (m2->glyphIdArray == NULL)
	    		{
				free(m2->data);
				free(m2);
				f->cmap[i].map.cmap2 = NULL;
				return 1;
	    		}
	    
	    		length /=2;
	    
	    		for(j=0;j<length;j++)
	      			m2->glyphIdArray[j] = get_USHORT(fp);
	    
	    		NotLoad = 0;
			break;
	  
	  		case 4:
	    
	    		m4 = f->cmap[i].map.cmap4 = TTFALLOC(CMAP4);
			length = get_USHORT(fp);
	    		/* ignore version */
	    		get_USHORT(fp);
	    		num_var = m4->segCount = get_USHORT(fp)/2;
	    		/*ignore optimized search */
	    		get_USHORT(fp); get_USHORT(fp); get_USHORT(fp);

	    		m4->endCount = (USHORT *) malloc(sizeof(USHORT)*num_var);
	    		m4->startCount = (USHORT *) malloc(sizeof(USHORT)*num_var);
	    		m4->idDelta = (SHORT *) malloc(sizeof(SHORT)*num_var);            
	    		m4->idRangeOffset = (USHORT *) malloc(sizeof(USHORT)*num_var);            

	    		for(j=0;j<num_var;j++) m4->endCount[j] = get_USHORT(fp);
	    		/*ignore reservedPad */
	    		get_USHORT(fp);
	    		for(j=0;j<num_var;j++) m4->startCount[j] = get_USHORT(fp);
	    		for(j=0;j<num_var;j++) m4->idDelta[j] = get_USHORT(fp);
	    		for(j=0;j<num_var;j++) m4->idRangeOffset[j] = get_USHORT(fp);
	    
	    		length -= (num_var*4+8)*2;
	    		m4->glyphIdArray = (USHORT *) malloc(length);
	    
	    		if (m4->glyphIdArray == NULL)
	    		{
				free(m4->endCount);
				free(m4->startCount);
				free(m4->idDelta);
				free(m4->idRangeOffset);
				free(m4);
				f->cmap[i].map.cmap4 = NULL;
				return 1;
	   		} 
	    
	   		length /=2;
	    
	    		for(j=0;j<length;j++)
	      			m4->glyphIdArray[j] = get_USHORT(fp);
	  
	  		NotLoad = 0;
			break;

	  		case 6:
	    
	    		break;
	 		}
      		}
		else
   		{
			f->cmap[i].map.cmap0 = NULL;
			f->cmap[i].format = 8;	
		}
    }
    
    free(offs);
    return 0;
}

int Font_load_GLYF(f, fp, off, len)
TTFFont *f;
FILE *fp;
int off, len;
{
    OUTLINE *ol;
    int totpoint,i;
    BYTE b,fl,c,st;
    SHORT sh;
    USHORT EM;
  
    fseek(fp,f->glyfoff+off,SEEK_SET);
    ol = f->glyf;
    ol->valid = 0;
    ol->numberOfContours = get_SHORT(fp);
    if (ol->numberOfContours > f->profile->maxContours)
      {
	  return 0;
      }
    ol->xMin = get_FWord(fp);
    ol->yMin = get_FWord(fp);
    ol->xMax = get_FWord(fp);
    ol->yMax = get_FWord(fp);
    if (ol->numberOfContours > 0)
      {
	  for(i = 0;i < ol->numberOfContours;i++)
	    ol->endPtsOfContours[i] = get_USHORT(fp);
	  totpoint = ol->endPtsOfContours[ol->numberOfContours-1]+1;
	  EM = get_USHORT(fp);
	  if (ol->instructionLength < EM)
	  {
	    ol->instructionLength = EM;
	    if(ol->instruction) free(ol->instruction);
	    ol->instruction = (BYTE *)malloc(sizeof(BYTE)*EM);
	  }
	  if(EM) fread(ol->instruction,1,EM,fp);
	  for(i=0;i<totpoint;)
	  {
	    ol->flags[i++] = c = get_BYTE(fp);
	    if (c& FLAG_REPEAT)
	    {
	      st = get_BYTE(fp);
	      while(st--) ol->flags[i++] = c;
	    }
	  }
	  
	  for(i=0;i<totpoint;i++)
	    {
		fl = ol->flags[i];
		if (fl & FLAG_X_SHORT_VECTOR)
		  {
		      b = get_BYTE(fp);
		      if (fl & FLAG_X_SAME)
			ol->xCoordinates[i] = b;
		      else
			ol->xCoordinates[i] = -b;
		  }
		else if (fl & FLAG_X_SAME)
		  ol->xCoordinates[i] = 0;
		else
		  ol->xCoordinates[i] = get_SHORT(fp);
	    }
	  for(i=0;i<totpoint;i++)
	    {
		fl = ol->flags[i];
		if (fl & FLAG_Y_SHORT_VECTOR)
		  {
		      b = get_BYTE(fp);
		      if (fl & FLAG_Y_SAME)
			ol->yCoordinates[i] = b;
		      else
			ol->yCoordinates[i] = -b;
		  }
		else if (fl & FLAG_Y_SAME)
		  ol->yCoordinates[i] = 0;
		else
		  ol->yCoordinates[i] = get_SHORT(fp);
	    }
      }
    else
      {
	  /* The composite font does not implement yet */
	  return 0;
      }
    for(i=1;i<totpoint;i++)
      {
	  ol->xCoordinates[i] +=ol->xCoordinates[i-1];
	  ol->yCoordinates[i] +=ol->yCoordinates[i-1];
      }
    f->glyf->valid = 1;
    return 1;
}
    
int Font_load_HEAD(f, fp, off, len)
TTFFont *f;
FILE *fp;
int off, len;
{
  Fixed version;
  HEAD *h;

  fseek(fp,off,SEEK_SET);
  version = get_Fixed(fp);
  if (version != 0x00010000)
    {
      return 1;
    }
  f->head = h = TTFALLOC(HEAD);
  if (h == NULL)
    return 1;

  h->fontRevision = get_Fixed(fp);
  /* forget checksum and magicnumber */
  get_ULONG(fp);get_ULONG(fp);
  h->flags = get_USHORT(fp);
  h->unitsPerEm = get_USHORT(fp);
  /* forget date data */
  get_ULONG(fp);get_ULONG(fp);get_ULONG(fp);get_ULONG(fp);
  
  h->xMin = get_FWord(fp);
  h->yMin = get_FWord(fp);
  h->xMax = get_FWord(fp);
  h->yMax = get_FWord(fp);
  h->macStyle = get_USHORT(fp);
  h->lowestRecPPEM = get_USHORT(fp);
  h->fontDirectionHint = get_SHORT(fp);
  h->indexToLocFormat = get_SHORT(fp);
  h->glyphDataFormat = get_SHORT(fp);

  return 0;
}

int Font_load_HHEA(f, fp, off, len)
TTFFont *f;
FILE *fp;
int off, len;
{
  int version;
  HORIZON *h;

  fseek(fp,off,SEEK_SET);
  version = get_Fixed(fp);
  if (version != 0x00010000)
    {
      return 1;
    }
  
  f->horz = h = TTFALLOC(HORIZON);
  if (h == NULL)
    return 1;

  h->ascender = get_FWord(fp);
  h->decender = get_FWord(fp);
  h->lineGap  = get_FWord(fp);
  h->advanceWidthMax = get_uFWord(fp);
  h->minLeftSideBearing = get_FWord(fp);
  h->minRightSideBearing= get_FWord(fp);
  h->xMaxExtent = get_FWord(fp);
  h->caretSlopeRise = get_SHORT(fp);
  h->caretSlopeRun  = get_SHORT(fp);
  /* stub bytes */
  get_SHORT(fp);get_SHORT(fp);get_SHORT(fp);get_SHORT(fp);
  h->metricDataFormat = get_SHORT(fp);
  h->numberOfHMetrics = get_USHORT(fp);
  h->longHorMetrics = NULL;
  h->leftSideBearing = NULL;
  return 0;
}

int Font_load_HMTX(f, fp, off, len)
TTFFont *f;
FILE *fp;
int off, len;
{
  int i;
  HORIZON *h;

  h = f->horz;

  fseek(fp,off,SEEK_SET);
  h->longHorMetrics = 
    (HorMetric *) malloc(sizeof(HorMetric)*h->numberOfHMetrics);
  len = f->profile->numglyphs - h->numberOfHMetrics;
  h->leftSideBearing = (FWord *) malloc(sizeof(SHORT)*len);
  for(i=0;i<h->numberOfHMetrics;i++)
    {
      h->longHorMetrics[i].advanceWidth = get_uFWord(fp);
      h->longHorMetrics[i].lsb = get_FWord(fp);
    }
  for(i=0;i<len;i++)
    h->leftSideBearing[i] = get_FWord(fp);

  return 0;
}

int Font_load_LOCA(f, fp, off, len)
TTFFont *f;
FILE *fp;
int off, len;
{
  int i;

  fseek(fp,off,SEEK_SET);
  if (f->head == NULL)
    {
      return 1;
    }
  if (f->profile == NULL)
    {
      return 1;
    }
  f->indexToLocation = (ULONG *) malloc(sizeof(ULONG)*(f->profile->numglyphs+1));
  if (f->head->indexToLocFormat == 0)
    for(i=0;i<=f->profile->numglyphs;i++)
      f->indexToLocation[i] = get_USHORT(fp)*2;
  else
    for(i=0;i<=f->profile->numglyphs;i++)
      f->indexToLocation[i] = get_ULONG(fp);
  return 0;
}

int Font_load_MAXP(f, fp, off, len)
TTFFont *f;
FILE *fp;
int off, len;
{
  int version;
  PROFILE *p;

  fseek(fp,off,SEEK_SET);
  version = get_Fixed(fp);
  if (version != 0x00010000)
    {
      return 1;
    }
  
  p = f->profile = TTFALLOC(PROFILE);

  if ( p == NULL)
    return 1;
  p-> numglyphs = get_USHORT(fp);
  p-> maxPoints = get_USHORT(fp);
  p-> maxContours = get_USHORT(fp);
  p-> maxCompositePoints = get_USHORT(fp);
  p-> maxCompositeContours = get_USHORT(fp);
  p-> maxZones = get_USHORT(fp);
  p-> maxTwilightPoints = get_USHORT(fp);
  p-> maxStorage = get_USHORT(fp);
  p-> maxFunctionDefs = get_USHORT(fp);
  p-> maxInstructionDefs = get_USHORT(fp);
  p-> maxStackElements = get_USHORT(fp);
  p-> maxSizeOfInstructions = get_USHORT(fp);
  p-> maxComponentElements = get_USHORT(fp);
  p-> maxComponentDepth = get_USHORT(fp);
  return 0;
}

int Font_load_NAME(f, fp, off, len)
TTFFont *f;
FILE *fp;
int off, len;
{
  return 0;
}

int Font_load_CVT(f, fp, off, len)
TTFFont *f;
FILE *fp;
int off, len;
{
  int i;

  fseek(fp,off,SEEK_SET);
  f->cvt = (USHORT *) malloc(sizeof(USHORT)*len/2);
  f->cvtlen = len/2;
  for(i=0;i<len/2;i++)
    f->cvt[i] = get_USHORT(fp);
  return 0;
}

int Font_load_FPGM(f, fp, off, len)
TTFFont *f;
FILE *fp;
int off, len;
{
  int i;

  fseek(fp,off,SEEK_SET);
  f->fpgm = (BYTE *) malloc(sizeof(BYTE)*len);
  f->fpgmlen = len;
  for(i=0;i<len;i++)
    f->fpgm[i] = get_BYTE(fp);
  return 0;
}

int Font_load_KERN(f, fp, off, len)
TTFFont *f;
FILE *fp;
int off, len;
{
  return 0;
}

int Font_load_LTSH(f, fp, off, len)
TTFFont *f;
FILE *fp;
int off, len;
{
  return 0;
}

int Font_load_PREP(f, fp, off, len)
TTFFont *f;
FILE *fp;
int off, len;
{
  int rlen;

  fseek(fp,off,SEEK_SET);
  f->prep = (BYTE *) malloc(len);
  rlen = fread(f->prep,1,len,fp);
  if (rlen != len)
    {
      return 1;
    }
  f->preplen = len;
  return 0;
}

int Font_load_HDMX(f, fp, off, len)
TTFFont *f;
FILE *fp;
int off, len;
{
  return 0;
}

int Font_load_PCLT(f, fp, off, len)
TTFFont *f;
FILE *fp;
int off, len;
{
  return 0;
}
  
void *Open_TTF(name)
char *name;
{
	TTFFont *fn;

	fn = TTF_Open_Font(name);

	if(fn) Font_select_CMAP(fn, (unsigned short)3, (unsigned short)4);

	return((void *)fn);
}

