/*
 * Copyright 1995, 1996, 1997 by Jun Yei.       All Rights Reserved
 *
 * Permission to retain, use, modify, copy, and distribute ChinesePower
 * in source or binary and its documentation (hereafter, the Software)
 * for noncommercial purpose is hereby granted to you without a fee,
 * provided that this entire copyright and permission notice appear in
 * all such copies, that no charge be associated with such copies,
 * that distribution of derivative works (including value-added
 * distributions such as with additional input dictionaries or fonts)
 * include clarification that such added or derived parts are not from
 * the original Software, and that the names of the author(s) not be
 * used to endorse or promote such works. The author(s) of the software
 * makes no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * THE AUTHOR(S) OF THE SOFTWARE DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 * IN NO EVENT SHALL THE AUTHOR(S) OF THE SOFTWARE BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Jun Yei, Ottawa, ON, Canada
 */

#include <signal.h>
#include "chpower.h"
#include "chpowerdata.h"
#include "hztoplevel.h"
#include "icon.h"

void MainExit();

TOplevelWg topWg;
Pixmap IconPixmap;
XtAppContext app;

void UninstallImage()
{
        XmUninstallImage(&saveImg);
        XmUninstallImage(&openImg);
        XmUninstallImage(&cutImg);
        XmUninstallImage(&copyImg);
        XmUninstallImage(&pasteImg);
        XmUninstallImage(&findImg);
        XmUninstallImage(&centerImg);
        XmUninstallImage(&undoImg);
        XmUninstallImage(&leftImg);
        XmUninstallImage(&rightImg);
        XmUninstallImage(&col1Img);
        XmUninstallImage(&col2Img);
        XmUninstallImage(&symbolImg);
        XmUninstallImage(&underImg);
	XmUninstallImage(&printImg);
	XmUninstallImage(&middleImg);
	XmUninstallImage(&nounImg);
	XmUninstallImage(&palletImg);
	XmUninstallImage(&inputImg);
	XmUninstallImage(&charImg);
	XmUninstallImage(&refreshImg);
	XmUninstallImage(&newImg);
	XmUninstallImage(&picImg);
}

void ExitViewPort(viewportlist)
MYviewportlist *viewportlist;
{
	if(viewportlist->myviewport.changed) 
	{
		ConfirmQuit(viewportlist, ExitSaveCB, ExitNoSaveCB);
		return;
	}
	else 
	{
		QuitViewport(viewportlist);
	}

	MainExitYesCB(NULL, NULL, NULL);
}

void YesExitSaveMChgCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	SaveAssocList(&MyHzInMod.cxtermInput, MyHzInMod.InputWin.misc.assoc_files, MyHzInMod.InputWin.misc.it_dirs);
	MainExitYesCB(NULL, NULL, NULL);
}

void MainExitYesCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	if(ViewPortRoot != NULL)
	{
		ExitViewPort(ViewPortRoot);
		return;	
	}

	if(MyHzInMod.assolstchg)
	{
		MyHzInMod.assolstchg = False;
		ConfirmSaveHZAssoc(NULL, YesExitSaveMChgCB, MainExitYesCB);
		return;
	}

	DistroySelection();
	DestroyCharSet();
	DestroyMyFontList();
	DestroyCXtermInput(&MyHzInMod.cxtermInput);
	DestroySoftKey();
	DestroyColor();
	PicImportDestroy();
	DestroyDefaults();
	UninstallImage();

	if(printset.PrCmd != NULL) free(printset.PrCmd);

	XtDestroyWidget(toplevel);

	exit(0);
}

void MainExitQueryCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	YESdoNOreturn(MainExitYesCB, (void *)client_data, "Sure to Quit ChPower?");
}

void SigDestroy(signo)
int signo;
{
	MainDestroyCB(NULL, NULL, NULL);
}
	
void MainDestroyCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	MYviewportlist *viewportlist;

	viewportlist = ViewPortRoot;

	while(viewportlist != NULL)
	{
		DestroySave(&viewportlist->myviewport, False);
		viewportlist = viewportlist->next;
	}

	MainExit();
}
		
void MainExit()
{
	while(ViewPortRoot != NULL)
	{
		FreeViewport(ViewPortRoot);
	}

	DistroySelection();
	DestroyCharSet();
	DestroyMyFontList();
	DestroyCXtermInput(&MyHzInMod.cxtermInput);
	DestroySoftKey();
	DestroyColor();
	PicImportDestroy();
	DestroyDefaults();
	UninstallImage();

	if(printset.PrCmd != NULL) free(printset.PrCmd);

	XtDestroyWidget(toplevel);

	exit(0);
}

void MainNewCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	if(FontListRoot == NULL) { ErrorPopup("No Font Installed!"); return; }
	if(EnglishFontHbf == NULL) { ErrorPopup("Missing 8x16.hbf Font!"); return; }
	
	IniViewport(toplevel, NULL);
}

void MainOpenCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	if(FontListRoot == NULL) { ErrorPopup("No Font Installed!"); return; }
	if(EnglishFontHbf == NULL) { ErrorPopup("Missing 8x16.hbf Font!"); return; }
	
	FileOpen(XtParent(w));
}

void MainInputCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	if(FontListRoot == NULL) { ErrorPopup("No Font Installed!"); return; }
	if(EnglishFontHbf == NULL) { ErrorPopup("Missing 8x16.hbf Font!"); return; }
		
	XtMapWidget(MyHzInMod.PopUpShell);
	XRaiseWindow(XtDisplay(MyHzInMod.PopUpShell), XtWindow(MyHzInMod.PopUpShell));
	MyHzInMod.mapped = 0;
}

void CreatMyIcon(w, hzbitmap, width, height)
Widget w;
char hzbitmap[];
int width, height;
{
	Display *dpy;
	XWMHints mywmhints;

	dpy = XtDisplay(w);

	IconPixmap = XCreateBitmapFromData(dpy, XtWindow(w), hzbitmap, width, height);

	mywmhints.icon_pixmap = IconPixmap;
	mywmhints.flags = IconPixmapHint;	
	XSetWMHints(dpy, XtWindow(w), &mywmhints);
	XSetWMHints(XtDisplay(MyHzInMod.PopUpShell), XtWindow(MyHzInMod.PopUpShell), &mywmhints);
	XSetWMHints(XtDisplay(printset.PopUpWg), XtWindow(printset.PopUpWg), &mywmhints);
	XSetWMHints(XtDisplay(pageset.PopUpWg), XtWindow(pageset.PopUpWg), &mywmhints);
	XSetWMHints(XtDisplay(gifset.PopUpWg), XtWindow(gifset.PopUpWg), &mywmhints);
	XSetWMHints(XtDisplay(charset.PopUpWg), XtWindow(charset.PopUpWg), &mywmhints);
	XSetWMHints(XtDisplay(pagedel.PopUpWg), XtWindow(pagedel.PopUpWg), &mywmhints);
	XSetWMHints(XtDisplay(pagebtn.PopUpWg), XtWindow(pagebtn.PopUpWg), &mywmhints);
	XSetWMHints(XtDisplay(findset.PopUpWg), XtWindow(findset.PopUpWg), &mywmhints);
	XSetWMHints(XtDisplay(titlerset.PopUpWg), XtWindow(titlerset.PopUpWg), &mywmhints);
	XSetWMHints(XtDisplay(colortab.PopUpWg), XtWindow(colortab.PopUpWg), &mywmhints);
}

void CreatHzLabel(w, gc, hzbitmapLSB, hzbitmapMSB, width, height, colorname)
Widget w;
GC gc;
char *hzbitmapLSB, *hzbitmapMSB;
int width, height;
char *colorname;
{
	Display *dpy;
	XImage *graph;
	Pixmap pixmap;
	XColor color, exact;
	Status result;

	Arg args[2];

	dpy = XtDisplay(w);

	pixmap = XCreatePixmap(dpy, XtWindow(w), width, height, DisplayPlanes(dpy, DefaultScreen(dpy)));

	result = XAllocNamedColor(dpy, DefaultColormap(dpy, DefaultScreen(dpy)), colorname, &color, &exact);

	if(result) XSetForeground(XtDisplay(topWg.main_window), gc, color.pixel);
	
	graph = XCreateImage(dpy, DefaultVisual(dpy, DefaultScreen(dpy)), 1, XYBitmap, 0, (char *) NULL, width, height, 8, 0);

	if(graph->bitmap_bit_order == LSBFirst)
	{
		graph->data = hzbitmapLSB;
	}
	else
	{
		graph->data = hzbitmapMSB;
	}

	XPutImage(dpy, pixmap, gc, graph, 0, 0, 0, 0, width, height);
	XFree(graph);

	XtSetArg(args[0], XmNlabelType, XmPIXMAP);
	XtSetArg(args[1], XmNlabelPixmap, pixmap);
	XtSetValues(w, args, 2);
}

Widget CreateTopLevel(parent)
Widget parent;
{
	Arg args[MAX_ARGS];
	register int n;
	
	/* Setup Main Window */
	
	n = 0;
	XtSetArg(args[n], XmNwidth, 140); n++;
	XtSetArg(args[n], XmNheight,140); n++;
	XtSetArg(args[n], XmNrubberPositioning, True); n++;
	topWg.main_window = XmCreateForm(parent, "main", args, n);
	XtManageChild(topWg.main_window);
	XtAddCallback(topWg.main_window, XmNdestroyCallback, MainDestroyCB, NULL);	

	/* Setup Main Buttons */
	
	n = 0;
	XtSetArg(args[n], XmNwidth, 35); n++;
	XtSetArg(args[n], XmNheight, 105); n++;
	XtSetArg(args[n], XmNhighlightThickness, 0); n++;
	XtSetArg(args[n], XmNshadowThickness, 2); n++;
	XtSetArg(args[n], XmNx, 0); n++;
	XtSetArg(args[n], XmNy, 0); n++;
	topWg.newBtn = XmCreatePushButtonGadget(topWg.main_window, "main_new", args, n);
	XtManageChild(topWg.newBtn);
	XtAddCallback(topWg.newBtn, XmNactivateCallback, MainNewCB, NULL);

	n = 0;
	XtSetArg(args[n], XmNwidth, 105); n++;
	XtSetArg(args[n], XmNheight, 35); n++;
	XtSetArg(args[n], XmNhighlightThickness, 0); n++;
	XtSetArg(args[n], XmNshadowThickness, 2); n++;
	XtSetArg(args[n], XmNx, 35); n++;
	XtSetArg(args[n], XmNy, 0); n++;
	topWg.openBtn = XmCreatePushButtonGadget(topWg.main_window, "main_open", args, n);
	XtManageChild(topWg.openBtn);  
	XtAddCallback(topWg.openBtn, XmNactivateCallback, MainOpenCB, NULL);

	n = 0;
	XtSetArg(args[n], XmNwidth, 35); n++;
	XtSetArg(args[n], XmNheight, 105); n++;
	XtSetArg(args[n], XmNhighlightThickness, 0); n++;
	XtSetArg(args[n], XmNshadowThickness, 2); n++;
	XtSetArg(args[n], XmNx, 105); n++;
	XtSetArg(args[n], XmNy, 35); n++;
	topWg.inputBtn = XmCreatePushButtonGadget(topWg.main_window, "main_input", args, n);
	XtManageChild(topWg.inputBtn);
	XtAddCallback(topWg.inputBtn, XmNactivateCallback, MainInputCB, NULL);

	n = 0;
	XtSetArg(args[n], XmNwidth, 70); n++;
	XtSetArg(args[n], XmNheight,70); n++;
	XtSetArg(args[n], XmNhighlightThickness, 0); n++;
	XtSetArg(args[n], XmNshadowThickness, 2); n++;
	XtSetArg(args[n], XmNx, 35); n++;
	XtSetArg(args[n], XmNy, 35); n++;
	topWg.infoBtn = XmCreatePushButtonGadget(topWg.main_window, "main_info", args, n);
	XtManageChild(topWg.infoBtn);
	XtAddCallback(topWg.infoBtn, XmNactivateCallback, OpenTitleCB, NULL);

	n = 0;
	XtSetArg(args[n], XmNwidth, 105); n++;
	XtSetArg(args[n], XmNheight, 35); n++;
	XtSetArg(args[n], XmNhighlightThickness, 0); n++;
	XtSetArg(args[n], XmNshadowThickness, 2); n++;
	XtSetArg(args[n], XmNx, 0); n++;
	XtSetArg(args[n], XmNy, 105); n++;
	topWg.exitBtn = XmCreatePushButtonGadget(topWg.main_window, "main_exit", args, n);
	XtManageChild(topWg.exitBtn);
	XtAddCallback(topWg.exitBtn, XmNactivateCallback, MainExitQueryCB, NULL);
}

void CreatToolBarIcons(w)
Widget w;
{
	int i;
	Arg args[2];
	unsigned long foreground, background;

	XtSetArg(args[0], XmNbackground, &background);
	XtSetArg(args[1], XmNforeground, &foreground);
	XtGetValues(w, args, 2);

	for(i = 0; i < ICON_NUM; i++)
	{
		ToolBarIcons[i] =  XmGetPixmap(XtScreen(w), TBIconsName[i], foreground, background);
	}
}
	
void SetTopLabels()
{
	Arg args[2];
	
	GC gc;
	XGCValues gc_init;

	XtSetArg(args[0], XmNbackground, &gc_init.background); 
	XtSetArg(args[1], XmNforeground, &gc_init.foreground);
	XtGetValues(topWg.main_window, args, 2);
				
	gc = XCreateGC(XtDisplay(topWg.main_window), XtWindow(topWg.main_window), GCForeground|GCBackground, &gc_init);

	CreatHzLabel(topWg.newBtn, gc, hz_xingLSB, hz_xingMSB, 24, 24, GlobalDefaults.topBtnColor[0]);
	CreatHzLabel(topWg.openBtn, gc, hz_jiuLSB, hz_jiuMSB, 24, 24, GlobalDefaults.topBtnColor[1]);
	CreatHzLabel(topWg.inputBtn, gc, hz_ruLSB, hz_ruMSB, 24, 24, GlobalDefaults.topBtnColor[2]);
	CreatHzLabel(topWg.exitBtn, gc, hz_chuLSB, hz_chuMSB, 24, 24, GlobalDefaults.topBtnColor[3]);
	CreatHzLabel(topWg.infoBtn, gc, hz_iconLSB, hz_iconMSB, 32, 32, GlobalDefaults.topBtnColor[4]);
	CreatToolBarIcons(topWg.main_window);	
	CreatMyIcon(toplevel, Wm_icon, 48, 48);

	XFreeGC(XtDisplay(topWg.main_window), gc);
}

void InstallImage()
{
	XmInstallImage(&saveImg, TBIconsName[SAVE_ICON]);
	XmInstallImage(&openImg, TBIconsName[OPEN_ICON]);
	XmInstallImage(&cutImg, TBIconsName[CUT_ICON]);
	XmInstallImage(&copyImg, TBIconsName[COPY_ICON]);
	XmInstallImage(&pasteImg, TBIconsName[PASTE_ICON]);
	XmInstallImage(&findImg, TBIconsName[FIND_ICON]);
	XmInstallImage(&centerImg, TBIconsName[CENTER_ICON]);
	XmInstallImage(&undoImg, TBIconsName[UNDO_ICON]);
	XmInstallImage(&leftImg, TBIconsName[LEFT_ICON]);
	XmInstallImage(&rightImg, TBIconsName[RIGHT_ICON]);
	XmInstallImage(&col1Img, TBIconsName[COL1_ICON]);
	XmInstallImage(&col2Img, TBIconsName[COL2_ICON]);
	XmInstallImage(&symbolImg, TBIconsName[SYM_ICON]);
	XmInstallImage(&underImg, TBIconsName[UNDER_ICON]);
	XmInstallImage(&printImg, TBIconsName[PRINT_ICON]);
        XmInstallImage(&middleImg, TBIconsName[MIDDLE_ICON]);
        XmInstallImage(&nounImg, TBIconsName[NOLINE_ICON]);
        XmInstallImage(&palletImg, TBIconsName[PALLET_ICON]);
        XmInstallImage(&inputImg, TBIconsName[INPUT_ICON]);
        XmInstallImage(&charImg, TBIconsName[CHAR_ICON]);
	XmInstallImage(&refreshImg,  TBIconsName[REFRESH_ICON]);
	XmInstallImage(&newImg,  TBIconsName[NEW_ICON]);
	XmInstallImage(&picImg,  TBIconsName[PIC_ICON]);
}

void main(argc, argv)
unsigned int argc;
char **argv;
{
	signal(SIGBUS, SigDestroy);
	signal(SIGFPE, SigDestroy);
	signal(SIGILL, SigDestroy);
	signal(SIGKILL, SigDestroy);
	signal(SIGSTOP, SigDestroy);
	signal(SIGTERM, SigDestroy);
	signal(SIGABRT, SigDestroy);
	signal(SIGTRAP, SigDestroy);
        signal(SIGSEGV, SigDestroy);

#ifndef linux
	signal(SIGEMT, SigDestroy);
	signal(SIGSYS, SigDestroy);
#endif

	XtSetLanguageProc(NULL, NULL, NULL);

	ShowTitle();
	
	toplevel = XtVaAppInitialize(&app, "main", NULL, 0, &argc, argv, fallbacks, NULL);

	XtAppAddActions(app, &actions, 1);
	FontDefaultInitial();
	InputTableInitial(False);
	IniHzInMod(toplevel);  
	InitialSimpleHZWin();
	InstallImage();
	GenerateFSGamma();
	CreateTopLevel(toplevel);
	CreateCharSet(toplevel);
	CreateFileSet(toplevel);
	CreatePrintSet(toplevel);
	CreateGifSet(toplevel);
	CreatePageSet(toplevel);
	CreatePageDel(toplevel);
	CreateErrorMsg(toplevel);
	CreatePromptMsg(toplevel);
	CreatePageNum(toplevel);
	CreateCursor(toplevel);
	CreateTitler(toplevel);
	CreateFinder(toplevel);
	CreateColorTab(toplevel);
	CreatePageNumSet(toplevel);
	CreateTTFSet(toplevel);
	CreatePicImport(toplevel);

	XtRealizeWidget(toplevel);
	
	SetTopLabels();
	CloseTitle();	
	CreateTitleSet(toplevel);

	XtAppMainLoop(app);
}

