/*
 * Copyright 1995, 1996, 1997 by Jun Yei.       All Rights Reserved
 *
 * Permission to retain, use, modify, copy, and distribute ChinesePower
 * in source or binary and its documentation (hereafter, the Software)
 * for noncommercial purpose is hereby granted to you without a fee,
 * provided that this entire copyright and permission notice appear in
 * all such copies, that no charge be associated with such copies,
 * that distribution of derivative works (including value-added
 * distributions such as with additional input dictionaries or fonts)
 * include clarification that such added or derived parts are not from
 * the original Software, and that the names of the author(s) not be
 * used to endorse or promote such works. The author(s) of the software
 * makes no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * THE AUTHOR(S) OF THE SOFTWARE DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 * IN NO EVENT SHALL THE AUTHOR(S) OF THE SOFTWARE BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Jun Yei, Ottawa, ON, Canada
 */

#include "chpower.h"
#include "fileformat.h"

extern void QuitViewport();
extern void NewFileSaveAndQuitStart();
extern void NewFileSaveAndExitStart();
extern void YesHzFormatCB();
extern void NoHzFormatCB();
extern void YesHzFormatRevertCB();
extern void NoHzFormatRevertCB();
extern void DoRevertCB();
extern void YesHzFormatImportCB();
extern void NoHzFormatImportCB();

Widget ErrMsg;
Widget ProMsg;

void ErrOkCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
        XtUnmanageChild(ErrMsg);
}

void CreateErrorMsg(parent)
Widget parent;
{
	Arg args[MAX_ARGS];
	register int n;
	Widget tmpWg;

	n = 0;
	XtSetArg(args[n], XmNdefaultButtonType, XmDIALOG_OK_BUTTON); n++;
	XtSetArg(args[n], XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL); n++;
	ErrMsg = XmCreateErrorDialog(parent, "Message", args, n);
	XtAddCallback(ErrMsg, XmNdestroyCallback, MainDestroyCB, NULL);

	tmpWg = XmMessageBoxGetChild(ErrMsg, XmDIALOG_CANCEL_BUTTON);
	XtUnmanageChild(tmpWg);

	tmpWg = XmMessageBoxGetChild(ErrMsg, XmDIALOG_HELP_BUTTON);
	XtUnmanageChild(tmpWg);
}

void ErrorPopup(msg)
char *msg;
{
	Arg args[MAX_ARGS];
	register int n;

	XtRemoveAllCallbacks(ErrMsg, XmNokCallback);
	n = 0;
	XtSetArg(args[n], XmNmessageString, XmStringCreate(msg, XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetValues(ErrMsg, args, n);

	XtAddCallback(ErrMsg, XmNokCallback, ErrOkCB, NULL);

	XBell(XtDisplay(ErrMsg), 100);

	XtManageChild(ErrMsg);
}

void ErrorFuncPopup(msg, func, data)
char *msg;
void (*func)();
char *data;
{
	Arg args[MAX_ARGS];
	register int n;

	XtRemoveAllCallbacks(ErrMsg, XmNokCallback);

	n = 0;
	XtSetArg(args[n], XmNmessageString, XmStringCreate(msg, XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetValues(ErrMsg, args, n);

	XtAddCallback(ErrMsg, XmNokCallback, func, data);

	XBell(XtDisplay(ErrMsg), 100);

	XtManageChild(ErrMsg);
}

void ErrorChgMsg(msg)
char *msg;
{
	Arg args[1];

	XtSetArg(args[0], XmNmessageString, XmStringCreate(msg, XmSTRING_DEFAULT_CHARSET));
	XtSetValues(ErrMsg, args, 1);
}

void CreatePromptMsg(parent)
Widget parent;
{
	Arg args[MAX_ARGS];
	register int n;

	n = 0;
	XtSetArg(args[n], XmNdefaultButtonType, XmDIALOG_OK_BUTTON); n++;
	XtSetArg(args[n], XmNokLabelString, XmStringCreate("Yes", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNcancelLabelString, XmStringCreate("No", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNhelpLabelString, XmStringCreate("Cancel", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL); n++;        
	ProMsg = XmCreateErrorDialog(parent, "Message", args, n);
	XtAddCallback(ProMsg, XmNdestroyCallback, MainDestroyCB, NULL);
}

void ProSaveCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	MYviewportlist *viewportlist = (MYviewportlist *)client_data;
	VIewportdata *viewport = &viewportlist->myviewport;

	XtUnmanageChild(ProMsg);

	fileset.TextOnly = False;
	fileset.Encoding = FILE_DEFAULT;

	if(viewport->filename != NULL)
	{
		viewport->fp = fopen(viewport->filename, "w");
		if(viewport->fp)
		{
			WriteForeWord(viewport);
			CodingFileWrite(viewport, viewport->fp, viewport->EditFile);
			fclose(viewport->fp);
			QuitViewport(viewportlist);
		}
		else
		{
			ErrorPopup("Unable to open file to write!");
		}
	}	
	else
	{
		NewFileSaveAndQuitStart(w, viewportlist, call_data);
	}
}

void ProNoSaveCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	XtUnmanageChild(ProMsg);
	QuitViewport((MYviewportlist *)client_data);
}

void ExitSaveCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	MYviewportlist *viewportlist = (MYviewportlist *)client_data;
	VIewportdata *viewport = &viewportlist->myviewport;
	
	XtUnmanageChild(ProMsg);

	fileset.TextOnly = False;
	fileset.Encoding = FILE_DEFAULT;
	
	if(viewport->filename != NULL)
	{
		viewport->fp = fopen(viewport->filename, "w");

		if(viewport->fp)
		{
			WriteForeWord(viewport);
			CodingFileWrite(viewport, viewport->fp, viewport->EditFile);
			fclose(viewport->fp);
			QuitViewport(viewportlist);
		}
		else
		{
			ErrorPopup("Unable to open file to write!");
			return;
		}	
	}
	else
	{
		NewFileSaveAndExitStart(w, viewportlist, call_data);
		return;	
	}

	MainExitYesCB(NULL, NULL, NULL);
}

void ExitNoSaveCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	MYviewportlist *viewportlist = (MYviewportlist *)client_data;

	XtUnmanageChild(ProMsg);
	QuitViewport(viewportlist);
	MainExitYesCB(NULL, NULL, NULL);
}

void ProCancelCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	XtUnmanageChild(ProMsg);
}

void ConfirmQuit(viewportlist, YesSaveCB, NoSaveCB)
MYviewportlist *viewportlist;
void (*YesSaveCB)();
void (*NoSaveCB)();
{
	Arg args[MAX_ARGS];
	register int n;
	Widget tmpWg;
	char question[50];
	question[0] = '\0';
	
	if(viewportlist->myviewport.filename != NULL)
	{
		strcat(question, "Save ");
		n = strlen(viewportlist->myviewport.filename);
		if(n > 34) n -= 34; else n = 0; 
		strcat(question, viewportlist->myviewport.filename + n);
		strcat(question, "?");
	}
	else strcat(question, "Save Unamed File?");
	
	n = 0;
	XtSetArg(args[n], XmNmessageString, XmStringCreate(question, XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetValues(ProMsg, args, n);

	tmpWg = XmMessageBoxGetChild(ProMsg, XmDIALOG_HELP_BUTTON);
	XtManageChild(tmpWg);

	XtRemoveAllCallbacks(ProMsg, XmNokCallback);
	XtRemoveAllCallbacks(ProMsg, XmNhelpCallback);    
	XtRemoveAllCallbacks(ProMsg, XmNcancelCallback);    

	XtAddCallback(ProMsg, XmNokCallback, YesSaveCB, viewportlist);
	XtAddCallback(ProMsg, XmNcancelCallback, NoSaveCB, viewportlist);
	XtAddCallback(ProMsg, XmNhelpCallback, ProCancelCB, viewportlist);

/*	XBell(XtDisplay(ProMsg), 100);
*/	XtManageChild(ProMsg);
}

int IsHzFormat(fp, data, func1, func2)
FILE *fp;
void *data;
void (*func1)(), (*func2)();
{
	Arg args[MAX_ARGS];
	register int n;
	Widget tmpWg;
	char *cp, line[100];

	rewind(fp);

	while((cp=fgets(line,99, fp)) != NULL)
	{
		while(*cp != '\n' && *cp != '\0')
		{
			if(*cp++ == '~')
			{
				if(*cp == '{')
				{
					if(fileset.Encoding == FILE_HZ)
					{
						func1(NULL, data, NULL);
						return True;
					}

					n = 0;
					XtSetArg(args[n], XmNmessageString, XmStringCreate("Is this file in HZ format?", XmSTRING_DEFAULT_CHARSET)); n++;
					XtSetValues(ProMsg, args, n);

					XtRemoveAllCallbacks(ProMsg, XmNokCallback);
					XtRemoveAllCallbacks(ProMsg, XmNhelpCallback);    
					XtRemoveAllCallbacks(ProMsg, XmNcancelCallback);    

					tmpWg = XmMessageBoxGetChild(ProMsg, XmDIALOG_HELP_BUTTON);
					XtUnmanageChild(tmpWg);

					XtAddCallback(ProMsg, XmNokCallback, func1, data);
					XtAddCallback(ProMsg, XmNcancelCallback, func2, data);

					XBell(XtDisplay(ProMsg), 100);
					XtManageChild(ProMsg);

					return True;
				}
			}
		}
	}

	return False;
}

void YESdoNOreturn(yesCB, data, message)
void (*yesCB)();
void *data;
char *message;
{
	Arg args[MAX_ARGS];
	register int n;
	Widget tmpWg;
	
	n = 0;
	XtSetArg(args[n], XmNmessageString, XmStringCreate(message, XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetValues(ProMsg, args, n);

	XtRemoveAllCallbacks(ProMsg, XmNokCallback);
	XtRemoveAllCallbacks(ProMsg, XmNhelpCallback);    
	XtRemoveAllCallbacks(ProMsg, XmNcancelCallback);    

	tmpWg = XmMessageBoxGetChild(ProMsg, XmDIALOG_HELP_BUTTON);
	XtUnmanageChild(tmpWg);

	XtAddCallback(ProMsg, XmNokCallback, yesCB, data);
	XtAddCallback(ProMsg, XmNcancelCallback, ProCancelCB, NULL);
	
	XBell(XtDisplay(ProMsg), 100);

	XtManageChild(ProMsg);
}

void ConfirmSaveHZAssoc(newhzmode, YesSaveCB, NoSaveCB)
char *newhzmode;
void (*YesSaveCB)();
void (*NoSaveCB)();
{
	Arg args[MAX_ARGS];
	register int n;
	Widget tmpWg;
	char question[20];
	question[0] = '\0';
	
	strcat(question, "Save ");
	strcat(question, MyHzInMod.InputWin.misc.hz_encoding);
	strcat(question, " phrases file?");
	
	n = 0;
	XtSetArg(args[n], XmNmessageString, XmStringCreate(question, XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetValues(ProMsg, args, n);

	tmpWg = XmMessageBoxGetChild(ProMsg, XmDIALOG_HELP_BUTTON);
	XtUnmanageChild(tmpWg);

	XtRemoveAllCallbacks(ProMsg, XmNokCallback);
	XtRemoveAllCallbacks(ProMsg, XmNhelpCallback);    
	XtRemoveAllCallbacks(ProMsg, XmNcancelCallback);    

	XtAddCallback(ProMsg, XmNokCallback, YesSaveCB, newhzmode);
	XtAddCallback(ProMsg, XmNcancelCallback, NoSaveCB, newhzmode);

/*	XBell(XtDisplay(ProMsg), 100);
*/	XtManageChild(ProMsg);
}

void YESdoNOdo(YesDoCB, yesdata, NoDoCB, nodata, msg)
char *yesdata, *nodata, *msg;
void (*YesDoCB)();
void (*NoDoCB)();
{
	Arg args[MAX_ARGS];
	register int n;
	Widget tmpWg;

	n = 0;
	XtSetArg(args[n], XmNmessageString, XmStringCreate(msg, XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetValues(ProMsg, args, n);

	tmpWg = XmMessageBoxGetChild(ProMsg, XmDIALOG_HELP_BUTTON);
	XtUnmanageChild(tmpWg);

	XtRemoveAllCallbacks(ProMsg, XmNokCallback);
	XtRemoveAllCallbacks(ProMsg, XmNhelpCallback);
	XtRemoveAllCallbacks(ProMsg, XmNcancelCallback);

	XtAddCallback(ProMsg, XmNokCallback, YesDoCB, yesdata);
	XtAddCallback(ProMsg, XmNcancelCallback, NoDoCB, nodata);

	XBell(XtDisplay(ProMsg), 100);
	XtManageChild(ProMsg);
}

void LongProcCancelCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	* (int *)client_data = True;
        XtUnmanageChild(ErrMsg);
}

