/*
 * Copyright 1995, 1996, 1997 by Jun Yei.       All Rights Reserved
 *
 * Permission to retain, use, modify, copy, and distribute ChinesePower
 * in source or binary and its documentation (hereafter, the Software)
 * for noncommercial purpose is hereby granted to you without a fee,
 * provided that this entire copyright and permission notice appear in
 * all such copies, that no charge be associated with such copies,
 * that distribution of derivative works (including value-added
 * distributions such as with additional input dictionaries or fonts)
 * include clarification that such added or derived parts are not from
 * the original Software, and that the names of the author(s) not be
 * used to endorse or promote such works. The author(s) of the software
 * makes no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * THE AUTHOR(S) OF THE SOFTWARE DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 * IN NO EVENT SHALL THE AUTHOR(S) OF THE SOFTWARE BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Jun Yei, Ottawa, ON, Canada
 */

#include "chpower.h"

void CancelPageNumCB();
void OkPageNumCB();
void PageNumColorCB();
void PageNumAlignCB();
void PageNumPosCB();

PAgeNumPopup pnset;

int PageNumSet(mypn)
PAgeNumData *mypn;
{	
	Arg args[5];
	register int n;
	XmString *newfont = NULL;
	char *name = NULL;
	int width, height;

	VIewportdata *viewport = (VIewportdata *) mypn->viewport;
	
	n = 0;
	XtSetArg(args[n], XmNvalue, &name); n++;
	XtGetValues(pnset.WidthWg, args, n);

	if(name)
	{
		sscanf(name, "%d", &width);
		XtFree(name); 
	}

	n = 0;
	XtSetArg(args[n], XmNvalue, &name); n++;
	XtGetValues(pnset.HeightWg, args, n);

	if(name)
	{
		sscanf(name, "%d", &height);
		XtFree(name);
	}
	
	if(height > MAX_FONT_SIZE || width > MAX_FONT_SIZE)     
	{
		ErrorPopup("Font Size is Too Big!");
		return(False);
	}

	if(height < MIN_FONT_SIZE || width < MIN_FONT_SIZE)     
	{
		ErrorPopup("Font Size is Too Small!");
		return(False);
	}

	mypn->hzdata.point_width = width * 100;
	mypn->hzdata.point_height = height * 100;

	n = 0;
	XtSetArg(args[n], XmNselectedItems, &newfont); n++;
	XtGetValues(pnset.HzFontLstWg, args, n);
	if(newfont) XmStringGetLtoR(*newfont, XmSTRING_DEFAULT_CHARSET, &name);

	if(name)
	{
		mypn->hzdata.hbf = GetFontHandle(name, &mypn->hzdata.FontId, &mypn->hzdata.FontType);
		
		if(mypn->hzdata.FontType == HBFONT) HBF_GetBitmapBoundingBox(mypn->hzdata.hbf, &(mypn->hzdata.hz_width), &(mypn->hzdata.hz_height),
		&width, &height);
		else mypn->hzdata.hz_width = mypn->hzdata.hz_height = 24;

		XtFree(name); 
	}

	FontCalc(viewport->resource.WorkArea, &mypn->hzdata, viewport->zoom);

	mypn->ColorNum = pnset.ColorNum;
	mypn->Position = pnset.Position;
	mypn->Align = pnset.Align;
	mypn->ShowIt = pnset.ShowIt;
	mypn->FirstShow = pnset.FirstShow;

	if(mypn->texta) 
	{
		free(mypn->texta);
		mypn->texta = NULL;
	}
	
	if(DBlen(pnset.texta.EditFile) > 0)
	{
		mypn->texta = (unsigned short*)DBdup(pnset.texta.EditFile);
	}
	
	if(mypn->textb) 
	{
		free(mypn->textb);
		mypn->textb = NULL;
	}
	
	if(DBlen(pnset.textb.EditFile) > 0)
	{
		mypn->textb = (unsigned short*)DBdup(pnset.textb.EditFile);
	}
	
	viewport->changed = True;
	viewport->autosavechanged = True;

	MyDrawPage(viewport, TRUEDRAW);

	return(True);
}


void CreatePageNumSet(parent)
Widget parent;
{
	Widget label, submenu;

	Arg args[MAX_ARGS];
	register int n;

	pnset.mapped = False;

	pnset.PopUpWg = XtCreatePopupShell("PageNum Win", transientShellWidgetClass, parent, NULL, 0);

	n = 0;
	XtSetArg(args[n], XmNheight, 310); n++;
	XtSetArg(args[n], XmNwidth, 360); n++;
	XtSetArg(args[n], XmNhorizontalSpacing, 0); n++;
	XtSetArg(args[n], XmNverticalSpacing, 0); n++;
	XtSetArg(args[n], XmNrubberPositioning, True); n++;
	pnset.Root = XmCreateForm(pnset.PopUpWg, "Set", args, n);
	XtManageChild(pnset.Root);
	XtAddCallback(pnset.Root, XmNdestroyCallback, MainDestroyCB, NULL);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Font Type:", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNx, 15); n++;
	XtSetArg(args[n], XmNy, 40); n++;
	label = XmCreateLabelGadget(pnset.Root, "Ftype", args, n);
	XtManageChild(label);
	
	n = 0;
	XtSetArg(args[n], XmNitems, charset.fontlist); n++;
	XtSetArg(args[n], XmNitemCount, TotalNumOfFont()+1); n++;
	XtSetArg(args[n], XmNx, 15); n++;
	XtSetArg(args[n], XmNy, 55); n++;
	XtSetArg(args[n], XmNheight, 145); n++;
	XtSetArg(args[n], XmNwidth, 100); n++;
	pnset.HzFontLstWg = XmCreateScrolledList(pnset.Root, "HZfontlist", args, n);
	XtManageChild(pnset.HzFontLstWg);
	
	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Width:", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNx, 130); n++;
	XtSetArg(args[n], XmNy, 55); n++;
	label = XmCreateLabelGadget(pnset.Root, "Width", args, n);
	XtManageChild(label);

	n = 0;
	XtSetArg(args[n], XmNwidth, 50); n++;
	XtSetArg(args[n], XmNscrollHorizontal, False); n++;
	XtSetArg(args[n], XmNscrollVertical, False); n++;
	XtSetArg(args[n], XmNresizeHeight, False); n++;
	XtSetArg(args[n], XmNresizeWidth, False); n++;
	XtSetArg(args[n], XmNmarginHeight, 0); n++;
	XtSetArg(args[n], XmNmarginWidth, 0); n++;
	XtSetArg(args[n], XmNmaxLength, 3); n++;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(args[n], XmNleftWidget, label); n++;
	XtSetArg(args[n], XmNy, 55); n++;
	pnset.WidthWg = XmCreateText(pnset.Root, "width", args, n);
	XtManageChild(pnset.WidthWg);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Height:", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNx, 240); n++;
	XtSetArg(args[n], XmNy, 55); n++;
	label = XmCreateLabelGadget(pnset.Root, "Height", args, n);
	XtManageChild(label);

	n = 0;
	XtSetArg(args[n], XmNwidth, 50); n++;
	XtSetArg(args[n], XmNscrollHorizontal, False); n++;
	XtSetArg(args[n], XmNscrollVertical, False); n++;
	XtSetArg(args[n], XmNresizeHeight, False); n++;
	XtSetArg(args[n], XmNresizeWidth, False); n++;
	XtSetArg(args[n], XmNmarginHeight, 0); n++;
	XtSetArg(args[n], XmNmarginWidth, 0); n++;
	XtSetArg(args[n], XmNmaxLength, 3); n++;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(args[n], XmNleftWidget, label); n++;
	XtSetArg(args[n], XmNy, 55); n++;
	pnset.HeightWg = XmCreateText(pnset.Root, "height", args, n);
	XtManageChild(pnset.HeightWg);

	n = 0;
	XtSetArg(args[n], XmNindicatorType, XmN_OF_MANY); n++;
	XtSetArg(args[n], XmNvisibleWhenOff, True); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Show Page Number", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNx, 100); n++;
	XtSetArg(args[n], XmNy, 15); n++;
	pnset.ShowNumTgWd = XmCreateToggleButtonGadget(pnset.Root, "Tog", args, n);
	XtAddCallback(pnset.ShowNumTgWd, XmNvalueChangedCallback, GifToggleCB, &pnset.ShowIt);
	XtManageChild(pnset.ShowNumTgWd);

	n = 0;
	XtSetArg(args[n], XmNindicatorType, XmN_OF_MANY); n++;
	XtSetArg(args[n], XmNvisibleWhenOff, True); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("First Page As Cover", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNx, 170); n++;
	XtSetArg(args[n], XmNy, 165); n++;
	pnset.FirstTgWd = XmCreateToggleButtonGadget(pnset.Root, "Tog", args, n);
	XtAddCallback(pnset.FirstTgWd, XmNvalueChangedCallback, GifToggleCB, &pnset.FirstShow);
	XtManageChild(pnset.FirstTgWd);

	submenu = (Widget) XmCreatePulldownMenu(pnset.Root, "Pull", NULL, 0);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Standard", XmSTRING_DEFAULT_CHARSET)); n++;
	pnset.ColorBtn[0] = XmCreatePushButtonGadget(submenu, "Co", args, n);
	XtAddCallback(pnset.ColorBtn[0], XmNactivateCallback, PageNumColorCB, (caddr_t *)0);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Color 1", XmSTRING_DEFAULT_CHARSET)); n++;
	pnset.ColorBtn[1] = XmCreatePushButtonGadget(submenu, "Co", args, n); 
	XtAddCallback(pnset.ColorBtn[1], XmNactivateCallback, PageNumColorCB, (caddr_t *)1);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Color 2", XmSTRING_DEFAULT_CHARSET)); n++;
	pnset.ColorBtn[2] = XmCreatePushButtonGadget(submenu, "Co", args, n); 
	XtAddCallback(pnset.ColorBtn[2], XmNactivateCallback, PageNumColorCB, (caddr_t *)2);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Color 3", XmSTRING_DEFAULT_CHARSET)); n++;
	pnset.ColorBtn[3] = XmCreatePushButtonGadget(submenu, "Co", args, n); 
	XtAddCallback(pnset.ColorBtn[3], XmNactivateCallback, PageNumColorCB, (caddr_t *)3);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Color 4", XmSTRING_DEFAULT_CHARSET)); n++;
	pnset.ColorBtn[4] = XmCreatePushButtonGadget(submenu, "Co", args, n); 
	XtAddCallback(pnset.ColorBtn[4], XmNactivateCallback, PageNumColorCB, (caddr_t *)4);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Color 5", XmSTRING_DEFAULT_CHARSET)); n++;
	pnset.ColorBtn[5] = XmCreatePushButtonGadget(submenu, "Co", args, n); 
	XtAddCallback(pnset.ColorBtn[5], XmNactivateCallback, PageNumColorCB, (caddr_t *)5);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Color 6", XmSTRING_DEFAULT_CHARSET)); n++;
	pnset.ColorBtn[6] = XmCreatePushButtonGadget(submenu, "Co", args, n); 
	XtAddCallback(pnset.ColorBtn[6], XmNactivateCallback, PageNumColorCB, (caddr_t *)6);

	XtManageChildren(pnset.ColorBtn, 7);

	n = 0;
	XtSetArg(args[n], XmNsubMenuId, submenu); n++;
	XtSetArg(args[n], XmNmenuHistory, pnset.ColorBtn[0]); n++;
	XtSetArg(args[n], XmNmarginHeight, 0); n++;
	XtSetArg(args[n], XmNmarginWidth, 0); n++;
	XtSetArg(args[n], XmNspacing, 0); n++;
	XtSetArg(args[n], XmNpacking, XmPACK_TIGHT); n++;
	XtSetArg(args[n], XmNx, 130); n++;
	XtSetArg(args[n], XmNy, 85); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Color:", XmSTRING_DEFAULT_CHARSET)); n++;
	pnset.ColorOpWg = XmCreateOptionMenu(pnset.Root, "Pull", args, n);
	XtManageChild(pnset.ColorOpWg);

	submenu = (Widget) XmCreatePulldownMenu(pnset.Root, "Pull", NULL, 0);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Left", XmSTRING_DEFAULT_CHARSET)); n++;
	pnset.AlignBtn[0] = XmCreatePushButtonGadget(submenu, "Co", args, n);
	XtAddCallback(pnset.AlignBtn[0], XmNactivateCallback, PageNumAlignCB, (caddr_t *)0);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Center", XmSTRING_DEFAULT_CHARSET)); n++;
	pnset.AlignBtn[1] = XmCreatePushButtonGadget(submenu, "Co", args, n); 
	XtAddCallback(pnset.AlignBtn[1], XmNactivateCallback, PageNumAlignCB, (caddr_t *)1);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Right", XmSTRING_DEFAULT_CHARSET)); n++;
	pnset.AlignBtn[2] = XmCreatePushButtonGadget(submenu, "Co", args, n); 
	XtAddCallback(pnset.AlignBtn[2], XmNactivateCallback, PageNumAlignCB, (caddr_t *)2);

	XtManageChildren(pnset.AlignBtn, 3);

	n = 0;
	XtSetArg(args[n], XmNsubMenuId, submenu); n++;
	XtSetArg(args[n], XmNmenuHistory, pnset.AlignBtn[0]); n++;
	XtSetArg(args[n], XmNmarginHeight, 0); n++;
	XtSetArg(args[n], XmNmarginWidth, 0); n++;
	XtSetArg(args[n], XmNspacing, 0); n++;
	XtSetArg(args[n], XmNpacking, XmPACK_TIGHT); n++;
	XtSetArg(args[n], XmNx, 130); n++;
	XtSetArg(args[n], XmNy, 125); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Align:", XmSTRING_DEFAULT_CHARSET)); n++;
	pnset.AlignOpWg = XmCreateOptionMenu(pnset.Root, "Pull", args, n);
	XtManageChild(pnset.AlignOpWg);

	submenu = (Widget) XmCreatePulldownMenu(pnset.Root, "Pull", NULL, 0);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Top", XmSTRING_DEFAULT_CHARSET)); n++;
	pnset.PosBtn[0] = XmCreatePushButtonGadget(submenu, "Co", args, n);
	XtAddCallback(pnset.PosBtn[0], XmNactivateCallback, PageNumPosCB, (caddr_t *)0);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Bottom", XmSTRING_DEFAULT_CHARSET)); n++;
	pnset.PosBtn[1] = XmCreatePushButtonGadget(submenu, "Co", args, n); 
	XtAddCallback(pnset.PosBtn[1], XmNactivateCallback, PageNumPosCB, (caddr_t *)1);

	XtManageChildren(pnset.PosBtn, 2);

	n = 0;
	XtSetArg(args[n], XmNsubMenuId, submenu); n++;
	XtSetArg(args[n], XmNmenuHistory, pnset.PosBtn[0]); n++;
	XtSetArg(args[n], XmNmarginHeight, 0); n++;
	XtSetArg(args[n], XmNmarginWidth, 0); n++;
	XtSetArg(args[n], XmNspacing, 0); n++;
	XtSetArg(args[n], XmNpacking, XmPACK_TIGHT); n++;
	XtSetArg(args[n], XmNx, 250); n++;
	XtSetArg(args[n], XmNy, 125); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Pos:", XmSTRING_DEFAULT_CHARSET)); n++;
	pnset.PosOpWg = XmCreateOptionMenu(pnset.Root, "Pull", args, n);
	XtManageChild(pnset.PosOpWg);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Text Before Page Num:", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNx, 15); n++;
	XtSetArg(args[n], XmNy, 210); n++;
	label = XmCreateLabelGadget(pnset.Root, "Text", args, n);
	XtManageChild(label);

	n = 0;
	XtSetArg(args[n], XmNwidth, 150); n++;
	XtSetArg(args[n], XmNheight,23); n++;
	XtSetArg(args[n], XmNx, 15); n++;
	XtSetArg(args[n], XmNy, 225); n++;
	pnset.TextBWg = CreateSimpleHzWin(pnset.Root, "Text:", args, n, &pnset.textb); 
	XtManageChild(pnset.TextBWg);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Text After Page Num:", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNx, 195); n++;
	XtSetArg(args[n], XmNy, 210); n++;
	label = XmCreateLabelGadget(pnset.Root, "Text", args, n);
	XtManageChild(label);

	n = 0;
	XtSetArg(args[n], XmNwidth, 150); n++;
	XtSetArg(args[n], XmNheight,23); n++;
	XtSetArg(args[n], XmNx, 195); n++;
	XtSetArg(args[n], XmNy, 225); n++;
	pnset.TextAWg = CreateSimpleHzWin(pnset.Root, "Text", args, n, &pnset.texta); 
	XtManageChild(pnset.TextAWg);

	n = 0;
	XtSetArg(args[n], XmNwidth, 100); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Ok", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNx, 80); n++;
	XtSetArg(args[n], XmNy, 270); n++;
	label = XmCreatePushButtonGadget(pnset.Root, "Btn", args, n);
	XtAddCallback(label, XmNactivateCallback, OkPageNumCB, NULL);
	XtManageChild(label);

	n = 0;
	XtSetArg(args[n], XmNwidth, 100); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Cancel", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNy, 270); n++;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(args[n], XmNleftWidget, label); n++;
	label = XmCreatePushButtonGadget(pnset.Root, "Btn", args, n);
	XtAddCallback(label, XmNactivateCallback, CancelPageNumCB, NULL);
	XtManageChild(label);

	XtRealizeWidget(pnset.PopUpWg);

	AddSHwCallbacks(&pnset.textb);
	AddSHwCallbacks(&pnset.texta);
}

void PageNumSetPopupCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	Arg args[5];
	register int n;
	PAgeNumData *mypn;
	int i;
	char line[10];
	VIewportdata *viewport = (VIewportdata *)client_data;

	if(pnset.mapped) 
	{
		XRaiseWindow(XtDisplay(pnset.PopUpWg), XtWindow(pnset.PopUpWg));
		ErrorPopup("Close Another Page Number Window First!");
		return;
	}

	active_viewport = viewport;
	
	mypn = &viewport->pndata;
	pnset.Ptrpn = mypn;

	mypn->mapped = True;

	i = SearchFontList(mypn->hzdata.FontId);

	n = 0;
	XtSetArg(args[n], XmNselectedItems, &charset.fontlist[i]); n++;
	XtSetArg(args[n], XmNselectedItemCount, 1); n++;
	XtSetValues(pnset.HzFontLstWg, args, n);

	sprintf(line, "%d\0", mypn->hzdata.point_width/100);
	
	n = 0;
	XtSetArg(args[n], XmNvalue, line); n++;
	XtSetArg(args[n], XmNmarginHeight, 0); n++;
	XtSetArg(args[n], XmNmarginWidth, 0); n++;
	XtSetValues(pnset.WidthWg, args, n);

	sprintf(line, "%d\0", mypn->hzdata.point_height/100);

	n = 0;
	XtSetArg(args[n], XmNvalue, line); n++;
	XtSetArg(args[n], XmNmarginHeight, 0); n++;
	XtSetArg(args[n], XmNmarginWidth, 0); n++;
	XtSetValues(pnset.HeightWg, args, n);

	if(mypn->ShowIt) XtSetArg(args[0], XmNset, True);
	else XtSetArg(args[0], XmNset, False);

	XtSetValues(pnset.ShowNumTgWd, args, 1);

	if(mypn->FirstShow) XtSetArg(args[0], XmNset, False);
	else XtSetArg(args[0], XmNset, True);
	
	XtSetValues(pnset.FirstTgWd, args, 1);

	pnset.ShowIt = mypn->ShowIt;

	pnset.FirstShow = mypn->FirstShow;

	pnset.ColorNum = mypn->ColorNum;

	XtSetArg(args[0], XmNmenuHistory, pnset.ColorBtn[pnset.ColorNum]); 
	XtSetValues(pnset.ColorOpWg, args, 1);

	pnset.Position = mypn->Position;

	XtSetArg(args[0], XmNmenuHistory, pnset.PosBtn[pnset.Position]);
	XtSetValues(pnset.PosOpWg, args, 1);

	pnset.Align = mypn->Align;

	XtSetArg(args[0], XmNmenuHistory, pnset.AlignBtn[pnset.Align]);
	XtSetValues(pnset.AlignOpWg, args, 1);

	if(mypn->texta) 
	{
		DBcpy(pnset.texta.EditFile, mypn->texta, SHZW_MEM_UNIT);
		pnset.texta.EditFile[SHZW_MEM_UNIT] = '\0';    
		pnset.texta.viewp.currentnum = DBbytelen(pnset.texta.EditFile);
		pnset.texta.viewp.DBcurrentnum = DBlen(pnset.texta.EditFile);	
	}
	else
	{
		*pnset.texta.EditFile = '\0';
		pnset.texta.viewp.currentnum = 0;
		pnset.texta.viewp.DBcurrentnum = 0;	
	}
	
	if(mypn->textb) 
	{
		DBcpy(pnset.textb.EditFile, mypn->textb, SHZW_MEM_UNIT);
		pnset.textb.EditFile[SHZW_MEM_UNIT] = '\0';    
		pnset.textb.viewp.currentnum = DBbytelen(pnset.textb.EditFile);
		pnset.textb.viewp.DBcurrentnum = DBlen(pnset.textb.EditFile);	
	}
	else
	{
		*pnset.textb.EditFile = '\0';
		pnset.textb.viewp.currentnum = 0;
		pnset.textb.viewp.DBcurrentnum = 0;	
	}
	
	pnset.texta.viewp.firstnum = 0; 
	pnset.texta.cursor.CurPos = 0;
	pnset.texta.cursor.MovePos = 0;
	pnset.texta.cursor.StartPos = 0;
	pnset.texta.cursor.EndPos = 0;
	pnset.texta.cursor.DBCurPos = 0;
	pnset.texta.cursor.DBMovePos = 0;
	pnset.texta.cursor.DBStartPos = 0;
	pnset.texta.cursor.DBEndPos = 0;

	pnset.textb.viewp.firstnum = 0; 
	pnset.textb.cursor.CurPos = 0;
	pnset.textb.cursor.MovePos = 0;
	pnset.textb.cursor.StartPos = 0;
	pnset.textb.cursor.EndPos = 0;
	pnset.textb.cursor.DBCurPos = 0;
	pnset.textb.cursor.DBMovePos = 0;
	pnset.textb.cursor.DBStartPos = 0;
	pnset.textb.cursor.DBEndPos = 0;

	pnset.mapped = True;

	PopupPosition(viewport->resource.PopUpShell, pnset.PopUpWg);
	XtPopup(pnset.PopUpWg, XtGrabNone); 
}

void PageNumColorCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	pnset.ColorNum = (int) client_data;
}

void PageNumAlignCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	pnset.Align = (int) client_data;
}

void PageNumPosCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	pnset.Position = (int) client_data;
}

void OkPageNumCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	if(!PageNumSet(pnset.Ptrpn)) return;
	pnset.mapped = False;
	pnset.Ptrpn->mapped = False;
	simple_active = False;
	active_simphzwin = NULL;
	XtPopdown(pnset.PopUpWg);
}

void CancelPageNumCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	pnset.mapped = False;
	pnset.Ptrpn->mapped = False;
	simple_active = False;
	active_simphzwin = NULL;
	XtPopdown(pnset.PopUpWg);
}

void FileReadSetPageNum(viewport, cp, Ptrpn, id)
VIewportdata *viewport;
char *cp;
PAgeNumData *Ptrpn;
int id;
{
	if(id) ReadSetPageNum(viewport, cp, Ptrpn);
	else 
	{
		Ptrpn->texta = (unsigned short*)strDBdup(cp);
	}
}

void ReadSetPageNum(viewport, cp, Ptrpn)
VIewportdata *viewport;
PAgeNumData *Ptrpn;
char *cp;
{
	int font_height = GlobalDefaults.fontheight;
	int font_width = GlobalDefaults.fontwidth;
	char *fontname = NULL, *searchprt;
	int i, j;

	Ptrpn->ColorNum = 0;
	Ptrpn->Position = 1;
	Ptrpn->Align = 2;
	Ptrpn->ShowIt = True;
	Ptrpn->FirstShow = True;

	for(searchprt = cp; *searchprt != '\n' && *searchprt != '\0'; searchprt++);

	*searchprt = '\0';

	if(Ptrpn->texta) 
	{
		free(Ptrpn->texta);
		Ptrpn->texta = NULL;
	}

	if(Ptrpn->textb)
	{
		free(Ptrpn->textb);
		Ptrpn->textb = NULL;
	}

	while(*cp == '@' && *(cp+1) == '[')
	{
		cp += 2;
		if(*cp >= '0' && *cp <= '9')
		{
			for(searchprt = cp; *searchprt != '\0' && *searchprt != ']'; searchprt++);
			
			if(*searchprt == '\0') break;

			*searchprt = '\0';

			if(sscanf(cp, "%d,%d", &i, &j) == 2)
			{
				if(i*j <= MAX_FONT_SIZE)
				{
					font_width = i;
					font_height = j;
				}	
			}
			else
			{
				if(i*i <= MAX_FONT_SIZE) font_width = font_height = i;
			}

			cp = searchprt + 1;
	
			continue;
		}
	
		if(*cp >= 'a' && *cp <= 'z')
		{
			for(searchprt = cp; *searchprt != '\0' && *searchprt != ']'; searchprt++);
				
			if(*searchprt == '\0') break;

			*searchprt = '\0';

			fontname = cp;
			
			cp = searchprt + 1;
	
			continue;
		}
	
		if(*cp == 'D' && *(cp+1) == ':')
		{
			cp += 2;
			
			for(searchprt = cp; *searchprt != '\0' && *searchprt != ']'; searchprt++);
				
			if(*searchprt == '\0') break;

			*searchprt = '\0';

			Ptrpn->textb = (unsigned short*)strDBdup(cp);
			
			cp = searchprt + 1;
	
			continue;
		}
	
		if(*cp == 'E' && *(cp+1) == ':')
		{
			cp += 2;
			
			for(searchprt = cp; *searchprt != '\0' && *searchprt != ']'; searchprt++);
				
			if(*searchprt == '\0') break;

			*searchprt = '\0';

			Ptrpn->texta = (unsigned short*)strDBdup(cp);
			
			cp = searchprt + 1;
	
			continue;
		}
	
		if(*cp == 'Z')
		{
			cp++;
			searchprt = cp + 1;

			if(*cp >= '0' && *cp <= '6' && *searchprt == ']')
			{
				Ptrpn->ColorNum = (int)(*cp) - (int)('0');
			}

			cp = searchprt + 1;
			
			continue;
		}
	
		if(*cp == 'U' && *(cp+1) == ']')
		{
			Ptrpn->ShowIt = False;
			cp += 2;
			
			continue;
		}

		if(*cp == 'C' && *(cp+1) == ']')
		{
			Ptrpn->FirstShow = False;
			cp += 2;

			continue;
		}

		if(*cp == 'A')
		{
			cp++;
			searchprt = cp + 1;
			
			if(*cp >= '0' && *cp <= '2' && *searchprt == ']')
			{
				Ptrpn->Align = (int)(*cp) - (int)('0');
			}

			cp = searchprt + 1;
			
			continue;
		}
	
		if(*cp == 'B')
		{
			cp++;
			searchprt = cp + 1;
			
			if(*cp >= '0' && *cp <= '1' && *searchprt == ']')
			{
				Ptrpn->Position = (int)(*cp) - (int)('0');
			}

			cp = searchprt + 1;
			
			continue;
		}
	}

	if(fontname) InitPointFont(viewport->resource.WorkArea, &Ptrpn->hzdata, fontname, font_width, font_height, viewport->zoom);
	else InitPointFont(viewport->resource.WorkArea, &Ptrpn->hzdata, GlobalDefaults.fontname, font_width, font_height, viewport->zoom);
}

void ResetPageNum(viewport, Ptrpn)
VIewportdata *viewport;
PAgeNumData *Ptrpn;
{
	if(Ptrpn->texta)
	{
		free(Ptrpn->texta);
		Ptrpn->texta = NULL;
	}

	if(Ptrpn->textb)
	{
		free(Ptrpn->textb);
		Ptrpn->textb = NULL;
	}

	Ptrpn->Align = 2;
	Ptrpn->ColorNum = 0;
	Ptrpn->Position = 1;
	Ptrpn->ShowIt = True;
	Ptrpn->FirstShow = True;

	InitPointFont(viewport->resource.WorkArea, &Ptrpn->hzdata, GlobalDefaults.fontname, GlobalDefaults.pagenum_width, 
	GlobalDefaults.pagenum_height, viewport->zoom);
}

void WritePageNumToFile(viewport, Ptrpn, note)
VIewportdata *viewport;
PAgeNumData *Ptrpn;
char *note;
{
	unsigned char *sp;	
	
	if(Ptrpn->texta || Ptrpn->textb || Ptrpn->hzdata.point_width/100 != GlobalDefaults.pagenum_width || Ptrpn->hzdata.point_height/100 !=
	GlobalDefaults.pagenum_height || Ptrpn->ColorNum != 0 || Ptrpn->Position != 1 || Ptrpn->Align != 2 || !Ptrpn->ShowIt || !Ptrpn->FirstShow
	|| strcmp(Ptrpn->hzdata.FontId, GlobalDefaults.fontname) != 0)
	{
		fprintf(viewport->fp, "%s@[%d,%d]@[%s]@[Z%d]@[B%d]@[A%d]", note, Ptrpn->hzdata.point_width/100, Ptrpn->hzdata.point_height/100, 
			Ptrpn->hzdata.FontId, Ptrpn->ColorNum, Ptrpn->Position, Ptrpn->Align);	
		if(!Ptrpn->ShowIt) fprintf(viewport->fp, "@[U]");
		if(!Ptrpn->FirstShow) fprintf(viewport->fp, "@[C]");
		if(Ptrpn->texta) 
		{
			sp = DBstrdup(Ptrpn->texta);
			fprintf(viewport->fp, "@[E:%s]", sp);
			free(sp);
		}
		if(Ptrpn->textb) 
		{
			sp = DBstrdup(Ptrpn->textb);
			fprintf(viewport->fp, "@[D:%s]", sp);
			free(sp);      
		}
		fprintf(viewport->fp, "\n");
	}
}

