/*
 * Copyright 1995, 1996, 1997 by Jun Yei.       All Rights Reserved
 *
 * Permission to retain, use, modify, copy, and distribute ChinesePower
 * in source or binary and its documentation (hereafter, the Software)
 * for noncommercial purpose is hereby granted to you without a fee,
 * provided that this entire copyright and permission notice appear in
 * all such copies, that no charge be associated with such copies,
 * that distribution of derivative works (including value-added
 * distributions such as with additional input dictionaries or fonts)
 * include clarification that such added or derived parts are not from
 * the original Software, and that the names of the author(s) not be
 * used to endorse or promote such works. The author(s) of the software
 * makes no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * THE AUTHOR(S) OF THE SOFTWARE DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 * IN NO EVENT SHALL THE AUTHOR(S) OF THE SOFTWARE BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Jun Yei, Ottawa, ON, Canada
 */

#include "chpower.h"

void ClosePageSetCB();
void DoPageSetCB();
int PageSet();
void PageLandscapeCB();
void PagePortraitCB();
void PageSizePreCB();
void LineVerticalCB();
void LineHorizonCB();

PageSetMenu pageset;

int PageSet(viewport)
VIewportdata *viewport;
{	
	char *line = NULL;
	Arg args[1];
	int height, width, pgh, pgw;

	XtSetArg(args[0], XmNvalue, &line);
	XtGetValues(pageset.VerMarWg, args, 1);

	sscanf(line, "%d", &height);

	XtFree(line);

	XtSetArg(args[0], XmNvalue, &line);
	XtGetValues(pageset.HorMarWg, args, 1);

	sscanf(line, "%d", &width);

	XtFree(line);

	if(pageset.Portrait)
	{
		pgw = PageData[pageset.PageSetId][0];
		pgh = PageData[pageset.PageSetId][1];
	}
	else
	{
		pgw = PageData[pageset.PageSetId][1];
		pgh = PageData[pageset.PageSetId][0];
	}

	if(2*height >= pgh) { ErrorPopup("Too Big Height Margin!"); return(False); }
	if(2*width >= pgw) { ErrorPopup("Too Big Width Margin!"); return(False); }
	
	viewport->origpage.PageSetId = pageset.PageSetId;
	viewport->origpage.Portrait = pageset.Portrait;

	viewport->page.PageMarginHeight = height;	
	viewport->page.PageMarginWidth = width;

	viewport->horOrder = pageset.horizonOrder;

	XtSetArg(args[0], XmNvalue, &line);
	XtGetValues(pageset.LineSpWg, args, 1);

	sscanf(line, "%d", &viewport->page.PageLineSpacing);

	XtFree(line);

	XtSetArg(args[0], XmNvalue, &line);
	XtGetValues(pageset.CharSpWg, args, 1);
	
	sscanf(line, "%d", &viewport->page.CharSpacing);
	
	XtFree(line);
	
	XtSetArg(args[0], XmNvalue, &line);
	XtGetValues(pageset.ColSpWg, args, 1);

	sscanf(line, "%d", &viewport->page.PageColSpacing);

	XtFree(line);

	ResetViewport(viewport, 1);		
	MyDrawPage(viewport, VIRDRAW);

	VerScroBarExpose(viewport);
	HorScroBarExpose(viewport);

	MyDrawPage(viewport, TRUEDRAW);
	
	viewport->changed = True;
	viewport->autosavechanged = True;

	return(True);
}


void CreatePageSet(parent)
Widget parent;
{
	Widget label, submenu, subBtn;
	int i;

	Arg args[MAX_ARGS];
	register int n;

	pageset.PopUpWg = XtCreatePopupShell("page Window", transientShellWidgetClass, parent, NULL, 0);

	n = 0;
	XtSetArg(args[n], XmNheight, 340); n++;
	XtSetArg(args[n], XmNwidth, 245); n++;
	XtSetArg(args[n], XmNhorizontalSpacing, 0); n++;
	XtSetArg(args[n], XmNverticalSpacing, 0); n++;
	XtSetArg(args[n], XmNrubberPositioning, True); n++;
	pageset.Root = XmCreateForm(pageset.PopUpWg, "CharSet", args, n);
	XtManageChild(pageset.Root);
	XtAddCallback(pageset.Root, XmNdestroyCallback, MainDestroyCB, NULL);

	submenu = (Widget) XmCreatePulldownMenu(pageset.Root, "pagesubmenu", NULL, 0);

	for(i = 0; i < NUMOFPAGESET; i++)
	{
		n = 0;
		XtSetArg(args[n], XmNlabelString, XmStringCreate(PageInfo[i], XmSTRING_DEFAULT_CHARSET)); n++;
		pageset.PageBtn[i] = XmCreatePushButtonGadget(submenu, PageInfo[i], args, n);
		XtAddCallback(pageset.PageBtn[i], XmNactivateCallback, PageSizePreCB, (caddr_t *)i);
	}
	
	XtManageChildren(pageset.PageBtn, NUMOFPAGESET);
	
	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Page Size", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNsubMenuId, submenu); n++;
	XtSetArg(args[n], XmNmenuHistory, pageset.PageBtn[LETTERPAGESETID]); n++;
	XtSetArg(args[n], XmNx, 15); n++;
	XtSetArg(args[n], XmNy, 15); n++;
	pageset.PageSize = XmCreateOptionMenu(pageset.Root, "PageS", args, n);
	XtManageChild(pageset.PageSize);

	n = 0;
	XtSetArg(args[n], XmNindicatorType, XmONE_OF_MANY); n++;
	XtSetArg(args[n], XmNvisibleWhenOff, True); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Landscape", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNx, 15); n++;
	XtSetArg(args[n], XmNy, 55); n++;
	pageset.LandBtn = XmCreateToggleButtonGadget(pageset.Root, "Land", args,n);
	XtManageChild(pageset.LandBtn);
	XtAddCallback(pageset.LandBtn, XmNvalueChangedCallback, PageLandscapeCB, NULL);

	n = 0;
	XtSetArg(args[n], XmNindicatorType, XmONE_OF_MANY); n++;
	XtSetArg(args[n], XmNvisibleWhenOff, True); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Portrait", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNx, 115); n++;
	XtSetArg(args[n], XmNy, 55); n++;
	pageset.PortBtn = XmCreateToggleButtonGadget(pageset.Root, "Port", args,n);
	XtManageChild(pageset.PortBtn);
	XtAddCallback(pageset.PortBtn, XmNvalueChangedCallback, PagePortraitCB, NULL);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Line Orientation:", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNx, 15); n++;
	XtSetArg(args[n], XmNy, 85); n++;
	label = XmCreateLabelGadget(pageset.Root, "LineLabel", args, n);
	XtManageChild(label);

	n = 0;
	XtSetArg(args[n], XmNindicatorType, XmONE_OF_MANY); n++;
	XtSetArg(args[n], XmNvisibleWhenOff, True); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Horizon", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNx, 15); n++;
	XtSetArg(args[n], XmNy, 105); n++;
	pageset.LHorBtn = XmCreateToggleButtonGadget(pageset.Root, "HorL", args,n);
	XtManageChild(pageset.LHorBtn);
	XtAddCallback(pageset.LHorBtn, XmNvalueChangedCallback, LineHorizonCB, NULL);

	n = 0;
	XtSetArg(args[n], XmNindicatorType, XmONE_OF_MANY); n++;
	XtSetArg(args[n], XmNvisibleWhenOff, True); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Vertical", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNx, 115); n++;
	XtSetArg(args[n], XmNy, 105); n++;
	pageset.LVerBtn = XmCreateToggleButtonGadget(pageset.Root, "VerL", args,n);
	XtManageChild(pageset.LVerBtn);
	XtAddCallback(pageset.LVerBtn, XmNvalueChangedCallback, LineVerticalCB, NULL);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("  Vertical Margin:", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNx, 35); n++;
	XtSetArg(args[n], XmNy, 145); n++;
	label = XmCreateLabelGadget(pageset.Root, "VerLabel", args, n);
	XtManageChild(label);
							
	n = 0;
	XtSetArg(args[n], XmNwidth, 50); n++;
	XtSetArg(args[n], XmNscrollHorizontal, False); n++;
	XtSetArg(args[n], XmNscrollVertical, False); n++;
	XtSetArg(args[n], XmNresizeHeight, False); n++;
	XtSetArg(args[n], XmNresizeWidth, False); n++;
	XtSetArg(args[n], XmNmarginHeight, 0); n++;
	XtSetArg(args[n], XmNmarginWidth, 0); n++;
	XtSetArg(args[n], XmNmaxLength, 3); n++;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(args[n], XmNleftWidget, label); n++;
	XtSetArg(args[n], XmNy, 145); n++;
	pageset.VerMarWg = XmCreateText(pageset.Root, "VerMar", args, n); 
	XtManageChild(pageset.VerMarWg);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Horizental Margin:", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNx, 35); n++;
	XtSetArg(args[n], XmNy, 175); n++;
	label = XmCreateLabelGadget(pageset.Root, "HorLabel", args, n);
	XtManageChild(label);
							
	n = 0;
	XtSetArg(args[n], XmNwidth, 50); n++;
	XtSetArg(args[n], XmNscrollHorizontal, False); n++;
	XtSetArg(args[n], XmNscrollVertical, False); n++;
	XtSetArg(args[n], XmNresizeHeight, False); n++;
	XtSetArg(args[n], XmNresizeWidth, False); n++;
	XtSetArg(args[n], XmNmarginHeight, 0); n++;
	XtSetArg(args[n], XmNmarginWidth, 0); n++;
	XtSetArg(args[n], XmNmaxLength, 3); n++;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(args[n], XmNleftWidget, label); n++;
	XtSetArg(args[n], XmNy, 175); n++;
	pageset.HorMarWg = XmCreateText(pageset.Root, "HorMar", args, n); 
	XtManageChild(pageset.HorMarWg);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("  Line Spacing(%):", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNx, 35); n++;
	XtSetArg(args[n], XmNy, 205); n++;
	label = XmCreateLabelGadget(pageset.Root, "LineS", args, n);
	XtManageChild(label);
							
	
	n = 0;
	XtSetArg(args[n], XmNwidth, 50); n++;
	XtSetArg(args[n], XmNscrollHorizontal, False); n++;
	XtSetArg(args[n], XmNscrollVertical, False); n++;
	XtSetArg(args[n], XmNresizeHeight, False); n++;
	XtSetArg(args[n], XmNresizeWidth, False); n++;
	XtSetArg(args[n], XmNmarginHeight, 0); n++;
	XtSetArg(args[n], XmNmarginWidth, 0); n++;
	XtSetArg(args[n], XmNmaxLength, 3); n++;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(args[n], XmNleftWidget, label); n++;
	XtSetArg(args[n], XmNy, 205); n++;
	pageset.LineSpWg = XmCreateText(pageset.Root, "LineSp", args, n); 
	XtManageChild(pageset.LineSpWg);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("  Char Spacing(%):", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNx, 35); n++;
	XtSetArg(args[n], XmNy, 235); n++;
	label = XmCreateLabelGadget(pageset.Root, "CharS", args, n);
	XtManageChild(label);
							
	
	n = 0;
	XtSetArg(args[n], XmNwidth, 50); n++;
	XtSetArg(args[n], XmNscrollHorizontal, False); n++;
	XtSetArg(args[n], XmNscrollVertical, False); n++;
	XtSetArg(args[n], XmNresizeHeight, False); n++;
	XtSetArg(args[n], XmNresizeWidth, False); n++;
	XtSetArg(args[n], XmNmarginHeight, 0); n++;
	XtSetArg(args[n], XmNmarginWidth, 0); n++;
	XtSetArg(args[n], XmNmaxLength, 3); n++;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(args[n], XmNleftWidget, label); n++;
	XtSetArg(args[n], XmNy, 235); n++;
	pageset.CharSpWg = XmCreateText(pageset.Root, "CharSp", args, n); 
	XtManageChild(pageset.CharSpWg);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("   Column Spacing:", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNx, 35); n++;
	XtSetArg(args[n], XmNy, 265); n++;
	label = XmCreateLabelGadget(pageset.Root, "ColS", args, n);
	XtManageChild(label);
							
	
	n = 0;
	XtSetArg(args[n], XmNwidth, 50); n++;
	XtSetArg(args[n], XmNscrollHorizontal, False); n++;
	XtSetArg(args[n], XmNscrollVertical, False); n++;
	XtSetArg(args[n], XmNresizeHeight, False); n++;
	XtSetArg(args[n], XmNresizeWidth, False); n++;
	XtSetArg(args[n], XmNmarginHeight, 0); n++;
	XtSetArg(args[n], XmNmarginWidth, 0); n++;
	XtSetArg(args[n], XmNmaxLength, 3); n++;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(args[n], XmNleftWidget, label); n++;
	XtSetArg(args[n], XmNy, 265); n++;
	pageset.ColSpWg = XmCreateText(pageset.Root, "ColSp", args, n); 
	XtManageChild(pageset.ColSpWg);

	n = 0;
	XtSetArg(args[n], XmNwidth, 100); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Set", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNx, 20); n++;
	XtSetArg(args[n], XmNy, 300); n++;
	pageset.SetBtn = XmCreatePushButtonGadget(pageset.Root, "pageBtn", args, n);
	XtManageChild(pageset.SetBtn);
	XtAddCallback(pageset.SetBtn, XmNactivateCallback, DoPageSetCB, NULL);

	n = 0;
	XtSetArg(args[n], XmNwidth, 100); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Cancel", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNy, 300); n++;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(args[n], XmNleftWidget, pageset.SetBtn); n++;
	pageset.CloseBtn = XmCreatePushButtonGadget(pageset.Root, "closeBtn", args, n);
	XtManageChild(pageset.CloseBtn);
	XtAddCallback(pageset.CloseBtn, XmNactivateCallback, ClosePageSetCB, NULL);

	XtRealizeWidget(pageset.PopUpWg);
}

void PageSetCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	Arg args[1];
	char line[10];

	active_viewport = (VIewportdata *)client_data;	
	pageset.Portrait = active_viewport->origpage.Portrait;
	pageset.PageSetId = active_viewport->origpage.PageSetId;
	pageset.horizonOrder = active_viewport->horOrder;

	if(pageset.Portrait)
	{
		XtSetArg(args[0], XmNset, True);
		XtSetValues(pageset.PortBtn, args, 1);

		XtSetArg(args[0], XmNset, False);
		XtSetValues(pageset.LandBtn, args, 1);
	}
	else
	{
		XtSetArg(args[0], XmNset, True);       
		XtSetValues(pageset.LandBtn, args, 1);

		XtSetArg(args[0], XmNset, False);
		XtSetValues(pageset.PortBtn, args, 1);
	}

	if(pageset.horizonOrder)
	{
		XtSetArg(args[0], XmNset, True);
		XtSetValues(pageset.LHorBtn, args, 1);

		XtSetArg(args[0], XmNset, False);
		XtSetValues(pageset.LVerBtn, args, 1);
	}
	else
	{
		XtSetArg(args[0], XmNset, True);       
		XtSetValues(pageset.LVerBtn, args, 1); 

		XtSetArg(args[0], XmNset, False);      
		XtSetValues(pageset.LHorBtn, args, 1); 
	}	

	XtSetArg(args[0], XmNmenuHistory, pageset.PageBtn[pageset.PageSetId]); 
	XtSetValues(pageset.PageSize, args, 1);

	sprintf(line, "%d\0", active_viewport->page.PageMarginHeight / 100);
	XtSetArg(args[0], XmNvalue, line); 
	XtSetValues(pageset.VerMarWg, args, 1);

	sprintf(line, "%d\0", active_viewport->page.PageMarginWidth / 100);
	XtSetArg(args[0], XmNvalue, line); 
	XtSetValues(pageset.HorMarWg, args, 1);

	sprintf(line, "%d\0", active_viewport->page.PageLineSpacing);
	XtSetArg(args[0], XmNvalue, line); 
	XtSetValues(pageset.LineSpWg, args, 1);

	sprintf(line, "%d\0", active_viewport->page.CharSpacing);
	XtSetArg(args[0], XmNvalue, line);
	XtSetValues(pageset.CharSpWg, args, 1);

	sprintf(line, "%d\0", active_viewport->page.PageColSpacing / 100);
	XtSetArg(args[0], XmNvalue, line);
	XtSetValues(pageset.ColSpWg, args, 1);

	PopupPosition(active_viewport->resource.PopUpShell, pageset.PopUpWg);
	XtPopup(pageset.PopUpWg, XtGrabExclusive); 
}

void DoPageSetCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	if(!PageSet(active_viewport)) return;
	XtPopdown(pageset.PopUpWg);
}

void ClosePageSetCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	XtPopdown(pageset.PopUpWg);
}

void PageSizePreCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	pageset.PageSetId = (int) client_data;
}

void PageLandscapeCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	Arg args[1];
	
	pageset.Portrait = False;

	XtSetArg(args[0], XmNset, True);
	XtSetValues(pageset.LandBtn, args, 1);

	XtSetArg(args[0], XmNset, False);
	XtSetValues(pageset.PortBtn, args, 1);
}

void PagePortraitCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	Arg args[1];

	pageset.Portrait = True;

	XtSetArg(args[0], XmNset, True);
	XtSetValues(pageset.PortBtn, args, 1);

	XtSetArg(args[0], XmNset, False);
	XtSetValues(pageset.LandBtn, args, 1);
}

void LineHorizonCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
        Arg args[1];

	pageset.horizonOrder = True;

	XtSetArg(args[0], XmNset, True);
	XtSetValues(pageset.LHorBtn, args, 1);

	XtSetArg(args[0], XmNset, False);
	XtSetValues(pageset.LVerBtn, args, 1); 
}

void LineVerticalCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	Arg args[1];

	pageset.horizonOrder = False;

	XtSetArg(args[0], XmNset, True);       
	XtSetValues(pageset.LVerBtn, args, 1); 

	XtSetArg(args[0], XmNset, False);      
	XtSetValues(pageset.LHorBtn, args, 1); 
}	

void ReadSetPageInfo(viewport, cp)
VIewportdata *viewport;
char *cp;
{
	sscanf(cp, "%d,%d,%d,%d,%d,%d,%d,%d", &viewport->page.PageMarginWidth, &viewport->page.PageMarginHeight, &viewport->page.PageColSpacing,
	   &viewport->page.PageLineSpacing, &viewport->page.CharSpacing, &viewport->origpage.PageSetId, &viewport->origpage.Portrait, &viewport->horOrder);
}

void WritePageInfoToFile(viewport, note)
VIewportdata *viewport;
char *note;
{
	if(viewport->origpage.Portrait != GlobalDefaults.page_orien ||
	viewport->origpage.PageSetId != GlobalDefaults.page_type ||
	viewport->page.PageLineSpacing != GlobalDefaults.line_spacing ||
	viewport->page.CharSpacing != GlobalDefaults.char_spacing ||
	viewport->page.PageMarginWidth/100 != GlobalDefaults.margin_width ||
	viewport->page.PageMarginHeight/100 != GlobalDefaults.margin_height ||
	viewport->page.PageColSpacing/100 != GlobalDefaults.column_spacing ||
	viewport->horOrder != GlobalDefaults.horOrder)
	{
	
		fprintf(viewport->fp, "%s%d,%d,%d,%d,%d,%d,%d,%d\n", 
		note, viewport->page.PageMarginWidth/100, viewport->page.PageMarginHeight/100, viewport->page.PageColSpacing/100,
        	viewport->page.PageLineSpacing, viewport->page.CharSpacing, viewport->origpage.PageSetId, viewport->origpage.Portrait, viewport->horOrder);
	}
}

void ResetPageInfo(viewport)
VIewportdata *viewport;
{
	viewport->page.PageMarginWidth = GlobalDefaults.margin_width;
	viewport->page.PageMarginHeight = GlobalDefaults.margin_height;
	viewport->page.PageColSpacing = GlobalDefaults.column_spacing;
	viewport->page.PageLineSpacing = GlobalDefaults.line_spacing;
	viewport->page.CharSpacing = GlobalDefaults.char_spacing;
	viewport->origpage.PageSetId = GlobalDefaults.page_type;
	viewport->origpage.Portrait = GlobalDefaults.page_orien;
	viewport->horOrder = GlobalDefaults.horOrder;
}

