/*
 * Copyright 1995, 1996, 1997 by Jun Yei.       All Rights Reserved
 *
 * Permission to retain, use, modify, copy, and distribute ChinesePower
 * in source or binary and its documentation (hereafter, the Software)
 * for noncommercial purpose is hereby granted to you without a fee,
 * provided that this entire copyright and permission notice appear in
 * all such copies, that no charge be associated with such copies,
 * that distribution of derivative works (including value-added
 * distributions such as with additional input dictionaries or fonts)
 * include clarification that such added or derived parts are not from
 * the original Software, and that the names of the author(s) not be
 * used to endorse or promote such works. The author(s) of the software
 * makes no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * THE AUTHOR(S) OF THE SOFTWARE DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 * IN NO EVENT SHALL THE AUTHOR(S) OF THE SOFTWARE BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Jun Yei, Ottawa, ON, Canada
 */

#include "chpower.h"

void CancelPicImportCB();
void ImportPicImportCB();
void RefreshPicImportCB();

PIcImportSet picImport;

void PicImportReleaseR()
{
	int i;

	for(i = 0; i < picImport.picnum; i++)
	{
		if(picImport.picnamelist[i]) XmStringFree(picImport.picnamelist[i]);
		else break;
	}

	if(picImport.picnamelist) free(picImport.picnamelist);

	picImport.picnum = 0;
	picImport.picnamelist = NULL;
}

void PicImportDestroy()
{
	PicImportReleaseR();
}

int TotalNumOfPic()
{
	char line[50], *cp;
	int i;
	FILE *fp;
	char *cmd = "ls $CHPPICPATH";
	
	fp = popen(cmd, "r");

	if(!fp)
	{
		return(1);
	}
	
	i = 0;

	while((cp = fgets(line, 49, fp)))
	{
		i++;
	}

	pclose(fp);

	picImport.picnum = i;

	picImport.picnamelist = (XmString *)malloc((i+1)*sizeof(XmString *));

	fp = popen(cmd, "r");
	
	i = 0;

	while((cp = fgets(line, 49, fp)))
	{
		for(; *cp != '\n' && *cp != '\0'; cp++);
		*cp = '\0';
		picImport.picnamelist[i] = XmStringCreateLtoR(line, XmSTRING_DEFAULT_CHARSET);
		i++;
	}

	pclose(fp);
	
	return(0);
}

void CreatePicImport(parent)
Widget parent;
{
	Arg args[MAX_ARGS];
	register int n;
	Widget label, Root, Btn;

	picImport.PopUpWg = XtCreatePopupShell("Image Import", transientShellWidgetClass, parent, NULL, 0);

	picImport.picnum = 0;
        picImport.picnamelist = NULL;

	n = 0;
	XtSetArg(args[n], XmNheight, 250); n++;
	XtSetArg(args[n], XmNwidth, 230); n++;
	XtSetArg(args[n], XmNhorizontalSpacing, 0); n++;
	XtSetArg(args[n], XmNverticalSpacing, 0); n++;
	XtSetArg(args[n], XmNrubberPositioning, True); n++;
	Root = XmCreateForm(picImport.PopUpWg, "picImport", args, n);
	XtManageChild(Root);
	XtAddCallback(Root, XmNdestroyCallback, MainDestroyCB, NULL);

	n = 0;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Image File", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNx, 15); n++;
	XtSetArg(args[n], XmNy, 10); n++;
	label = XmCreateLabelGadget(Root, "ImgType", args, n);
	XtManageChild(label);

	n = 0;

/*        XtSetArg(args[n], XmNitems, NULL); n++;
        XtSetArg(args[n], XmNitemCount, 0); n++;
*/	XtSetArg(args[n], XmNx, 15); n++;
	XtSetArg(args[n], XmNy, 30); n++;
	XtSetArg(args[n], XmNheight, 150); n++;
	XtSetArg(args[n], XmNwidth, 200); n++;
	picImport.ImgLstWg = XmCreateScrolledList(Root, "ImgList", args, n);
	XtManageChild(picImport.ImgLstWg);

	/*Add seperator*/

	n = 0;
	XtSetArg(args[n], XmNwidth, 230); n++;
	XtSetArg(args[n], XmNheight, 5); n++; 
	XtSetArg(args[n], XmNx, 0); n++;
	XtSetArg(args[n], XmNy, 190); n++; 
	XtSetArg(args[n], XmNorientation, XmHORIZONTAL); n++;
	XtSetArg(args[n], XmNseparatorType, XmSHADOW_ETCHED_IN); n++;
	Btn = XmCreateSeparator(Root, "imIsep", args, n);
	XtManageChild(Btn);
 
	/*Add double click*/

	n = 0;
	XtSetArg(args[n], XmNwidth, 60); n++;
	XtSetArg(args[n], XmNheight, 25); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Import", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNdefaultButtonShadowThickness, 1); n++;
	XtSetArg(args[n], XmNshowAsDefault, True); n++;
	XtSetArg(args[n], XmNx, 10); n++;
	XtSetArg(args[n], XmNy, 205); n++;
	picImport.ImBtn = XmCreatePushButtonGadget(Root, "importBtn", args, n);
	XtManageChild(picImport.ImBtn);

	n = 0;
	XtSetArg(args[n], XmNwidth, 60); n++;
	XtSetArg(args[n], XmNheight, 25); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Cancel", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNdefaultButtonShadowThickness, 0); n++;
	XtSetArg(args[n], XmNx, 90); n++;
	XtSetArg(args[n], XmNy, 210); n++;
	Btn = XmCreatePushButtonGadget(Root, "cancelBtn", args, n);
	XtManageChild(Btn);
	XtAddCallback(Btn, XmNactivateCallback, CancelPicImportCB, NULL);

	n = 0;
	XtSetArg(args[n], XmNwidth, 60); n++;
	XtSetArg(args[n], XmNheight, 25); n++;
	XtSetArg(args[n], XmNlabelString, XmStringCreate("Refresh", XmSTRING_DEFAULT_CHARSET)); n++;
	XtSetArg(args[n], XmNdefaultButtonShadowThickness, 0); n++;
	XtSetArg(args[n], XmNx, 160); n++;
	XtSetArg(args[n], XmNy, 210); n++;
	Btn = XmCreatePushButtonGadget(Root, "refreBtn", args, n);
	XtManageChild(Btn);
	XtAddCallback(Btn, XmNactivateCallback, RefreshPicImportCB, NULL);

	XtRealizeWidget(picImport.PopUpWg);
	return;
}

int GetPicImport(viewport)
VIewportdata *viewport;
{
	unsigned short result[55];
	Arg args[MAX_ARGS];
	register int n;
	char *imgname = NULL;
	XmString *imgXname = NULL;
	int NumOfInput;

	n = 0;
	XtSetArg(args[n], XmNselectedItems, &imgXname); n++;
	XtGetValues(picImport.ImgLstWg, args, n);

	XmStringGetLtoR(*imgXname, XmSTRING_DEFAULT_CHARSET, &imgname);

	result[0] = 0x00;
	
	if(imgname)
	{
		if(UpdatePicture(viewport, imgname))
		{
			return(False);
		}
		
		DBstrcat(result, "@[G");
		DBstrcat(result, imgname);
		DBstrcat(result, "]");
		
		if(imgname) XtFree(imgname);
	}
	else
	{
		return(False);
	}
	
	NumOfInput = DBlen(result);

	Input(viewport, VIRINPUT, result, NumOfInput);
	return(True);
}

void PicImportCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	active_viewport = (VIewportdata *)client_data;

	RefreshPicImportCB(NULL, NULL, NULL);


	XtRemoveAllCallbacks(picImport.ImgLstWg, XmNdefaultActionCallback);
	XtRemoveAllCallbacks(picImport.ImBtn, XmNactivateCallback);

	XtAddCallback(picImport.ImgLstWg, XmNdefaultActionCallback, ImportPicImportCB, client_data);
	XtAddCallback(picImport.ImBtn, XmNactivateCallback, ImportPicImportCB, client_data);
	
	PopupPosition(active_viewport->resource.PopUpShell, picImport.PopUpWg);	
	XtPopup(picImport.PopUpWg, XtGrabExclusive);
}

void ImportPicImportCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	VIewportdata *viewport = (VIewportdata *)client_data;

	if(GetPicImport(viewport))
	{
		MyDrawPage(viewport, INVIRDRAW);	
		CheckCurrentPage(viewport);
		XtPopdown(picImport.PopUpWg);
		MyDrawPage(viewport, CLEARDRAW);	
	}
}

void CancelPicImportCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
        XtPopdown(picImport.PopUpWg);
}

void RefreshPicImportCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	Arg args[2];

	PicImportReleaseR();
	
	if(TotalNumOfPic())
	{
		XtPopdown(picImport.PopUpWg);
		ErrorPopup("No Picture Path or File available!");
		return;
	}
	
	XtSetArg(args[0], XmNitems, picImport.picnamelist);
	XtSetArg(args[1], XmNitemCount, picImport.picnum);
	XtSetValues(picImport.ImgLstWg, args, 2);
}


