/*
 * Copyright 1995, 1996, 1997 by Jun Yei.       All Rights Reserved
 *
 * Permission to retain, use, modify, copy, and distribute ChinesePower
 * in source or binary and its documentation (hereafter, the Software)
 * for noncommercial purpose is hereby granted to you without a fee,
 * provided that this entire copyright and permission notice appear in
 * all such copies, that no charge be associated with such copies,
 * that distribution of derivative works (including value-added
 * distributions such as with additional input dictionaries or fonts)
 * include clarification that such added or derived parts are not from
 * the original Software, and that the names of the author(s) not be
 * used to endorse or promote such works. The author(s) of the software
 * makes no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * THE AUTHOR(S) OF THE SOFTWARE DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 * IN NO EVENT SHALL THE AUTHOR(S) OF THE SOFTWARE BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Jun Yei, Ottawa, ON, Canada
 */

#include 	"chpower.h"

Cursor myxterm = 0;
Cursor mywatch = 0;

void CreateCursor(parent)
Widget parent;
{
	myxterm = XCreateFontCursor(XtDisplay(parent), XC_xterm);
	mywatch = XCreateFontCursor(XtDisplay(parent), XC_watch);
}

void BusyWindows()
{
	MYviewportlist *viewportlist;
	XEvent myevent;

	if(mywatch)
	{
		XDefineCursor(XtDisplay(MyHzInMod.Root), XtWindow(MyHzInMod.Root), mywatch);
		XDefineCursor(XtDisplay(printset.Root), XtWindow(printset.Root), mywatch);
		XDefineCursor(XtDisplay(pageset.Root), XtWindow(pageset.Root), mywatch);       
		XDefineCursor(XtDisplay(gifset.Root), XtWindow(gifset.Root), mywatch);       
		XDefineCursor(XtDisplay(charset.Root), XtWindow(charset.Root), mywatch);       
		XDefineCursor(XtDisplay(pagedel.Root), XtWindow(pagedel.Root), mywatch);
		XDefineCursor(XtDisplay(pagebtn.Root), XtWindow(pagebtn.Root), mywatch);
		XDefineCursor(XtDisplay(findset.Root), XtWindow(findset.Root), mywatch);
		XDefineCursor(XtDisplay(titlerset.Root), XtWindow(titlerset.Root), mywatch);        
		XDefineCursor(XtDisplay(colortab.Root), XtWindow(colortab.Root), mywatch);	
		XDefineCursor(XtDisplay(toplevel), XtWindow(toplevel), mywatch);       
		
		viewportlist = ViewPortRoot;

		while(viewportlist != NULL && viewportlist->myviewport.resource.PopUpShell != NULL && viewportlist->myviewport.resource.dpy != NULL)
		{
			XDefineCursor(XtDisplay(viewportlist->myviewport.resource.PopUpShell), XtWindow(viewportlist->myviewport.resource.PopUpShell), mywatch);       
			XDefineCursor(viewportlist->myviewport.resource.dpy, viewportlist->myviewport.resource.wind, mywatch);
			viewportlist = viewportlist->next;
		}

		XCheckTypedEvent(XtDisplay(toplevel), EnterNotify, &myevent);
	}
}

void UnBusyWindows()
{
	MYviewportlist *viewportlist;
	
	if(mywatch)
	{
		XUndefineCursor(XtDisplay(MyHzInMod.Root), XtWindow(MyHzInMod.Root));
		XUndefineCursor(XtDisplay(printset.Root), XtWindow(printset.Root));
		XUndefineCursor(XtDisplay(pageset.Root), XtWindow(pageset.Root));       
		XUndefineCursor(XtDisplay(pagedel.Root), XtWindow(pagedel.Root));
		XUndefineCursor(XtDisplay(pagebtn.Root), XtWindow(pagebtn.Root));
		XUndefineCursor(XtDisplay(gifset.Root), XtWindow(gifset.Root));       
		XUndefineCursor(XtDisplay(charset.Root), XtWindow(charset.Root));       
		XUndefineCursor(XtDisplay(findset.Root), XtWindow(findset.Root));        
		XUndefineCursor(XtDisplay(titlerset.Root), XtWindow(titlerset.Root));    
		XUndefineCursor(XtDisplay(colortab.Root), XtWindow(colortab.Root));
		XUndefineCursor(XtDisplay(toplevel), XtWindow(toplevel));       

		viewportlist = ViewPortRoot;

		while(viewportlist != NULL && viewportlist->myviewport.resource.
PopUpShell != NULL && viewportlist->myviewport.resource.dpy != NULL)
		{
			XUndefineCursor(XtDisplay(viewportlist->myviewport.resource.PopUpShell), XtWindow(viewportlist->myviewport.resource.PopUpShell));
			XDefineCursor(viewportlist->myviewport.resource.dpy, viewportlist->myviewport.resource.wind, myxterm);
			viewportlist = viewportlist->next;
		}
	}
}

void BusySomeWindows()
{
	MYviewportlist *viewportlist;
	
	if(mywatch)
	{
		XDefineCursor(XtDisplay(MyHzInMod.Root), XtWindow(MyHzInMod.Root), mywatch);
		XDefineCursor(XtDisplay(charset.Root), XtWindow(charset.Root), mywatch);       
		XDefineCursor(XtDisplay(toplevel), XtWindow(toplevel), mywatch);       

		viewportlist = ViewPortRoot;

		while(viewportlist != NULL && viewportlist->myviewport.resource.
PopUpShell != NULL && viewportlist->myviewport.resource.dpy != NULL)
		{
			if(viewportlist->next == NULL) break;
			XDefineCursor(XtDisplay(viewportlist->myviewport.resource.PopUpShell), XtWindow(viewportlist->myviewport.resource.PopUpShell), mywatch);       
			XDefineCursor(viewportlist->myviewport.resource.dpy, viewportlist->myviewport.resource.wind, mywatch);
			viewportlist = viewportlist->next;
		}
	}
}

void UnBusySomeWindows()
{
	MYviewportlist *viewportlist;
	
	if(mywatch)
	{
		XUndefineCursor(XtDisplay(MyHzInMod.Root), XtWindow(MyHzInMod.Root));
		XUndefineCursor(XtDisplay(charset.Root), XtWindow(charset.Root));       
		XUndefineCursor(XtDisplay(toplevel), XtWindow(toplevel));       

		viewportlist = ViewPortRoot;

		while(viewportlist != NULL && viewportlist->myviewport.resource.
PopUpShell != NULL && viewportlist->myviewport.resource.dpy != NULL)
		{
			if(viewportlist->next == NULL) break;
			XUndefineCursor(XtDisplay(viewportlist->myviewport.resource.PopUpShell), XtWindow(viewportlist->myviewport.resource.PopUpShell));
			XDefineCursor(viewportlist->myviewport.resource.dpy, viewportlist->myviewport.resource.wind, myxterm);
			viewportlist = viewportlist->next;
		}
	}
}

