/*
 * Copyright 1995 by Jun Yei.			All Rights Reserved
 *
 * Permission to retain, use, modify, copy, and distribute ChinesePower 1.0
 * in source or binary and its documentation (hereafter, the Software)
 * for noncommercial purpose is hereby granted to you without a fee,
 * provided that this entire copyright and permission notice appear in
 * all such copies, that no charge be associated with such copies,
 * that distribution of derivative works (including value-added
 * distributions such as with additional input dictionaries or fonts)
 * include clarification that such added or derived parts are not from
 * the original Software, and that the names of the author(s) not be
 * used to endorse or promote such works. The author(s) of the software
 * makes no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * THE AUTHOR(S) OF THE SOFTWARE DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 * IN NO EVENT SHALL THE AUTHOR(S) OF THE SOFTWARE BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Jun Yei, Ottawa, ON, Canada
 */

/* $Id: ptyx.h,v 3.2 1995/01/17 10:56:02 ygz Exp $ */
/***********************************************************************
* Copyright 1994 by Yongguang Zhang.
* Copyright 1990, 1991 by Yongguang Zhang and Pong Man-Chi.
*
* All rights reserved.  Under the same copyright and permission term as
* the original.  Absolutely no warranties of any kinds.
************************************************************************/

/*
 *	$XConsortium: ptyx.h,v 1.63 94/08/02 19:24:44 converse Exp $
 */

/*
 * Copyright 1987 by Digital Equipment Corporation, Maynard, Massachusetts.
 *
 *                         All Rights Reserved
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of Digital Equipment
 * Corporation not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission.
 *
 *
 * DIGITAL DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
 * ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
 * DIGITAL BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
 * ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 */

/* ptyx.h */
/* @(#)ptyx.h	X10/6.6	11/10/86 */

#include        "fonts.h"
#include        "hbf.h"
#include <X11/IntrinsicP.h>

#ifdef X11R5  
#include "Xfuncs.h"
#else
#include <X11/Xfuncs.h>
#endif

#define MAX_COLS	200
#define MAX_ROWS	128

/*
** System V definitions
*/

#ifdef SYSV
#ifdef X_NOT_POSIX
#ifndef CRAY
#define	dup2(fd1,fd2)	((fd1 == fd2) ? fd1 : \
				(close(fd2), fcntl(fd1, F_DUPFD, fd2)))
#endif
#endif
#endif /* SYSV */

/* Until the translation manager comes along, I have to do my own translation of
 * mouse events into the proper routines. */

typedef enum {NORMAL, LEFTEXTENSION, RIGHTEXTENSION} EventMode;

/*
 * The origin of a screen is 0, 0.  Therefore, the number of rows
 * on a screen is screen->max_row + 1, and similarly for columns.
 */

typedef unsigned char Char;		/* to support 8 bit chars */

#ifdef HANZI
#define	FGCS_GL		0	/* font in GL plane */
#define	FGCS_GR		1	/* font in GR plane */
#define	FGCS_MIX	2	/* mixture of GL and GR planes */
#endif /* HANZI */

typedef struct {
/* These parameters apply to both windows */
	Widget		inputwidg;
	Display		*display;	/* X display for screen		*/
	GC		normalGC;	/* normal painting		*/
	GC		reverseGC;
	Window	window;		/* X window id			*/
	int	width;		/* width of columns		*/
	int	height;		/* height of rows		*/
	HZdata 	hzdata;
/*	int             max_col;
        int             scrollbar; 
*/
	int		lspacing;	/* space in pixel between lines	*/
	/* Hanzi input area stuff					*/
	struct {
		int	rows;		/* number of rows in input area	*/
		int	cols;
		int	x, y;		/* upperleft of the input area	*/
		int	width, height;	/* width, height of input area  */
		int	rule_x, rule_y;	/* left end of the rule line	*/
		int	rule_length;	/* length of the rule line	*/
	} hzIwin;
	int		hzIwin_height;	/* height of Hanzi input area	*/

	Widget		configPopup;	/* popup panel for HANZI input */
} TScreen;


typedef struct _Misc {
    char *f_hn;
    char *f_hb;
    char *hz_mode;
    char *hz_encoding;		/* Hanzi encoding identifier */
    char *it_dirs;		/* directories for input tables	*/
    char *assoc_files;		/* Hanzi association-list files */
} Misc;

typedef struct _XtermWidgetRec {
    TScreen	screen;		/* terminal screen		*/
    Misc	misc;		/* miscellaneous parameters	*/
} XtermWidgetRec, *XtermWidget;

#define BUF_SIZE 4096
#define DEFAULT_ASSO_FILE "simple.lx"

/*
 * terminal flags
 * There are actually two namespaces mixed together here.
 * One is the set of flags that can go in screen->buf attributes
 * and which must fit in a char.
 * The other is the global setting stored in
 * term->flags and screen->save_modes.  This need only fit in an unsigned.
 */

#ifdef HANZI
#define	ATTRIBUTES	0x87	/* mask: user-visible attributes */
#else  /* HANZI */
#define	ATTRIBUTES	0x07	/* mask: user-visible attributes */
#endif /* HANZI */
/* global flags and character flags (visible character attributes) */
#define INVERSE		0x01	/* invert the characters to be output */
#define UNDERLINE	0x02	/* true if underlining */
#define BOLD		0x04
/* character flags (internal attributes) */
#define LINEWRAPPED	0x08	/* used on the first character in a
				 * line to indicate that it wraps onto
				 * the next line so we can tell the
				 * difference between lines that have
				 * wrapped around and lines that have
				 * ended naturally with a CR at column
				 * max_col.
				 */
#define CHARDRAWN	0x10    /* a character has been drawn here on the
				   screen.  Used to distinguish blanks from
				   empty parts of the screen when selecting */
#ifdef HANZI
#define FIRSTBYTE	0x40	/* is the first byte in a Hanzi (internal) */
#define CHARHANZI	0x80	/* is part of a Hanzi   (global & visible) */
#endif /* HANZI */
/* global flags */
#define WRAPAROUND	0x400	/* true if auto wraparound mode */
#define	REVERSEWRAP	0x800	/* true if reverse wraparound mode */
#define REVERSE_VIDEO	0x1000	/* true if screen white on black */
#define LINEFEED	0x2000	/* true if in auto linefeed mode */
#define ORIGIN		0x4000	/* true if in origin mode */
#define INSERT		0x8000	/* true if in insert mode */
#define SMOOTHSCROLL	0x10000	/* true if in smooth scroll mode */
#define IN132COLUMNS	0x20000	/* true if in 132 column mode */


#define TextWindow(screen)      (screen->window)
#define Width(screen)		(screen->width)
#define Height(screen)		(screen->height)
#define FontWidth(screen)	(screen->hzdata.hz_swidth/2)
#define FontHeight(screen)	(screen->hzdata.hz_sheight+2)

/*#DEFINe CursorX(screen,col) ((col) * FontWidth(screen) + screen->border \
			+ screen->scrollbar)
#define CursorY(screen,row) ((((row) - screen->topline) * FontHeight(screen)) \
			+ screen->border)

#define	TWINDOWEVENTS	(KeyPressMask | ExposureMask | ButtonPressMask |\
			 ButtonReleaseMask | StructureNotifyMask |\
			 EnterWindowMask | LeaveWindowMask | FocusChangeMask)

#define	WINDOWEVENTS	(TWINDOWEVENTS | PointerMotionMask)
*/
#ifdef HANZI
#define Is1stByteHZ(f)	(((f) & CHARHANZI) &&  ((f) & FIRSTBYTE))
#define Is2ndByteHZ(f)	(((f) & CHARHANZI) && !((f) & FIRSTBYTE))
#define ROWSINPUTAREA	2	/* number of rows in hanzi input areas */

#define FGCSinScreen(s,f)	\
		(((f) & BOLD) ? (s)->hz_boldFGCS : (s)->hz_normalFGCS)
#endif /* HANZI */

