/*
 * Copyright 1995, 1996, 1997 by Jun Yei.       All Rights Reserved
 *
 * Permission to retain, use, modify, copy, and distribute ChinesePower
 * in source or binary and its documentation (hereafter, the Software)
 * for noncommercial purpose is hereby granted to you without a fee,
 * provided that this entire copyright and permission notice appear in
 * all such copies, that no charge be associated with such copies,
 * that distribution of derivative works (including value-added
 * distributions such as with additional input dictionaries or fonts)
 * include clarification that such added or derived parts are not from
 * the original Software, and that the names of the author(s) not be
 * used to endorse or promote such works. The author(s) of the software
 * makes no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * THE AUTHOR(S) OF THE SOFTWARE DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 * IN NO EVENT SHALL THE AUTHOR(S) OF THE SOFTWARE BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Jun Yei, Ottawa, ON, Canada
 */

#include "ttf.h"
#include "fonts.h"
#include "ttffont.h"

#define STACKSIZE 256
#define INCSTACKNUM() { stackcount++; count = 0; fprintf(ps,"} J\n/%sx%x-%x {", ID, Num, stackcount);}

/*
   render a solid font.
*/

int TTF_GetBitmap(bitmap, map, cc, byte_order)
HZdata *bitmap;
unsigned char *map;
unsigned short cc;
int byte_order;
{
    int index,i;
    OUTLINE *ol;
    int x,y;
    LONG *xcoor,*ycoor;
    USHORT *endp,*flags;
    USHORT startp,EM,totpoint;
    TTFFont *f = (TTFFont *)bitmap->hbf;

    /* 
       get the start address of outline data.The Font_select_CMAP()
       should be called first to setup character-to-index table
       */
    index = Font_getFontOffset(f,cc);
    if (index == -1)
      {
	  memset(map,0,bitmap->hz_sheight * ((bitmap->hz_swidth + 7)/8));
	  return 1;
      }
    /* load outline data to f->glyf */
    Font_load_GLYF(f,f->fp,index,0);
    /* 
       if data has error or the index has no data or index point to
       a composite font.
       */
    if (f->glyf->valid==0)
      {
	/* fill bitmap with blink */
	memset(map,0,bitmap->hz_sheight * ((bitmap->hz_swidth + 7)/8));
	return 1;
      }
    ol = f->glyf;
    EM = f->head->unitsPerEm / 4;
    totpoint= ol->endPtsOfContours[ol->numberOfContours-1]+1;
    /* rescale the outline data to the size describe by (dpoint,res) */
    for(i=0;i<totpoint;i++)
      {
	  ol->xCoordinates[i] = (ol->xCoordinates[i] - f->head->xMin)* bitmap->hz_swidth/EM - 2;
	  ol->yCoordinates[i] = (ol->yCoordinates[i] - f->head->yMin)* bitmap->hz_sheight/EM - 2;
     }
     
/*       call a block of stolen codes to convert outline data
       into bitmap data.This program need an extra extra rewrite
       */
    convert(bitmap, map, byte_order);

    return 1;
}

#define xcoor ol->xCoordinates
#define ycoor ol->yCoordinates		

static FILE *ps;
static char *ID;
static int Num;
static int count;
static int stackcount;

int PS_TTF_GetChar(f, cc, psfile, FontID, FontNum)
TTFFont *f;
unsigned short cc;
FILE *psfile;
char *FontID;
int FontNum;
{
    int i,j, k;
    int index;
    OUTLINE *ol;
    int x,y;
    USHORT *endp,*flags;
    USHORT EM, startp,totpoint;
    int fst,start_offpt,flag,end_offpt;
    int stacknumax; 
     
    /* 
       get the start address of outline data.The Font_select_CMAP()
       should be called first to setup character-to-index table
       */
    index = Font_getFontOffset(f,cc);
    
    /* load outline data to f->glyf */
    if(index == -1)
    {
	return(-1);
    }
    
    Font_load_GLYF(f,f->fp,index,0);
    /* 
       if data has error or the index has no data or index point to
       a composite font.
       */
    if (f->glyf->valid==0)
      {
	  /* fill bitmap with blink */
	  return(-1);
      }
    ol = f->glyf;
    totpoint= ol->endPtsOfContours[ol->numberOfContours-1]+1;
    ps = psfile;
    ID = FontID;
    Num = FontNum;
    EM = f->head->unitsPerEm;

    for(i=0;i<totpoint;i++)
    {
	ol->xCoordinates[i] = (long)(ol->xCoordinates[i] - f->head->xMin) * 1024 / EM;
	ol->yCoordinates[i] = (long)(ol->yCoordinates[i] - f->head->yMin) * 1024 / EM;
    }

    count = 0;
    stacknumax = 1;

    for (i = 0, j = 0; i < ol->numberOfContours; i++) 
      { 
	  if(count + 3 > STACKSIZE) { count = 0; stacknumax++; }
	  count += 3;
	  start_offpt = 0; /*start at least 1*/ 
	  
	  for (j++; j <= ol->endPtsOfContours[i]; j++) 
	  { 
	      if (!(ol->flags[j]&1)) 
		{ /*Off curve*/ 
		    if (!start_offpt) { start_offpt = end_offpt = j; } 
		    else end_offpt++; 
		} 
	      else { /*On Curve*/ 
		  if (start_offpt) 
		    { 
			for(k = 0; k < (end_offpt - start_offpt + 1); k++) 
			{
				if(count + 7 > STACKSIZE) { count = 0; stacknumax++; }
				count += 7;
			}
			start_offpt = 0; 
		    } 
		  else 
		    {
			if(count + 3 > STACKSIZE) { count = 0; stacknumax++; }
			count += 3;
		    }
	      } 
	  } 
	  
	  if (start_offpt) 
	    {
		 for(k = 0; k < (end_offpt - start_offpt + 1); k++) 
		 {
			if(count + 7 > STACKSIZE) { count = 0; stacknumax++; }
			count += 7;
		}
	    }
	  else 
	    {
		if(count + 3 > STACKSIZE) { count = 0; stacknumax++; }
		count += 3;
	    }
      } 
   
    for (i = 0; i < stacknumax; i++) fprintf(psfile, "%sx%x-%x ", FontID, FontNum, i);

    fprintf(psfile, "Q G} J\n");
    
    fprintf(psfile, "/%sx%xdict %d dict def %sx%xdict begin\n", FontID, FontNum, stacknumax, FontID, FontNum);

    stackcount = 0;
    count = 0;

    fprintf(psfile,"/%sx%x-%x {", FontID, FontNum, stackcount);

    for (i = 0, j = 0; i < ol->numberOfContours; i++) 
      { 
	  fst = j; 
	  if(count + 3 > STACKSIZE) INCSTACKNUM(); 
	  fprintf(psfile,"%d %d M ", xcoor[j], ycoor[j]);
	  count += 3;
	  start_offpt = 0; /*start at least 1*/ 
	  
	  for (j++; j <= ol->endPtsOfContours[i]; j++) 
	  { 
	      if (!(ol->flags[j]&1)) 
		{ /*Off curve*/ 
		    if (!start_offpt) { start_offpt = end_offpt = j; } 
		    else end_offpt++; 
		} 
	      else { /*On Curve*/ 
		  if (start_offpt) 
		    { 
			PSCurveto(ol,psfile,xcoor[j],ycoor[j],start_offpt,end_offpt);
			start_offpt = 0; 
		    } 
		  else 
		    {
			if(count + 3 > STACKSIZE) INCSTACKNUM(); 
			fprintf(psfile,"%d %d L ",xcoor[j], ycoor[j]);
		   	count += 3; 
		    }
	      } 
	  } 
	  
	  if (start_offpt) 
	    {
		PSCurveto(ol,psfile,xcoor[fst],ycoor[fst], start_offpt,end_offpt);
	    }
	  else 
	    {
		if(count + 3 > STACKSIZE) INCSTACKNUM();
		fprintf(psfile,"%d %d L ", xcoor[fst], ycoor[fst]);
	   	count += 3;
	    }
     
      } 
  
      fprintf(psfile,"} J\n");

      fprintf(psfile, "end %%%sx%xdict\n", FontID, FontNum);
      return(1);
}

int PSCurveto(ol, psfile, x, y, s, t) 
OUTLINE *ol;
FILE *psfile;
int x;
int y;
int s;
int t;
{ 
     int N, i; 
     double sx[3], sy[3], cx[4], cy[4]; 
 
     N = t-s+2; 
     for(i=0; i<N-1; i++) { 
	 sx[0] = i==0?xcoor[s-1]:(xcoor[i+s]+xcoor[i+s-1])/2; 
	 sy[0] = i==0?ycoor[s-1]:(ycoor[i+s]+ycoor[i+s-1])/2; 
	 sx[1] = xcoor[s+i]; 
	 sy[1] = ycoor[s+i]; 
	 sx[2] = i==N-2?x:(xcoor[s+i]+xcoor[s+i+1])/2; 
	 sy[2] = i==N-2?y:(ycoor[s+i]+ycoor[s+i+1])/2; 
	 cx[3] = sx[2]; 
	 cy[3] = sy[2]; 
	 cx[1] = (2*sx[1]+sx[0])/3; 
	 cy[1] = (2*sy[1]+sy[0])/3; 
	 cx[2] = (sx[2]+2*sx[1])/3; 
	 cy[2] = (sy[2]+2*sy[1])/3; 
	 if(count + 7 > STACKSIZE) INCSTACKNUM(); 
	 fprintf(psfile,"%.1f %.1f %.1f %.1f %.1f %.1f N ", 
		cx[1],cy[1],cx[2],cy[2],cx[3],cy[3]); 
     
    	 count += 7; 
     } 
 } 

