/*
 * Copyright 1995, 1996, 1997 by Jun Yei.       All Rights Reserved
 *
 * Permission to retain, use, modify, copy, and distribute ChinesePower
 * in source or binary and its documentation (hereafter, the Software)
 * for noncommercial purpose is hereby granted to you without a fee,
 * provided that this entire copyright and permission notice appear in
 * all such copies, that no charge be associated with such copies,
 * that distribution of derivative works (including value-added
 * distributions such as with additional input dictionaries or fonts)
 * include clarification that such added or derived parts are not from
 * the original Software, and that the names of the author(s) not be
 * used to endorse or promote such works. The author(s) of the software
 * makes no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * THE AUTHOR(S) OF THE SOFTWARE DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 * IN NO EVENT SHALL THE AUTHOR(S) OF THE SOFTWARE BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Jun Yei, Ottawa, ON, Canada
 */

#include "chpower.h"

int PageMoveUp();
int PageMoveDown();
int PageMoveRight();
int PageMoveLeft();

void VerScroBarExpose(viewport)
VIewportdata *viewport;
{
	register int n;
	Arg args[10];
	int max, min;
	int slidsize, slidevalue, inc;
	long i, j;
	Dimension a;

	n = 0;
	XtSetArg(args[n], XmNheight, &a); n++;
	XtGetValues(viewport->resource.VerScroBar, args, n);

	max = (int)a;
	i = (long)max * (long)viewport->viewp.ViewPortHeight / 100;
	j = (long)viewport->page.MaxPageNum * ((long)viewport->page.PageHeight / 100);
	i = i / j;

	slidsize = (int)i;	
	if(slidsize < 1) slidsize = 1;
	if(slidsize >= max) 
	{
		slidsize = max;
		PageMoveUp(viewport, (long) viewport->page.MaxPageNum * (long) viewport->page.PageHeight);
	}

	inc = slidsize * 5000 / viewport->viewp.ViewPortHeight;
	if(inc < 1) inc = 1;
	if(inc > max) inc = max;

	viewport->bar.VerBarInc = viewport->viewp.ViewPortHeight * inc / slidsize;
	
	i = ((long)viewport->viewp.ViewPortPage - 1) * ((long)viewport->page.PageHeight / 100) + (long)viewport->viewp.ViewPort_y / 100;
	i = (long)max * i;
	j = (long)viewport->page.MaxPageNum * ((long)viewport->page.PageHeight / 100);	
	i = i / j;
	slidevalue = (int)i;

	if(slidevalue > max - slidsize) 
	{
		slidevalue = max - slidsize;
		PageMoveDown(viewport, (long) viewport->page.MaxPageNum * (long) viewport->page.PageHeight);
	}
			
	n = 0;
	XtSetArg(args[n], XmNsliderSize, slidsize); n++;
	XtSetArg(args[n], XmNpageIncrement, slidsize); n++;
	XtSetArg(args[n], XmNmaximum, max); n++;
	XtSetArg(args[n], XmNincrement, inc); n++;
	XtSetArg(args[n], XmNvalue, slidevalue); n++;	
	XtSetValues(viewport->resource.VerScroBar, args, n);
}

void HorScroBarExpose(viewport)
VIewportdata *viewport;
{
	register int n;
	Arg args[10];
	int max, min;
	int slidsize, slidevalue, inc;
	long i;
	Dimension a;

	n = 0;
	XtSetArg(args[n], XmNwidth, &a); n++;
	XtGetValues(viewport->resource.HorScroBar, args, n);

	max = (int)a;
	i = (long)max * (long)viewport->viewp.ViewPortWidth;
	i = i / (long)viewport->page.PageWidth;
	
	slidsize = (int)i;
	if(slidsize < 1) slidsize = 1;
	if(slidsize >= max) 
	{
		slidsize = max;
		viewport->viewp.ViewPort_x = 0;
	}

	inc = slidsize * 5000 / viewport->viewp.ViewPortWidth;
	if(inc < 1) inc = 1;
	if(inc > max) inc = max;

	viewport->bar.HorBarInc = viewport->viewp.ViewPortWidth * inc / slidsize;

	i = (long)viewport->viewp.ViewPort_x * (long)max;
	i = i / (long)viewport->page.PageWidth;
	slidevalue = (int)i;

	if(slidevalue > max - slidsize) 
	{
		slidevalue = max - slidsize;
		PageMoveRight(viewport, viewport->page.PageWidth);
	}

	n = 0;
	XtSetArg(args[n], XmNsliderSize, slidsize); n++;
	XtSetArg(args[n], XmNpageIncrement, slidsize); n++;
	XtSetArg(args[n], XmNmaximum, max); n++;
	XtSetArg(args[n], XmNincrement, inc); n++;
	XtSetArg(args[n], XmNprocessingDirection, XmMAX_ON_RIGHT); n++;
	XtSetArg(args[n], XmNvalue, slidevalue); n++;
	XtSetValues(viewport->resource.HorScroBar, args, n);
}

void VerScroBarPos(viewport)
VIewportdata *viewport;
{
	register int n;
	Arg args[2];
	int max;
	int slidevalue, slidsize;
	long i, j;

	n = 0;
	XtSetArg(args[n], XmNmaximum, &max); n++;
	XtSetArg(args[n], XmNsliderSize, &slidsize); n++;
	XtGetValues(viewport->resource.VerScroBar, args, n);

	i = ((long)viewport->viewp.ViewPortPage - 1) * ((long)viewport->page.PageHeight / 100) + ((long)viewport->viewp.ViewPort_y / 100);
	i = (long)max * i;	
	j = (long)viewport->page.MaxPageNum * ((long)viewport->page.PageHeight / 100);
	i = i / j;
	slidevalue = (int)i;

	if(slidevalue > max - slidsize) 
	{
		slidevalue = max - slidsize;
		PageMoveDown(viewport, (long) viewport->page.MaxPageNum * (long) viewport->page.PageHeight);
	}
			
	n = 0;
	XtSetArg(args[n], XmNvalue, slidevalue); n++;	
	XtSetValues(viewport->resource.VerScroBar, args, n);
}

void HorScroBarPos(viewport)
VIewportdata *viewport;
{
	register int n;
	Arg args[2];
	int max;
	int slidevalue, slidsize;
	long i;

	n = 0;
	XtSetArg(args[n], XmNmaximum, &max); n++;
	XtSetArg(args[n], XmNsliderSize, &slidsize); n++;
	XtGetValues(viewport->resource.HorScroBar, args, n);

	i = (long)viewport->viewp.ViewPort_x * (long)max;
	i = i / (long)viewport->page.PageWidth;
	slidevalue = (int)i;

	if(slidevalue > max - slidsize) 
	{
		slidevalue = max - slidsize;
		PageMoveRight(viewport, viewport->page.PageWidth);
	}

	n = 0;
	XtSetArg(args[n], XmNvalue, slidevalue); n++;
	XtSetValues(viewport->resource.HorScroBar, args, n);
}

long CalVerDragDistance(viewport, act)
VIewportdata *viewport;
int act;
{
	register int n;
	Arg args[3];
	int max;
	int slidvalue;
	long curloc, newloc;

	n = 0;
	XtSetArg(args[n], XmNmaximum, &max); n++;
	XtSetArg(args[n], XmNvalue, &slidvalue); n++;
	XtGetValues(viewport->resource.VerScroBar, args, n);

	if(act)
	{
		return((long)((slidvalue * viewport->page.MaxPageNum / max)+1));			
	
	}
	else
	{
		curloc = ((long)viewport->viewp.ViewPortPage - 1) * (long)viewport->page.PageHeight + (long)viewport->viewp.ViewPort_y;
		newloc = (long)(viewport->page.MaxPageNum) * (long)viewport->page.PageHeight / (long)max;
		newloc *= (long)slidvalue;
		return(curloc - newloc);
	}
}

int PageMoveUp(viewport, inc)
VIewportdata *viewport;
long inc;
{
	PAgedata *curpage;
	int id = 0;

       	if(viewport->virdrawn == False) 
	{
		viewport->cursor.RedrawCurPtr = viewport->cursor.CurPtr;
		MyDrawPage(viewport, INVIRDRAW);
	}

	curpage = viewport->viewp.CurPage;

	if (curpage->Last == NULL && viewport->viewp.ViewPort_y == 0) return 0;

	if(inc < (long) viewport->viewp.ViewPortHeight)
	{
		id = (int) inc;
	}
	
	viewport->draw.DrawScr_x = 0;
	viewport->draw.DrawScr_y = 0;
	viewport->draw.DrawPos_x = 0;
	viewport->draw.DrawPos_y = 0;

	while(inc > (long) viewport->page.PageHeight)
	{
		if(curpage->Last == NULL) break;
		inc -= (long) viewport->page.PageHeight;
		viewport->viewp.CurPage = curpage->Last;
		curpage = viewport->viewp.CurPage;
		viewport->viewp.ViewPortPage = curpage->PageNum;
	}
	
	if(viewport->viewp.ViewPort_y > (int)inc)
	{
		viewport->viewp.ViewPort_y -= (int) inc;
		viewport->draw.DrawPos_y = id;
	}
	else
	{
		if (curpage->Last == NULL)
			viewport->viewp.ViewPort_y = 0;
		else
		{
			viewport->draw.DrawPos_y = id;
			viewport->viewp.CurPage = curpage->Last;
			viewport->viewp.ViewPortPage = viewport->viewp.CurPage->PageNum;
			viewport->viewp.ViewPort_y -= (int) inc;
			viewport->viewp.ViewPort_y += viewport->page.PageHeight;
		}
	}
	
	return 1;
}

int PageMoveDown(viewport, inc)
VIewportdata *viewport;
long inc;
{
	PAgedata *curpage;
	int id = 0;

	if(viewport->virdrawn == False) 
	{
		viewport->cursor.RedrawCurPtr = viewport->cursor.CurPtr;
		MyDrawPage(viewport, INVIRDRAW); 
	}

	curpage = viewport->viewp.CurPage;
	
	if(curpage->Next == NULL && viewport->viewp.ViewPort_y + viewport->viewp.ViewPortHeight >= viewport->page.PageHeight)
		return 0;
		
	if(inc < (long) viewport->viewp.ViewPortHeight)
	{
		id = (int) inc;
	}
	
	viewport->draw.DrawScr_x = 0;
	viewport->draw.DrawScr_y = 0;
	viewport->draw.DrawPos_x = 0;
	viewport->draw.DrawPos_y = 0;

	while(inc > (long) viewport->page.PageHeight)
	{
		if(curpage->Next == NULL) break;
		inc -= (long) viewport->page.PageHeight;
		viewport->viewp.CurPage = curpage->Next;
		curpage = viewport->viewp.CurPage;
		viewport->viewp.ViewPortPage = curpage->PageNum;
	}

	if(curpage->Next == NULL)
	{
		if(viewport->viewp.ViewPort_y + viewport->viewp.ViewPortHeight + (int) inc < viewport->page.PageHeight)
		{
			viewport->viewp.ViewPort_y += (int) inc;
			viewport->draw.DrawScr_y = id;
		}
		else
		{
			viewport->viewp.ViewPort_y = viewport->page.PageHeight;
			PageMoveUp(viewport, (long) viewport->viewp.ViewPortHeight);
		}
	}
	else
	{
		viewport->draw.DrawScr_y = id;

		if(viewport->viewp.ViewPort_y + (int) inc < viewport->page.PageHeight)
			viewport->viewp.ViewPort_y += (int) inc;
		else
		{
			viewport->viewp.CurPage = curpage->Next;
			viewport->viewp.ViewPortPage = viewport->viewp.CurPage->PageNum;
			viewport->viewp.ViewPort_y += (int) inc;
			viewport->viewp.ViewPort_y -= viewport->page.PageHeight;
		}
	}
	return 1;
}

void VerPageMove(viewport, inc)
VIewportdata *viewport;
long inc;
{
	if(inc >= 0)
		PageMoveUp(viewport, inc);
	else
		PageMoveDown(viewport, -inc);
}

void PageScroUp(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	VIewportdata *viewport = (VIewportdata *)client_data;  
	
	PageMoveUp(viewport, (long) viewport->viewp.ViewPortHeight);
	MyDrawPage(viewport, TRUEDRAW);
}

void PageScroDown(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	VIewportdata *viewport = (VIewportdata *)client_data;  
	
	PageMoveDown(viewport, (long) viewport->viewp.ViewPortHeight);
	MyDrawPage(viewport, TRUEDRAW);
}

void LineScroUp(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	VIewportdata *viewport = (VIewportdata *)client_data;  
	
	PageMoveUp(viewport, (long) viewport->bar.VerBarInc);
	MyDrawPage(viewport, TRUEDRAW);
}

void LineScroDown(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	VIewportdata *viewport = (VIewportdata *)client_data;  
	
	PageMoveDown(viewport, (long) viewport->bar.VerBarInc);
	MyDrawPage(viewport, TRUEDRAW);
}

void VerScroDraged(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	long inc;
	VIewportdata *viewport = (VIewportdata *)client_data;  

	inc = CalVerDragDistance(viewport, 0);
	VerPageMove(viewport, inc);
	MyDrawPage(viewport, TRUEDRAW);
	viewport->bar.DragPageNum = viewport->viewp.ViewPortPage;
}

void VerScroDraging(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	int pagenum;
	VIewportdata *viewport = (VIewportdata *)client_data;  
	char num[20];
	Arg args[1];

	if(viewport->virdrawn == False)
	{
		viewport->cursor.RedrawCurPtr = viewport->cursor.CurPtr;
		MyDrawPage(viewport, INVIRDRAW);
	}

	pagenum = (int) CalVerDragDistance(viewport, 1);

	if(pagenum != viewport->bar.DragPageNum)  
	{
		viewport->bar.DragPageNum = pagenum;
		sprintf(num, "Page: %d (of %d)\0", pagenum, viewport->page.MaxPageNum);

		XtSetArg(args[0], XmNlabelString, XmStringCreate(num, XmSTRING_DEFAULT_CHARSET));
		XtSetValues(viewport->resource.PageBtn, args, 1);
	}
}


int CalHorDragDistance(viewport)
VIewportdata *viewport;
{
	register int n;
	Arg args[3];
	int max;
	int slidvalue;
	long curloc, newloc;

	n = 0;
	XtSetArg(args[n], XmNmaximum, &max); n++;
	XtSetArg(args[n], XmNvalue, &slidvalue); n++;
	XtGetValues(viewport->resource.HorScroBar, args, n);

	curloc = (long)viewport->viewp.ViewPort_x;
	newloc = (long)slidvalue * (long)(viewport->page.PageWidth);
	newloc /= (long)max;
	
	return((int)(curloc - newloc));
}

int PageMoveLeft(viewport, inc)
VIewportdata *viewport;
int inc;
{
	int id = 0;
	
	if(viewport->viewp.ViewPort_x == 0) return 0;
	
	if(inc < viewport->viewp.ViewPortWidth)
		id = inc;
	
	viewport->draw.DrawScr_x = 0;
	viewport->draw.DrawScr_y = 0;
	viewport->draw.DrawPos_x = 0;
	viewport->draw.DrawPos_y = 0;
	
	if(viewport->viewp.ViewPort_x > inc)
	{
		viewport->viewp.ViewPort_x -= inc;
		viewport->draw.DrawPos_x = id;
	}
	else
		viewport->viewp.ViewPort_x = 0;
	
	return 1;
}

int PageMoveRight(viewport, inc)
VIewportdata *viewport;
int inc;
{
	int id = 0;

	if(viewport->viewp.ViewPort_x == 0 && viewport->viewp.ViewPort_x >= viewport->page.PageWidth - viewport->viewp.ViewPortWidth) return 0;
	else if(viewport->viewp.ViewPort_x == viewport->page.PageWidth - viewport->viewp.ViewPortWidth) return 0;
	
	if(inc < viewport->viewp.ViewPortWidth)
		id = inc;

	viewport->draw.DrawScr_x = 0;
	viewport->draw.DrawScr_y = 0;
	viewport->draw.DrawPos_x = 0;
	viewport->draw.DrawPos_y = 0;
	
	if(viewport->viewp.ViewPort_x + viewport->viewp.ViewPortWidth + inc < viewport->page.PageWidth)
	{
		viewport->viewp.ViewPort_x += inc;
		viewport->draw.DrawScr_x = id;
	}
	else
		viewport->viewp.ViewPort_x = viewport->page.PageWidth - viewport->viewp.ViewPortWidth;
	
	if(viewport->viewp.ViewPort_x < 0) viewport->viewp.ViewPort_x = 0;

	return 1;
}

void HorPageMove(viewport, inc)
VIewportdata *viewport;
int inc;
{
	if(inc >= 0)
		PageMoveLeft(viewport, inc);
	else
		PageMoveRight(viewport, -inc);
}

void PageScroLeft(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	VIewportdata *viewport = (VIewportdata *)client_data;  
	
	PageMoveLeft(viewport, viewport->viewp.ViewPortWidth);
	MyDrawPage(viewport, TRUEDRAW);
}

void PageScroRight(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	VIewportdata *viewport = (VIewportdata *)client_data;  
	
	PageMoveRight(viewport, viewport->viewp.ViewPortWidth);
	MyDrawPage(viewport, TRUEDRAW);
}

void LineScroLeft(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	VIewportdata *viewport = (VIewportdata *)client_data;  
	
	PageMoveLeft(viewport, viewport->bar.HorBarInc);
	MyDrawPage(viewport, TRUEDRAW);
}

void LineScroRight(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	VIewportdata *viewport = (VIewportdata *)client_data;  
	
	PageMoveRight(viewport, viewport->bar.HorBarInc);
	MyDrawPage(viewport, TRUEDRAW);
}

void HorScroDraged(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	int inc;
	VIewportdata *viewport = (VIewportdata *)client_data;

	inc = CalHorDragDistance(viewport);
	HorPageMove(viewport, inc);
	MyDrawPage(viewport, TRUEDRAW);
}


