/*
 * Copyright 1995, 1996, 1997 by Jun Yei.       All Rights Reserved
 *
 * Permission to retain, use, modify, copy, and distribute ChinesePower
 * in source or binary and its documentation (hereafter, the Software)
 * for noncommercial purpose is hereby granted to you without a fee,
 * provided that this entire copyright and permission notice appear in
 * all such copies, that no charge be associated with such copies,
 * that distribution of derivative works (including value-added
 * distributions such as with additional input dictionaries or fonts)
 * include clarification that such added or derived parts are not from
 * the original Software, and that the names of the author(s) not be
 * used to endorse or promote such works. The author(s) of the software
 * makes no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * THE AUTHOR(S) OF THE SOFTWARE DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 * IN NO EVENT SHALL THE AUTHOR(S) OF THE SOFTWARE BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Jun Yei, Ottawa, ON, Canada
 */
#include "chpower.h"

#include <X11/Xatom.h>
#include <stdio.h>

void SelectionReceived();
Boolean ConvertSelection();
void LoseSelection();
void SelectionDone();

struct _SelectionList {
    Cardinal count;
    Time time;
};

MYSelection myselect = { NULL, 0, 0, 0, NULL, 0, NULL, 0, NULL};

void Sel_Unhilite(port, VPType)
void *port;
int VPType;
{
	VIewportdata *viewport;
	SImpleHzWin *sviewport;

	if(port == NULL) return;
	
	if(VPType) 
	{
		viewport = (VIewportdata *) port;

		if(viewport->cursor.StartCurPtr != viewport->cursor.EndCurPtr)
		{
			MyDrawPage(viewport, UNHIGHLITE);
			MyDrawPage(viewport, CURSEARCH);
			viewport->cursor.MoveCurPage = viewport->cursor.ViewPortCurPage;
			viewport->cursor.MoveCur_x = viewport->cursor.ViewPortCur_x;
			viewport->pointer.DrawCur_x =  (viewport->horOrder ? viewport->cursor.ViewPortCur_x : viewport->cursor.ViewPortCur_y);
			viewport->cursor.MoveCur_y = viewport->cursor.ViewPortCur_y;
			viewport->cursor.MoveCurPtr = viewport->cursor.CurPtr;
			viewport->cursor.StartCurPtr = viewport->cursor.CurPtr;
			viewport->cursor.EndCurPtr = viewport->cursor.CurPtr;
			viewport->cursor.Mohzdata = viewport->cursor.hzdata;
			viewport->cursor.MoveColNum = viewport->cursor.ColNum;
			viewport->cursor.MoveLineAlignment = viewport->cursor.LineAlignment;
			viewport->cursor.MoveColorNum = viewport->cursor.ColorNum;
			viewport->cursor.MoveUnderLine = viewport->cursor.UnderLine;
		}	
	}
	else
	{
		sviewport = (SImpleHzWin *) port;

		if(sviewport->cursor.StartPos != sviewport->cursor.EndPos)
		{
			DrawSimpleHzWin(sviewport, UNHIGHLITE);
			ShwDrawCursor(sviewport, False);
			sviewport->cursor.MovePos = sviewport->cursor.CurPos;
			sviewport->cursor.StartPos = sviewport->cursor.CurPos;
			sviewport->cursor.EndPos = sviewport->cursor.CurPos;
		}
	}
}

void GetSelection(w, time, num_params, ReceiveSelection)
Widget w;
Time time;
int num_params;
void (*ReceiveSelection)();
{
    Atom selection;
    int cutbuffer;

    if(num_params == 1) 
    {
	cutbuffer = 0; 
   	selection = XA_CUT_BUFFER0;
    }
    else 
    {
	cutbuffer = -1;
   	selection = XA_PRIMARY;
    }

    if (cutbuffer >= 0) {
	int inbytes;
	unsigned long nbytes;
	int fmt8 = 8;
	Atom type = XA_STRING;
	char *line = XFetchBuffer(XtDisplay(w), &inbytes, cutbuffer);
	nbytes = (unsigned long) inbytes;
	if (nbytes > 0)
	    ReceiveSelection(w, NULL, &selection, &type, (XtPointer)line,
			      &nbytes, &fmt8);
	else if (num_params > 1)
	    GetSelection(w, time, num_params-1, ReceiveSelection);
    } else {
	struct _SelectionList* list;
	if (--num_params) {
	    list = XtNew(struct _SelectionList);
	    list->count = num_params; /* decremented above */
	    list->time = time;
	} else list = NULL;
	XtGetSelectionValue(w, selection, XA_STRING, ReceiveSelection,
			    (XtPointer)list, time);
    }
}

void SelectionReceived(w, client_data, selection, type,
			      value, length, format)
Widget w;
XtPointer client_data;
Atom *selection, *type;
XtPointer value;
unsigned long *length;
int *format;
{
    unsigned char *source;
    unsigned short *tmp, *target;
    int i;
    unsigned long count;
    int ishzform;

    if (*type == 0 || *length == 0 || value == NULL) {
	
	struct _SelectionList* list = (struct _SelectionList*)client_data;
	if (list != NULL) {
	    GetSelection(w, list->time, list->count, SelectionReceived);
	    XtFree(client_data);
	}
	return;
    }

    count = *length;

    if(count > 0) 
    {
	tmp = NULL;
	tmp = (unsigned short *)malloc((count+1) * sizeof(unsigned short));

	if(tmp)
	{
		ishzform = False;

		for(source = (unsigned char*)value, target = tmp; source < (unsigned char*)value + count; source++)
		{
			if(*source == '\n')
			{
				ishzform = False;

				*target++ = '\n';
				
				continue;
			}

			if(*source == '~')
			{
				switch(*(source+1))
				{
					case '{':
				
					source++;
					ishzform = True;
					continue;
				
					case '}':
			
					source++;
					ishzform = False;
					continue;

					case '~':

					*target++ = *source++;
					continue;

					case '\n':
					case '\r':

					source++;
					continue;
				}
			}

			if(ishzform)
			{
				*target = 0x80 | *source++;
				*target <<= 8;
				*target |= 0x80 | *source;
			}
			else if(*source == '\r')
			{
				continue;
			} 
			else 
			{
				*target = *source;
				
				if(ISACHZ(*source))
				{
					source++;	
					*target <<= 8;
					*target |= *source;
				}	
			}

			target++;
		}

		*target = '\0';

    		if(active_viewport != NULL && simple_active == False)
    		{
			MyDrawPage(active_viewport, Input(active_viewport, KEYINPUT, tmp, (int)DBlen(tmp)));
    		}
    		else
    		{
			if(active_simphzwin != NULL && simple_active == True)
			{
				DrawSimpleHzWin(active_simphzwin, SimpleHzWinInput(active_simphzwin, KEYINPUT, tmp, (int)DBlen(tmp)));
			}
    		}
   
 	 	free(tmp); 
   	}
    }

    XtFree(client_data);
    XtFree(value);
}

void PhraseAddSelReceived(w, client_data, selection, type,
			      value, length, format)
Widget w;
XtPointer client_data;
Atom *selection, *type;
XtPointer value;
unsigned long *length;
int *format;
{
    if (*type == 0 || *length == 0 || value == NULL) {
	
	struct _SelectionList* list = (struct _SelectionList*)client_data;
	if (list != NULL) {
/*	    GetSelection(w, list->time, list->count, PhraseAddSelReceived);
*/	    XtFree(client_data);
	}
	ErrorPopup("No Selection!");
	return;
    }


    if(*length > 0) 
    {
	*((char *)value + *length) = '\0';
	PhraseOperation((char *)value, (int)*length, 0);	
    }
    else ErrorPopup("No Selection!");

    XtFree(client_data);
    XtFree(value);

}

void PhraseDelSelReceived(w, client_data, selection, type,
			      value, length, format)
Widget w;
XtPointer client_data;
Atom *selection, *type;
XtPointer value;
unsigned long *length;
int *format;
{
    if (*type == 0 || *length == 0 || value == NULL) {
	
	struct _SelectionList* list = (struct _SelectionList*)client_data;
	if (list != NULL) {
/*	    GetSelection(w, list->time, list->count, PhraseDelSelReceived);
*/	    XtFree(client_data);
	}
	ErrorPopup("No Selection!");
	return;
    }


    if(*length > 0) 
    {
	*((char *)value + *length) = '\0';
	PhraseOperation((char *)value, (int)*length, 1);
    }
    else ErrorPopup("No Selection!");

    XtFree(client_data);
    XtFree(value);

}

void PhraseChkSelReceived(w, client_data, selection, type,
			      value, length, format)
Widget w;
XtPointer client_data;
Atom *selection, *type;
XtPointer value;
unsigned long *length;
int *format;
{
    if (*type == 0 || *length == 0 || value == NULL) {
	
	struct _SelectionList* list = (struct _SelectionList*)client_data;
	if (list != NULL) {
/*	    GetSelection(w, list->time, list->count, PhraseChkSelReceived);
*/	    XtFree(client_data);
	}
	ErrorPopup("No Selection!");
	return;
    }


    if(*length > 0) 
    {
	*((char *)value + *length) = '\0';
	PhraseOperation((char *)value, (int)*length, 2);	
    }
    else ErrorPopup("No Selection!");

    XtFree(client_data);
    XtFree(value);
}


Boolean ConvertSelection(w, selection, target,
				type, value, length, format)
Widget w;
Atom *selection, *target, *type;
XtPointer *value;
unsigned long *length;
int *format;
{
    if (myselect.sel == NULL) return False; 

    if (*target == XA_STRING)
    {	
	*type = *target;
	*value = (XtPointer) myselect.sel;
	*length = myselect.length;
	*format = 8;
	return True;
    }
    return False;

}

void LoseSelection(w, selection)
  Widget w;
  Atom *selection;
{
    register Atom* atomP;
    int i;
    for (i = 0, atomP = myselect.atoms;
	 i < myselect.count; i++, atomP++)
    {
	if (*selection == *atomP) *atomP = (Atom)0;
	switch (*atomP) {
	  case XA_CUT_BUFFER0:
	  case XA_CUT_BUFFER1:
	  case XA_CUT_BUFFER2:
	  case XA_CUT_BUFFER3:
	  case XA_CUT_BUFFER4:
	  case XA_CUT_BUFFER5:
	  case XA_CUT_BUFFER6:
	  case XA_CUT_BUFFER7:	*atomP = (Atom)0;
	}
    }

    for (i = myselect.count; i; i--) {
	if (myselect.atoms[i-1] != 0) break;
    }
    myselect.count = i;

    for (i = 0, atomP = myselect.atoms;
	 i < myselect.count; i++, atomP++)
    {
	if (*atomP == (Atom)0) {
	    *atomP = myselect.atoms[--myselect.count];
	}
    }

    if (myselect.count == 0)
    {
	DisownSelection();
    }
}


void SelectionDone(w, selection, target)
Widget w;
Atom *selection, *target;
{
}


void OwnSelection(w, count)
Widget w;
int count;
{
    int i;
    Boolean have_selection = False;

    if (myselect.length <= 0) return;

    if (count > myselect.atoms_size) {
	if(myselect.atoms) XtFree((char*)myselect.atoms);
	myselect.atoms = (Atom*)XtMalloc(count*sizeof(Atom));
   	myselect.atoms_size = count; 
    }
    
    for (i = 0; i < count; i++) {
	int cutbuffer;
	
	if(i == 0) 
	{
		cutbuffer = -1;
		myselect.atoms[0] = XA_PRIMARY;
	}
	else 
	{
		cutbuffer = 0;
		myselect.atoms[i] = XA_CUT_BUFFER0;
	}
	
	if (cutbuffer >= 0)
	    if ( myselect.length >
		 4*XMaxRequestSize(XtDisplay(w))-32)
		ErrorPopup("Too Big Selection!");
	    else
		XStoreBuffer( XtDisplay(w), myselect.sel, myselect.length, cutbuffer );
	else {
	    have_selection |=
		XtOwnSelection(w, myselect.atoms[i], myselect.time, ConvertSelection, LoseSelection, SelectionDone);
	}
    }
    myselect.count = count;
    myselect.w = w;

   if(!have_selection) 
   {
	DisownSelection();
   }
}

void DisownSelection()
{
    int i;

    if(myselect.w == NULL) return;

    for (i = 0; i < myselect.count; i++) {
	int cutbuffer;
	switch (myselect.atoms[i]) {
	  case XA_CUT_BUFFER0: cutbuffer = 0; break;
	  case XA_CUT_BUFFER1: cutbuffer = 1; break;
	  case XA_CUT_BUFFER2: cutbuffer = 2; break;
	  case XA_CUT_BUFFER3: cutbuffer = 3; break;
	  case XA_CUT_BUFFER4: cutbuffer = 4; break;
	  case XA_CUT_BUFFER5: cutbuffer = 5; break;
	  case XA_CUT_BUFFER6: cutbuffer = 6; break;
	  case XA_CUT_BUFFER7: cutbuffer = 7; break;
	  default:	       cutbuffer = -1;
	}
	if (cutbuffer < 0)
	    XtDisownSelection(myselect.w, myselect.atoms[i], myselect.time);
    }
    myselect.count = 0;

    Sel_Unhilite(myselect.viewport, myselect.VPType);

   myselect.viewport = NULL;
   myselect.w = NULL;
   myselect.viewport = NULL; 
   myselect.length = 0;
   if(myselect.sel) 
   {
	free(myselect.sel);
	myselect.sel = NULL;
   }
}

void DistroySelection()
{
	DisownSelection();
	if(myselect.atoms) XtFree((char*)myselect.atoms);
}

