/*
 * Copyright 1995, 1996, 1997 by Jun Yei.       All Rights Reserved
 *
 * Permission to retain, use, modify, copy, and distribute ChinesePower
 * in source or binary and its documentation (hereafter, the Software)
 * for noncommercial purpose is hereby granted to you without a fee,
 * provided that this entire copyright and permission notice appear in
 * all such copies, that no charge be associated with such copies,
 * that distribution of derivative works (including value-added
 * distributions such as with additional input dictionaries or fonts)
 * include clarification that such added or derived parts are not from
 * the original Software, and that the names of the author(s) not be
 * used to endorse or promote such works. The author(s) of the software
 * makes no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * THE AUTHOR(S) OF THE SOFTWARE DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 * IN NO EVENT SHALL THE AUTHOR(S) OF THE SOFTWARE BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Jun Yei, Ottawa, ON, Canada
 */

#include "chpower.h"

void SimpleHzWinResizeCB();
void SimpleHzWinExposeCB();
void SimpleMiscCB();

void SimpleHzWinDestroyCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	SImpleHzWin *viewport = (SImpleHzWin *)client_data;
	
	if(viewport->cursor.TimerId) XtRemoveTimeOut(viewport->cursor.TimerId);
	
	XtRemoveEventHandler(viewport->resource.WorkArea, FocusChangeMask, False, SimpleMiscCB, 
	(caddr_t *) viewport);

	XtRemoveEventHandler(XtParent(XtParent(viewport->resource.WorkArea)), EnterWindowMask | LeaveWindowMask,
	False, SimpleMiscCB, (caddr_t *) viewport);

	free(viewport->EditFile);
}

void SimpleMiscCB(w, client_data, event)
Widget w;
XtPointer client_data;
XEvent *event;
{
	SImpleHzWin *viewport = (SImpleHzWin *)client_data;

	switch(event->type)
	{
		case FocusIn:

		viewport->cursor.Focused = True;

		case EnterNotify:

		if(event->type == EnterNotify && event->xcrossing.detail == NotifyInferior) break;
		
		if(viewport->cursor.Focused && viewport->cursor.TimerId == (XtIntervalId) NULL)
		{
			viewport->cursor.TimerId =
			XtAppAddTimeOut(XtWidgetToApplicationContext(viewport->resource.WorkArea),500,
			SimCurFlash, viewport);
		}
		break;

		case FocusOut:

		viewport->cursor.Focused = False;

		case LeaveNotify:

		if(event->type == LeaveNotify && event->xcrossing.detail == NotifyInferior) break;
		
		ShwDrawCursor(viewport, False);

		if(viewport->cursor.TimerId)
		{
			XtRemoveTimeOut(viewport->cursor.TimerId);
			viewport->cursor.TimerId = (XtIntervalId) NULL;
		}
	}
}

Widget CreateSimpleHzWin(parent, name, args, n, viewport)
Widget parent;
char *name;
Arg *args;
int n;
SImpleHzWin *viewport;
{
	Widget hzwin;

	XtSetArg(args[n], XmNmarginHeight, 0); n++;
	XtSetArg(args[n], XmNmarginWidth, 0); n++;
	XtSetArg(args[n], XmNshadowThickness, 2); n++;
	XtSetArg(args[n], XmNshadowType, XmSHADOW_IN); n++;
	hzwin = XmCreateFrame(parent, name, args, n);
	parent = hzwin;

	n = 0;
	XtSetArg(args[n], XmNmarginWidth, 0); n++;
	XtSetArg(args[n], XmNmarginHeight, 0); n++;
	hzwin = XmCreateDrawingArea(parent, name, args, n);
	XtManageChild(hzwin);

	viewport->resource.WorkArea = hzwin;
	viewport->viewp.firstnum = 0;
	viewport->viewp.currentnum = 0;	
	viewport->viewp.DBcurrentnum = 0;
	viewport->EditFile = (unsigned short *)malloc(SHZW_MEM_UNIT*sizeof(unsigned short));
	*viewport->EditFile = '\0';
	return(parent);
}

void AddSHwCallbacks(viewport)
SImpleHzWin *viewport;
{
	viewport->resource.dpy = XtDisplay(viewport->resource.WorkArea);
	viewport->resource.wind = XtWindow(viewport->resource.WorkArea);
	XtAddEventHandler(viewport->resource.WorkArea, FocusChangeMask,
	False, SimpleMiscCB, (caddr_t *) viewport);
	XtAddEventHandler(XtParent(XtParent(viewport->resource.WorkArea)), EnterWindowMask | LeaveWindowMask,
	False, SimpleMiscCB, (caddr_t *) viewport);
	XtAddCallback(viewport->resource.WorkArea, XmNinputCallback, SimpleHzWinInputCB, (caddr_t *)viewport);
	XtAddCallback(viewport->resource.WorkArea, XmNresizeCallback, SimpleHzWinResizeCB, (caddr_t *)viewport);
	XtAddCallback(viewport->resource.WorkArea, XmNexposeCallback, SimpleHzWinExposeCB, (caddr_t *)viewport);
	XtAddCallback(viewport->resource.WorkArea, XmNdestroyCallback, SimpleHzWinDestroyCB, (caddr_t *)viewport);
}

void SimpleHzWinExpose(viewport)
SImpleHzWin *viewport;
{
	XGCValues gc_init;
	register int n;
	Arg args[10];
	Dimension a, b;
	int i;

	n = 0;
	XtSetArg(args[n], XmNbackground, &gc_init.background); n++;
	XtSetArg(args[n], XmNforeground, &gc_init.foreground); n++;
	XtSetArg(args[n], XmNheight, &a); n++;
	XtSetArg(args[n], XmNwidth, &b); n++;
	XtGetValues(viewport->resource.WorkArea, args, n);
	viewport->resource.gc = XtGetGC(viewport->resource.WorkArea, GCForeground|GCBackground, &gc_init);
	
	i = gc_init.background;
	gc_init.background = gc_init.foreground;
	gc_init.foreground = i;
	viewport->resource.reverse_gc = XtGetGC(viewport->resource.WorkArea, GCForeground|GCBackground, &gc_init);

	viewport->viewp.height = (int)a;
	viewport->viewp.width = (int)b * 2/ SHZW_FONT_SIZE;
}

void SimpleHzWinExposeCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	SImpleHzWin *viewport = (SImpleHzWin *)client_data;
	XmDrawingAreaCallbackStruct *mydata = (XmDrawingAreaCallbackStruct *)call_data;
	XEvent *event = mydata->event;	

	if(event->xexpose.count==0)
	{
		SimpleHzWinExpose(viewport);
		DrawSimpleHzWin(viewport, EXPOSEDR);
	}
}

void SimpleHzWinResizeCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	SImpleHzWin *viewport = (SImpleHzWin *)client_data;
	
	SimpleHzWinExpose(viewport);
	DrawSimpleHzWin(viewport, TRUEDRAW);
}

