/*
 * Copyright 1995, 1996, 1997 by Jun Yei.       All Rights Reserved
 *
 * Permission to retain, use, modify, copy, and distribute ChinesePower
 * in source or binary and its documentation (hereafter, the Software)
 * for noncommercial purpose is hereby granted to you without a fee,
 * provided that this entire copyright and permission notice appear in
 * all such copies, that no charge be associated with such copies,
 * that distribution of derivative works (including value-added
 * distributions such as with additional input dictionaries or fonts)
 * include clarification that such added or derived parts are not from
 * the original Software, and that the names of the author(s) not be
 * used to endorse or promote such works. The author(s) of the software
 * makes no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * THE AUTHOR(S) OF THE SOFTWARE DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 * IN NO EVENT SHALL THE AUTHOR(S) OF THE SOFTWARE BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Jun Yei, Ottawa, ON, Canada
 */

#include "chpower.h"
#include "title.h"

void ExposeTitleCB();
void OpenTitleCB();
void CloseTitleCB();

TitleShow title;

void ShowTitle()
{
	XEvent myevent;
	XSizeHints myhint;
	XWMHints mywmhint;
	int myscreen;
	unsigned long myforeground, mybackground;
	Cursor cur;
	Font font;

	title.dpy = XOpenDisplay("");

	if(!title.dpy) 
	{
		fprintf(stderr,"Fatal error: Can not open Dispaly\n");
		exit(1);
	}

	myscreen = DefaultScreen(title.dpy);

	mybackground = WhitePixel(title.dpy, myscreen);
	myforeground = BlackPixel(title.dpy, myscreen);

	myhint.x = 150;
	myhint.y = 250;
	myhint.width = 300;
	myhint.height = 150;
	myhint.flags = PPosition | PSize;
	mywmhint.flags = InputHint;
	mywmhint.input = True;

	title.wind = XCreateSimpleWindow(title.dpy,
	DefaultRootWindow(title.dpy), myhint.x, myhint.y, myhint.width,
	myhint.height, 5, myforeground, mybackground);
	XSetStandardProperties(title.dpy, title.wind, "hello", "hello", None, NULL,
	0, &myhint);
	XSetWMHints(title.dpy, title.wind, &mywmhint);
	
	title.gc = XCreateGC(title.dpy, title.wind, 0, 0);
	font = XLoadFont(title.dpy, "6x13");
	XSetBackground(title.dpy, title.gc, mybackground);
	XSetForeground(title.dpy, title.gc, myforeground);
	XSetFont(title.dpy, title.gc, font);
	cur = XCreateFontCursor(title.dpy, XC_watch);
	XDefineCursor(title.dpy, title.wind, cur);
	XSelectInput(title.dpy, title.wind, ButtonPressMask | KeyPressMask | ExposureMask);

	XMapRaised(title.dpy, title.wind);

	XNextEvent(title.dpy, &myevent);
	ExposeTitleCB(title.PopUpWg, NULL, NULL);
	XCheckTypedEvent(title.dpy, EnterNotify, &myevent);
}

void CloseTitle()
{
	XFreeGC(title.dpy, title.gc);
	XDestroyWindow(title.dpy, title.wind);
	XCloseDisplay(title.dpy);
}

void CreateTitleSet(parent)
Widget parent;
{
	Arg args[MAX_ARGS];
	register int n;
	XGCValues gc_init;
	Font font;

	title.PopUpWg = XtCreatePopupShell("Title Window", transientShellWidgetClass, parent, NULL, 0);
	title.mapped = False;

	n = 0;
	XtSetArg(args[n], XmNheight, 150); n++;
	XtSetArg(args[n], XmNwidth, 300); n++;
	XtSetArg(args[n], XmNx, 600); n++;
	XtSetArg(args[n], XmNy, 400); n++;
	XtSetArg(args[n], XmNresizePolicy, XmRESIZE_NONE); n++;
	title.Root = XmCreateDrawingArea(title.PopUpWg, "Title", args, n);
	XtManageChild(title.Root);
	
	XtAddCallback(title.Root, XmNinputCallback, CloseTitleCB, NULL);
	XtAddCallback(title.Root, XmNexposeCallback, ExposeTitleCB, NULL);

	XtRealizeWidget(title.PopUpWg);
	
	title.dpy = XtDisplay(title.Root);
	title.wind = XtWindow(title.Root);
	
	n = 0;
	XtSetArg(args[n], XmNbackground, &gc_init.background); n++;
	XtSetArg(args[n], XmNforeground, &gc_init.foreground); n++;
	XtGetValues(title.Root, args, n);
	title.gc = XtGetGC(title.Root, GCForeground|GCBackground, &gc_init);
	font = XLoadFont(title.dpy, "6x13");
	XSetFont(title.dpy, title.gc, font);
}

void ExposeTitleCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	XImage *graph;

	graph = XCreateImage(title.dpy, DefaultVisual(title.dpy, DefaultScreen(title.dpy)), 1, XYBitmap, 0, (char *)NULL, 112, 112, 8, 0);
	if(graph->bitmap_bit_order == LSBFirst)
	{
		graph->data = (char *) TitleLSB;
	}
	else
	{
		graph->data = (char *) TitleMSB;
	}
		
	XPutImage(title.dpy, title.wind, title.gc, graph, 0, 0, 20, 20, 112, 112);
	XFree(graph);
	
	graph = XCreateImage(title.dpy, DefaultVisual(title.dpy, DefaultScreen(title.dpy)), 1, XYBitmap, 0, (char *)NULL, 32, 16, 8, 0);
	if(graph->bitmap_bit_order == LSBFirst)
	{
		graph->data = (char *) myhz_nameLSB;
	}
	else
	{
		graph->data = (char *) myhz_nameMSB;
	}
	
	XPutImage(title.dpy, title.wind, title.gc, graph, 0, 0, 237, 82, 32, 16);
	XFree(graph);

	XDrawImageString(title.dpy, title.wind, title.gc, 152, 45, "ChinesePower Ver3.0", 19);
	XDrawImageString(title.dpy, title.wind, title.gc, 152, 60, "Developed by Jun Yei", 20);
	XDrawImageString(title.dpy, title.wind, title.gc, 152, 85, "Copyright (c)", 13);
	XDrawImageString(title.dpy, title.wind, title.gc, 176, 98, "1996,1997", 9);
	XDrawImageString(title.dpy, title.wind, title.gc, 152, 115, "All rights reserved", 19);
}

void OpenTitleCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	if(!title.mapped)
	{
		title.mapped = True;
		PopupPosition(w, title.PopUpWg);
		XtPopup(title.PopUpWg, XtGrabExclusive); 
	}
}

void CloseTitleCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	
	title.mapped = False;
	XtPopdown(title.PopUpWg);
}

