/*
 * Copyright 1995, 1996, 1997 by Jun Yei.       All Rights Reserved
 *
 * Permission to retain, use, modify, copy, and distribute ChinesePower
 * in source or binary and its documentation (hereafter, the Software)
 * for noncommercial purpose is hereby granted to you without a fee,
 * provided that this entire copyright and permission notice appear in
 * all such copies, that no charge be associated with such copies,
 * that distribution of derivative works (including value-added
 * distributions such as with additional input dictionaries or fonts)
 * include clarification that such added or derived parts are not from
 * the original Software, and that the names of the author(s) not be
 * used to endorse or promote such works. The author(s) of the software
 * makes no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * THE AUTHOR(S) OF THE SOFTWARE DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 * IN NO EVENT SHALL THE AUTHOR(S) OF THE SOFTWARE BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Jun Yei, Ottawa, ON, Canada
 */

/* TTF to BMP */
/* Written by Lin YawJen (LYJ) 1994 */ 
/* edited by Wu LiangSheng (LSN) 1994 */ 
/* steal by Yu-Chung Wang 1994 */
/* MSC: cl /AL /Ox /G2 demottf.c */ 
/* GCC: gcc -o ttf2bmp ttf2bmp.c -lpc -lgrx */ 
 
/* usage: e.g. compiled as ttf2bmp.exe, 
read ntu_li_m.ttf with 256 points (max) 
 
> ttf2bmp ntu_li_m.ttf 256 
 
Hold escape key for a moment to quit. 
*/ 
 
/* 
Input: TTF Font File 
Output: font_buf 1 bit per pixel array 
To display it on screen, need to write 
translation routine for particular graphics mode. 
Email me for more information. 
Note: some function names may vary between compiler implementations 
Try out new ones if you know. Minor changes will be OK. 
But don't ask me about your own compiler because I don't know. 
 
Porting tips: use your own Graphics library. The rest may require  
minor changes only. 
*/ 
 
/* #define NOV -> No Video, will print some info in text */ 
/* #define NOV */ 

#include <stdio.h> 
#include <math.h> 
#include <stdlib.h> 
#include <string.h> 
#include "ttf.h"
#include "fonts.h" 
#include "ttffont.h"
#include <X11/X.h>

#define MAXXSIZE 3200
#define MAXYSIZE 3200
#define MAX(a,b) ((a)>(b)? (a):(b))

typedef short fixp; /* May use long to prevent overflow */ 
/* Fix point : S_############.## bit, range +-4096 with 1/4 precision */ 
 
struct SCANLINE 
     { 
     short max, cur; 
     fixp *grid; 
     char *dir; 
     } *rowarray = NULL, *colarray = NULL; 
 
/* drawdot and hline mask */ 

unsigned char mask_pixelLSB[8] = {0x01, 0x02, 0x04, 0x08, 0x10, 0x20, 0x40, 0x80};
unsigned char mask_fromLSB[8] = {0xff, 0xfe, 0xfc, 0xf8, 0xf0, 0xe0, 0xc0, 0x80};
unsigned char mask_toLSB[8] = {0x01, 0x03, 0x07, 0x0f, 0x1f, 0x3f, 0x7f, 0xff};

unsigned char mask_pixelMSB[8] = {0x80,0x40,0x20,0x10,8,4,2,1}; 
unsigned char mask_fromMSB[8] = {0xff,0x7f,0x3f,0x1f,0xf,7,3,1}; 
unsigned char mask_toMSB[8] = {0x80,0xc0,0xe0,0xf0,0xf8,0xfc,0xfe,0xff}; 

void convert(); 
void clear_data(); 
 
void DestroyTTF()
{
	int i;
	
	if(rowarray)
	{
		for (i = 0; i < MAXYSIZE; i++)
		{
			free(rowarray[i].grid);
			free(rowarray[i].dir);	
		}
		free(rowarray);
	}

	if(colarray)
	{
		for (i = 0; i < MAXXSIZE; i++)
		{
			free(colarray[i].grid);
			free(colarray[i].dir);	
		}
		free(colarray);
	}
}
 
void Init_TTF_Env() /* Initial the data space */ 
{ 
  short i; 

  rowarray = (struct SCANLINE *)calloc(sizeof(struct SCANLINE),MAXYSIZE); 
  if (rowarray == NULL) 
    { 
      fprintf(stderr, "Memory error when initial.\n"); 
      exit(1); 
    } 
  colarray = (struct SCANLINE *)calloc(sizeof(struct SCANLINE),MAXXSIZE); 
  if (colarray == NULL) 
    { 
      fprintf(stderr, "Memory error when initial.\n"); 
      exit(1); 
    } 
  for (i = 0; i < MAXYSIZE; i++) 
    { 
      rowarray[i].max = 32;  /* from max of NTU_MM */ 
      rowarray[i].grid = (fixp *)calloc(sizeof(fixp),32); 
      rowarray[i].dir = (char *)calloc(sizeof(char),32); 
      if (rowarray[i].grid == NULL || rowarray[i].dir == NULL) 
	{ 
	  fprintf(stderr, "Memory overflow when init rowarray.\n"); 
	  exit(2); 
	} 
    } 
  for (i = 0; i < MAXXSIZE; i++) 
    { 
      colarray[i].max = 32; 
      colarray[i].grid = (fixp *)calloc(sizeof(fixp),32); 
      colarray[i].dir = (char *)calloc(sizeof(char),32);
      if (colarray[i].grid == NULL || colarray[i].dir == NULL) 
	{ 
	  fprintf(stderr, "Memory overflow when init colarray.\n"); 
	  exit(2); 
	} 
    } 
} 

/* Must free all and allocate new ones when resize the resolution. */ 
 
void clear_data(bit, map) 
HZdata *bit;
unsigned char *map;
{ 
  short i; 

  for (i = 0; i < bit->hz_sheight; i++) rowarray[i].cur = 0; 
  for (i = 0; i < bit->hz_swidth; i++) colarray[i].cur = 0; 
  memset(map,0,bit->hz_sheight*((bit->hz_swidth + 7)/8));
} 
 
 
 
static fixp lastx, lasty; 
 
#define Moveto(x, y)  {lastx = (x); lasty = (y);} 

void Lineto();
void Curveto();
void SplitCurve();
void Draw(); 
static LONG *xcoor,*ycoor;
static USHORT *epts_ctr;
static BYTE *flags;
static int num_ctr;
 
void convert(bit, map, byte_order) 
HZdata *bit;
unsigned char *map;
int byte_order;
{ 
  short i; 
  unsigned short j, start_offpt, end_offpt, fst; 
  TTFFont *f = (TTFFont *)bit->hbf;


  clear_data(bit, map);
  xcoor = f->glyf->xCoordinates;
  ycoor = f->glyf->yCoordinates;
  epts_ctr = f->glyf->endPtsOfContours;
  flags = f->glyf->flags;
  num_ctr = f->glyf->numberOfContours;

  for (i = 0, j = 0; i < num_ctr; i++) 
    { 
      fst = j; 
      Moveto(xcoor[j],ycoor[j]); 
      start_offpt = 0; /*start at least 1*/ 
      for (j++; j <= epts_ctr[i]; j++) 
	{ 
	  if (!(flags[j]&1)) 
	    { 
	      /*Off curve*/ 
	      if (!start_offpt) 
		  start_offpt = end_offpt = j;
	      else 
		end_offpt++; 
	    } 
	  else { 
	    /*On Curve*/ 
	    if (start_offpt) 
	      { 
		Curveto(xcoor[j],ycoor[j],start_offpt,end_offpt); 
		start_offpt = 0; 
	      } 
	    else 
	      Lineto(xcoor[j],ycoor[j]); 
	  } 
	} 
      if (start_offpt) 
	Curveto(xcoor[fst],ycoor[fst],start_offpt,end_offpt); 
      else 
	Lineto(xcoor[fst],ycoor[fst]); 
    } 
  Draw(bit, map, byte_order); 
} 

void Lineto(x, y) 
fixp x;
fixp y;
{ 
  short start, end; 
  long delta, sy, sx; /* 10 bit point */ 
  fixp *tmp; 
  char *adir;

  if (x != lastx ) /* usual apply under 96X96 */ 
    { 
	/* calculate the slope of line */
	delta = ((long)(y-lasty))*1024/((long)(x-lastx)); 
	if (x < lastx) 
	  { 
	      /******************************************************/
	      /* round up ,the coordinates here is 14DF2            */
	      /* (4*3+1)/4 to (4*3+4)/4 will round to 4 for example */
	      /******************************************************/
	      start = (x+3)/4; 
	      end = (lastx+3)/4; 
	      sy = ((long)y)*256 + (((long)start)*4-((long)x))*delta/4; 
	  } 
	else
	  { 
	      start = (lastx+3)/4; 
	      end = (x+3)/4; 
	      sy = ((long)lasty)*256 + (((long)start)*4-((long)lastx))*delta/4; 
	  } 
	if (start < 0) 
	  { 
	      /* trunce to x = 0 */
	      sy += delta*(-start); 
	      start = 0;
	  } 
	if (end >= MAXXSIZE) 
	  end = MAXXSIZE-1; 
	/* add = for hack */
	for (; start < end; start++, sy += delta) 
	  { 
	      /* xadd(start,(fixp)(sy/256));*/ 
	      if (colarray[start].cur == colarray[start].max) 
		{
		    /* Growth 8 each time */ 
		    tmp = (fixp *)realloc(colarray[start].grid, 
					  (colarray[start].max+8)*sizeof(fixp)); 
		    if (tmp == NULL) { fprintf(stderr, "Memory over.\n"); continue; } 
		    colarray[start].grid = tmp; 
		    
		    adir = (char *)realloc(colarray[start].dir, (colarray[start].max+8)*sizeof(char));
		    if(adir == NULL) { fprintf(stderr, "Memory over.\n"); continue; }
		    colarray[start].dir = adir;
		    
		    colarray[start].max += 8; 
		} 
	      /**************************************************/
	      /* we scale 1024 at beginning but we calculate in */
	      /* F14.2 so divide 4                              */
	      /*   1024/4 = 256                                 */
	      /**************************************************/
	      
	      if(x > lastx) colarray[start].dir[colarray[start].cur] = 1;
	      else colarray[start].dir[colarray[start].cur] = -1;
	      
	      colarray[start].grid[colarray[start].cur++] = (fixp)(sy/256); 
	  } 
    } 
  
  if (y != lasty) 
    { 
	delta = ((long)(x-lastx))*1024/((long)(y-lasty)); 
	if (y < lasty) 
	  { 
	      start = (y+3)/4; 
	      end = (lasty+3)/4; 
	      sx = ((long)x)*256 + (((long)start)*4-((long)y))*delta/4; 
	  } 
	else
	  { 
	      start = (lasty+3)/4; 
	      end = (y+3)/4; 
	      sx = ((long)lastx)*256 + ((long)(start*4-lasty))*delta/4; 
	  } 
	if (start < 0) 
	  { 
	      sx += delta*(-start); 
	      start = 0;
	  } 
	if (end >= MAXYSIZE) 
	  end = MAXYSIZE-1; 
	/* add = for hack */
	for (; start < end; start++, sx += delta) 
	  { 
	      /* yadd(start,(fixp)(sx/256));*/ 
	      if (rowarray[start].cur == rowarray[start].max) 
		{ 
		    /* Growth 8 each time */ 
		    tmp = (fixp *)realloc(rowarray[start].grid, 
					  (rowarray[start].max+8)*sizeof(fixp)); 
		    if (tmp == NULL) { fprintf(stderr, "Memory over.\n"); continue; } 
		    rowarray[start].grid = tmp; 
		    
		    adir = (char *)realloc(rowarray[start].dir, (rowarray[start].max+8)*sizeof(char));
	       	    if(adir == NULL) { fprintf(stderr, "Memory over.\n"); continue; }
		    rowarray[start].dir = adir;

		    rowarray[start].max += 8; 
		} 
	      
	      if(y > lasty) rowarray[start].dir[rowarray[start].cur] = 1;
	      else rowarray[start].dir[rowarray[start].cur] = -1;

	      rowarray[start].grid[rowarray[start].cur++] = (fixp)(sx/256); 
	  } 
    } 
  lastx = x; lasty = y; 
} 
 
void Curveto(x, y, ctrl_fst, ctrl_last) 
fixp x, y;
unsigned short ctrl_fst;
unsigned short ctrl_last;
{ 
  unsigned short ctrl_next = ctrl_fst+1; 
  fixp _x = xcoor[ctrl_fst], _y = ycoor[ctrl_fst]; 
  fixp tx, ty; 

  Lineto((lastx+_x)/2, (lasty+_y)/2); /*Uniform B-spline*/ 

  for (; ctrl_fst <= ctrl_last; ctrl_fst++, ctrl_next++) 
    { 
      if (ctrl_next <= ctrl_last) 
	{ 
	  tx = xcoor[ctrl_next]; 
	  ty = ycoor[ctrl_next];
	} 
      else 
	{ 
	  tx = x;
	  ty = y; 
	} 
	
      SplitCurve(_x, _y, (_x+tx)/2, (_y+ty)/2); 
      _x = tx; 
      _y = ty; 
    }
  
  Lineto(x, y); 
  
} 
 
void SplitCurve(x1, y1, x2, y2) /* Great Modify */ 
fixp x1, y1, x2, y2;
{ 
  /*Delta routine for split nonuniform part of B-spline*/ 
  short n, i, m; 
  long p1x = ((long)lastx-2*(long)x1+(long)x2) << 8; 
  long p1y = ((long)lasty-2*(long)y1+(long)y2) << 8; 
  long ddx = p1x, ddy = p1y, p2x, p2y, fx, fy, dx, dy; 
  long ratio; /* 10 bit point */ 
  
  if(ddx < 0) ddx = -ddx;
  if(ddy < 0) ddy = -ddy;
  
  ratio = MAX(ddx, ddy);
  
  for (n = 0; ratio >= 512L; n++) ratio >>= 2; /* Compute coef. of spline */ 
  if (n) 
    {                                       /* delta = 1/2^n */ 
      p2x = ((long)x1 - (long)lastx) << 9; 
      p2y = ((long)y1 - (long)lasty) << 9; 
      ddx = ((p1x*2) >> (2*n)); /* 2 order */ 
      ddy = ((p1y*2) >> (2*n)); 
      dx = (p2x >> n)+(p1x >> (2*n)); 
      dy = (p2y >> n)+(p1y >> (2*n)); 
      fx = dx + ((long)lastx << 8); 
      fy = dy + ((long)lasty << 8); 
      for (m = (1 << n), i = 1; i < m; i++) 
	{ 
	  Lineto((fixp)(fx/256),(fixp)(fy/256)); 
	  dx += ddx; fx += dx; 
	  dy += ddy; fy += dy; 
	} 
    } 
  Lineto(x2,y2); 
} 
 
void Draw(bit, map, byte_order) 
HZdata *bit;
unsigned char *map;
int byte_order;
{ 
  short flag, i, j, k, cnt, len, x, y, x1, y1, xsize = (bit->hz_swidth + 7)/8; /* Modify */ 
  fixp *tmp, t; 
  int ycur;
  unsigned char *ptr, *mask_pixel, *mask_from, *mask_to;
  char *mydir;
  char count, tdir;

  if(byte_order == LSBFirst)
  {
	mask_pixel = mask_pixelLSB;
	mask_from = mask_fromLSB;
	mask_to = mask_toLSB;
  }
  else
  {
	mask_pixel = mask_pixelMSB;
	mask_from = mask_fromMSB;
	mask_to = mask_toMSB;
  }
  
  for (i = 0; i < bit->hz_sheight; i++) 
    { 
      if (!rowarray[i].cur) continue; 
      ycur = (bit->hz_sheight-i-1)*xsize; 
      tmp = rowarray[i].grid; 
      cnt = rowarray[i].cur; 
      mydir = rowarray[i].dir;
      
      for (j = 1; j < cnt; j++) /* Insert sort for average length 6*/ 
	{ 
	  for (t = tmp[j], tdir = mydir[j], k = j-1; (k >= 0) && (tmp[k] > t); k--) 
	  {
		tmp[k+1] = tmp[k]; 
	 	mydir[k+1] = mydir[k];
	  }

	  if (k < j-1) 
	  {
		tmp[k+1] = t; 
		mydir[k+1] = tdir;
	  }
	} 
      
	count = mydir[0];

	for (j = 1; j < cnt; j++) 
	{ 
	  k = j - 1;
	  
		if(count != 0)
		{
			x = (tmp[k]+3)/4;
			y = (tmp[j]+3)/4;
			
			if(tmp[j] - tmp[k] < 4) 
			{
				if(x != y)
				{
					y = x;
					tmp[j] = y * 4 - 3;
				}
			}

			flag = 1;

			if(x < 0)
			{
				if(y < 0) flag = 0;
				else if(y >= bit->hz_swidth) { x = 0; y = bit->hz_swidth - 1;}
				else x = 0;
			}
			else if(x >= bit->hz_swidth) flag = 0;
			else if(y >= bit->hz_swidth) y = bit->hz_swidth - 1;

			if(flag)
			{
				x1 = x >> 3;
				y1 = y >> 3;
				ptr = map + ycur + x1;

				if(x1 == y1) *ptr |= (mask_from[x&7] & mask_to[y&7]);
				else 
				{
					*ptr++ |= mask_from[x&7];
					for (x1++; x1 < y1; x1++) *ptr++ = 0xff;
					*ptr |= mask_to[y&7];
				}	
			}
		}	

	  count += mydir[j];

	}    /* Scan convert rule 0 & 1 */ 
    } 
 
  if(bit->hz_sheight <= 96)
  {
	for (i = 0; i < bit->hz_swidth; i++) 
  	{ 
      
      		/* Only check rule 3, can be skipped when res > 96 */ 
      		
		if (!colarray[i].cur) continue; 
      		
		tmp = colarray[i].grid; 
      		cnt = colarray[i].cur; 
      		mydir = colarray[i].dir;

		for (j = 1; j < cnt; j++) /* Insert sort for average length 6*/ 
		{ 
	    		for (t = tmp[j], tdir = mydir[j], k = j-1; (k >= 0) && (tmp[k] > t); k--) 
	      		{
				tmp[k+1] = tmp[k]; 
	    			mydir[k+1] = tmp[k];
			}
			
			if (k < j-1) 
			{
				tmp[k+1] = t; 
	  			mydir[k+1] = tdir;
			}
		} 
      	
		count = mydir[0];

		for (j = 1; j < cnt; j++) /* Modify */ 
		{
			k = j - 1;
			
			if (count != 0 && tmp[j]-tmp[k] < 6) 
			{
				y = bit->hz_sheight-1-(tmp[k] >> 2);
				if (y >= 0 && y <= bit->hz_sheight)
					map[y*xsize+ (i >> 3)] |= mask_pixel[i&7];
     			}
	
			count += mydir[j];

		}
		/* Scan convert rule 3, usual appling under 96X96 */ 
    	} 
  }
} 

