#ifndef __TTF_FONT_H
#define __TTF_FONT_H

/*
 * Copyright 1995, 1996, 1997 by Jun Yei.       All Rights Reserved
 *
 * Permission to retain, use, modify, copy, and distribute ChinesePower
 * in source or binary and its documentation (hereafter, the Software)
 * for noncommercial purpose is hereby granted to you without a fee,
 * provided that this entire copyright and permission notice appear in
 * all such copies, that no charge be associated with such copies,
 * that distribution of derivative works (including value-added
 * distributions such as with additional input dictionaries or fonts)
 * include clarification that such added or derived parts are not from
 * the original Software, and that the names of the author(s) not be
 * used to endorse or promote such works. The author(s) of the software
 * makes no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * THE AUTHOR(S) OF THE SOFTWARE DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 * IN NO EVENT SHALL THE AUTHOR(S) OF THE SOFTWARE BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Jun Yei, Ottawa, ON, Canada
 */

/*
   The name of structure and its member is according to Microsoft's
   TTF font reference's manual,so you can learn the meaning of variables
   by reading it.
*/
#ifndef __TTF_H
#include "ttf.h"
#endif

typedef struct {
    BYTE valid;
    SHORT numberOfContours;
    FWord xMin,yMin,xMax,yMax;
    USHORT *endPtsOfContours;
    USHORT instructionLength;
    BYTE *instruction;
    BYTE *flags;
    LONG *xCoordinates;
    LONG *yCoordinates;
} OUTLINE;

#define FLAG_ON_CURVE                   1
#define FLAG_X_SHORT_VECTOR             2
#define FLAG_Y_SHORT_VECTOR             4
#define FLAG_REPEAT                     8
#define FLAG_X_SAME                    0x10
#define FLAG_Y_SAME                    0x20
 
typedef struct {
  Fixed fontRevision;
  USHORT flags;
  USHORT unitsPerEm;
  FWord xMin,yMin,xMax,yMax;
  USHORT macStyle;
  USHORT lowestRecPPEM;
  SHORT fontDirectionHint;
  SHORT indexToLocFormat;
  SHORT glyphDataFormat;
} HEAD;

typedef struct {
  uFWord advanceWidth;
  FWord lsb;
} HorMetric;

typedef struct {
  FWord ascender;
  FWord decender;
  FWord lineGap;
  uFWord advanceWidthMax;
  FWord minLeftSideBearing;
  FWord minRightSideBearing;
  FWord xMaxExtent;
  SHORT caretSlopeRise;
  SHORT caretSlopeRun;
  SHORT metricDataFormat;
  USHORT numberOfHMetrics;
  HorMetric *longHorMetrics;
  FWord *leftSideBearing;
} HORIZON;

typedef struct {
  USHORT numglyphs;
  USHORT maxPoints;
  USHORT maxContours;
  USHORT maxCompositePoints;
  USHORT maxCompositeContours;
  USHORT maxZones;
  USHORT maxTwilightPoints;
  USHORT maxStorage;
  USHORT maxFunctionDefs;
  USHORT maxInstructionDefs;
  USHORT maxStackElements;
  USHORT maxSizeOfInstructions;
  USHORT maxComponentElements;
  USHORT maxComponentDepth;
} PROFILE;

typedef struct {
  BYTE glyphIdArray[256];
} CMAP0;

typedef struct {
  USHORT subHeaderKeys[256];
  USHORT *data,*glyphIdArray;
} CMAP2;

typedef struct {
USHORT segCount;
USHORT *endCount;
USHORT *startCount;
SHORT *idDelta;
USHORT *idRangeOffset;
USHORT *glyphIdArray;
} CMAP4;

typedef struct {
USHORT firstCode;
USHORT entryCount;
USHORT *glyphIdArray;
} CMAP6;

typedef struct {
  USHORT format;
  USHORT PlateformID,EncodingID;
  union {
    CMAP0 *cmap0;
    CMAP2 *cmap2;
    CMAP4 *cmap4;
    CMAP6 *cmap6;
  } map;
} CMAP;

typedef struct {
    Fixed version;
    char *name;
    int res,EM;
    FILE *fp;
    /* curmap is the active CMAP encoding */
    CMAP *cmap,*curmap;
    USHORT numberOfEncoding;
    ULONG glyfoff;
    OUTLINE *glyf;
    HEAD *head;
    HORIZON *horz;
    ULONG *indexToLocation;
    PROFILE *profile;
    USHORT *cvt;
    BYTE *fpgm,*prep;
    USHORT cvtlen,fpgmlen,preplen;
} TTFFont;

#endif

