/*
 * Copyright 1995, 1996, 1997 by Jun Yei.       All Rights Reserved
 *
 * Permission to retain, use, modify, copy, and distribute ChinesePower
 * in source or binary and its documentation (hereafter, the Software)
 * for noncommercial purpose is hereby granted to you without a fee,
 * provided that this entire copyright and permission notice appear in
 * all such copies, that no charge be associated with such copies,
 * that distribution of derivative works (including value-added
 * distributions such as with additional input dictionaries or fonts)
 * include clarification that such added or derived parts are not from
 * the original Software, and that the names of the author(s) not be
 * used to endorse or promote such works. The author(s) of the software
 * makes no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * THE AUTHOR(S) OF THE SOFTWARE DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 * IN NO EVENT SHALL THE AUTHOR(S) OF THE SOFTWARE BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Jun Yei, Ottawa, ON, Canada
 */
#include "chpower.h"

/* $Id: unicode.c,v 2.0 1995/01/29 08:10:04 ygz Exp $ */

void U2fileRead(fp, tp)
FILE *fp;
unsigned short *tp;
{
        int sample;
 
        while((sample = getc(fp)) != EOF)
        {
		*tp = sample << 8;

		sample = getc(fp);
		if(sample == EOF) break;

		*tp |= sample;
 
		tp++;
        }
	*tp = 0;
}

void U2fileWrite(fp, tp)
FILE *fp;
unsigned short *tp;
{
	unsigned short tmp;
 
	while(*tp != 0)
	{
		SKIPIDS(tp);
		tmp = *tp >> 8;
		fputc((unsigned char) tmp, fp);
		fputc((unsigned char) *tp, fp);
		tp++;
	}
}
 
/*
 * The UTF-FSS (aka UTF-2) encoding of UCS, as described in the following
 * quote from Ken Thompson's utf-fss.c:
 *
 * Bits  Hex Min  Hex Max  Byte Sequence in Binary
 *   7  00000000 0000007f 0vvvvvvv
 *  11  00000080 000007FF 110vvvvv 10vvvvvv
 *  16  00000800 0000FFFF 1110vvvv 10vvvvvv 10vvvvvv
 *  21  00010000 001FFFFF 11110vvv 10vvvvvv 10vvvvvv 10vvvvvv
 *  26  00200000 03FFFFFF 111110vv 10vvvvvv 10vvvvvv 10vvvvvv 10vvvvvv
 *  31  04000000 7FFFFFFF 1111110v 10vvvvvv 10vvvvvv 10vvvvvv 10vvvvvv 10vvvvvv
 *
 * The UCS value is just the concatenation of the v bits in the multibyte
 * encoding.  When there are multiple ways to encode a value, for example
 * UCS 0, only the shortest encoding is legal.
 */
 
/* This implementation does not enforce the last restriction on input */
 
/*** UTF8 => UNICODE ***/

void U8fileRead(fp, tp)
unsigned short *tp;
FILE *fp;
{
	int sample;
	int extras;
	unsigned int bit;
	
	while((sample = getc(fp)) != EOF)
	{
		*tp = sample;

		if ((sample & 0x80) == 0)
		{
			/* ASCII character */
		}
		else if((sample & 0xc0) == 0x80)
		{
			/* unexpected tail character */
		} 
		else 
		{
		        /* how many extra bytes? */
		        for (bit = 0x40, extras = 0; (sample & bit) != 0; bit >>= 1, extras++);

			if (extras <= 5) 
			{
				/* put all the bits together */

				*tp = sample & (bit-1);
				while (extras-- > 0)
				{
					if ((sample = getc(fp)) != EOF)
					{
						if ((sample & 0xc0) == 0x80)
						{
							*tp = (*tp<<6) | (sample&0x3f);
						}
						else
						{
							ungetc(sample, fp);
							break;
						}
					}
					else break;
				}
			}
		}
		tp++;
	}
	*tp = 0;
}

/*** UNICODE => UTF8 ***/

void U8fileWrite(fp, tp)
unsigned short *tp;
FILE *fp;
{
	int extras;
	unsigned short bit;
	
	while (*tp != 0)
	{
		SKIPIDS(tp);
		if((*tp & 0xff80) == 0)
		{	/* ASCII */
			fputc((unsigned char) *tp, fp);
		}
		else
		{
			/* how many extra bytes are required? */
       			for(extras = 1, bit = *tp >> 11; bit != 0; bit >>= 5, extras++);

                        /* put header byte */                                                                                         
			bit = 0x1f80 >> extras;
			bit |= *tp >> (extras*6);

			fputc(bit, fp);
	
			/* put tail bytes */
			while (extras != 0)
			{
				extras--;
				bit = *tp >> (extras*6);
				bit &= 0x3f;
				bit |= 0x80;

				fputc(bit, fp);
			}
		}
		tp++;
	}
}

/* ASCII subsets */
static  const   unsigned char   base64[] =
        "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";

#define CHARSIZE 16
#define SEGSIZE  6

/* character classes */
#define BASE64  0x01
#define SAFE    0x02
#define OPTIONAL 0x04
#define SPACE   0x08

static  char char_type[] =
{
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x8, 0x8, 0x0, 0x0, 0x8, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x8, 0x4, 0x4, 0x4, 0x4, 0x4, 0x4, 0x2, 0x2, 0x2, 0x4, 0x1, 0x2, 0x2, 0x2, 0x1, 
0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x2, 0x4, 0x4, 0x4, 0x4, 0x2, 
0x4, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 
0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x4, 0x0, 0x4, 0x4, 0x4, 
0x4, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 
0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x4, 0x4, 0x4, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0
};

static  char inv_base64[] =
{
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x3e, 0x0, 0x0, 0x0, 0x3f, 
0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3a, 0x3b, 0x3c, 0x3d, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x1, 0x2, 0x3, 0x4, 0x5, 0x6, 0x7, 0x8, 0x9, 0xa, 0xb, 0xc, 0xd, 0xe, 
0xf, 0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x1a, 0x1b, 0x1c, 0x1d, 0x1e, 0x1f, 0x20, 0x21, 0x22, 0x23, 0x24, 0x25, 0x26, 0x27, 0x28, 
0x29, 0x2a, 0x2b, 0x2c, 0x2d, 0x2e, 0x2f, 0x30, 0x31, 0x32, 0x33, 0x0, 0x0, 0x0, 0x0, 0x0
};

void U7fileRead(fp, tp)
unsigned short *tp;
FILE *fp;
{
        int c;
        int nbits = 0;
        unsigned long bit_buffer = 0;
	int in_base64 = False;

        while((c = getc(fp)) != EOF)
	{
		c &= 0x7f;

                if (! in_base64)
		{
                        if (c != '+')
			{
				*tp++ = c;
				continue;
			} else if ((c = getc(fp)) != EOF)
			{
				c &= 0x7f;

                        	if (c == '-') 
				{
					*tp++ = '+';
					continue;
				}
				else
				{
                        		in_base64 = True;
                        		bit_buffer = 0;
					nbits = 0;
				}
			}
                }

                /* now we're in Base64 mode */
		while (char_type[c]&BASE64)
		{
                       	bit_buffer <<= SEGSIZE;
               		bit_buffer |= inv_base64[c];
                        nbits += SEGSIZE;
                        if (nbits >= CHARSIZE)
			{
                                nbits -= CHARSIZE;
                      		*tp++ = (bit_buffer >> nbits)&0xffffL;
                       	}
			if ((c = getc(fp)) == EOF) break;

			c &= 0x7f;

                }

               	in_base64 = False;
       		if (c != '-') *tp++ = c;
        }

	*tp = 0;
}

/*
 * Sloppiness:
 * - uses same bit_buffer as utf7_getc(): change if both needed at once.
 * - assumes last character output is safe (e.g. newline) so no flush
 *   is needed.
 */

void U7fileWrite(fp, tp)
unsigned short *tp;
FILE *fp;
{
        int nbits = 0;
        unsigned long bit_buffer = 0;
	int in_base64 = False;

        while(*tp != 0)
        {
		SKIPIDS(tp);
                if (*tp <= 0x7f && (char_type[*tp] & (BASE64|SAFE|SPACE)))
                {
                        if (in_base64)
                        {
                                if (nbits > 0)
                                        fputc(base64[(bit_buffer<<(6-nbits))&0x3f], fp);
                                if ((char_type[*tp] & BASE64) || *tp == '-')
                                        fputc('-', fp);
                                in_base64 = False;
                        }
                        fputc(*tp, fp);
                        if (*tp == '+') fputc('-', fp);
                }
                else
                {
                        if (! in_base64)
                        {
                                fputc('+', fp);
                                in_base64 = True;
				bit_buffer = 0;
                                nbits = 0;
                        }
                        bit_buffer <<= CHARSIZE;
                        bit_buffer |= *tp;
                        nbits += CHARSIZE;

                        while (nbits >= SEGSIZE)
                        {
                                nbits -= SEGSIZE;
                                fputc(base64[(bit_buffer >> nbits)&0x3f], fp);
                        }
                }
                tp++;
        }
}


