/*
 * Copyright 1995, 1996, 1997 by Jun Yei.       All Rights Reserved
 *
 * Permission to retain, use, modify, copy, and distribute ChinesePower
 * in source or binary and its documentation (hereafter, the Software)
 * for noncommercial purpose is hereby granted to you without a fee,
 * provided that this entire copyright and permission notice appear in
 * all such copies, that no charge be associated with such copies,
 * that distribution of derivative works (including value-added
 * distributions such as with additional input dictionaries or fonts)
 * include clarification that such added or derived parts are not from
 * the original Software, and that the names of the author(s) not be
 * used to endorse or promote such works. The author(s) of the software
 * makes no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * THE AUTHOR(S) OF THE SOFTWARE DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 * IN NO EVENT SHALL THE AUTHOR(S) OF THE SOFTWARE BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Jun Yei, Ottawa, ON, Canada
 */

#include "chpower.h"

void HzDataZoom(hzdata, zoom)
HZdata *hzdata;
int zoom;
{
	int i;
	
	hzdata->hz_swidth = hzdata->orighz_swidth * zoom / 100;
	hzdata->hz_sheight = hzdata->orighz_sheight * zoom / 100;
	
	hzdata->wcharpointsize =  hzdata->hz_swidth / hzdata->hz_width;
	i = hzdata->hz_swidth % hzdata->hz_width;
	hzdata->winsertpoint = (float)i / (float)hzdata->hz_width;
	
	hzdata->ewcharpointsize = (hzdata->hz_swidth / 2) / ENGLISH_FONT_WIDTH;
	i = (hzdata->hz_swidth / 2) % ENGLISH_FONT_WIDTH;
	hzdata->ewinsertpoint = (float)i / (float)ENGLISH_FONT_WIDTH;
	
	hzdata->hcharpointsize =  hzdata->hz_sheight / hzdata->hz_height;
	i = hzdata->hz_sheight % hzdata->hz_height;
	hzdata->hinsertpoint = (float)i / (float)hzdata->hz_height;

	hzdata->ehcharpointsize = hzdata->hz_sheight / ENGLISH_FONT_HEIGHT;
	i = hzdata->hz_sheight % ENGLISH_FONT_HEIGHT;
	hzdata->ehinsertpoint = (float)i / (float)ENGLISH_FONT_HEIGHT;
}

int ZoomCount(viewport, zoomvalue)
VIewportdata *viewport;
int zoomvalue;
{
	if(zoomvalue < 0 && viewport->zoom <= LEAST_ZOOM) return 0;
	if(zoomvalue > 0 && viewport->zoom >= MOST_ZOOM) return 0;
	viewport->oldzoom = viewport->zoom;
	viewport->zoom += zoomvalue;
	if(viewport->zoom < LEAST_ZOOM) viewport->zoom = LEAST_ZOOM;
	if(viewport->zoom > MOST_ZOOM) viewport->zoom = MOST_ZOOM;
	return 1;
}

void ZoomOpr(viewport, ShowZoom)
VIewportdata *viewport;
int ShowZoom;
{
	register int n;
	Arg args[5];
	PAgedata *pagedata;
	char num[10];

	viewport->viewp.ViewPortHeight = viewport->origpage.ViewPortHeight * 100 / viewport->zoom;
	viewport->viewp.ViewPortWidth = viewport->origpage.ViewPortWidth * 100 / viewport->zoom;  
	
	HzDataZoom(&viewport->hzdata, viewport->zoom);
	HzDataZoom(&viewport->pndata.hzdata, viewport->zoom);
	HzDataZoom(&viewport->titler.hzdata, viewport->zoom);
	HzDataZoom(&viewport->footer.hzdata, viewport->zoom);

	for(pagedata = viewport->page.PageRoot; pagedata != NULL; pagedata = pagedata->Next)
	{
		HzDataZoom(&pagedata->hzdata, viewport->zoom);
	}

	if(viewport->page.PageWidth - viewport->viewp.ViewPortWidth < 0) viewport->viewp.ViewPort_x = 0;

	if(ShowZoom)
	{
		viewport->draw.DrawScr_x = 0;
		viewport->draw.DrawPos_x = 0;
		viewport->draw.DrawScr_y = 0;
		viewport->draw.DrawPos_y = 0;

		sprintf(num, "%d%%\0", viewport->zoom);
		n = 0;
		XtSetArg(args[n], XmNlabelString, XmStringCreate(num, XmSTRING_DEFAULT_CHARSET)); n++;
		XtSetValues(viewport->resource.ZoomBtn, args, n);

	}
}

void ZoomFix(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	VIewportdata *viewport = (VIewportdata *)client_data;
	
	ZoomOpr(viewport, 1);
	MyDrawPage(viewport, TRUEDRAW);
}

void ZoomOutCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	VIewportdata *viewport = (VIewportdata *)client_data;
	
	if(ZoomCount(viewport, ZOOM_UNIT))
	{
		ZoomOpr(viewport, 1);
		MyDrawPage(viewport, TRUEDRAW);
		VerScroBarExpose(viewport);
		HorScroBarExpose(viewport);
	}
}

void ZoomInCB(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	VIewportdata *viewport = (VIewportdata *)client_data;
	
	if(ZoomCount(viewport, -ZOOM_UNIT))
	{
		ZoomOpr(viewport, 1);
		MyDrawPage(viewport, TRUEDRAW);
		VerScroBarExpose(viewport);
		HorScroBarExpose(viewport);
	}
}

void ZoomBtnZoom(viewport, zoomsize)
VIewportdata *viewport;
int zoomsize;
{
	register int n;
        Arg args[3];
	char num[10];

	switch(zoomsize)
	{
		case 0:
		case 1:
		case 2:

		WindowFitPage(viewport, zoomsize, False);
		break;
			
		case 3:
		
		PageFitWindow(viewport);	
		break;

		default:
		
		viewport->zoom = zoomsize;
	}
	
	if(zoomsize >= 3)
	{
		ZoomOpr(viewport, 1);
		MyDrawPage(viewport, TRUEDRAW);
		VerScroBarExpose(viewport);
		HorScroBarExpose(viewport);
	}
	else
	{
		MyDrawPage(viewport, TRUEDRAW);
		sprintf(num, "%d%%\0", viewport->zoom);
		n = 0;
		XtSetArg(args[n], XmNlabelString, XmStringCreate(num, XmSTRING_DEFAULT_CHARSET)); n++;
		XtSetValues(viewport->resource.ZoomBtn, args, n);
	}

}

void WindowFitPage(viewport, option, init)
VIewportdata *viewport;
int option, init;
{
        int width, height;
        register int n;
        Arg args[3];
	
	if(viewport->origpage.Portrait)
	{
		width = PageData[viewport->origpage.PageSetId][0];
		height = PageData[viewport->origpage.PageSetId][1];
	}
	else
	{
		height = PageData[viewport->origpage.PageSetId][0];
		width = PageData[viewport->origpage.PageSetId][1];
	}

	if(option == FIT_TO_MARGIN_WIN)
	{
		if(viewport->horOrder)
		{
			width -= 2*(viewport->page.PageMarginWidth / 100);
			width += 16 * viewport->zoom / 100;
			viewport->viewp.ViewPort_x = viewport->page.PageMarginWidth - 900;
		}
		else
		{
			height -= 2*(viewport->page.PageMarginHeight / 100);
			height += 16 * viewport->zoom / 100;
			viewport->viewp.ViewPort_y = viewport->page.PageMarginHeight - 900;
		}
	}

	width = XmConvertUnits(viewport->resource.WorkArea, XmHORIZONTAL, Xm100TH_POINTS, width * viewport->zoom, XmPIXELS) + 30;
	height = XmConvertUnits(viewport->resource.WorkArea, XmVERTICAL, Xm100TH_POINTS, height * viewport->zoom, XmPIXELS) + 45;

	n = 0;

	if(option == FIT_TO_PAGE_WIN || !viewport->horOrder) 
	{ 
		XtSetArg(args[n], XmNheight, height); n++;
	}
	else if(init)
	{
		XtSetArg(args[n], XmNheight, GlobalDefaults.win_height); n++;
	} 

	if(option == FIT_TO_PAGE_WIN || viewport->horOrder)
	{
		XtSetArg(args[n], XmNwidth, width); n++;
	}
	else if(init)
	{
		XtSetArg(args[n], XmNwidth, GlobalDefaults.win_width); n++;
	}	

	XtSetValues(viewport->resource.PopUpShell, args, n);
}

void PageFitWindow(viewport)
VIewportdata *viewport;
{
	int width, height;

	if(viewport->origpage.Portrait)
	{
		width = PageData[viewport->origpage.PageSetId][0];
		height = PageData[viewport->origpage.PageSetId][1];
	}
	else
	{
		height = PageData[viewport->origpage.PageSetId][0];
		width = PageData[viewport->origpage.PageSetId][1];
	}

	height = viewport->origpage.ViewPortHeight / height;
	width = viewport->origpage.ViewPortWidth / width;
		
	if(height > width) viewport->zoom = width;
	else	viewport->zoom = height;
}

